#****************************************************************************
#**
#**  File     :  /lua/ScenarioFramework.lua
#**  Author(s): John Comes, Drew Staltman
#**
#**  Summary  : Functions for use in the Operations.
#**
#**  Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
#****************************************************************************
local TriggerFile = import('scenariotriggers.lua')
local ScenarioUtils = import('/lua/ScenarioUtilities.lua')
local UnitUpgradeTemplates = import('/lua/modules/upgradeTemplates.lua').UnitUpgradeTemplates
local VizMarker = import('/lua/modules/defaultvizmarkers.lua').VizMarker
local SimCamera = import('/lua/modules/SimCamera.lua').SimCamera
local Cinematics = import('/lua/modules/cinematics.lua')

Objectives = import('/lua/modules/SimObjectives.lua')

# Cause the game to exit immediately
function ExitGame()
    Sync.RequestingExit = true
end

# Call to end an operation
#   string _opKey - unique identifier for the current operation (ie SCCA_E01 would be a good key)
#   bool _success - instructs UI which dialog to show
#   int _difficulty - 1,2,3 currently supported
#   bool _allPrimary - true if all primary objectives completed, otherwise, false
#   bool _allSecondary - true if all secondary objectives completed, otherwise, false
#   bool _allBonus - true if all bonus objectives completed, otherwise, false
function EndOperation(_opKey, _success, _difficulty, _allPrimary, _allSecondary, _allBonus)
    Sync.OperationComplete = {
        opKey = _opKey,
        success = _success,
        difficulty = _difficulty,
        allPrimary = _allPrimary,
        allSecondary = _allSecondary,
        allBonus = _allBonus,
    }
end

# Call to end an operation where the data is already provided in table form (just a wrapper for sync
function EndOperationT(opData)
    Sync.OperationComplete = opData
end

### Single Area Trigger Creation
# This will create an area trigger around <rectangle>.  It will fire when <categoy> is met of <aiBrain>.
# onceOnly means it will not continue to run after the first time it fires.
# invert meants it will fire when units are NOT in the area.  Useful for testing if someone has defeated a base.
# number refers to the number of units it will take to fire.  If not inverted.  It will fire when that many are in the area# If inverted, it will fire when less than that many are in the area
function CreateAreaTrigger( callbackFunction, rectangle, category, onceOnly, invert, aiBrain, number, requireBuilt)
    return TriggerFile.CreateAreaTrigger(callbackFunction, rectangle, category, onceOnly, invert, aiBrain, number, requireBuilt)
end

### Table of Areas Trigger Creations
# same as above except you can supply the function with a table of Rects.
# If you have an odd shaped area for an area trigger
function CreateMultipleAreaTrigger(callbackFunction, rectangleTable, category, onceOnly, invert, aiBrain, number, requireBuilt)
    return TriggerFile.CreateMultipleAreaTrigger(callbackFunction, rectangleTable, category, onceOnly, invert, aiBrain, number, requireBuilt)
end

### Single Line timer Trigger creation
# Fire the <cb> function after <seconds> number of seconds.
# you can have the function repeant <repeatNum> times which will fire every <seconds>
# until <repeatNum> is met
function CreateTimerTrigger( cb, seconds, displayBool)
    return TriggerFile.CreateTimerTrigger(cb, seconds, displayBool)
end

function ResetUITimer()
    if not Sync.ObjectiveTimer then
        Sync.ObjectiveTimer = 0
    end
end

### Single Line unit damaged trigger creation
# When <unit> is damaged it will call the <callbackFunction> provided
# If <percent> provided, will check if damaged percent EXCEEDS number provided before callback
# function repeats up to repeatNum ... or once if not declared
function CreateUnitDamagedTrigger( callbackFunction, unit, amount, repeatNum )
    unit:AddOnDamagedCallback(callbackFunction, amount, repeatNum)
end

function CreateUnitPercentageBuiltTrigger(callbackFunction, aiBrain, category, percent)
    aiBrain:AddUnitBuiltPercentageCallback(callbackFunction, category, percent)
end

### Single Line unit death trigger creation
# When <unit> dies it will call the <cb> function provided
function CreateUnitDeathTrigger( cb, unit )
    unit:AddOnKilledCallback(cb)
end

### Single Line unit death trigger creation
# When <unit> is killed, reclaimed, or captured it will call the <cb> function provided
function CreateUnitDestroyedTrigger( cb, unit )
    unit:AddOnReclaimedCallback(cb)
    unit:AddOnCapturedCallback(cb)
    unit:AddOnKilledCallback(cb)
end

### Single Line Unit built trigger
# Tests when <unit> builds a unit of type <category> calls <cb>
function CreateUnitBuiltTrigger( cb, unit, category )
    unit:AddOnUnitBuiltCallback(cb, category)
end

### Single line unit captured trigger creation
# When <unit> is captured cbOldUnit is called passing the old unit BEFORE it has switched armies,
# cbNewUnit is called passing in the unit AFTER it has switched armies
function CreateUnitCapturedTrigger( cbOldUnit, cbNewUnit, unit )
    unit:AddOnCapturedCallback(cbOldUnit, cbNewUnit)
end

### Single line unit start being captured trigger
# when <unit> begins to be captured function is called
function CreateUnitStartBeingCapturedTrigger( cb, unit )
    unit:AddOnStartBeingCapturedCallback(cb)
end

### Single line unit stop being captured trigger
# when <unit> stops being captured, the function is called
function CreateUnitStopBeingCapturedTrigger( cb, unit )
    unit:AddOnStopBeingCapturedCallback(cb)
end

### Single line unit failed being captured trigger
# when capture of <unit> fails, the function is called
function CreateUnitFailedBeingCapturedTrigger( cb, unit )
    unit:AddOnFailedBeingCapturedCallback(cb)
end

### Single line unit started capturing trigger creation
# When <unit> starts capturing the <cb> function provided is called
function CreateUnitStartCaptureTrigger( cb, unit )
    unit:AddOnStartCaptureCallback(cb)
end

### Single line unit finished capturing trigger creation
# When <unit> finishes capturing the <cb> function provided is called
function CreateUnitStopCaptureTrigger( cb, unit )
    unit:AddOnStopCaptureCallback(cb)
end

### Single line failed capturing trigger creation
# When <unit> fails to capture a unit, the <cb> function is called
function CreateUnitFailedCaptureTrigger( cb, unit )
    unit:AddOnFailedCaptureCallback(cb)
end

### Single line unit has been reclaimed trigger creation
# When <unit> has been reclaimed the <cb> function provided is called
function CreateUnitReclaimedTrigger( cb, unit )
    unit:AddOnReclaimedCallback(cb)
end

### Single line unit started reclaiming trigger creation
# When <unit> starts reclaiming the <cb> function provided is called
function CreateUnitStartReclaimTrigger( cb, unit )
    unit:AddOnStartReclaimCallback(cb)
end

### Single line unit finished reclaiming trigger creation
# When <unit> finishes reclaiming the <cb> function provided is called
function CreateUnitStopReclaimTrigger( cb, unit )
    unit:AddOnStopReclaimCallback(cb)
end

### Single line unit veterancy trigger creation
# When <unit> achieves veterancy, <cb> is called with parameters of the unit then level achieved
function CreateUnitVeterancyTrigger(cb, unit)
    unit:AddOnVeteranCallback(cb)
end

### Single line Group Death Trigger creation
# When all units in <group> are destoyed, <cb> function will be called
function CreateGroupDeathTrigger( cb, group )
   return TriggerFile.CreateGroupDeathTrigger(cb, group)
end

### Single line Platoon Death Trigger creation
# When all units in <platoon> are destroyed, <cb> function will be called
function CreatePlatoonDeathTrigger( cb, platoon )
    platoon:AddDestroyCallback(cb)
end

### Single line Sub Group Death Trigger creation
# When <num> <cat> units in <group> are destroyed, <cb> function will be called
function CreateSubGroupDeathTrigger(cb, group, num)
    return TriggerFile.CreateSubGroupDeathTrigger(cb, group, num)
end

### Checks if units of Cat are within the provided rectangle
# Checks the <Rectangle> to see if any units of <Cat> category are in it
function UnitsInAreaCheck( Cat, Rectangle )
    if type(Rectangle) == 'string' then
        Rectangle = ScenarioUtils.AreaToRect(Rectangle)
    end
    local entities = GetUnitsInRect( Rectangle )
    if not entities then
        return false
    end
    for k,v in entities do
        if EntityCategoryContains( Cat, v ) then
            return true
        end
    end
    return false
end

# Retruns the number of <cat> units in <area> belonging to <brain>
function NumCatUnitsInArea(cat, area, brain)
    if type(area) == 'string' then
        area = ScenarioUtils.AreaToRect(area)
    end
    local entities = GetEntitiesInRect(area)
    local result = 0
    if entities then
        local filteredList = EntityCategoryFilterDown(cat, entities)

        for k, v in filteredList do
            if(v:GetAIBrain() == brain) then
                result = result + 1
            end
        end
    end

    return result
end

# Retruns the units in <area> of <cat> belonging to <brain>
function GetCatUnitsInArea(cat, area, brain)
    if type(area) == 'string' then
        area = ScenarioUtils.AreaToRect(area)
    end
    local entities = GetEntitiesInRect(area)
    local result = {}
    if entities then
        local filteredList = EntityCategoryFilterDown(cat, entities)

        for k, v in filteredList do
            if(v:GetAIBrain() == brain) then
                table.insert(result, v)
            end
        end
    end

    return result
end

### Destroys a group
# Goes through every unit in <group> and destroys them without explosions
function DestroyGroup( group )
    for k,v in group do
        v:Destroy()
    end
end

# Checks if <unitOne> and <unitTwo> are less than <distance> from each other
# if true calls <callbackFunction>
function CreateUnitDistanceTrigger( callbackFunction, unitOne, unitTwo, distance )
    TriggerFile.CreateUnitDistanceTrigger( callbackFunction, unitOne, unitTwo, distance )
end



# Stat trigger creation
# === triggerTable spec === #
# {
#     { StatType = string, -- Examples: Units_Active, Units_Killed, Enemies_Killed, Economy_Trend_Mass, Economy_TotalConsumed_Energy
#       CompareType = string, -- GreaterThan, GreatherThanOrEqual, LessThan, LessThanOrEqual
#       Value = integer,
#       Category = category, -- Only used with "Units" triggers
#     },
#  }

# === COMPLETE LIST OF STAT TYPES === #
#     "Units_Active",
#     "Units_Killed",
#     "Units_History",
#     "Enemies_Killed",
#     "Economy_TotalProduced_Energy",
#     "Economy_TotalConsumed_Energy",
#     "Economy_Income_Energy",
#     "Economy_Output_Energy",
#     "Economy_Stored_Energy",
#     "Economy_Reclaimed_Energy",
#     "Economy_MaxStorage_Energy",
#     "Economy_PeakStorage_Energy",
#     "Economy_TotalProduced_Mass",
#     "Economy_TotalConsumed_Mass",
#     "Economy_Income_Mass",
#     "Economy_Output_Mass",
#     "Economy_Stored_Mass",
#     "Economy_Reclaimed_Mass",
#     "Economy_MaxStorage_Mass",
#     "Economy_PeakStorage_Mass",

function CreateArmyStatTrigger(callbackFunction, aiBrain, name, triggerTable)
    TriggerFile.CreateArmyStatTrigger(callbackFunction, aiBrain, name, triggerTable)
end



# Fires when the threat level of <position> of size <rings> is related to <value>
# if <greater> is true it will fire if the threat is greater than <value>
function CreateThreatTriggerAroundPosition(callbackFunction, aiBrain, posVector, rings, onceOnly, value, greater)
    TriggerFile.CreateThreatTriggerAroundPosition(callbackFunction, aiBrain, posVector, rings, onceOnly, value, greater)
end

# Fires when the threat level of <unit> of size <rings> is related to <value>
function CreateThreatTriggerAroundUnit(callbackFunction, aiBrain, unit, rings, onceOnly, value, greater)
    TriggerFile.CreateThreatTriggerAroundUnit(callbackFunction, aiBrain, unit, rings, onceOnly, value, greater)
end

#Type = 'LOSNow'/'Radar'/'Sonar'/'Omni',
#        Blip = blip handle or false if you don't care,
#Category = category of unit to trigger off of
#OnceOnly = run it once
#        Value = true/false, true = when you get it, false = when you first don't have it
# <aiBrain> refers to the intelligence you are monitoring.
# <targetAIBrain> requires that the intelligence fires on seeing a specific brain's units
function CreateArmyIntelTrigger(callbackFunction, aiBrain, reconType, blip, value, category, onceOnly, targetAIBrain)
    TriggerFile.CreateArmyIntelTrigger(callbackFunction, aiBrain, reconType, blip, value, category, onceOnly, targetAIBrain)
end

function CreateArmyUnitCategoryVeterancyTrigger(callbackFunction, aiBrain, category, level)
    TriggerFile.CreateArmyUnitCategoryVeterancyTrigger(callbackFunction, aiBrain, category, level)
end

# Fires when <unit> and <marker> are less than or equal to <distance> apart
function CreateUnitToMarkerDistanceTrigger( callbackFunction, unit, marker, distance )
    TriggerFile.CreateUnitToPositionDistanceTrigger( callbackFunction, unit, marker, distance )
end

# Function that fires when <unit> is near any unit of type <category> belonging to <brain> withing <distance>
function CreateUnitNearTypeTrigger( callbackFunction, unit, brain, category, distance )
    return TriggerFile.CreateUnitNearTypeTrigger( callbackFunction, unit, brain, category, distance )
end

# tells platoon to move along table of coordinates
function PlatoonMoveRoute( platoon, route, squad )
    for k,v in route do
        if type(v) == 'string' then
            if squad then
                platoon:MoveToLocation(ScenarioUtils.MarkerToPosition(v), false, squad)
            else
                platoon:MoveToLocation(ScenarioUtils.MarkerToPosition(v), false)
            end
        else
            if squad then
                platoon:MoveToLocation(v, false, squad)
            else
                platoon:MoveToLocation(v, false)
            end
        end
    end
end

# commands platoon to patrol a route
function PlatoonPatrolRoute( platoon, route, squad )
    for k,v in route do
        if type(v) == 'string' then
            if squad then
                platoon:Patrol(ScenarioUtils.MarkerToPosition(v), squad)
            else
                platoon:Patrol(ScenarioUtils.MarkerToPosition(v))
            end
        else
            if squad then
                platoon:Patrol(v, squad)
            else
                platoon:Patrol(v)
            end
        end
    end
end

function PlatoonMoveChain( platoon, chain, squad )
    for k,v in ScenarioUtils.ChainToPositions(chain) do
        if squad then
            platoon:MoveToLocation(v, false, squad)
        else
            platoon:MoveToLocation(v, false)
        end
    end
end

# orders a platoon to patrol all teh points in a chain
function PlatoonPatrolChain( platoon, chain, squad )
    for k,v in ScenarioUtils.ChainToPositions(chain) do
        if squad then
            platoon:Patrol(v, squad)
        else
            platoon:Patrol(v)
        end
    end
end

# Orders a platoon to attack move through a chain
function PlatoonAttackChain( platoon, chain, squad )
    for k,v in ScenarioUtils.ChainToPositions(chain) do
        if squad then
            platoon:AggressiveMoveToLocation(v,squad)
        else
            platoon:AggressiveMoveToLocation(v)
        end
    end
end

# orders group to patrol a chain
function GroupPatrolChain( group, chain )
    for k,v in ScenarioUtils.ChainToPositions(chain) do
        IssuePatrol( group, v )
    end
end

# orders group to patrol a route
function GroupPatrolRoute( group, route )
    for k,v in route do
        if type(v) == 'string' then
            IssuePatrol(group, ScenarioUtils.MarkerToPosition(v))
        else
            IssuePatrol(group, v)
        end
    end
end

# orders group to patrol a route in formation
function GroupFormPatrolChain( group, chain, formation )
    for k,v in ScenarioUtils.ChainToPositions(chain) do
        IssueFormPatrol(group, v, formation, 0)
    end
end

# order group to attack a chain
function GroupAttackChain( group, chain )
    for k,v in ScenarioUtils.ChainToPositions(chain) do
        IssueAggressiveMove( group, v )
    end
end

# dialogueTable format
# it's a table of 4 variables - vid, cue, text, and duration
# ex. Hello = {
#   { vid=video, cue=false, bank=false, text='Hello World', duration = 5 },
# }
#
#   - bank = audio bank
#   - cue = audio cue
#   - vid = video cue
#   - text:     text to be displayed on the screen
#   - delay: time before begin next dialogue in table in second
function Dialogue( dialogueTable, callback )
    if callback then
        dialogueTable.Callback = callback
    end
    if ScenarioInfo.DialogueLock == nil then
        ScenarioInfo.DialogueLock = false
        ScenarioInfo.DialogueLockPosition = 0
        ScenarioInfo.DialogueQueue = {}
    end
    table.insert(ScenarioInfo.DialogueQueue, dialogueTable)
    if not ScenarioInfo.DialogueLock then
        ScenarioInfo.DialogueLock = true
        ForkThread( PlayDialogue )
    end
end

### The actual thread used by Dialogue
function PlayDialogue()
    while table.getn(ScenarioInfo.DialogueQueue) > 0 do
        local dTable = table.remove(ScenarioInfo.DialogueQueue, 1)
        for k,v in dTable do
            if v ~= nil then
                #if v.bank and v.cue then
                #    table.insert(Sync.Sounds, {Bank = v.bank, Cue = v.cue} )
                #end
                if not v.vid and v.bank and v.cue then
                    table.insert(Sync.Sounds, {Cue=v.cue, Bank=v.bank} )
                    if not v.delay then
                        WaitSeconds(5)
                    end
                end
                if v.text and not v.vid then
                    if not v.vid then
                        DisplayMissionText( v.text )
                    end
                end
                if v.vid then
                    if v.text then
                        DisplayVideoText( v.text )
                    end
                    if GetMovieDuration('/movies/' .. v.vid) == 0 then
                        Sync.PlayMFDMovie = { '/movies/_placeholder_movie.sfd', v.bank, v.cue, v.faction }
                        WaitSeconds(GetMovieDuration( '/movies/_placeholder_movie.sfd' ) )
                    else
                        Sync.PlayMFDMovie = {'/movies/' .. v.vid, v.bank, v.cue, v.faction}
						if v.faction then
                        	WaitSeconds(GetMovieDuration('/movies/' .. v.vid) + GetMovieDuration('movies/comm_' .. v.faction .. '.sfd')) # add duration of intro faction movie
                        else
                        	WaitSeconds(GetMovieDuration('/movies/' .. v.vid))
                        end
                    end
                end
                if v.delay and v.delay > 0 then
                    WaitSeconds( v.delay )
                end
            end
        end
        if dTable.Callback then
            ForkThread(dTable.Callback)
        end
    end
    ScenarioInfo.DialogueLock = false
end

function PlayUnlockDialogue()
    local factionIndex = ArmyBrains[1]:GetFactionIndex()
    #LOG('Debug: playing unit reveal for faction '..factionIndex)
    if factionIndex == 1 then
        if Random(1,2) == 1 then
            Dialogue( {import('/lua/modules/ui/game/vo_computer.lua').Computer_UEFComputer_UnitRevalation_01370 } )
        else
            Dialogue( {import('/lua/modules/ui/game/vo_computer.lua').Computer_UEFComputer_UnitRevalation_01372 } )
        end
    elseif factionIndex == 2 then
        if Random(1,2) == 1 then
            Dialogue( {import('/lua/modules/ui/game/vo_computer.lua').Computer_AeonComputer_UnitRevalation_01370 } )
        else
            Dialogue( {import('/lua/modules/ui/game/vo_computer.lua').Computer_AeonComputer_UnitRevalation_01372 } )
        end
    elseif factionIndex == 3 then
        if Random(1,2) == 1 then
            Dialogue( {import('/lua/modules/ui/game/vo_computer.lua').Computer_CybranComputer_UnitRevalation_01370 } )
        else
            Dialogue( {import('/lua/modules/ui/game/vo_computer.lua').Computer_CybranComputer_UnitRevalation_01372 } )
        end
    end
end

### Given a head and taunt number, tells the UI to play the relating taunt
function PlayTaunt(head, taunt)
    Sync.MPTaunt = {head, taunt}
end

### Mission Text
function DisplayMissionText(string)
    if(not Sync.MissionText) then
        Sync.MissionText = {}
    end

    table.insert(Sync.MissionText, string)
end

### Video Text
function DisplayVideoText(string)
    if(not Sync.VideoText) then
        Sync.VideoText = {}
    end

    table.insert(Sync.VideoText, string)
end

### Play an NIS
function PlayNIS(pathToMovie)
    if not Sync.NISVideo then
        Sync.NISVideo = pathToMovie
    end
end

function PlayEndGameMovie(faction)
    if not Sync.EndGameMovie then
        Sync.EndGameMovie = faction
    end
end

### Plays an XACT sound if needed - currently all VOs are videos
function PlayVoiceOver( voSound )
    table.insert(Sync.Sounds, voSound)
    local pauseHere = nil
end

### Set enhancement restrictions
### Supply a table of strings of the names of enhancements you do not want the player to build
function RestrictEnhancements( table )
    Sync.EnhanceRestrict = table
end

### returns true if all units in group are dead
function GroupDeathCheck( group )
    for k,v in group do
        if not v:IsDead() then
            return false
        end
    end
    return true
end

#### iterates through objects in list.  if any in list are not true ... returns false
function CheckObjectives( list )
    for k,v in list do
        if not v then
            return false
        end
    end
    return true
end

#### FakeTeleportUnitThread
#### Run teleport effect then delete unit                
function FakeTeleportUnit(unit,killUnit)
    IssueStop({unit})
    IssueClearCommands({unit})
    unit:SetCanBeKilled( false )

    unit:PlayTeleportChargeEffects()
    unit:PlayUnitSound('GateCharge')
    WaitSeconds(2)

    unit:CleanupTeleportChargeEffects()
    unit:PlayTeleportOutEffects()
	unit:PlayUnitSound('GateOut')    
    WaitSeconds(1)

    if killUnit then
        unit:Destroy()
    end
end

#### Upgrades unit - for use with engineers, factories, radar, and other single upgrade path units.
####                 Commander upgrades are too complicated for this
function UpgradeUnit(unit)
    local upgradeBP = unit:GetBlueprint().General.UpgradesTo
    IssueStop({unit})
    IssueClearCommands({unit})
    IssueUpgrade( {unit}, upgradeBP )
end

#### triggers a help text prompt to appear in the UI
#### see /modules/ui/help/helpstrings.lua for a list of valid Help Prompt IDs
function HelpPrompt(show)
    if not Sync.HelpPrompt then
        Sync.HelpPrompt = show
    end
end

#### returns lists of factories by category
#### <point> and <radius> are optional
#### this allows you to know which factories can build and which can't
#### useful if you want to track what factories can do what
function GetFactories( brain, point, radius )
    if type(point) == 'string' then
        point = ScenarioUtils.MarkerToPosition(point)
    end
    local available = {}
    if point then
        available = brain:GetAvailableFactories(point, radius)
    else
        available = brain:GetAvailableFactories()
    end
    local retTable = {
        T1Air = {}, T2Air = {}, T3Air = {},
        T1Land = {}, T2Land = {}, T3Land = {},
        T1Naval = {}, T2Naval = {}, T3Naval = {},
    }
    for k,v in available do
        if not v:IsUnitState('Building') and ( v:GetNumBuildOrders(categories.ALLUNITS) == 0 ) then
            if EntityCategoryContains( categories.TECH1 * categories.AIR, v ) then
                table.insert( retTable.T1Air, v )
            elseif EntityCategoryContains( categories.TECH2 * categories.AIR, v ) then
                table.insert( retTable.T2Air, v )
            elseif EntityCategoryContains( categories.TECH3 * categories.AIR, v ) then
                table.insert( retTable.T3Air, v )
            elseif EntityCategoryContains( categories.TECH1 * categories.LAND, v ) then
                table.insert( retTable.T1Land, v )
            elseif EntityCategoryContains( categories.TECH2 * categories.LAND, v ) then
                table.insert( retTable.T2Land, v )
            elseif EntityCategoryContains( categories.TECH3 * categories.LAND, v ) then
                table.insert( retTable.T3Land, v )
            elseif EntityCategoryContains( categories.TECH1 * categories.NAVAL, v ) then
                table.insert( retTable.T1Naval, v )
            elseif EntityCategoryContains( categories.TECH2 * categories.NAVAL, v ) then
                table.insert( retTable.T2Naval, v )
            elseif EntityCategoryContains( categories.TECH3 * categories.NAVAL, v ) then
                table.insert( retTable.T3Naval, v )
            end
        end
    end
    return retTable
end

### Creates a visible area for <vizArmy> at <vizLocation> of <vizRadius> size.
### If vizLifetime is 0, the entity lasts forever.  Otherwise for <vizLifetime> seconds.
### Function returns an entitiy so you can destroy it later if you want
function CreateVisibleAreaLocation( vizRadius, vizLocation, vizLifetime, vizArmy )
    if type(vizLocation) == 'string' then
        vizLocation = ScenarioUtils.MarkerToPosition(vizLocation)
    end
    local spec = {
        X = vizLocation[1],
        Z = vizLocation[3],
        Radius = vizRadius,
        LifeTime = vizLifetime,
        Army = vizArmy:GetArmyIndex(),
    }
    local vizEntity = VizMarker(spec)
    return vizEntity
end

function CreateVisibleAreaAtUnit( vizRadius, vizUnit, vizLifetime, vizArmy )
    local pos = vizUnit:GetPosition()
    local spec = {
        X = pos[1],
        Z = pos[3],
        Radius = vizRadius,
        LifeTime = vizLifetime,
        Army = vizArmy:GetArmyIndex(),
    }
    local vizEntity = VizMarker(spec)
    return vizEntity
end


### Similar to the above funtion except it taks in an { X, Z } location rather
### Than an { X, Y, Z } position
function CreateVisibleArea( vizRadius, vizX, vizZ, vizLifetime, vizArmy )
    local spec = {
        X = vizX,
        Z = vizZ,
        Radius = vizRadius,
        LifeTime = vizLifetime,
        Army = vizArmy,
    }
    local vizEntity = VizMarker(spec)
    return vizEntity
end

# Sets the playable area for an operation to rect size.
# this function allows you to use ScenarioUtilities function AreaToRect for the rectangle.
function SetPlayableArea( rect, voFlag )
     if (voFlag == nil) then
         voFlag = true 
    end 
    if type(rect) == 'string' then
    	if rect == 'M1Area' then
    		voFlag = false
    	end
        rect = ScenarioUtils.AreaToRect(rect)
    end
    local x0 = rect.x0
    local y0 = rect.y0
    local x1 = rect.x1
    local y1 = rect.y1
    SetPlayableRect( x0, y0, x1, y1 )
    if voFlag then
    	Dialogue( {import('/lua/modules/ui/game/vo_computer.lua').Computer_CybranComputer_MapExpansion_01380 } )
    end
end

### Sets platoon to only be built once
function BuildOnce(platoon)
    local aiBrain = platoon:GetBrain()
    aiBrain:PBMSetPriority(platoon, 0)
end

# TODO: Stop mission scripts from using this function, then remove it.
function AddObjective(Type,Complete,Title,Description,Image,Progress,Target)
    Objectives.AddObjective(Type,Complete,Title,Description,Image,Progress,Target)
end

# TODO: Stop mission scripts from using this function, then remove it.
function UpdateObjective(Title, UpdateField, NewData, objTag)
    Objectives.UpdateObjective(Title,UpdateField,NewData,objTag)
end

### Moves the camera to the specified area in 1 second
function StartCamera(area)
    local cam = SimCamera('WorldCamera')

    cam:ScaleMoveVelocity(0.03)
    cam:MoveTo(area, 1)
    cam:WaitFor()

    cam:Release()
end

### Sets an army color to the Aeon colors given by art
function SetAeonColor( number )
    SetArmyColor( number, 41, 191, 41 )
end

### Sets an army color to the Cybran colors given by art
function SetCybranColor( number )
    SetArmyColor( number, 128, 39, 37 )
end

### Sets an army color to the UEF colors given by art
function SetUEFColor( number )
    SetArmyColor( number, 36, 35, 129 )
end

### Sets an army color to the UEF ally colors given by art
function SetUEFAllyColor( number )
    SetArmyColor( number, 71, 114, 148 )
end

function AMPlatoonCounter( aiBrain, name )
    if not aiBrain.AttackData.AMPlatoonCount[name] then
        aiBrain.AttackData.AMPlatoonCount[name] = 0
    end
    return aiBrain.AttackData.AMPlatoonCount[name]
end

### Automatically attaches <units> to attach points on <transports>
function AttachUnitsToTransports(units, transports)
    local locUnits = {}
    for k,v in units do
        table.insert(locUnits, v)
    end
    local transportBones = {}
    local numTransports = table.getn(transports)
    for k, unit in transports do
        transportBones[k] = {
            Lrg = {},
            Med = {},
            Sml = {},
        }
        for i=1,unit:GetBoneCount() do
            if unit:GetBoneName(i) ~= nil then
                if string.find(unit:GetBoneName(i), 'Attachpoint_Lrg') then
                    table.insert(transportBones[k].Lrg,unit:GetBoneName(i))
                elseif string.find(unit:GetBoneName(i), 'Attachpoint_Med') then
                    table.insert(transportBones[k].Med,unit:GetBoneName(i))
                elseif string.find(unit:GetBoneName(i), 'Attachpoint') then
                    table.insert(transportBones[k].Sml,unit:GetBoneName(i))
                end
            end
        end
    end
    local sortedGroup = {}
    for i = 1, table.getn(locUnits) do
        local highest = 0
        local key, value
        for k, v in locUnits do
            local bp = v:GetBlueprint()
            if not bp.Transport.TransportClass then
                if 1 > highest then
                    highest = 1
                    value = v
                    key = k
                end
            else
                if bp.Transport.TransportClass > highest then
                    highest = bp.Transport.TransportClass
                    value = v
                    key = k
                end
            end
        end
        sortedGroup[i] = value
        table.remove(locUnits, key)
    end
    locUnits = sortedGroup
    for k, v in locUnits do
        if not v:IsUnitState('Attached') then
            # attach locUnits and remove bones when locUnits attached
            local bp = v:GetBlueprint()
            local notInserted = true
            local attachBone = -1
            if v:IsValidBone('AttachPoint', false) then
                attachBone = 'AttachPoint'
            end
            local i = 1
            if bp.Transport.TransportClass == 3 then
                while notInserted and i <= numTransports do
                    if table.getn(transportBones[i].Lrg) > 0 then
                        notInserted = false
                        local bone = table.remove(transportBones[i].Lrg, 1)
                        v:AttachBoneTo(attachBone, transports[i], bone)
                        local bonePos = transports[i]:GetPosition(bone)
                        for j=1,2 do
                            local lowDist = 100
                            local key
                            for kbone, vbone in transportBones[i].Med do
                                local dist = VDist3(transports[i]:GetPosition(vbone), bonePos)
                                if dist < lowDist then
                                    lowDist = dist
                                    key = kbone
                                end
                            end
                            table.remove(transportBones[i].Med, key)
                        end
                        for j=1,4 do
                            local lowDist = 100
                            local key
                            for kbone, vbone in transportBones[i].Sml do
                                local dist = VDist3(transports[i]:GetPosition(vbone), bonePos)
                                if dist < lowDist then
                                    lowDist = dist
                                    key = kbone
                                end
                            end
                            table.remove(transportBones[i].Sml, key)
                        end
                    end
                    i = i + 1
                end
            elseif bp.Transport.TransportClass == 2 then
                while notInserted and i <= numTransports do
                    if table.getn(transportBones[i].Med) > 0 then
                        notInserted = false
                        local bone = table.remove(transportBones[i].Med, 1)
                        v:AttachBoneTo(attachBone, transports[i], bone)
                        local bonePos = transports[i]:GetPosition(bone)
                        for j=1,2 do
                            local lowDist = 100
                            local key
                            for kbone, vbone in transportBones[i].Sml do
                                local dist = VDist3(transports[i]:GetPosition(vbone), bonePos)
                                if dist < lowDist then
                                    lowDist = dist
                                    key = kbone
                                end
                            end
                            table.remove(transportBones[i].Sml, key)
                        end
                    end
                    i = i + 1
                end
            else
                while notInserted and i <= numTransports do
                    if table.getn(transportBones[i].Sml) > 0 then
                        notInserted = false
                        local bone = table.remove(transportBones[i].Sml, 1)
                        v:AttachBoneTo(attachBone, transports[i], bone)
                    end
                    i = i + 1
                end
            end
        end
    end
end

### Take a table of markers, and return the marker with the largest number of target units in a specified radius around it.
### ex: attackLocation = ScenarioFramework.DetermineBestAttackLocation( ArmyBrains[Aeon], ArmyBrains[Player], 'enemy', ScenarioUtils.ChainToPositions( 'Attack_Grid' ), 64 )
function DetermineBestAttackLocation( attackingBrain, targetBrain, relationship, tableOfAttackLocations, pointRadius )
    local highestUnitCountFound = 0
    local targetBrainUnitCount = 0
    local foundUnits = nil
    local attackLocation = nil

    for locationsCount, position in tableOfAttackLocations do
        foundUnits = attackingBrain:GetUnitsAroundPoint( relationship, categories.ALLUNITS, position, pointRadius )

        for throwawayCounter, unit in foundUnits do
            if unit:GetAIBrain() == targetBrain then
                targetBrainUnitCount = targetBrainUnitCount + 1
            end
        end

        if( targetBrainUnitCount > highestUnitCountFound ) or ( attackLocation == nil ) then
            highestUnitCountFound = targetBrainUnitCount
            attackLocation = position
        end

        targetBrainUnitCount = 0
    end

    return attackLocation
end

### Returns a random entry from a table.
### ex: takeoff_Spot = ScenarioFramework.GetRandomEntry( ScenarioUtils.ChainToPositions('Offscreen_Launch_Points'))
function GetRandomEntry( tableOfData )
    return tableOfData[Random( 1, table.getn( tableOfData ))]
end


function KillBaseInArea( brain, area, category )
    local rect = area
    local unitTable = {}
    
    if type(area) == 'table' then 
        for k,subArea in area do
            if type(subArea) == 'string' then
                rect = ScenarioUtils.AreaToRect(subArea)
            else 
                rect = subArea
            end   
            for k2,unit in GetUnitsInRect(rect) do
                table.insert(unitTable, unit)
            end
        end
    else       
        if type(area) == 'string' then
            rect = ScenarioUtils.AreaToRect(area)
        end
        unitTable = GetUnitsInRect(rect)
    end
    if not category then
        category = categories.STRUCTURE + categories.ENGINEER
    end
    local filteredUnits = {}
    if unitTable then
        for num,unit in unitTable do
            if not unit:IsDead() and unit:GetAIBrain() == brain and EntityCategoryContains( category, unit ) then
                table.insert( filteredUnits, unit )
            end
        end
        ForkThread(KillBaseInAreaThread, filteredUnits )
    end
end

function KillBaseInAreaThread(unitTable)
    local numUnits = table.getn(unitTable)
    local waitNum = math.floor(numUnits / 20)
    for num, unit in unitTable do
        if not unit:IsDead() then
            unit:Kill()
        end
        if waitNum > 0 and num >= waitNum and math.mod(waitNum, num) == 0 then
            WaitTicks( 5 )
        end
    end
end

function StartOperationJessZoom( areaName, callbackFunction, duration )
    ForkThread( StartOperationJessZoomThread, areaName, callbackFunction, duration )
end

function StartOperationJessZoomThread( areaName, callbackFunction, duration )
    Cinematics.EnterNISMode()
    duration = duration or 3
    Cinematics.CameraMoveToRectangle(ScenarioUtils.AreaToRect(areaName), duration)
    Cinematics.ExitNISMode()
    if callbackFunction then
        callbackFunction()
    end
end

function EndOperationSafety( units )

    ScenarioInfo.OpEnded = true

    for k,v in ArmyBrains do
        for subk, subv in ArmyBrains do
            SetAlliance( v.Name, subv.Name, 'Neutral' )
        end
        for subk, subv in v:GetListOfUnits( categories.COMMAND, false ) do
            subv:SetCanTakeDamage(false)
            subv:SetCanBeKilled(false)
        end
        if units and table.getn( units ) > 0 then
            for subk, subv in units do
                subv:SetCanTakeDamage(false)
                subv:SetCanBeKilled(false)
            end
        end
    end
end
