#
# SimCamera
#
# SimCamera buffers control requests to push to the user layer at sync time. It provides facilities
# for waiting on the camera to perform certain actions. At the moment, the facilities that wait can
# not be used in multiplayer and are considered to finish immediately when running the simulation in
# a headless mode. Thus the primary use for such features is in the single player campaign. We have
# a plan to add multiplayer and headless support for these features but it may be some time before
# this gets implemented.
#
# Copyright  2005 Gas Powered Games, Inc.  All rights reserved.

SingleEvent = import('/lua/system/SingleEvent.lua').SingleEvent

# Name / Object table for cameras
Cameras = {}

# The user layer calls this via SimCallback when the camera finishes moving to its target.
function OnCameraFinish(name)
    #LOG('Signal')
    Cameras[name]:EventSet()
end

SimCamera = Class(SingleEvent) {
    __init = function(self,name)
        self.CameraName = name
        self.Callback = {
            Func = "OnCameraFinish",
            Args = name,
        }
        Cameras[name] = self
    end,

    ScaleMoveVelocity = function(self,val)
        WARN('ScaleMoveVelocity is defunct. Please remove.')
    end,

    MoveTo = function(self,rectRegion,seconds)
        #LOG('Camera:MoveTo ', repr(rectRegion), ' ',seconds)
        request = {
            Name = self.CameraName,
            Type = 'CAMERA_MOVE',
            Region = rectRegion,
            Time = seconds or 0,
            Callback = self.Callback
        }
        table.insert( Sync.CameraRequests, request )
    end,

    MoveToMarker = function(self,marker,seconds)
        #LOG('Camera:MoveToMarker ', repr(marker.type), ' ',seconds)
        request = {
            Name = self.CameraName,
            Type = 'CAMERA_MOVE',
            Marker = marker,
            Time = seconds or 0,
            Callback = self.Callback
        }
        table.insert( Sync.CameraRequests, request )
    end,

    SyncPlayableRect = function(self,rectRegion)
        #LOG('Camera:LockCameraRect ', repr(rectRegion))
        request = {
            Name = self.CameraName,
            Type = 'CAMERA_SYNC_VIEW_RECT',
            Region = rectRegion,
        }
        table.insert( Sync.CameraRequests, request )
    end,

    SnapToMarker = function(self,marker)
        #LOG('Camera:SnapToMarker('..repr(marker.type)..')')
        request = {
            Name = self.CameraName,
            Type = 'CAMERA_SNAP',
            Marker = marker,
        }
        table.insert( Sync.CameraRequests, request )
    end,

    Lock = function(self)
        #LOG('Camera:Lock')
        table.insert( Sync.CameraRequests, { Name = self.CameraName, Exec = 'Lock' } )
    end,

    Release = function(self)
        #LOG('Camera:Release')
        table.insert( Sync.CameraRequests, { Name = self.CameraName, Exec = 'Release' } )
    end,

    TrackEntity = function(self, ent, zoom, seconds)
        #LOG('Camera:TrackEntity')
        request = {
            Name = self.CameraName,
            Type = 'CAMERA_TRACK_ENTITIES',
            Ents = {},
            Time = seconds or 0,
            Zoom = zoom,
            Callback = self.Callback
        }
        table.insert( request.Ents, ent:GetEntityId() )
        table.insert( Sync.CameraRequests, request )
    end,

    TrackEntities = function(self, units, zoom, seconds)
        #LOG('Camera:TrackEntities')
        request = {
            Name = self.CameraName,
            Type = 'CAMERA_TRACK_ENTITIES',
            Ents = {},
            Time = seconds or 0,
            Zoom = zoom,
            Callback = self.Callback
        }
        for k,v in units do
            table.insert( request.Ents, v:GetEntityId() )
        end
        table.insert( Sync.CameraRequests, request )
    end,

    NoseCam = function(self, ent, pitchAdjust, zoom, seconds)
        #LOG('Camera:NoseCam')
        request = {
            Name = self.CameraName,
            Type = 'CAMERA_NOSE_CAM',
            Entity = ent:GetEntityId(),
            PitchAdjust = pitchAdjust,
            Time = seconds or 0,
            Zoom = zoom,
            Callback = self.Callback
        }
        table.insert( Sync.CameraRequests, request )
    end,

    SetZoom = function(self,zoom,seconds)
        #LOG('Camera:SetZoom')
        request = {
            Name = self.CameraName,
            Type = 'CAMERA_SET_ZOOM',
            Zoom = zoom,
            Time = seconds or 0,
            Callback = self.Callback
        }
        table.insert( Sync.CameraRequests, request )
    end,

    Spin = function(self,headingRate,zoomRate)
        #LOG('Camera:Spin(',headingRate,',',zoomRate,')')
        table.insert( Sync.CameraRequests, { Name = self.CameraName, Exec = 'Spin', Params = {headingRate,zoomRate} } )
    end,

    RevertRotation = function(self)
        #LOG('Camera:RevertRotation')
        table.insert( Sync.CameraRequests, { Name = self.CameraName, Exec = 'RevertRotation' } )
    end,

    UseGameClock = function(self)
        #LOG('Camera:UseGameClock')
        table.insert( Sync.CameraRequests, { Name = self.CameraName, Exec = 'UseGameClock' } )
    end,

    UseSystemClock = function(self)
        #LOG('Camera:UseSystemClock')
        table.insert( Sync.CameraRequests, { Name = self.CameraName, Exec = 'UseSystemClock' } )
    end,

    EnableEaseInOut = function(self)
        #LOG('Camera:EnableEaseInOut')
        table.insert( Sync.CameraRequests, { Name = self.CameraName, Exec = 'EnableEaseInOut' } )
    end,

    DisableEaseInOut = function(self)
        #LOG('Camera:DisableEaseInOut')
        table.insert( Sync.CameraRequests, { Name = self.CameraName, Exec = 'DisableEaseInOut' } )
    end,

    Reset = function(self)
        #LOG('Camera:Reset')
        table.insert( Sync.CameraRequests, {Name=self.CameraName, Exec='Reset'} )
    end,
}
