#****************************************************************************
#**
#**  File     :  /lua/modules/SimObjectives.lua
#**
#**  Summary  : Sim side objectives
#**
#**  Copyright  2006 Gas Powered Games, Inc.  All rights reserved.
#****************************************************************************
##
## SUPPORTED OBJECTIVE TYPES:
##      Kill
##      Capture
##      KillOrCapture
##      Reclaim
##      Locate
##      SpecificUnitsInArea
##      CategoriesInArea
##      ArmyStatCompare
##      UnitStatCompare
##      CategoryStatCompare
##      Protect
##      Timer
##      Unknown
##
local ScenarioUtils = import('/lua/ScenarioUtilities.lua')
local VizMarker = import('/lua/modules/defaultvizmarkers.lua').VizMarker
local objNum = 0 # Used to create unique tags for objectives

local objectiveDecal = '/env/utility/decals/objective_debug_albedo.dds'

#
# CreateGroup
#
#   Takes list of objective tables which are produced by the objective creation
# functions such as Kill, Protect, Capture, etc.
#
#   UserCallback is executed when all objectives in the list are complete
#
function CreateGroup( name, userCallback, numRequired )
    LOG('Creating objective group ',name)
    local objectiveGroup =  {
        Name = name,
        Active = true,
        Objectives = {},
        NumRequired = numRequired,
        NumCompleted = 0,
        AddObjective = function(self,objective) end, # defined later
        RemoveObjective = function(self,objective) end, # defined later
        OnComplete = userCallback,
    }

    local function OnResult(result)
        if not objectiveGroup.Active then
            LOG('ObjectiveGroup ',objectiveGroup.Name,' is not active.')
            return
        end

        if result then
            objectiveGroup.NumCompleted = objectiveGroup.NumCompleted + 1
        end

        if objectiveGroup.NumRequired then
            LOG('ObjectiveGroup ',objectiveGroup.Name,' Progress ',objectiveGroup.NumRequired, '/',objectiveGroup.NumCompleted)
            if objectiveGroup.NumCompleted < objectiveGroup.NumRequired then
                return
            end
        else
            if objectiveGroup.Objectives then
                for k,v in objectiveGroup.Objectives do
                    if v.Active then
                        return
                    end
                end
            end
        end

        # Complete!
        objectiveGroup.Active = false
        objectiveGroup.OnComplete()
    end

    objectiveGroup.AddObjective = function(self,objective)
        table.insert(self.Objectives,objective)
        objective:AddResultCallback(OnResult)
    end

    objectiveGroup.RemoveObjective = function(self,objective)
        table.removeByValue(self.Objectives,objective)
    end

    return objectiveGroup
end

#
# Kill
#   Kill units
function Kill(Type,Complete,Title,Description,Target)
    Target.killed = 0
    Target.total = table.getn(Target.Units)

    local image = GetActionIcon('kill')
    local objective = AddObjective(Type,Complete,Title,Description,image,Target)

    local function OnUnitKilled(unit)
        Target.killed = Target.killed + 1

        local progress = string.format('(%s/%s)', Target.killed, Target.total)
        UpdateObjective( Title, 'Progress', progress, objective.Tag )
        objective:OnProgress(Target.killed,Target.total)

        if Target.killed == Target.total then
            UpdateObjective( Title, 'complete', "complete", objective.Tag )
            objective.Active = false
            objective:OnResult(true)
        end
    end

    for k,unit in Target.Units do
        if not unit:IsDead() then
            # Mark the units unless MarkUnits == false
            if ( Target.MarkUnits == nil ) or Target.MarkUnits then
                local ObjectiveArrow = import('objectiveArrow.lua').ObjectiveArrow
                local arrow = ObjectiveArrow { AttachTo = unit }
            end
            if Target.FlashVisible then
                FlashViz( unit )
            end
            unit:AddOnKilledCallback(OnUnitKilled)
        else
            OnUnitKilled(unit)
        end
    end

    local progress = string.format('(%s/%s)', Target.killed, Target.total)
    UpdateObjective( Title, 'Progress', progress, objective.Tag )

    return objective
end

#
# Capture
#   Capture units
#
# Target = {
#       Units = <units>,
#       NumRequired = <x>,
# }
#
function Capture(Type,Complete,Title,Description,Target)
    Target.captured = 0
    Target.total = table.getn(Target.Units)
    local required = Target.NumRequired or Target.total
    local returnUnits = {}

    local image = GetActionIcon('capture')
    local objective = AddObjective(Type,Complete,Title,Description,image,Target)

    local function OnUnitCaptured(unit, captor)
        table.insert(returnUnits, unit)
        if not objective.Active then
            return
        end

        Target.captured = Target.captured + 1
        local progress = string.format('(%s/%s)', Target.captured, required)
        objective:OnProgress(Target.captured,required)
        UpdateObjective( Title, 'Progress', progress, objective.Tag )
        if Target.captured >= required then
            objective.Active = false
            objective:OnResult(true, returnUnits)
            UpdateObjective( Title, 'complete', "complete", objective.Tag )
        end
    end

    local function OnUnitKilled(unit)
        if not objective.Active then
            return
        end
        Target.total = Target.total - 1
        if Target.total < required then
            objective.Active = false
            objective:OnResult(false)
            UpdateObjective( Title, 'complete', 'failed', objective.Tag )
        end
    end

    for k,unit in Target.Units do
        if not unit:IsDead() then
            # Mark the units unless MarkUnits == false
            if ( Target.MarkUnits == nil ) or Target.MarkUnits then
                local ObjectiveArrow = import('objectiveArrow.lua').ObjectiveArrow
                local arrow = ObjectiveArrow { AttachTo = unit }
            end

            unit:AddOnCapturedCallback(nil,OnUnitCaptured)
            unit:AddOnKilledCallback(OnUnitKilled)
            unit:AddOnReclaimedCallback(OnUnitKilled) #same functionality as killed

            if Target.FlashVisible then
                FlashViz( unit )
            end
        else
            #treat as killed Matt 8.30.06
            OnUnitKilled(unit)
        end
    end

    local progress = string.format('(%s/%s)', Target.captured, required)
    UpdateObjective( Title, 'Progress', progress, objective.Tag )

    return objective
end

#
# Kill or Capture
#   Kill or Capture units
function KillOrCapture(Type,Complete,Title,Description,Target)
    Target.killed_or_captured = 0
    Target.total = table.getn(Target.Units)

    local image = GetActionIcon('KillOrCapture')
    local objective = AddObjective(Type,Complete,Title,Description,image,Target)

    # keep track of captured units so subsequent kills don't get counted
    local captured = {}

    local function OnUnitKilled(unit)
        for k,v in captured do
            if v == unit then
                # ignore units already captured
                return
            end
        end


        Target.killed_or_captured = Target.killed_or_captured + 1
        local progress = string.format('(%s/%s)', Target.killed_or_captured, Target.total)
        objective:OnProgress(Target.killed_or_captured,Target.total)
        UpdateObjective( Title, 'Progress', progress, objective.Tag )
        if Target.killed_or_captured == Target.total then
            objective.Active = false
            objective:OnResult(true)
            UpdateObjective( Title, 'complete', "complete", objective.Tag )
        end
    end

    local function OnUnitCaptured(unit)
        table.insert(captured,unit)
        Target.killed_or_captured = Target.killed_or_captured + 1
        local progress = string.format('(%s/%s)', Target.killed_or_captured, Target.total)
        objective:OnProgress(Target.killed_or_captured,Target.total)
        UpdateObjective( Title, 'Progress', progress, objective.Tag )
        if Target.killed_or_captured == Target.total then
            objective.Active = false
            objective:OnResult(true)
            UpdateObjective( Title, 'complete', "complete", objective.Tag )
        end
    end


    local function OnUnitReclaimed(unit)
        if not objective.Active then
            return
        end

        Target.killed_or_captured = Target.killed_or_captured + 1
        local progress = string.format('(%s/%s)', Target.killed_or_captured, Target.total)
        objective:OnProgress(Target.killed_or_captured,Target.total)
        UpdateObjective( Title, 'Progress', progress, objective.Tag )
        if Target.killed_or_captured == Target.total then
            objective.Active = false
            objective:OnResult(true)
            UpdateObjective( Title, 'complete', "complete", objective.Tag )
        end
    end

    for k,unit in Target.Units do
        if not unit:IsDead() then
            # Mark the units unless MarkUnits == false
            if ( Target.MarkUnits == nil ) or Target.MarkUnits then
                local ObjectiveArrow = import('objectiveArrow.lua').ObjectiveArrow
                local arrow = ObjectiveArrow { AttachTo = unit }
            end

            if Target.FlashVisible then
                FlashViz( unit )
            end

            # note: you won't get an OnKilled after an OnCaptured because once
            # captured it's actually a new unit with no callback.
            unit:AddOnKilledCallback(OnUnitKilled)
            unit:AddOnCapturedCallback(OnUnitCaptured)
            unit:AddOnReclaimedCallback(OnUnitReclaimed)
        else
            OnUnitKilled(unit)
        end
    end

    local progress = string.format('(%s/%s)', Target.killed_or_captured, Target.total)
    UpdateObjective( Title, 'Progress', progress, objective.Tag )

    return objective
end

#
# Reclaim
#   Reclaim units
function Reclaim(Type,Complete,Title,Description,Target)
    Target.reclaimed = 0
    Target.total = table.getn(Target.Units)

    local image = GetActionIcon("reclaim")
    local objective = AddObjective(Type,Complete,Title,Description,image,Target)

    local function OnUnitReclaimed(unit)
        if not objective.Active then
            return
        end

        Target.reclaimed = Target.reclaimed + 1
        local progress = string.format('(%s/%s)', Target.reclaimed, Target.total)
        objective:OnProgress(Target.reclaimed,Target.total)
        UpdateObjective( Title, 'Progress', progress, objective.Tag )
        if Target.reclaimed == Target.total then
            objective.Active = false
            objective:OnResult(true)
            UpdateObjective( Title, 'complete', "complete", objective.Tag )
        end
    end

    local function OnUnitKilled(unit)
        objective.Active = false
        objective:OnResult(false)
        UpdateObjective( Title, 'complete', 'failed', objective.Tag )
    end

    # If the unit is captured it can still be reclaimed to complete the
    # objective, so track the new unit created on a capture.
    local function OnUnitCaptured(newUnit,captor)
        newUnit:AddOnKilledCallback(OnUnitKilled)
        newUnit:AddOnReclaimedCallback(OnUnitReclaimed)
        local ObjectiveArrow = import('objectiveArrow.lua').ObjectiveArrow
        local arrow = ObjectiveArrow { AttachTo = newUnit }
    end

    for k,unit in Target.Units do
        local ObjectiveArrow = import('objectiveArrow.lua').ObjectiveArrow
        local arrow = ObjectiveArrow { AttachTo = unit }
        unit:AddOnReclaimedCallback(OnUnitReclaimed)
        unit:AddOnKilledCallback(OnUnitKilled)
        unit:AddOnCapturedCallback(nil,OnUnitCaptured)
    end

    local progress = string.format('(%s/%s)', Target.reclaimed, Target.total)
    UpdateObjective( Title, 'Progress', progress, objective.Tag )

    return objective
end

#
# Locate
#   Locate units
function Locate(Type,Complete,Title,Description,Target)
    Target.located = 0
    Target.total = table.getn(Target.Units)

    local image = GetActionIcon("locate")
    local objective = AddObjective(Type,Complete,Title,Description,image,Target)

    local function OnUnitLocated(unit)
        Target.located = Target.located + 1
        local progress = string.format('(%s/%s)', Target.located, Target.total)
        UpdateObjective( Title, 'Progress', progress, objective.Tag )
        objective:OnProgress(Target.located,Target.total)
        if Target.located == Target.total then
            objective.Active = false
            objective:OnResult(true)
            UpdateObjective( Title, 'complete', "complete", objective.Tag )
        end
    end

    for k,unit in Target.Units do
        local IntelTrigger = import('/lua/modules/ScenarioTriggers.lua').CreateArmyIntelTrigger
        IntelTrigger(OnUnitLocated,
                    GetArmyBrain(GetFocusArmy()),
                    'LOSNow',
                    unit,
                    true,
                    categories.ALLUNITS,
                    true,
                    unit:GetAIBrain() )
    end

    local progress = string.format('(%s/%s)', Target.located, Target.total)
    UpdateObjective( Title, 'Progress', progress, objective.Tag )

    return objective
end

#
# SpecificUnitsInArea
#   Complete when specified units are in the target area. We don't care how
# they got there (cheat teleport, etc), we just check if they're in there
function SpecificUnitsInArea(Type,Complete,Title,Description,Target)
    local image = GetActionIcon('Move')
    local objective = AddObjective(Type,Complete,Title,Description,image,Target)
    local total = table.getn(Target.Units)
    local numRequired = Target.NumRequired or total
    Target.Count = 0

    local function WatchArea(units,rect)
        while objective.Active do
            local cnt = 0
            for k,unit in units do
                if not unit:IsDead() then
                    if ScenarioUtils.InRect( unit:GetPosition(), rect ) then
                        cnt = cnt + 1
                    end
                end
            end

            if cnt != Target.Count then
                Target.Count = cnt
                local progress = string.format('(%s/%s)', Target.Count, numRequired)
                objective:OnProgress(Target.Count,numRequired)
                UpdateObjective( Title, 'Progress', progress, objective.Tag)
            end

            if cnt >= numRequired then
                objective.Active = false
                objective:OnResult(true)
                UpdateObjective( Title, 'complete', 'complete', objective.Tag)
                return
            end
            WaitTicks(5)
        end
    end

    local rect = ScenarioUtils.AreaToRect( Target.Area )

    local w = rect.x1 - rect.x0
    local h = rect.y1 - rect.y0
    local x = rect.x0 + ( w / 2.0 )
    local z = rect.y0 + ( h / 2.0 )

    if Target.MarkArea then
        objective.Decals[Target.Area] = CreateDecal(Vector(x,0,z), 0, objectiveDecal, '', 'Albedo', w, h, 500, 0, 1)
    end

    local watchThread = ForkThread( WatchArea, Target.Units, rect )

    local function OnUnitKilled(unit)
        total = total - 1
        if objective.Active and total < numRequired then
            objective.Active = false
            objective:OnResult(false)
            UpdateObjective( Title, 'complete', 'failed', objective.Tag)
            KillThread(watchThread)
        end
    end

    for k,v in Target.Units do
        v:AddOnKilledCallback(OnUnitKilled)
    end

    local progress = string.format('(%s/%s)', Target.Count, numRequired)
    UpdateObjective( Title, 'Progress', progress, objective.Tag )

    return objective
end

#
# CategoriesInArea
#   Complete when specified units matching the target blueprint types are in
# the target area. We don't care exactly which units they are (pre-built or
# newly constructed) or how they got there (cheat teleport, etc). We just check
# the area for what units are inside and look at the blueprints (and optionally
# match the army, use -1 for don't care).
#
# Target = {
#   Requirements = {
#       { Area = <areaName>, Category=<cat1>, CompareOp=<op>, Value=<x>, [ArmyIndex=<index>]},
#       { Area = <areaName>, Category=<cat2>, CompareOp=<op>, Value=<y>, [ArmyIndex=<index>] },
#       ...
#       { Area = <areaName>, Category=<cat3>, CompareOp=<op>, Value=<z>, [ArmyIndex=<index>] },
#   }
# }
#
# -- op is one of: '<=', '>=', '<', '>', or '=='
#
function CategoriesInArea(Type,Complete,Title,Description,Action,Target)

    local image = GetActionIcon(Action)
    local objective = AddObjective(Type,Complete,Title,Description,image,Target)
    local lastReqsMet = 0

    # call ManualResult
    objective.ManualResult = function(result)
        objective.Active = false
        objective:OnResult(result)
        local resultStr
        if result then
            resultStr = 'complete'
        else
            resultStr = 'failed'
        end
        UpdateObjective( Title, 'complete', resultStr, objective.Tag)
    end

    local function WatchArea(requirements)
        local totalReqs = table.getn(requirements)
        while objective.Active do
            local reqsMet = 0

            for i,requirement in requirements do
                local units = GetUnitsInRect(requirement.Rect)
                local cnt = 0
                if units then
                    for k,unit in units do
                        if not unit:IsDead() and not unit:IsBeingBuilt() then
                            if not requirement.ArmyIndex or (requirement.ArmyIndex == unit:GetArmy()) then
                                if EntityCategoryContains(requirement.Category, unit) then
                                    cnt = cnt+1
                                end
                            end
                        end
                    end
                    #LOG('debug: CategoriesInArea: reqsmet '..reqsMet..'count '.. cnt )
                end
                if requirement.CompareFunc(cnt,requirement.Value) then
                    reqsMet = reqsMet +1
                end
            end

            if lastReqsMet != reqsMet then
                local progress = string.format('(%s/%s)', reqsMet, totalReqs)
                UpdateObjective( Title, 'Progress', progress, objective.Tag )
                objective:OnProgress(reqsMet,totalReqs)
                lastReqsMet = reqsMet
            end

            if reqsMet == totalReqs then
                objective.Active = false
                objective:OnResult(true)
                UpdateObjective( Title, 'complete', 'complete', objective.Tag)
                return
            end
            WaitTicks(10)
        end
    end

    for k,requirement in Target.Requirements do
        local rect = ScenarioUtils.AreaToRect( requirement.Area )

        local w = rect.x1 - rect.x0
        local h = rect.y1 - rect.y0
        local x = rect.x0 + ( w / 2.0 )
        local z = rect.y0 + ( h / 2.0 )

        if Target.MarkArea and not objective.Decals[requirement.Area] then
            local decal = CreateDecal(Vector(x,0,z), 0, objectiveDecal, '', 'Albedo', w, h, 500, 0, 1)
            objective.Decals[requirement.Area] = decal
        elseif Target.FlashVisible then
            FlashViz( requirement.Area )
        end

        local reqRef = requirement
        reqRef.Rect = rect
        reqRef.CompareFunc = GetCompareFunc(requirement.CompareOp)
    end

    ForkThread( WatchArea, Target.Requirements )

    return objective
end

#
# ArmyStatCompare
#   Army stat is compared <=, >=, >, <, or == to some value.
#
# Target = {
#       Army=<index>,
#       StatName=<name>,
#       CompareOp=<op>,   -- op is one of: '<=', '>=', '<', '>', or '=='
#       Value=<value>,
#       [Category=<category>], -- optional to compare to a blueprint stat
#       ShowProgress, optional: shows #/#. may not make sense for all compare types
# }
#
# Note: Be careful when using '==' as the stat is only checked every 5 ticks.
#
function ArmyStatCompare(Type,Complete,Title,Description,Action,Target)
    local image = GetActionIcon(Action)
    local objective = AddObjective(Type,Complete,Title,Description,image,Target)

    local function WatchStat(statName,brain,compareFunc,value,category)
        while objective.Active do
            local result = false
            local testVal = 0
            local oldVal

            if category then
                testVal = brain:GetBlueprintStat(statName,category)

            else
                testVal = brain:GetArmyStat(statName,value).Value
            end

            if (Target.ShowProgress) then
                if (testVal ~= oldVal) then
                    local progress = string.format('(%s/%s)', testVal, value)
                    UpdateObjective( Title, 'Progress', progress, objective.Tag )
                    oldVal = testVal
                end
            end

            result = compareFunc(testVal,value)

            if result then
                objective.Active = false
                objective:OnResult(true)
                UpdateObjective( Title, 'complete', 'complete', objective.Tag )
                return
            end
            WaitTicks(5)
        end
    end

    local op = GetCompareFunc(Target.CompareOp)
    if op then
        ForkThread( WatchStat, Target.StatName, GetArmyBrain(Target.Army), op, Target.Value, Target.Category )
    end

    return objective
end

#
# UnitStatCompare
#   A specified unit's stat is <=, >=, >, <, or == to some value.
#
# Target = {
#       Unit=<unit>,
#       StatName=<name>,
#       CompareOp=<op>,   -- op is one of: '<=', '>=', '<', '>', or '=='
#       Value=<value>,
# }
#
# Note: Be careful when using '==' as the stat is only checked every 5 ticks.
#
function UnitStatCompare(Type,Complete,Title,Description,Action,Target)
    local image = GetActionIcon(Action)
    local objective = AddObjective(Type,Complete,Title,Description,image,Target)

    local function WatchStat(statName,unit,compareFunc,value)
        while objective.Active do
            if compareFunc(unit:GetStat(statName,value).Value,value) then
                objective.Active = false
                objective:OnResult(true)
                UpdateObjective( Title, 'complete', 'complete', objective.Tag )
                return
            end
            WaitTicks(5)
        end
    end

    local op = GetCompareFunc(Target.CompareOp)
    if op then
        ForkThread( WatchStat, Target.StatName, Target.Unit, op, Target.Value )
    end

    return objective
end

#
# CategoryStatCompare
#   Some unit belonging to specified category has a stat <=, >=, >, <, or ==
# to some value.
#
# Target = {
#       Army=<index>,
#       Category=<unit>,
#       StatName=<name>,
#       CompareOp=<op>,   -- op is one of: '<=', '>=', '<', '>', or '=='
#       Value=<value>,
# }
#
# Note: Be careful when using '==' as the stat is only checked every 5 ticks.
#
function CategoryStatCompare(Type,Complete,Title,Description,Action,Target)
    local image = GetActionIcon(Action)
    local objective = AddObjective(Type,Complete,Title,Description,image,Target)

    local function WatchStat(statName,brain,category,compareFunc,value)
        while objective.Active do
            local unitsInCategory = brain:GetListOfUnits(category,false)
            if unitsInCategory then
                for k,unit in unitsInCategory do
                    if compareFunc(unit:GetStat(statName,value).Value,value) then
                        objective.Active = false
                        objective:OnResult(true)
                        UpdateObjective( Title, 'complete', 'complete', objective.Tag )
                        return
                    end
                end
            end
            WaitTicks(5)
        end
    end

    local op = GetCompareFunc(Target.CompareOp)
    if op then
        ForkThread( WatchStat, Target.StatName, GetArmyBrain(Target.Army), Target.Category, op, Target.Value )
    end

    return objective
end

#
# Protect
#   Fails if # of units in list falls below NumRequired before the timer expires
# or, in the case of no timer, the objective is manually update to complete.
#
# Target = {
#       Units = {},
#       Timer = <seconds> or nil,   -- if nil, requires manual completion
#       NumRequired = <#>,          -- how many must survive
# }
function Protect(Type,Complete,Title,Description,Target)

    local image = GetActionIcon("protect")
    local objective = AddObjective(Type,Complete,Title,Description,image,Target)
    local total = table.getn(Target.Units)
    local numRequired = Target.NumRequired or total
    local timer = nil

    local function OnUnitKilled(unit)
        if not objective.Active then
            return
        end

        total = total - 1
        objective:OnProgress( total, numRequired )

        if (Target.ShowProgress) then
            local progress = string.format('(%s/%s)', total, numRequired)
            UpdateObjective( Title, 'Progress', progress, objective.Tag )
        end

        if objective.Active and total < numRequired then
            objective.Active = false
            objective:OnResult(false)
            UpdateObjective( Title, 'complete', 'failed', objective.Tag)
            Sync.ObjectiveTimer = 0
            if timer then
                KillThread(timer)
            end
        end
    end

    local function OnExpired()
        if objective.Active then
            objective.Active = false
            objective:OnResult(true)
            UpdateObjective( Title, 'complete', 'complete', objective.Tag)
        end
        Sync.ObjectiveTimer = 0
    end

    objective.ManualResult = function(result)
        objective.Active = false
        objective:OnResult(result)
        local resultStr
        if result then
            resultStr = 'complete'
        else
            resultStr = 'failed'
        end
        UpdateObjective( 'UNUSED', 'complete', resultStr, objective.Tag)
    end

    if Target.Timer then
        timer = import('/lua/modules/ScenarioTriggers.lua').CreateTimerTrigger(
            OnExpired,
            Target.Timer,
            true
        )
    end

    for k,v in Target.Units do
        v:AddOnKilledCallback(OnUnitKilled)
    end

    if (Target.ShowProgress) then
        local progress = string.format('(%s/%s)', total, numRequired)
        UpdateObjective( Title, 'Progress', progress, objective.Tag )
    end

    return objective
end

#
# Timer
#   OnResult() is called when the timer expires. The result depends on whether
# ExpireResult is set to complete or failed.
#
# Target = {
#       Timer = <seconds>
#       ExpireResult = 'complete' or 'failed'
# }
function Timer(Type,Complete,Title,Description,Target)

    local image = GetActionIcon("timer")
    local objective = AddObjective(Type,Complete,Title,Description,image,Target)

    # call ManualResult
    objective.ManualResult = function(result)
        objective.Active = false
        objective:OnResult(result)
        local resultStr
        if result then
            resultStr = 'complete'
        else
            resultStr = 'failed'
        end
        UpdateObjective( Title, 'complete', resultStr, objective.Tag)

        # what do we do with the timer?

    end

    local function OnExpired()
        objective.Active = false
        if Target.ExpireResult == 'complete' then
            objective:OnResult(true)
            UpdateObjective( Title, 'complete', 'complete', objective.Tag)
        else
            objective:OnResult(false)
            UpdateObjective( Title, 'complete', 'failed', objective.Tag)
        end
        Sync.ObjectiveTimer = 0
    end

    if Target.Timer then
        import('/lua/modules/ScenarioTriggers.lua').CreateTimerTrigger(
            OnExpired,
            Target.Timer,
            true
        )
    end

    return objective
end

function Unknown(Type,Complete,Title,Description)
    local objective = AddObjective(Type,Complete,Title,Description)

    # call ManualResult
    objective.ManualResult = function(result)
        objective.Active = false
        objective:OnResult(result)
        local resultStr
        if result then
            resultStr = 'complete'
        else
            resultStr = 'failed'
        end
        UpdateObjective( Title, 'complete', resultStr, objective.Tag)
    end

    return objective
end

function Basic(Type,Complete,Title,Description,Image,Target)
    local objective = AddObjective(Type,Complete,Title,Description,Image,Target)

    # call ManualResult
    objective.ManualResult = function(self, result)
        objective.Active = false
        objective:OnResult(result)
        local resultStr
        if result then
            resultStr = 'complete'
        else
            resultStr = 'failed'
        end
        UpdateObjective( Title, 'complete', resultStr, objective.Tag)
    end

    objective.AddTarget = function(self, target)
        if (target.Area) then
            local rect = ScenarioUtils.AreaToRect( target.Area )

            local w = rect.x1 - rect.x0
            local h = rect.y1 - rect.y0
            local x = rect.x0 + ( w / 2.0 )
            local z = rect.y0 + ( h / 2.0 )

            if target.MarkArea then
                objective.Decals[target.Area] = CreateDecal(Vector(x,0,z), 0, objectiveDecal, '', 'Albedo', w, h, 500, 0, 1)
            end
            if Target.FlashVisible then
                FlashViz( target.Area )
            end
        end
        if target.Areas and target.MarkArea then
            for k,v in target.Areas do
                local rect = ScenarioUtils.AreaToRect( v )
    
                local w = rect.x1 - rect.x0
                local h = rect.y1 - rect.y0
                local x = rect.x0 + ( w / 2.0 )
                local z = rect.y0 + ( h / 2.0 )
    
                objective.Decals[v] = CreateDecal(Vector(x,0,z), 0, objectiveDecal, '', 'Albedo', w, h, 500, 0, 1)
                if Target.FlashVisible then
                    FlashViz( v )
                end
            end
        end
        if (target.Units) then
            if (target.MarkUnits) then
                for k,unit in target.Units do
                    if not unit:IsDead() then
                        local ObjectiveArrow = import('objectiveArrow.lua').ObjectiveArrow
                        local arrow = ObjectiveArrow { AttachTo = unit }
                        table.insert( objective.UnitMarkers, arrow )
                        if target.AlwaysVisible then
                            SetupVizMarker(self, unit)
                        end
                        if Target.FlashVisible then
                            FlashViz( unit )
                        end
                    end
                end
            end
        end
    end

    if Target then
        objective:AddTarget(Target)
    end

    return objective
end


### Adds objective for the objectives screen
function AddObjective(Type,         # 'primary', 'bonus', etc
                      Complete,     # 'complete', 'incomplete'
                      Title,        # e.g. "Destroy Radar Stations"
                      Description,  # e.g. "A reason why you need to destroy the radar stations"
                      ActionImage,        # '/textures/ui/common/missions/mission1.dds'
                      Target        # Can be one of:
                                    #   Units = { unit1, unit2, ... }
                                    #   Areas = { 'areaName1', 'areaName2', ... }
                      )

    if(not Sync.ObjectivesTable) then
        Sync.ObjectivesTable = {}
    end

    local tag = 'Objective' .. objNum
    objNum = objNum + 1

    LOG("AddObjective: ", Title, " (Tag=",tag,")")

    # Set up objective table to return.
    local objective = {
        # Used to synchronize sim objectives with user side objectives
        Tag = tag,

        # Whether the objective is in progress or not and does not indicate
        # success or failure.
        Active = true,

        # success or failure.
        Complete = false,

        # Decal table, key'd by area names
        Decals = {},

        # Unit arrow table
        UnitMarkers = {},

        # Visibility markers that we manage
        VizMarkers = {},

        # Single decal
        Decal = false,

        # Strategic icon overrides
        IconOverrides = {},

        # For tracking targets
        NextTargetTag = 0,

        # Called on success or failure
        ResultCallbacks = {},
        AddResultCallback = function(self,cb)
            table.insert(self.ResultCallbacks,cb)
        end,

        # Some objective types can provide progress updates (not success/fail)
        ProgressCallbacks = {},
        AddProgressCallback = function(self,cb)
            table.insert(self.ProgressCallbacks,cb)
        end,

        # Don't override these if you want notification. Call Add???Callback
        # intead
        OnResult = function(self,success,data)

            self.Complete = success

            for k,v in self.ResultCallbacks do v(success,data) end

            # Destroy decals
            for k,v in self.Decals do v:Destroy() end

            # Destroy unit marker things
            for k,v in self.UnitMarkers do
                v:Destroy()
            end

            # Revert strategic icons
            for k,v in self.IconOverrides do
                if not v:BeenDestroyed() then
                    v:SetStrategicUnderlay("")
                end
            end

            # Destroy visibility markers
            for k,v in self.VizMarkers do
                v:Destroy()
            end

            if self.PositionUpdateThread then
                KillThread(self.PositionUpdateThread)
                self.PositionUpdateThread = false
            end
        end,

        OnProgress = function(self,current,total)
            for k,v in self.ProgressCallbacks do v(current,total) end
        end,

        # Call this to manually fail the objective
        Fail = function(self)
            self.Active = false
            self:OnResult(false)
            UpdateObjective(self.Title,'complete','failed',self.Tag)
        end,

        AddUnitTarget = function(self,unit) end, # defined below
        AddAreaTarget = function(self,area) end, # defined below
    }

    # Takes a unit that is an objective target and uses its recon detect
    # event to notify the objectives that we have a blip for the unit.
    function SetupNotify(obj,unit,targetTag)

        # Add a detectedBy callback to notify the user layer when our recon
        # on the target comes in and out.
        local detectedByCB = function(cbunit,armyindex)
            if not obj.Active then
                return
            end

            # now if we've been detected by the focus army ...
            if armyindex == GetFocusArmy() then

                # get the blip that is associated with the unit
                local blip = cbunit:GetBlip(armyindex)

                # Only provide the target position to the user layer if
                # the blip IsSeenEver() (i.e. has been identified).
                obj.PositionUpdateThread = ForkThread(
                    function()
                        while obj.Active and not blip:BeenDestroyed() do
                            if blip:IsSeenEver() then
                                UpdateObjective(Title,
                                                'Target',
                                                {
                                                    Type = 'Position',
                                                    Value = blip:GetPosition(),
                                                    BlueprintId = blip:GetBlueprint().BlueprintId,
                                                    TargetTag=targetTag
                                                },
                                                obj.Tag )
                            end

                            # If it's not mobile we can exit the thread since
                            # the blip won't move.
                            if not unit:IsDead() and not unit:BeenDestroyed() and not EntityCategoryContains( categories.MOBILE, unit ) then
                                return
                            end

                            WaitTicks(10)
                        end
                    end
                )

                local destroyCB = function(cbblip)
                    if not obj.Active then
                        return
                    end

                    if obj.PositionUpdateThread then
                        KillThread( obj.PositionUpdateThread )
                        obj.PositionUpdateThread = false
                    end

                    # when the blip is destroyed, tell objectives we dont
                    # have a blip anymore. This doesn't necessarily mean the
                    # unit is killed, we simply lost the blip.
                    UpdateObjective(Title,
                                    'Target',
                                    {
                                        Type = 'Position',
                                        Value = nil,
                                        BlueprintId = nil,
                                        TargetTag=targetTag,
                                    },
                                    obj.Tag )
                end

                # When the blip is destroyed, have it call this callback
                # function (defined above)
                blip:AddDestroyHook(destroyCB)
            end
        end

        # When the unit is detected by an army, have it call this callback
        # function (defined above)
        unit:AddDetectedByHook(detectedByCB)
    end

    function SetupVizMarker(objective, object)
        if IsEntity(object) then
            local pos = object:GetPosition()
            local spec = {
                X = pos[1],
                Z = pos[2],
                Radius = 8,
                LifeTime = -1,
                Omni = false,
                Vision = true,
                Army = GetFocusArmy(),
            }
            local vizmarker = VizMarker(spec)
            object.Trash:Add(vizmarker)
            vizmarker:AttachBoneTo(-1,object,-1)
        else
            local rect = ScenarioUtils.AreaToRect(Target.Area)
            local width = rect.x1 - rect.x0
            local height = rect.y1 - rect.y0
            local spec = {
                X = rect.x0 + width/2,
                Z = rect.y0 + height/2,
                Radius = math.max( width, height ),
                LifeTime = -1,
                Omni = false,
                Vision = true,
                Army = GetFocusArmy(),
            }
            local vizmarker = VizMarker(spec)
            table.insert(objective.VizMarkers,vizmarker);
        end
    end

    function FlashViz (object)
        if IsEntity(object) then

            local pos = object:GetPosition()
            local spec = {
                X = pos[1],
                Z = pos[2],
                Radius = 2,
                LifeTime = 1.00,
                Omni = false,
                Vision = true,
                Radar = false,
                Army = GetFocusArmy(),
            }
            local vizmarker = VizMarker(spec)
            object.Trash:Add(vizmarker)
            vizmarker:AttachBoneTo(-1,object,-1)
        else
            local rect = ScenarioUtils.AreaToRect(object)
            local width = rect.x1 - rect.x0
            local height = rect.y1 - rect.y0
            local spec = {
                X = rect.x0 + width/2,
                Z = rect.y0 + height/2,
                Radius = math.max( width, height ),
                LifeTime = 0.01,
                Omni = false,
                Vision = true,
                Radar = false,
                Army = GetFocusArmy(),
            }
            local vizmarker = VizMarker(spec)
        end
    end

    local userTargets = {}
    if Target and Target.Requirements then
        for k,req in Target.Requirements do
            if req.Area then
                table.insert(userTargets, { Type = 'Area', Value = ScenarioUtils.AreaToRect(req.Area) })
            end
        end
    end

    local userObjectiveData = {
        tag = tag,
        type = Type,
        complete = Complete,
        title = Title,
        description = Description,
        actionImage = ActionImage,
        targetImage = Target.Image,
        progress = "",
        targets = userTargets,
    }

    Sync.ObjectivesTable[tag] = userObjectiveData

    objective.AddUnitTarget = function(self,unit)
        self.NextTargetTag = self.NextTargetTag + 1
        SetupNotify(self,unit,self.NextTargetTag)
        if Target.AlwaysVisible then
            SetupVizMarker(self,unit)
        end

        table.insert(self.IconOverrides,unit)

        # Mark the units unless MarkUnits == false
        if ( Target.MarkUnits == nil ) or Target.MarkUnits then
            if Type == 'primary' then
                unit:SetStrategicUnderlay('icon_objective_primary')
            elseif Type == 'secondary' then
                unit:SetStrategicUnderlay('icon_objective_secondary')
            end
        end

    end

    objective.AddAreaTarget = function(self,area)
        self.NextTargetTag = self.NextTargetTag + 1
        UpdateObjective(Title,
                        'Target',
                        {
                            Type = 'Area',
                            Value = ScenarioUtils.AreaToRect(area),
                            TargetTag=self.NextTargetTag
                        },
                        self.Tag )

        if Target.AlwaysVisible then
            SetupVizMarker(self,area)
        end
    end

    if Target then
        if Target.Units then
            for k,v in Target.Units do
                if not v:IsDead() then
                    objective:AddUnitTarget(v)
                end
            end
        end

        if Target.Unit then
            objective:AddUnitTarget(Target.Unit)
        end

        if Target.Areas then
            for k,v in Target.Areas do
                objective:AddAreaTarget(v)
            end
        end

        if Target.Area then
            objective:AddAreaTarget(Target.Area)
        end
    end

    return objective
end

### Update legacy style objective using correct syntax
function UpdateBasicObjective(Objective, UpdateField, NewData)
    UpdateObjective(Objective.Title, UpdateField, NewData, Objective.Tag)
end
### Updates an objective, referencing it by objective title
function UpdateObjective(Title, UpdateField, NewData, objTag)

    if(not Sync.ObjectivesUpdateTable) then
        Sync.ObjectivesUpdateTable = {}
    end

    if type(objTag) ~= 'string' then
        error('SimObjectives error: Invalid type for objTag in UpdateObjective.  String expected but got '
        .. type(objTag), 2)
    end
    if type(UpdateField) ~= 'string' then
        error('SimObjectives error: Invalid type for UpdateField in UpdateObjective. String expected but got ' .. type(UpdateField), 2)
    end
    #if type(Title) ~= 'string' then
    #    error('SimObjectives error: Invalid type for Title in UpdateObjective. String expected but got ' .. type(Title), 2)
    #end

    # All fields are stored with lowercase names
    UpdateField = string.lower(UpdateField)
    if not (
        (UpdateField == 'type') or
        (UpdateField == 'complete') or
        (UpdateField == 'title') or
        (UpdateField == 'description') or
        (UpdateField == 'image') or
        (UpdateField == 'progress') or
        (UpdateField == 'target')
        )
      then
        error( 'Unknown UpdateField: ' .. UpdateField .. '.  Cannot process UpdateObjective request.', 2 )
    else
        local userObjectiveUpdate = {
            title = Title,
            updateField = UpdateField,
            updateData = NewData,
            tag = objTag,
        }

        table.insert(Sync.ObjectivesUpdateTable, userObjectiveUpdate)
    end
end

function GetCompareFunc( op )
    function gt(a,b) return a > b end
    function lt(a,b) return a < b end
    function gte(a,b) return a >= b end
    function lte(a,b) return a <= b end
    function eq(a,b) return a == b end

    if op == '<=' then return lte end
    if op == '>=' then return gte end
    if op == '<' then return lt end
    if op == '>' then return gt end
    if op == '==' then return eq end

    WARN("Unsupported CompareOp '",op,"'")
end

function GetActionIcon(actionString)
    local action = string.lower(actionString)
    if action == "kill"     then return "/game/orders/attack_btn_up.dds"    end
    if action == "capture"  then return "/game/orders/convert_btn_up.dds"   end
    if action == "build"    then return "/game/orders/production_btn_up.dds" end
    if action == "protect"  then return "/game/orders/guard_btn_up.dds"     end
    if action == "timer"    then return "/game/orders/guard_btn_up.dds"     end
    if action == "move"     then return "/game/orders/move_btn_up.dds"      end
    if action == "reclaim"  then return "/game/orders/reclaim_btn_up.dds"   end
    if action == "repair"   then return "/game/orders/repair_btn_up.dds"   end
    if action == "locate"   then return "/game/orders/omni_btn_up.dds"    end

     # todo: make 'kill or capture' icon?
    if action == "killorcapture" then return "/game/orders/attack_btn_up.dds" end

    return ""
end

function IsComplete(obj)
    if obj then
        if obj.Complete then
            return true
        end
    end

    return false
end