
local Group = import('group.lua').Group
local Bitmap = import('bitmap.lua').Bitmap
local Dragger = import('dragger.lua').Dragger
local LayoutHelpers = import('layouthelpers.lua')
local lazyvar = import('/lua/modules/lazyvar.lua')

Slider = Class(Group)
{
    -- TODO make it possible for the start value to be greate than the end value and have that work the opposite way
    __init = function(self, parent, isVertical, startValue, endValue, thumb, background, debugname)
        Group.__init(self, parent)
        self:SetName(debugname or "Slider")

        self._isVertical = isVertical
        self._startValue = startValue
        self._currentValue = lazyvar.Create(startValue)
        self._endValue = endValue
        self._thumb = Bitmap(self, thumb)
        self._background = Bitmap(self, background)

        -- set size of group relative to background
        self.Width:Set(self._background.Width)
        self.Height:Set(self._background.Height)

        -- set background to top left of group
        self._background.Left:Set(self.Left)
        self._background.Top:Set(self.Top)

        -- place the thumb in the center of the group on the non-moving axis
        -- set the bottom or left to a function that is determined by the current and end values
        self._thumb.Depth:Set(function() return self._background.Depth() + 1 end)
        if self._isVertical then
            LayoutHelpers.AtHorizontalCenterIn(self._thumb, self)
            self._thumb.Bottom:Set(function()
                    return math.floor(self.Bottom() - (((self._currentValue() - self._startValue) / (self._endValue - self._startValue)) * (self.Bottom() - (self.Top() + self._thumb.Height()))))
                end)
        else
            LayoutHelpers.AtVerticalCenterIn(self._thumb, self)
            self._thumb.Left:Set(function()
                    return math.floor(self.Left() + (((self._currentValue() - self._startValue) / (self._endValue - self._startValue)) * (self.Right() - (self.Left() + self._thumb.Width()))))
                end)
        end

        -- set up thumb event handler
        self._thumb.HandleEvent = function(control, event)
            eventHandled = false
            if event.Type == 'ButtonPress' then
                local dragger = Dragger()
                dragger.OnMove = function(control, x, y)
                    self:SetValue(self:CalculateValueFromMouse(x, y))
                end

                dragger.OnRelease = function(control, x, y)
                    local value = self:CalculateValueFromMouse(x, y)
                    self:SetValue(value)
                    self:OnValueSet(self:_Constrain(value))
                    dragger:Destroy()
                end

                PostDragger(self:GetRootFrame(), event.KeyCode, dragger)
                eventHandled = true
            end

            return eventHandled
        end

        -- set up background event handler
        self._background.HandleEvent = function(control, event)
            eventHandled = false
            if event.Type == 'ButtonPress' or event.Type == 'ButtonDClick' then
                local value = self:CalculateValueFromMouse(event.MouseX, event.MouseY)
                self:SetValue(value)
                self:OnValueSet(self._currentValue())
                eventHandled = true
            end
            return eventHandled
        end
    end,

    -- this will constrain your values to not exceed min or max
    SetValue = function(self, value)
        self._currentValue:Set(self:_Constrain(value))
        self:OnValueChanged(self._currentValue())
    end,

    GetValue = function(self)
        return self._currentValue()
    end,

    SetStartValue = function(self, startValue)
        self._startValue = startValue
        self:SetValue(self._currentValue())
    end,

    SetEndValue = function(self, endValue)
        self._endValue = endValue
        self:SetValue(self._currentValue())
    end,

    CalculateValueFromMouse = function(self, x, y)
        local newValue = self._currentValue()
        if self._isVertical then
            newValue = self._startValue + (((self.Bottom() - y) / (self.Bottom() - (self.Top() + self._thumb.Height()))) * (self._endValue - self._startValue))
        else
            newValue = self._startValue + (((x - self.Left()) / (self.Right() - (self.Left() + self._thumb.Width()))) * (self._endValue - self._startValue))
        end
        return newValue
    end,

    _Constrain = function(self, value)
        return math.max(math.min(value, self._endValue), self._startValue)
    end,

    -- overload to be informed when the value is set by a mouse release
    OnValueSet = function(self, newValue) end,

    -- overload to be informed when the value is changed
    OnValueChanged = function(self, newValue) end,
}

IntegerSlider = Class(Slider)
{
    __init = function(self, parent, isVertical, startValue, endValue, indentValue, thumb, background)
        Slider.__init(self, parent, isVertical, math.floor(startValue), math.floor(endValue), thumb, background)
        self._indentValue = math.floor(indentValue)
    end,

    SetValue = function(self, value)
        value = self:_Constrain(value)
        value = math.floor(value / self._indentValue) * self._indentValue
        self._currentValue:Set(value)
        self:OnValueChanged(self._currentValue())
    end,

}
