-- Class methods:
-- SetNewFont(family, pointsize)
-- SetNewColor(color)
-- SetText(text)
-- string GetText()
-- SetDropShadow(bool)
-- SetNewClipToWidth(bool)
-- SetCenteredVertically(bool)
-- SetCenteredHorizontally(bool)

local Control = import('control.lua').Control

Text = Class(moho.text_methods, Control) {

    __init = function(self, parent, debugname)
        InternalCreateText(self, parent)
        if debugname then
            self:SetName(debugname)
        end

        local LazyVar = import('/lua/modules/lazyvar.lua')
        self._font = {_family = LazyVar.Create(), _pointsize = 12}
        self._font._family.OnDirty = function(var)
            self:SetNewFont(self._font._family(), self._font._pointsize)
        end

        self._color = LazyVar.Create()
        self._color.OnDirty = function(var)
            self:SetNewColor(var())
        end
    end,

    OnInit = function(self)
        Control.OnInit(self)
        self.Height:Set(function() return math.floor(self.FontAscent() + self.FontDescent()) end)
        self:SetClipToWidth(false)
    end,

    SetClipToWidth = function(self, clipToWidth)
        if clipToWidth then
            self.Width:Set(function() return self.Right() - self.Left() end)
        else
            self.Width:Set(function() return math.floor(self.TextAdvance()) end)
        end
        self:SetNewClipToWidth(clipToWidth)
    end,

    -- lazy var support
    SetFont = function(self, family, pointsize)
        if self._font then
            self._font._pointsize = pointsize
            self._font._family:Set(family)
        end
    end,


    StreamText = function(self, text, speed)
        ForkThread(function()
            local tempString = ""
            local index = 1
            while string.len(tempString) < string.len(text) do
                missionTextInactivityTime = 0
                tempString = string.sub(text, 1, index)
                self:SetText(tempString)
                index = index + speed
                WaitSeconds(.01)
            end
            self:SetText(LOC(text))
        end)
    end,
       
    SetColor = function(self, color)
        if self._color then
            self._color:Set(color)
        end
    end,

    OnDestroy = function(self)
       self._font._family:Destroy()
       self._font = nil
       self._color:Destroy()
       self._color = nil
    end,
}

-- given a string, a line width and a function to calculate the advance of a string, returns an array of wrapped text
function WrapText(text, lineWidth, advanceFunction)
    local result = {}
    local pos = 0
    local curLine = 1
    local spaceWidth = advanceFunction(" ")
    for packedWord in string.gfind(text, "[^ \t]+") do
        local words = {}
        local lfStartIndex = string.find(packedWord, "\n")
        local len = string.len(packedWord)
        local curIndex = 1
        if lfStartIndex then
            -- split out new lines from packedWords and make them their own entry in the table
            while lfStartIndex do
                if lfStartIndex - curIndex > 0 then
                    -- found a word before this line feed, so get it and push in to the word list
                    table.insert(words, string.sub(packedWord, curIndex, lfStartIndex - 1))
                end
                table.insert(words, "\n")   -- add the line feed to the word list
                curIndex = lfStartIndex + 1
                lfStartIndex = string.find(packedWord, "\n", curIndex)
                -- pick up any trailing word
                if not lfStartIndex and curIndex < len then
                    table.insert(words, string.sub(packedWord, curIndex, len))
                end
            end
        else
            table.insert(words, packedWord)
        end

        for index, word in words do

            if word == "\n" then
                if not result[curLine] then result[curLine] = "" end    -- result can't have nil lines
                curLine = curLine + 1
                pos = 0
                continue
            end

            local wordWidth = advanceFunction(word)

            if wordWidth > lineWidth then
                -- start long word on its own line
                if result[curLine] then
                    curLine = curLine + 1
                end

                local wordLines = math.ceil(wordWidth / lineWidth)
                local charsPerLine = math.floor(string.len(word) / wordLines)
                local curCharLineStart = 1
                for line = curLine, curLine + wordLines do
                    local endChar = curCharLineStart + charsPerLine - 1
                    if endChar > string.len(word) then endChar = string.len(word) end
                    result[curLine] = string.sub(word, curCharLineStart, endChar)
                    curLine = curLine + 1
                    curCharLineStart = endChar + 1
                end

                pos = 0
            else
                if pos + wordWidth < lineWidth then
                    if result[curLine] then
                        result[curLine] = result[curLine] .. word .. " "
                    else
                        result[curLine] = word .. " "
                    end
                else
                    curLine = curLine + 1
                    pos = 0
                    result[curLine] = word .. " "
                end

                pos = pos + wordWidth + spaceWidth
            end
        end
    end
    return result
end