#****************************************************************************
#**
#**  File     :  /cdimage/lua/modules/platoon.lua
#**  Author(s):
#**
#**  Summary  :
#**
#**  Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
#****************************************************************************
#########################################
# Platoon Lua Module                    #
#########################################
local AIUtils = import('ai/aiutilities.lua')
local Utilities = import('/lua/modules/utilities.lua')
local AIBuildStructures = import('/lua/modules/ai/aibuildstructures.lua')
local UnitUpgradeTemplates = import('/lua/modules/upgradetemplates.lua').UnitUpgradeTemplates
local StructureUpgradeTemplates = import('/lua/modules/upgradetemplates.lua').StructureUpgradeTemplates
local Behaviors = import('/lua/modules/ai/aibehaviors.lua')
local AIAttackUtils = import('/lua/modules/AI/aiattackutilities.lua')

Platoon = Class(moho.platoon_methods) {
    NeedCoolDown = false,

    OnCreate = function(self, plan)
        self.Trash = TrashBag()
        if self[plan] then
            self.AIThread = self:ForkThread(self[plan])
        end
        self.PlatoonData = {}
        self.EventCallbacks = {
            OnDestroyed = {},
        }
        self.PartOfAttackForce = false
    end,

    SetPlatoonData = function(self, dataTable)
        self.PlatoonData = table.deepcopy(dataTable)
    end,

    SetPartOfAttackForce = function(self)
        if not self.PlatoonData then
            self.PlatoonData = {}
        end
        if self.PlatoonData.NotPartOfAttackForce then return end
        local platoonsGiven = false
        local aiBrain = self:GetBrain()
        self.PartOfAttackForce = true
        # Because of how the PlatoonData in the editor exports table.getn will not work here
        if self.PlatoonData.AMPlatoons then
            for k,v in self.PlatoonData.AMPlatoons do
                platoonsGiven = true
                if not aiBrain.AttackData.AMPlatoonCount[v] then
                    aiBrain.AttackData.AMPlatoonCount[v] = 1
                else
                    aiBrain.AttackData.AMPlatoonCount[v] = aiBrain.AttackData.AMPlatoonCount[v] + 1
                end
            end
        end
        if not platoonsGiven then
            local testUnit = self:GetPlatoonUnits()[1]
            if testUnit then
                self.PlatoonData.AMPlatoons = {}
                if EntityCategoryContains( categories.MOBILE * categories.AIR, testUnit ) then
                    aiBrain.AttackData.AMPlatoonCount['DefaultGroupAir'] = aiBrain.AttackData.AMPlatoonCount['DefaultGroupAir'] + 1
                    table.insert( self.PlatoonData.AMPlatoons, 'DefaultGroupAir' )

                elseif EntityCategoryContains( categories.MOBILE * categories.LAND, testUnit ) then
                    aiBrain.AttackData.AMPlatoonCount['DefaultGroupLand'] = aiBrain.AttackData.AMPlatoonCount['DefaultGroupLand'] + 1
                    table.insert( self.PlatoonData.AMPlatoons, 'DefaultGroupLand' )

                elseif EntityCategoryContains( categories.MOBILE * categories.NAVAL, testUnit ) then
                    aiBrain.AttackData.AMPlatoonCount['DefaultGroupSea'] = aiBrain.AttackData.AMPlatoonCount['DefaultGroupSea'] + 1
                    table.insert( self.PlatoonData.AMPlatoons, 'DefaultGroupSea' )
                end
            end
        end
        self:AddDestroyCallback(aiBrain.AMDecrementCount)
    end,


    IsPartOfAttackForce = function(self)
        return self.PartOfAttackForce
    end,

    ForkAIThread = function(self, fn, ...)
        if fn then
            self.AIThread = ForkThread(fn, self, unpack(arg))
            self.Trash:Add(self.AIThread)
            return self.AIThread
        else
            return nil
        end
    end,

    StopAI = function(self)
        if self.AIThread ~= nil then
            self.AIThread:Destroy()
        end
    end,

    AddDestroyCallback = function(self, callbackFunction)
        if not callbackFunction then
            error('*ERROR: Tried to add an OnDestroy on a platoon callback with a nil function')
            return
        end
        table.insert(self.EventCallbacks.OnDestroyed, callbackFunction)
    end,

    DoDestroyCallbacks = function(self)
        if self.EventCallbacks.OnDestroyed then
            for k, cb in self.EventCallbacks.OnDestroyed do
                if cb then
                    cb(self:GetBrain(), self)
                end
            end
        end
    end,

    RemoveDestroyCallback = function(self, fn)
        for k,v in self.EventCallbacks.OnDestroyed do
            if v == fn then
                self.EventCallbacks.OnDestroyed[k] = nil
            end
        end
    end,

    OnDestroy = function(self)
        self:DoDestroyCallbacks()
        if self.Trash then
            self.Trash:Destroy()
        end
    end,

    ForkThread = function(self, fn, ...)
        if fn then
            local thread = ForkThread(fn, self, unpack(arg))
            self.Trash:Add(thread)
            return thread
        else
            return nil
        end
    end,

    SetAIPlan = function(self, plan)
        if not self[plan] then return end
        if self.AIThread then
            self.AIThread:Destroy()
        end
        self.PlanName = plan
        self:ForkAIThread(self[plan])
    end,

    GetPlan = function(self)
        if self.PlanName then
            return self.PlanName
        end
    end,

    GetThreatLevel = function(self, rings)
        local brain = self:GetBrain()
        return brain:GetThreatAtPosition(self:GetPlatoonPosition(), rings, false)
    end,

    CheckCommandsCompleted = function(self, commands)
        for k, v in commands do
            if self:IsCommandsActive(commands) then
                return false
            end
        end
        return true
    end,

    TurnOffPoolAI = function(self)
        if self.PoolAIOn then
            self.AIThread:Destroy()
            self.PoolAIOn = false
            self.AIThread = nil
        end
    end,

    TurnOnPoolAI = function(self)
        if not self.PoolAIOn and not self.AIThread then
            self.AIThread = self:ForkAIThread(self.PoolAI)
        end
    end,

    PoolAI = function(self)

    end,

    # ===== AI THREADS ===== #
    BuildOnceAI = function(self)
        local aiBrain = self:GetBrain()
        for k,v in self:GetPlatoonUnits() do
            if not v:IsDead() then
                v.PreviousPriority = aiBrain:PBMGetPriority(self)
                v.Platoon = self
            end
        end
        aiBrain:PBMSetPriority(self, 0)
    end,
    
    EnhanceAI = function(self)
        local aiBrain = self:GetBrain()
        local unit
        local data = self.PlatoonData
        for k,v in self:GetPlatoonUnits() do
            unit = v
            break
        end
        if unit then
            IssueStop({unit})
            IssueClearCommands({unit})
            for k,v in data.Enhancement do
                IssueEnhance({unit}, v)
            end
            WaitSeconds(data.TimeBetweenEnhancements or 1)
            repeat
                WaitSeconds(5)
                if not aiBrain:PlatoonExists(self) then
                    return
                end
            until unit:IsDead() or not unit:IsUnitState('Upgrading')
        end
        if data.DoNotDisband then return end
        aiBrain:DisbandPlatoon(self)
    end,

    HuntAI = function(self)
        self:Stop()
        local aiBrain = self:GetBrain()
        local armyIndex = aiBrain:GetArmyIndex()
        local target
        local blip
        while aiBrain:PlatoonExists(self) do
            if self:IsOpponentAIRunning() then
                target = self:FindClosestUnit('Attack', 'Enemy', true, categories.ALLUNITS - categories.WALL)
                if target then
                    blip = target:GetBlip(armyIndex)
                    self:Stop()
                    self:AggressiveMoveToLocation(target:GetPosition())
                end
            end
            WaitSeconds(17)
        end
    end,
    
    TacticalAI = function(self)
        self:Stop()
        local aiBrain = self:GetBrain()
        local armyIndex = aiBrain:GetArmyIndex()
        local platoonUnits = self:GetPlatoonUnits()
        local unit
        #GET THE Launcher OUT OF THIS PLATOON
        for k, v in platoonUnits do
            if EntityCategoryContains(categories.STRUCTURE * categories.TACTICALMISSILEPLATFORM, v) then
                unit = v
                break
            end
        end
        local bp = unit:GetBlueprint()
        local weapon = bp.Weapon[1]
        local maxRadius = weapon.MaxRadius
        local minRadius = weapon.MinRadius
        unit:SetAutoMode(true)
        local atkPri = { 'COMMAND', 'STRUCTURE STRATEGIC', 'STRUCTURE DEFENSE', 'CONSTRUCTION', 'EXPERIMENTAL MOBILE LAND', 'TECH3 MOBILE LAND',
            'TECH2 MOBILE LAND', 'TECH1 MOBILE LAND', 'ALLUNITS' }
        self:SetPrioritizedTargetList( 'Attack', { categories.COMMAND, categories.CONSTRUCTION, categories.STRUCTURE * categories.DEFENSE, 
            categories.EXPERIMENTAL * categories.MOBILE, categories.TECH3 * categories.MOBILE, categories.TECH2 * categories.MOBILE, 
            categories.TECH1 * categories.MOBILE, categories.ALLUNITS } )
        while aiBrain:PlatoonExists(self) do
            local target = false
            local blip = false
            while unit:GetTacticalSiloAmmoCount() < 1 or not target do
                WaitSeconds(7)
                target = false
                while not target do
                    if aiBrain:GetCurrentEnemy() and aiBrain:GetCurrentEnemy():IsDefeated() then
                        aiBrain:PickEnemyLogic()
                    end
                    if aiBrain.PlatoonNameCounter['AttackForce'] > 0 then
                        # Attack enemy base to help attacking platoon
                        target = AIAttackUtils.AIFindStrongestInRoute( aiBrain, maxRadius, atkPri, aiBrain.LastAttackForceRoute, aiBrain:GetCurrentEnemy() )
                    else
                        target = AIUtils.AIFindBrainTargetInRange( aiBrain, self, 'Attack', maxRadius, atkPri, aiBrain:GetCurrentEnemy() )
                    end                       
                    if not target then
                        target = self:FindPrioritizedUnit('Attack', 'Enemy', true, unit:GetPosition(), maxRadius)
                    end
                    if target then
                        break
                    end
                    WaitSeconds(3)
                    if not aiBrain:PlatoonExists(self) then
                        return
                    end
                end
            end
            if not target:IsDead() then
                #LOG('*AI DEBUG: Firing Tactical Missile at enemy swine!')
                IssueTactical({unit}, target)
            end
            WaitSeconds(3)
        end
    end,

    NukeAI = function(self)
        self:Stop()
        local aiBrain = self:GetBrain()
        local platoonUnits = self:GetPlatoonUnits()
        local unit
        #GET THE Launcher OUT OF THIS PLATOON
        for k, v in platoonUnits do
            if EntityCategoryContains(categories.SILO * categories.NUKE, v) then
                unit = v
                break
            end
        end
        local bp = unit:GetBlueprint()
        local weapon = bp.Weapon[1]
        local maxRadius = weapon.MaxRadius
        unit:SetAutoMode(true)
        while aiBrain:PlatoonExists(self) do
            while unit:GetNukeSiloAmmoCount() < 1 do
                WaitSeconds( 11 )
                if not  aiBrain:PlatoonExists(self) then
                    return
                end
            end
            IssueNuke({unit}, (aiBrain:GetHighestThreatPosition(2, false)))
            WaitSeconds(11)
        end
    end,


    AntiNukeAI = function(self)
        self:Stop()
        local platoonUnits = self:GetPlatoonUnits()
        local aiBrain = self:GetBrain()
        local antiNuke
        #GET THE AntiNuke OUT OF THIS PLATOON
        for k, v in platoonUnits do
            if EntityCategoryContains(categories.STRUCTURE * categories.ANTIMISSILE * categories.TECH3, v) then
                antiNuke = v
                break
            end
        end
        # Toggle on auto build ?!?!?
        antiNuke:SetAutoMode(true)
    end,

    PauseAI = function(self)
        local platoonUnits = self:GetPlatoonUnits()
        local aiBrain = self:GetBrain()
        IssuePause(platoonUnits)
        local econ = AIUtils.AIGetEconomyNumbers(aiBrain)
        while econ.EnergyStorageRatio < 0.4 do
            WaitSeconds(2)
            econ = AIUtils.AIGetEconomyNumbers(aiBrain)
        end
        IssuePause(platoonUnits)
        aiBrain:DisbandPlatoon(self)
    end,

    ScoutingAI = function(self)
        local aiBrain = self:GetBrain()
        local personality = self:GetPersonality()
        local needAttackCheck = false
        local targetSwitch = true
        local scoutPath = {}
        while aiBrain:PlatoonExists(self) do
            if self:IsOpponentAIRunning() then
                scoutPath = AIUtils.AIGetSortedScoutingLocations(self:GetBrain())
                for k, v in scoutPath do
                    if v[1] < 0 or v[3] < 0 or v[1] > ScenarioInfo.size[1] or v[3] > ScenarioInfo.size[2] then
                        #LOG('*AI DEBUG: SCOUT SENDING UNITS TO WRONG LOCATION - ' .. v[1] .. ', ' .. v[3] )
                    end
                    self:MoveToLocation(v, false)
                end
                local startPosX, startPosZ = self:GetBrain():GetArmyStartPos()
                local height = GetTerrainHeight( startPosX, startPosZ )
                if GetSurfaceHeight( startPosX, startPosZ ) > height then
                    height = GetSurfaceHeight( startPosX, startPosZ )
                end
                self:MoveToLocation(Vector(startPosX, height, startPosZ), false)
            end
            WaitSeconds(11)
            if not aiBrain:PlatoonExists(self) then
                return
            end
            while self:IsMoving('scout') do
                WaitSeconds(31)
                if not aiBrain:PlatoonExists(self) then
                    return
                end
            end
        end
    end,

    AirCraftStagingAI = function(self)
        self:SetPartOfAttackForce()
    end,
    
    SubCommanderAssistAI = function(self)
        local aiBrain = self:GetBrain()
        local location = self.PlatoonData.LocationType or self.PlatoonData.Position or self:GetPlatoonPosition()
        local radius = self.PlatoonData.Radius or 100
        local assistingBuildings = {}
        #local 
        while aiBrain:PlatoonExists(self) do
        end
    end,

    PatrolNearbyDefensiveAreaAI = function(self)
        self:SetPartOfAttackForce()
        if not self.PlatoonData then
            self.PlatoonData = {}
        end
        local personality = self:GetPersonality()
        local needAttackCheck = false
        local targetSwitch = true
        local scoutPath = {}
        if self:IsOpponentAIRunning() then
            scoutPath = AIUtils.AIGetSortedDefensiveLocations(self:GetBrain(), 3)
            for k, v in scoutPath do
                self:Patrol(v)
            end
        end
    end,
    
    PatrolLocationFactoriesAI = function(self)
        local aiBrain = self:GetBrain()
        local location = self.PlatoonData.LocationType or 'MAIN'
        local position = aiBrain:PBMGetLocationCoords(location)
        local radius = aiBrain:PBMGetLocationRadius(location)
        while aiBrain:PlatoonExists(self) do
            local factories = aiBrain:PBMGetLocationFactories(location)
            if factories then
                for fType,fac in factories do
                    if not fac:IsDead() then
                        self:Patrol(fac:GetPosition())
                        local guards = fac:GetGuards()
                        if guards then
                            for num,guard in guards do
                                self:Patrol(guard:GetPosition())
                            end
                        end
                    end
                end
            else
                aiBrain:DisbandPlatoon(self)
            end
            WaitSeconds(71)
        end
    end,

    PatrolBaseVectorsAI = function(self)
        self:SetPartOfAttackForce()
        local aiBrain = self:GetBrain()
        local location = self.PlatoonData.LocationType or self.PlatoonData.Position or self:GetPlatoonPosition()
        local radius = self.PlatoonData.Radius or 100
        local first = true
        local unit = false
        for k,v in self:GetPlatoonUnits() do
            if not v:IsDead() and EntityCategoryContains( categories.MOBILE * categories.LAND, v ) then
                unit = v
                break
            end
        end
        for k,v in AIUtils.GetBasePatrolPoints(aiBrain, location, radius) do
            if not unit or AIUtils.CheckUnitPathing( v, unit:GetPosition(), unit ) then
                if first then
                    self:MoveToLocation(v, false)
                    first = false
                else
                    self:Patrol(v)
                end
            end
        end
    end,

    PlatoonCallForHelpAI = function(self)
        local aiBrain = self:GetBrain()
        local checkTime = self.PlatoonData.DistressCheckTime or 7
        while aiBrain:PlatoonExists(self) do
            if not self.DistressCall then
                local pos = self:GetPlatoonPosition()
                if pos then
                    local threat = aiBrain:GetThreatAtPosition( self:GetPlatoonPosition(), 1, false )
                    if threat and threat > 1 then
                        aiBrain:BaseMonitorPlatoonDistress(self, threat)
                        self.DistressCall = true
                    end
                end
            end
            WaitSeconds(checkTime)
        end
    end,

    DistressResponseAI = function(self)
        local aiBrain = self:GetBrain()
        local distressRange = self.PlatoonData.DistressRange or aiBrain.BaseMonitor.DefaultDistressRange
        local reactionTime = self.PlatoonData.DistressReactionTime or aiBrain.BaseMonitor.PlatoonDefaultReactionTime
        while aiBrain:PlatoonExists(self) do
            if not self.DistressCall then
                local distressLocation = aiBrain:BaseMonitorDistressLocation(self:GetPlatoonPosition())
                local moveLocation
                if distressLocation then
                    local platPos = self:GetPlatoonPosition()
                    local distance = VDist2( platPos[1], platPos[3], distressLocation[1], distressLocation[3] )
                    if distance < distressRange then
                        #LOG('*AI DEBUG: ARMY '.. aiBrain:GetArmyIndex() ..': --- DISTRESS RESPONSE AI ACTIVATION ---')
                        local oldPlan = self:GetPlan()
                        if self.AiThread then
                            self.AIThread:Destroy()
                        end
                        repeat
                            moveLocation = distressLocation
                            self:Stop()
                            local cmd = self:AggressiveMoveToLocation( distressLocation )
                            local numEnemies
                            repeat
                                WaitSeconds(reactionTime)
                                if not aiBrain:PlatoonExists(self) then
                                    return
                                end
                                #numEnemies = aiBrain:GetNumUnitsAroundPoint( 'Enemy', aiBrain.BaseMonitor.UnitCategoryCheck, self:GetPlatoonPosition(), aiBrain.BaseMonitor.UnitCheckRadius*2 )
                            until not self:IsCommandsActive(cmd) # or numEnemies <= aiBrain.BaseMonitor.UnitRadiusThreshold
                            distressLocation = aiBrain:BaseMonitorDistressLocation()
                            if distressLocation then
                                if distressLocation[1] <= 0 or distressLocation[3] <= 0 or distressLocation[1] >= ScenarioInfo.size[1] or
                                        distressLocation[3] >= ScenarioInfo.size[2] then
                                    #LOG('*AI DEBUG: DISTRESS RESPONSE SENDING UNITS TO WRONG LOCATION')
                                end
                                distance = VDist2( platPos[1], platPos[3], distressLocation[1], distressLocation[3] )
                            end
                        until not distressLocation or distance > distressRange or ( distressLocation[1] == moveLocation[1] and distressLocation[3] == moveLocation[3] )
                        #LOG('*AI DEBUG: '..aiBrain.Name..' DISTRESS RESPONSE AI DEACTIVATION - oldPlan: '..oldPlan)
                        # Return to old shit
                        self:SetAIPlan(oldPlan)
                    end
                end
            end
            WaitSeconds(11)
        end
    end,

    PoolDistressAI = function(self)
        local aiBrain = self:GetBrain()
        local distressRange = aiBrain.BaseMonitor.PoolDistressRange
        local reactionTime = aiBrain.BaseMonitor.PoolReactionTime
        while aiBrain:PlatoonExists(self) do
            local platoonUnits = self:GetPlatoonUnits()
            for locNum, locData in aiBrain.PBM.Locations do
                if not locData.DistressCall then
                    local distressLocation = aiBrain:BaseMonitorDistressLocation( locData.Location )
                    local moveLocation
                    if distressLocation and VDist2( distressLocation[1], distressLocation[3], locData.Location[1], locData.Location[3] ) <= aiBrain.BaseMonitor.PoolDistressRange then
                        #LOG('*AI DEBUG: ARMY '.. aiBrain:GetArmyIndex() ..': --- POOL DISTRESS RESPONSE ---')
                        local group = {}
                        for k,v in platoonUnits do
                            vPos = v:GetPosition()
                            if VDist2( vPos[1], vPos[3], locData.Location[1], locData.Location[3] ) < locData.Radius then
                                table.insert(group, v)
                            end
                        end
                        IssueClearCommands( group )
                        if distressLocation[1] <= 0 or distressLocation[3] <= 0 or distressLocation[1] >= ScenarioInfo.size[1] or
                                distressLocation[3] >= ScenarioInfo.size[2] then
                            #LOG('*AI DEBUG: POOLDISTRESSAI SENDING UNITS TO WRONG LOCATION')
                        end
                        IssueMove( group, distressLocation )
                        IssueMove( group, aiBrain:PBMGetLocationCoords( locData.LocationType ) )
                        locData.DistressCall = true
                        self:ForkThread( self.UnlockPBMDistressLocation, locData )
                    end
                end
            end
            WaitSeconds( aiBrain.BaseMonitor.PoolReactionTime )
        end
    end,

    UnlockPBMDistressLocation = function(self, locData)
        WaitSeconds(15)
        locData.DistressCall = false
    end,

    DisbandAI = function(self)
        self:Stop()
        self:GetBrain():DisbandPlatoon(self)
    end,

    NavalPatrolAI = function(self)
        self:Stop()
        local aiBrain = self:GetBrain()
        local subs = {}
        for k,v in self:GetPlatoonUnits() do
            if EntityCategoryContains( categories.SUBMERSIBLE, v ) then
                table.insert( subs, v )
            end
        end

        local atkPri = { 'SPECIALHIGHPRI', 'MOBILE NAVAL', 'STRUCTURE NAVAL', 'STRUCTURE STRATEGIC', 'STRUCTURE DEFENSE', 'COMMAND', 
                        'STRUCTURE', 'MOBILE', 'SPECIALLOWPRI', 'ALLUNITS' }
        local atkPriTable = {}
        for k,v in atkPri do
            table.insert( atkPriTable, ParseEntityCategory( v ) )
        end
        self:SetPrioritizedTargetList( 'Attack', atkPriTable )

        if table.getn( subs ) > 0 then
            local subsOnWater = {}
            for k, v in subs do
                if v:GetCurrentLayer() == 'Water' then
                    table.insert(subsOnWater, v)
                end
            end
            local cmd = IssueDive( subsOnWater )
            local diving = false
            repeat
                WaitSeconds(2)
                if not aiBrain:PlatoonExists(self) then
                    return
                end
                for k,v in subs do
                    if not v:IsDead() and v:IsUnitState( 'Diving' ) then
                        diving = true
                        break
                    end
                end
            until not diving
        end

        local scoutPath = {}
        if self:IsOpponentAIRunning() then
            scoutPath = AIUtils.AIGetSortedNavalLocations(self:GetBrain())
            for k, v in scoutPath do
                self:Patrol(v)
            end
        end
    end,

    EngineerAI = function(self)
        self:Stop()
        local aiBrain = self:GetBrain()
        aiBrain:DisbandPlatoon(self)
    end,
    
    CaptureAI = function(self)
        self:Stop()
        local aiBrain = self:GetBrain()
        local index = aiBrain:GetArmyIndex()
        local data = self.PlatoonData
        local pos = self:GetPlatoonPosition()
        local radius = data.Radius or 100
        if not data.Categories then
            error( 'PLATOON.LUA ERROR- CaptureAI requires Categories field',2)
        end
        
        local checkThreat = false
        if data.ThreatMin and data.ThreatMax and data.ThreatRings then
            checkThreat = true
        end
        while aiBrain:PlatoonExists( self ) do
            local target = AIAttackUtils.AIFindUnitRadiusThreat( aiBrain, 'Enemy', data.Categories, pos, radius, data.ThreatMin, data.ThreatMax, data.ThreatRings )
            if target then
                local blip = target:GetBlip(index)
                if blip then
                    IssueClearCommands( self:GetPlatoonUnits() )
                    IssueCapture( self:GetPlatoonUnits(), target )
                    local allIdle
                    repeat
                        WaitSeconds(2)
                        if not aiBrain:PlatoonExists(self) then
                            return
                        end
                        allIdle = true
                        for k,v in self:GetPlatoonUnits() do
                            if not v:IsDead() and not v:IsIdleState() then
                                allIdle = false
                                break
                            end
                        end
                    until allIdle or blip:BeenDestroyed() or blip:IsKnownFake() or blip:IsMaybeDead()
                end
            else
                local location = AIUtils.RandomLocation(aiBrain:GetArmyStartPos())
                self:MoveToLocation( location, false )
                aiBrain:DisbandPlatoon(self)
            end
            WaitSeconds(1)
        end
    end,
    
    ReclaimAI = function(self)
        self:Stop()
        local aiBrain = self:GetBrain()
        local pos = self:GetPlatoonPosition()
        local radius = 100
        local x1 = pos[1] - 100
        local x2 = pos[1] + 100
        local z1 = pos[3] - 100
        local z2 = pos[3] + 100
        local rect = Rect( x1, z1, x2, z2 )
        local counter = 0
        while aiBrain:PlatoonExists(self) do
            local massRatio = aiBrain:GetEconomyStoredRatio('MASS')
            local energyRatio = aiBrain:GetEconomyStoredRatio('ENERGY')
            local massFirst = true
            if energyRatio < massRatio then
                massFirst = false
            end
            local ents = GetEntitiesInRect( rect )
            local reclaimables = { Mass = {}, Energy = {} }
            for k,v in ents do
                if v.MassReclaim and v.MassReclaim > 0 then
                    table.insert( reclaimables.Mass, v )
                elseif v.EnergyReclaim and v.EnergyReclaim > 0 then
                    table.insert( reclaimables.Energy, v )
                end
            end
            local unitPos = self:GetPlatoonPosition()
            local closest, entType, distance
            for num, recType in reclaimables do
                for k,v in recType do
                    local tempDist = VDist2( unitPos[1], unitPos[3], v:GetPosition()[1], v:GetPosition()[3] )
                    if ( not closest or tempDist < distance ) then
                        closest = v
                        entType = num
                        distance = tempDist
                    end
                end
                if num == 'Mass' and closest and massFirst then
                    break
                end
            end
            if closest and ( entType == 'Mass' and massRatio < .9 ) or ( entType == 'Energy' and energyRatio < .9 ) then
                counter = 0
                IssueReclaim( self:GetPlatoonUnits(), closest )
                local allIdle
                repeat
                    WaitSeconds(2)
                    if not aiBrain:PlatoonExists(self) then
                        return
                    end
                    allIdle = true
                    for k,v in self:GetPlatoonUnits() do
                        if not v:IsDead() and not v:IsIdleState() then
                            allIdle = false
                            break
                        end
                    end
                until allIdle
            elseif not closest or counter >= 5 then
                aiBrain:DisbandPlatoon(self)
            else
                counter = counter + 1
                WaitSeconds(5)
            end
        end
    end,
    
    ReclaimUnitsAI = function(self)
        self:Stop()
        local aiBrain = self:GetBrain()
        local index = aiBrain:GetArmyIndex()
        local data = self.PlatoonData
        local pos = self:GetPlatoonPosition()
        local radius = data.Radius or 500
        if not data.Categories then
            error( 'PLATOON.LUA ERROR- ReclaimUnitsAI requires Categories field',2)
        end
        
        local checkThreat = false
        if data.ThreatMin and data.ThreatMax and data.ThreatRings then
            checkThreat = true
        end
        while aiBrain:PlatoonExists(self) do
            local target = AIAttackUtils.AIFindUnitRadiusThreat( aiBrain, 'Enemy', data.Categories, pos, radius, data.ThreatMin, data.ThreatMax, data.ThreatRings )
            if target then
                local blip = target:GetBlip(index)
                if blip then
                    IssueClearCommands( self:GetPlatoonUnits() )
                    IssueReclaim( self:GetPlatoonUnits(), target )
                    local allIdle
                    repeat
                        WaitSeconds(2)
                        if not aiBrain:PlatoonExists(self) then
                            return
                        end
                        allIdle = true
                        for k,v in self:GetPlatoonUnits() do
                            if not v:IsDead() and not v:IsIdleState() then
                                allIdle = false
                                break
                            end
                        end
                    until allIdle or blip:BeenDestroyed() or blip:IsKnownFake() or blip:IsMaybeDead()
                else
                    WaitSeconds(2)
                end
            else
                local location = AIUtils.RandomLocation(aiBrain:GetArmyStartPos())
                self:MoveToLocation( location, false )
                aiBrain:DisbandPlatoon(self)
            end
            WaitSeconds(1)
        end
    end,

    EngineerAssistAI = function(self)
        #LOG('*DEBUG: ENGINEER ASSIST AI ACTIVE')
        self:Stop()
        local platoonUnits = self:GetPlatoonUnits()
        local aiBrain = self:GetBrain()
        local assistData = self.PlatoonData.Assist
        local platoonPos = self:GetPlatoonPosition()
        local assistee = false
        local assistingBool = false
        if assistData then
            local assistRange = assistData.AssistRange or 80
            # Check for units being built
            if assistData.BeingBuiltCategories then
                local unitsBuilding = aiBrain:GetListOfUnits( categories.CONSTRUCTION, false )
                for catNum, buildeeCat in assistData.BeingBuiltCategories do
                    local buildCat = ParseEntityCategory(buildeeCat)
                    for unitNum, unit in unitsBuilding do
                        if not unit:BeenDestroyed() and unit:IsUnitState('Building') then
                            local buildingUnit = unit:GetUnitBeingBuilt()
                            if buildingUnit and not buildingUnit:IsDead() and EntityCategoryContains( buildCat, buildingUnit ) then
                                local unitPos = unit:GetPosition()
                                if VDist2(platoonPos[1], platoonPos[3], unitPos[1], unitPos[3]) < assistRange then
                                    assistee = unit
                                    break
                                end
                            end
                        end
                    end
                    if assistee then
                        break
                    end
                end
            end
            # Check for builders
            if not assistee and assistData.BuilderCategories then
                for catNum, buildCat in assistData.BuilderCategories do
                    self:SetPrioritizedTargetList('support', {ParseEntityCategory(buildCat)})
                    assistee = self:FindPrioritizedUnit('support','Ally',false, platoonPos, assistRange)
                    if assistee and not assistee:IsDead() and assistee:IsUnitState('Building') then
                        break
                    else
                        assistee = false
                    end
                end
            end
            # If the unit to be assisted is a factory, assist whatever it is assisting or is assisting it
            # Makes sure all factories have someone helping out to load balance better
            if assistee and not assistee:IsDead() and EntityCategoryContains( categories.FACTORY, assistee ) then
                local guardee = assistee:GetGuardedUnit()
                if guardee and not guardee:IsDead() and EntityCategoryContains( categories.FACTORY, guardee ) then
                    local factories = AIUtils.AIReturnAssistingFactories( guardee )
                    AIUtils.AIEngineersAssistFactories( aiBrain, platoonUnits, factories )
                    assistingBool = true
                elseif table.getn(assistee:GetGuards()) > 0 then
                    local factories = AIUtils.AIReturnAssistingFactories( assistee )
                    AIUtils.AIEngineersAssistFactories( aiBrain, platoonUnits, factories )
                    assistingBool = true
                end
            end
        end
        if assistee and not assistee:IsDead() then
            if not assistingBool then
                IssueGuard( platoonUnits, assistee )
            end
            WaitSeconds(assistData.Time or 60)
        end
        if not aiBrain:PlatoonExists(self) then
            return
        end
        aiBrain:DisbandPlatoon(self)
    end,

    EngineerBuildAI = function(self)
        self:Stop()
        local aiBrain = self:GetBrain()
        local platoonUnits = self:GetPlatoonUnits()
        local factionIndex = self:GetFactionIndex()
        local armyIndex = aiBrain:GetArmyIndex()
        local x,z = aiBrain:GetArmyStartPos()
        local cons = self.PlatoonData.Construction
        local buildingTmpl, buildingTmplFile, baseTmpl, baseTmplFile
        if not cons then
            error('*ERROR: No Construction table in PlatoonData for EngineerBuildAI.  Aborting Platoon AI Thread.', 2)
            self:Stop()
            aiBrain:PBMAdjustPriority(self, -20)
            aiBrain:DisbandPlatoon(self)
            return
        else
            buildingTmplFile = import(cons.BuildingTemplateFile or '/lua/modules/basetemplates.lua')
            baseTmplFile = import(cons.BaseTemplateFile or '/lua/modules/basetemplates.lua')
            buildingTmpl = buildingTmplFile[(cons.BuildingTemplate or 'BuildingTemplates')][factionIndex]
            baseTmpl = baseTmplFile[(cons.BaseTemplate or 'BaseTemplates')][factionIndex]

            if cons.Location then
                local coords = aiBrain:PBMGetLocationCoords(cons.Location)
                self:MoveToLocation(coords, false)
                WaitSeconds(5)
                if not aiBrain:PlatoonExists(self) then
                    return
                end
            end
            local eng
            for k, v in platoonUnits do
                if not v:IsDead() and EntityCategoryContains(categories.CONSTRUCTION, v) then
                    if not eng then
                        eng = v
                    else
                        IssueClearCommands( {v} )
                        IssueGuard({v}, eng)
                    end
                end
            end

            if not eng or eng:IsDead() then
                aiBrain:DisbandPlatoon(self)
                return
            end

            #### CHOOSE APPROPRIATE BUILD FUNCTION AND SETUP BUILD VARIABLES ####
            local reference = false
            local refName = false
            local buildFunction
            local closeToBuilder
            local relative
            local baseTmplList = {}
            if cons.BuildStructures then
                if cons.NearUnitCategory then
                    self:SetPrioritizedTargetList('support', {ParseEntityCategory(cons.NearUnitCategory)})
                    local unitNearBy = self:FindPrioritizedUnit('support', 'Ally', false, self:GetPlatoonPosition(), 50)
                    if unitNearBy then
                        reference = unitNearBy:GetPosition()
                    else
                        reference = eng:GetPosition()
                    end
                    relative = false
                    buildFunction = AIBuildStructures.AIExecuteBuildStructure
                    table.insert( baseTmplList, AIBuildStructures.AIBuildBaseTemplateFromLocation( baseTmpl, reference ) )
                elseif cons.Wall then
                    local pos = aiBrain:PBMGetLocationCoords(cons.LocationType) or cons.Position or self:GetPlatoonPosition()
                    local radius = cons.LocationRadius or aiBrain:PBMGetLocationRadius(cons.LocationType) or 100                    
                    relative = false
                    reference = AIUtils.GetLocationNeedingWalls( aiBrain, 200, 4, 'STRUCTURE - WALLS', cons.ThreatMin, cons.ThreatMax, cons.ThreatRings )
                    table.insert( baseTmplList, 'Blank' )
                    buildFunction = AIBuildStructures.WallBuilder
                elseif cons.NearBasePatrolPoints then
                    relative = false
                    reference = AIUtils.GetBasePatrolPoints(aiBrain, cons.Location or 'MAIN', cons.Radius or 100)
                    baseTmpl = baseTmplFile['ExpansionBaseTemplates'][factionIndex]
                    for k,v in reference do
                        table.insert( baseTmplList, AIBuildStructures.AIBuildBaseTemplateFromLocation( baseTmpl, v ) )
                    end
                    buildFunction = AIBuildStructures.AIBuildBaseTemplateOrdered
                elseif cons.NearMarkerType and cons.MarkerUnitCount then
                    local pos = aiBrain:PBMGetLocationCoords(cons.LocationType) or cons.Position or self:GetPlatoonPosition()
                    local radius = cons.LocationRadius or aiBrain:PBMGetLocationRadius(cons.LocationType) or 100
                    reference, refName = AIUtils.AIGetMarkerLeastUnits( aiBrain, cons.NearMarkerType, (cons.MarkerRadius or 100),
                            pos, radius, cons.MarkerUnitCount, ParseEntityCategory( cons.MarkerUnitCategory ), cons.ThreatMin,
                            cons.ThreatMax, cons.ThreatRings )
                    if not cons.BaseTemplate and ( cons.NearMarkerType == 'Defensive Point' or cons.NearMarkerType == 'Expansion Area' ) then
                        baseTmpl = baseTmplFile['ExpansionBaseTemplates'][factionIndex]
                    end
                    if cons.ExpansionBase and refName then
                        AIBuildStructures.AINewExpansionBase( aiBrain, refName, reference, (cons.ExpansionRadius or 100), cons.ExpansionTypes )
                    end
                    relative = false
                    if reference and aiBrain:GetThreatAtPosition( reference, 1, false ) > -5 then
                        #aiBrain:ExpansionHelp( eng, reference )
                    end
                    table.insert( baseTmplList, AIBuildStructures.AIBuildBaseTemplateFromLocation( baseTmpl, reference ) )
                    buildFunction = AIBuildStructures.AIBuildBaseTemplateOrdered
                elseif cons.NearMarkerType then
                    if not cons.ThreatMin or not cons.ThreatMax or not cons.ThreatRings then
                        cons.ThreatMin = -1000000
                        cons.ThreatMax = 1000000
                        cons.ThreatRings = 0
                    end
                    if not cons.BaseTemplate and ( cons.NearMarkerType == 'Defensive Point' or cons.NearMarkerType == 'Expansion Area' ) then
                        baseTmpl = baseTmplFile['ExpansionBaseTemplates'][factionIndex]
                    end
                    relative = false
                    local pos = self:GetPlatoonPosition()
                    reference, refName = AIUtils.AIGetClosestThreatMarkerLoc(aiBrain, cons.NearMarkerType, pos[1], pos[3],
                                                                    cons.ThreatMin, cons.ThreatMax, cons.ThreatRings)
                    if cons.ExpansionBase and refName then
                        AIBuildStructures.AINewExpansionBase( aiBrain, refName, reference, (cons.ExpansionRadius or 100), cons.ExpansionTypes )
                    end
                    if reference and aiBrain:GetThreatAtPosition( reference, 1, false ) > -5 then
                        #aiBrain:ExpansionHelp( eng, reference )
                    end
                    table.insert( baseTmplList, AIBuildStructures.AIBuildBaseTemplateFromLocation( baseTmpl, reference ) )
                    buildFunction = AIBuildStructures.AIExecuteBuildStructure
                elseif cons.AdjacencyCategory then
                    relative = false
                    local pos = self:GetPlatoonPosition()
                    local cat = ParseEntityCategory(cons.AdjacencyCategory)
                    local radius = ( cons.AdjacencyDistance or 50 )
                    if not pos or not pos then
                        aiBrain:DisbandPlatoon(self)
                        return
                    end
                    reference  = AIUtils.GetOwnUnitsAroundPoint( aiBrain, cat, pos, radius, cons.ThreatMin,
                                                                cons.ThreatMax, cons.ThreatRings)
                    buildFunction = AIBuildStructures.AIBuildAdjacency
                    table.insert( baseTmplList, baseTmpl )
                else
                    table.insert( baseTmplList, baseTmpl )
                    relative = true
                    buildFunction = AIBuildStructures.AIExecuteBuildStructure
                end
                if cons.BuildClose then
                    closeToBuilder = eng
                end

                #### BUILD BUILDINGS HERE ####
                for baseNum, baseListData in baseTmplList do
                    for k, v in cons.BuildStructures do
                        if not eng:IsDead() then
                            IssueStop({eng})
                            IssueClearCommands({eng})
                            local retVal, pos = buildFunction(aiBrain, eng, v, closeToBuilder, relative, buildingTmpl, baseListData, reference)
                            if not retVal and pos then
                                #LOG('*AI DEBUG: ARMY ' .. aiBrain:GetArmyIndex() .. ' - Engineer moving to capture at ' .. pos[1] .. ', ' .. pos[3] )
                                IssueMove( {eng}, pos )
                            end
                            if retVal or pos then
                                repeat
                                    WaitSeconds(3)
                                    if not aiBrain:PlatoonExists(self) then
                                        return
                                    end
                                until eng:IsDead() or eng:IsIdleState()
                            end
                            if pos then
                                # Check if unit at location
                                local checkUnits = aiBrain:GetUnitsAroundPoint( 'Enemy', categories.STRUCTURE + ( categories.MOBILE * categories.LAND), pos, 10 )
                                #( Rect( pos[1] - 7, pos[3] - 7, pos[1] + 7, pos[3] + 7 ) )
                                if checkUnits then
                                    for num,unit in checkUnits do
                                        IssueCapture( {eng}, unit )
                                    end
                                    repeat
                                        WaitSeconds(3)
                                        if not aiBrain:PlatoonExists(self) then
                                            return
                                        end
                                    until eng:IsDead() or eng:IsIdleState()
                                end
                            end
                        else
                            aiBrain:DisbandPlatoon(self)
                        end
                    end
                end
            end
        end
        local location = AIUtils.RandomLocation(aiBrain:GetArmyStartPos())
        self:MoveToLocation(location, false)
        aiBrain:DisbandPlatoon(self)
    end,


    #UpgradeAnEngineeringPlatoon
    UnitUpgradeAI = function(self)
        local aiBrain = self:GetBrain()
        local platoonUnits = self:GetPlatoonUnits()
        local factionIndex = aiBrain:GetFactionIndex()
        self:Stop()
        for k, v in platoonUnits do
            local upgradeID
            if EntityCategoryContains(categories.MOBILE, v) then
                upgradeID = aiBrain:FindUpgradeBP(v:GetUnitId(), UnitUpgradeTemplates[factionIndex])
            else
                upgradeID = aiBrain:FindUpgradeBP(v:GetUnitId(), StructureUpgradeTemplates[factionIndex])
            end
            if upgradeID then
                IssueUpgrade({v}, upgradeID)
            end
        end
        local upgrading = true
        while aiBrain:PlatoonExists(self) and upgrading do
            WaitSeconds(3)
            upgrading = false
            for k, v in platoonUnits do
                if v and not v:IsDead() then
                    upgrading = true
                end
            end
        end
        if not aiBrain:PlatoonExists(self) then
            return
        end
        aiBrain:DisbandPlatoon(self)
    end,

    ClosestUnitAI = function (self)
        local target = self:FindClosestUnit('Attack', 'Enemy', true, categories.ALLUNITS - categories.WALL)
        local cmd
        local aiBrain = self:GetBrain()
        while aiBrain:PlatoonExists(self) do
            if not target then
                if not cmd or not platoon:IsCommandsActive(cmd) then
                    if not target:IsDead() then
                        self:Stop()
                        self:AggressiveMoveToLocation(target:GetPosition())
                    else
                        target = self:FindClosestUnit('Attack', 'Enemy', true, categories.ALLUNITS - categories.WALL)
                    end
                else
                    if target:IsDead() then
                        if not target then
                            target = self:FindClosestUnit('Attack', 'Enemy', true, categories.ALLUNITS - categories.WALL)
                            if target and Utilities.GetDistanceBetweenTwoVectors( target:GetPosition(), self:GetPlatoonPosition()) < 40 then
                                self:Stop()
                                self:AggressiveMoveToLocation( target:GetPosition() )
                            end
                        end
                    end
                end
            else
                target = self:FindClosestUnit('Attack', 'Enemy', true, categories.ALLUNITS - categories.WALL)
            end
            WaitSeconds(7)
        end
    end,

    HighestThreatAI = function(self)
        local aiBrain = self:GetBrain()
        local cmd = false
        repeat
            if not cmd or not self:IsCommandsActive(cmd) then
                location,threat = self:GetBrain():GetHighestThreatPosition(2, false)
                self:Stop()
                cmd = self:AggressiveMoveToLocation(location)
            end
            WaitSeconds(7)
        until not aiBrain:PlatoonExists(self)
    end,
    
    RepairCDRAI = function(self)
        local aiBrain = self:GetBrain()
        local cdrUnits = aiBrain:GetListOfUnits( categories.COMMAND, false )
        for k,v in cdrUnits do
            if v:GetHealthPercent() < .8 then
                IssueRepair( self:GetPlatoonUnits(), v )
            end
        end
        aiBrain:DisbandPlatoon( self )
    end,

    MassExtractorHunterAI = function(self)
        local threatMin = self.PlatoonData.ThreatMin or 1
        local threatMax = self.PlatoonData.ThreatMax or 25
        local threatRings = self.PlatoonData.ThreatRings or 2
        local aiBrain = self:GetBrain()
        local recentPoints = {}
        local cmd = false
        local atkPri = { 'SPECIALHIGHPRI', 'STRUCTURE ECONOMIC', 'COMMAND', 'MOBILE TECH3', 'MOBILE TECH2', 'MOBILE TECH1', 
                        'STRUCTURE FACTORY', 'STRUCTURE DEFENSE', 'SPECIALLOWPRI', 'ALLUNITS' }
        local atkPriTable = {}
        for k,v in atkPri do
            table.insert( atkPriTable, ParseEntityCategory( v ) )
        end
        self:SetPrioritizedTargetList( 'Attack', atkPriTable )
        for k,v in self:GetPlatoonUnits() do
            v:SetTargetPriorities( atkPriTable )
        end

        while aiBrain:PlatoonExists(self) do
            if table.getn(recentPoints) > 10 or not cmd then
                recentPoints = {}
            end
            cmd = false
            local locs = AIUtils.AIGetSortedMassLocations(aiBrain, 1000, threatMin, threatMax, threatRings, self)
            local unit = false
            for k,v in self:GetPlatoonUnits() do
                if not v:IsDead() then
                    unit = v
                    break
                end
            end
            if not unit then
                return
            end
            local pos = unit:GetPosition()
            local orderSent = false
            self:Stop()
            for k, v in locs do
                if not AIUtils.FindPointInTable(v, recentPoints) and ( VDist2(pos[1], pos[3], v[1], v[3]) > 16 ) then
                    if not AIUtils.CheckUnitPathing( v, pos, unit ) then
                        if orderSent then
                            break
                        end
                    else
                        cmd = self:AggressiveMoveToLocation(v)
                        if v[1] <= 0 or v[3] <= 0 or v[1] >= ScenarioInfo.size[1] or v[3] >= ScenarioInfo.size[2] then
                            #LOG('*AI DEBUG: MASS HUNTER SENDING UNITS TO WRONG SPOT')
                        end
                        pos = v
                        orderSent = true
                        table.insert( recentPoints, v )
                    end
                end
            end
            repeat
                WaitSeconds(3)
                if not aiBrain:PlatoonExists(self) then
                    return
                end
            until not cmd or not self:IsCommandsActive(cmd)
        end
    end,

    StrikeForceAI = function(self)
        local aiBrain = self:GetBrain()
        local armyIndex = aiBrain:GetArmyIndex()
        local data = self.PlatoonData
        local categoryList = {}
        local atkPri = {}
        if data.PrioritizedCategories then
            for k,v in data.PrioritizedCategories do
                table.insert( atkPri, v )
                table.insert( categoryList, ParseEntityCategory( v ) )
            end
        end
        table.insert( atkPri, 'ALLUNITS' )
        table.insert( categoryList, categories.ALLUNITS )
        self:SetPrioritizedTargetList( 'Attack', categoryList )
        local target
        local blip
        local maxRadius = data.SearchRadius or 200
        local movingToScout = false
        while aiBrain:PlatoonExists(self) and not self.BreakOff do
            if not blip or blip:BeenDestroyed() or blip:IsKnownFake() or blip:IsMaybeDead() then
                if aiBrain:GetCurrentEnemy() and aiBrain:GetCurrentEnemy():IsDefeated() then
                    aiBrain:PickEnemyLogic()
                end
                target = AIUtils.AIFindBrainTargetInRange( aiBrain, self, 'Attack', maxRadius, atkPri, aiBrain:GetCurrentEnemy() )
                #target = self:FindPrioritizedUnit('Attack', 'Enemy', true, self:GetPlatoonPosition(), maxRadius)
                if target then
                    blip = target:GetBlip( armyIndex )
                    self:Stop()
                    if not data.UseMoveOrder then
                        self:AttackTarget( target )
                    else
                        self:MoveToLocation(target:GetPosition(), false)
                    end
                    movingToScout = false
                elseif not movingToScout then
                    movingToScout = true
                    for k,v in AIUtils.AIGetSortedScoutingLocations(aiBrain, 10) do
                        if v[1] < 0 or v[3] < 0 or v[1] > ScenarioInfo.size[1] or v[3] > ScenarioInfo.size[2] then
                            #LOG('*AI DEBUG: STRIKE FORCE SENDING UNITS TO WRONG LOCATION - ' .. v[1] .. ', ' .. v[3] )
                        end
                        self:MoveToLocation( (v), false )
                    end
                end
            end
            WaitSeconds( 7 )
        end
    end,

    DummyAI = function(self)
    end,
    
    ArtilleryAI = function(self)
        local aiBrain = self:GetBrain()
        
        local atkPri = { 'SPECIALHIGHPRI', 'STRUCTURE STRATEGIC', 'STRUCTURE DEFENSE', 'COMMAND', 'EXPERIMENTAL LAND', 
            'MOBILE TECH3 LAND', 'MOBILE TECH2 LAND', 'MOBILE TECH1 LAND', 'STRUCTURE FACTORY', 'SPECIALLOWPRI', 'ALLUNITS' }
        local atkPriTable = {}
        for k,v in atkPri do
            table.insert( atkPriTable, ParseEntityCategory( v ) )
        end
        self:SetPrioritizedTargetList( 'Attack', atkPriTable )
        
        local unit = false
        for k,v in self:GetPlatoonUnits() do
            if not v:IsDead() then
                unit = v
                break
            end
        end
        if not unit then
            return
        end
        unit:SetTargetPriorities( atkPriTable )
        local bp = unit:GetBlueprint()
        local maxRange = false
        for k,v in bp.Weapon do
            if not maxRange or v.MaxRadius > maxRange then
                maxRange = v.MaxRadius
            end
        end
        while aiBrain:PlatoonExists(self) do
            local enemy = aiBrain:GetCurrentEnemy()
            if aiBrain:GetCurrentEnemy() and aiBrain:GetCurrentEnemy():IsDefeated() then
                aiBrain:PickEnemyLogic()
            end
            if aiBrain.PlatoonNameCounter['AttackForce'] > 0 then
                # Attack enemy base to help attacking platoon
                local target = AIAttackUtils.AIFindStrongestInRoute( aiBrain, maxRange, atkPri, aiBrain.LastAttackForceRoute, aiBrain:GetCurrentEnemy() )
                self:Stop()
                if target and not unit:IsDead() then
                    IssueClearCommands( {unit} )
                    IssueAttack( {unit}, target:GetPosition() )
                end
            else
                self:Stop()
                local target = AIUtils.AIFindBrainTargetInRange( aiBrain, self, 'Attack', maxRange, atkPri, aiBrain:GetCurrentEnemy() )
                if target and not unit:IsDead() then
                    IssueClearCommands( {unit} )
                    IssueAttack( {unit}, target:GetPosition() )
                end
            end
            WaitSeconds(20)
        end
    end,

    AttackForceAI = function(self)
        self:Stop()
        local aiBrain = self:GetBrain()
        aiBrain.PlatoonNameCounter['AttackForce'] = aiBrain.PlatoonNameCounter['AttackForce'] + 1
        self:AddDestroyCallback( function(aiBrain, self)
                                     aiBrain.PlatoonNameCounter['AttackForce'] = aiBrain.PlatoonNameCounter['AttackForce'] - 1
                                 end )
        local pos = self:GetPlatoonPosition()
        
        # move units closer to each other
        local unitsSet = true
        for k,v in self:GetPlatoonUnits() do
            if VDist2( v:GetPosition()[1], v:GetPosition()[3], pos[1], pos[3] ) > 40 then
               unitsSet = false
               break
            end
        end
        if not unitsSet then
            AIUtils.AIGetClosestMarkerLocation(aiBrain, 'Defensive Point', pos[1], pos[3])
            local cmd = self:MoveToLocation( self:GetPlatoonPosition(), false )
            local counter = 0
            repeat
                WaitSeconds(1)
                counter = counter + 1
                if not aiBrain:PlatoonExists(self) then
                    return
                end
            until not self:IsCommandsActive(cmd) or counter >= 30
        end

        local artilleryGroup = {}
        local guardGroup = {}
        # use scout group as attack group 2
        local scoutGroup = {}
        local attackGroup = {}

        local flip = false
        for k,v in self:GetSquadUnits('Attack') do
            if not v:BeenDestroyed() then
                if EntityCategoryContains( categories.ANTIAIR, v ) then
                    table.insert( guardGroup, v )
                elseif EntityCategoryContains( categories.INDIRECTFIRE, v ) then
                    table.insert( artilleryGroup, v )
                end
            end
        end

        #LOG('*AI DEBUG: ARMY ' .. aiBrain:GetArmyIndex() .. ' - ATTACK FORCE SENT')
        aiBrain:AssignUnitsToPlatoon( self, artilleryGroup, 'Artillery', 'GrowthFormation' )
        aiBrain:AssignUnitsToPlatoon( self, guardGroup, 'Guard', 'GrowthFormation' )
#        LOG( '*AI DEBUG: Num Attack Units = ' .. table.getn( self:GetSquadUnits('Attack') ) )
#        LOG( '*AI DEBUG: Num Artilery Units = ' .. table.getn( self:GetSquadUnits('Artillery') ) )
#        LOG( '*AI DEBUG: Num Guard Units = ' .. table.getn( self:GetSquadUnits('Guard') ) )


#        if table.getn( scoutGroup ) >= 8 then
#            local plat = aiBrain:MakePlatoon( '', 'AttackForceSplitAI' )
#            aiBrain:AssignUnitsToPlatoon( plat, scoutGroup, 'Attack', 'GrowthFormation' )
#        end

        local guardCmd = false
        local artilleryCmd = false
        local attackCmd = false
        local groupCmd = false
        
        WaitTicks(1)
        self:Stop()
        
        local t1AtkPri = { 'SHIELD STRUCTURE', 'MASSEXTRACTION', 'MASSPRODUCTION', 'FACTORY TECH3', 'FACTORY TECH2', 'ENERGYPRODUCTION TECH3',
                            'ENERGYPRODUCTION TECH2', 'ENERGYPRODUCTION TECH1', 'FACTORY TECH1', 'COMMAND', 'STRUCTURE', 'ALLUNITS' }
        local t2AtkPri = { 'SHIELD STRUCTURE', 'MASSEXTRACTION', 'MASSPRODUCTION', 'FACTORY TECH3', 'FACTORY TECH2', 'COMMAND', 
                            'ENERGYPRODUCTION TECH3', 'ENERGYPRODUCTION TECH2', 'ENERGYPRODUCTION TECH1', 'FACTORY TECH1', 'STRUCTURE', 'ALLUNITS' }
        local t3AtkPri = { 'COMMAND', 'SHIELD STRUCTURE', 'MASSEXTRACTION', 'MASSPRODUCTION', 'FACTORY TECH3', 'FACTORY TECH2', 
                           'ENERGYPRODUCTION TECH3', 'ENERGYPRODUCTION TECH2', 'ENERGYPRODUCTION TECH1', 'FACTORY TECH1', 'STRUCTURE', 'ALLUNITS' }
        ### Set the pri
        AIAttackUtils.SetGroupAttackPri( self, 'Attack', t1AtkPri, t2AtkPri, t3AtkPri )

        local t1ArtPri = { 'SHIELD STRUCTURE', 'DEFENSE', 'MASSEXTRACTION', 'MASSPRODUCTION', 'FACTORY TECH3', 'FACTORY TECH2', 'ENERGYPRODUCTION TECH3',
                            'ENERGYPRODUCTION TECH2', 'ENERGYPRODUCTION TECH1', 'FACTORY TECH1', 'COMMAND', 'STRUCTURE', 'ALLUNITS' }
        local t2ArtPri = { 'SHIELD STRUCTURE', 'DEFENSE', 'MASSEXTRACTION', 'MASSPRODUCTION', 'FACTORY TECH3', 'FACTORY TECH2', 'COMMAND', 
                            'ENERGYPRODUCTION TECH3', 'ENERGYPRODUCTION TECH2', 'ENERGYPRODUCTION TECH1', 'FACTORY TECH1', 'STRUCTURE', 'ALLUNITS' }
        local t3ArtPri = { 'COMMAND', 'SHIELD STRUCTURE', 'DEFENSE', 'MASSEXTRACTION', 'MASSPRODUCTION', 'FACTORY TECH3', 'FACTORY TECH2', 
                           'ENERGYPRODUCTION TECH3', 'ENERGYPRODUCTION TECH2', 'ENERGYPRODUCTION TECH1', 'FACTORY TECH1', 'STRUCTURE', 'ALLUNITS' }
        ### Set the pri
        AIAttackUtils.SetGroupAttackPri( self, 'Artillery', t1ArtPri, t2ArtPri, t3ArtPri )


        local grdPri = { 'SPECIALHIGHPRI', 'COMMAND', 'MOBILE TECH3', 'MOBILE TECH2', 'MOBILE TECH1', 'STRUCTURE', 'SPECIALLOWPRI', 
                        'ALLUNITS' }
        local grdPriTable = {}
        for k,v in grdPri do
            table.insert( grdPriTable, ParseEntityCategory( v ) )
        end
        for k,v in self:GetSquadUnits( 'Guard' ) do
            v:SetTargetPriorities( grdPriTable )
        end
        
        local artillery = false
        local artRange = 0
        local artUnits = self:GetSquadUnits('Artillery')
        if artUnits then
            for k,v in artUnits do
                artillery = true
                for n,data in v:GetBlueprint().Weapon do
                    if data.MaxRadius > artRange then
                        artRange = data.MaxRadius
                    end
                end
            end
        end
        local gates = AIUtils.FindIdleGates(aiBrain)
        while aiBrain:PlatoonExists(self) do
            local squadCheck = AIAttackUtils.CheckSquads( self )
            if aiBrain:GetCurrentEnemy() and aiBrain:GetCurrentEnemy():IsDefeated() then
                aiBrain:PickEnemyLogic()
            end
            if not groupCmd then
                if squadCheck.Guard or squadCheck.Attack or squadCheck.Artillery then
                    local groupUnits = {}
                    for k,v in self:GetSquadUnits( 'Attack' ) do
                        table.insert( groupUnits, v )
                    end
                    for k,v in self:GetSquadUnits( 'Artillery' ) do
                        table.insert( groupUnits, v )
                    end
                    for k,v in self:GetSquadUnits( 'Guard' ) do
                        table.insert( groupUnits, v )
                    end
                    IssueStop( groupUnits )
                    IssueClearCommands( groupUnits )
                    groupCmd = AIAttackUtils.AIPlatoonSquadAttackVector( aiBrain, self, 'All', 'HighestVector', 'GrowthFormation', gates )
                end
            elseif self:IsCommandsActive( groupCmd ) then
                if artillery and squadCheck.Artillery then
                    local target = self:FindPrioritizedUnit( 'Artillery', 'Enemy', true, self:GetSquadPosition('Artillery'), artRange )
                    if target and not target:IsDead() then # and EntityCategoryContains( categories.STRUCTURE - categories.MASSEXTRACTION, target ) then
                        #LOG( '*AI DEBUG: ARMY ' .. aiBrain:GetArmyIndex() .. ' ATTACK FORCE DEBUG - Attack force break off')
                        self:Stop()
                        # Unit found break off.
                        if squadCheck.Guard then
                            if not guardCmd or not self:IsCommandsActive( guardCmd ) then
                                self:Stop( 'Guard' )
                                if squadCheck.Artillery then
                                    guardCmd = AIAttackUtils.AISquadFollowSquad( aiBrain, self, 'Guard', 'Artillery', 'GrowthFormation' )
                                elseif squadCheck.Attack then
                                    guardCmd = AIAttackUtils.AISquadFollowSquad( aiBrain, self, 'Guard', 'Attack', 'GrowthFormation' )
                                else
                                    guardCmd = AIAttackUtils.AIPlatoonSquadAttackVector( aiBrain, self, 'Guard', 'HighestVector', 'GrowthFormation' )
                                end
                            end
                        end
                        if squadCheck.Artillery then
                            if not artilleryCmd or not self:IsCommandsActive( artilleryCmd ) then
                                self:Stop('Artillery')
                                artilleryCmd = AIAttackUtils.AIPlatoonSquadAttackVector( aiBrain, self, 'Artillery', 'HighestVector', 'GrowthFormation' )
                            end
                        end
                        if squadCheck.Attack then
                            if not attackCmd or not self:IsCommandsActive( attackCmd ) then
                                self:Stop( 'Attack' )
                                attackCmd = AIAttackUtils.AIPlatoonSquadAttackVector( aiBrain, self, 'Attack', 'HighestVector', 'GrowthFormation' )
                            end
                        end
                    end
                end
            else
                if squadCheck.Guard then
                    if not guardCmd or not self:IsCommandsActive( guardCmd ) then
                        self:Stop( 'Guard' )
                        if squadCheck.Artillery then
                            guardCmd = AIAttackUtils.AISquadFollowSquad( aiBrain, self, 'Guard', 'Artillery', 'GrowthFormation' )
                        elseif squadCheck.Attack then
                            guardCmd = AIAttackUtils.AISquadFollowSquad( aiBrain, self, 'Guard', 'Attack', 'GrowthFormation' )
                        else
                            guardCmd = AIAttackUtils.AIPlatoonSquadAttackVector( aiBrain, self, 'Guard', 'HighestVector', 'GrowthFormation' )
                        end
                    end
                end
                if squadCheck.Artillery then
                    if not artilleryCmd or not self:IsCommandsActive( artilleryCmd ) then
                        self:Stop('Artillery')
                        artilleryCmd = AIAttackUtils.AIPlatoonSquadAttackVector( aiBrain, self, 'Artillery', 'HighestVector', 'GrowthFormation' )
                    end
                end
                if squadCheck.Attack then
                    if not attackCmd or not self:IsCommandsActive( attackCmd ) then
                        self:Stop( 'Attack' )
                        attackCmd = AIAttackUtils.AIPlatoonSquadAttackVector( aiBrain, self, 'Attack', 'HighestVector', 'GrowthFormation' )
                    end
                end
            end
            WaitSeconds(3)
        end
    end,
    
    
    NameUnits = function(self)
        local units = self:GetPlatoonUnits()
        if units and table.getn(units) > 0 then
            for k, v in units do
                local bp = v:GetBlueprint().Display
                if bp.AINames then
                    local num = Random(1, table.getn(bp.AINames))
                    v:SetCustomName(bp.AINames[num])
                end
            end
        end
    end,
}
