#****************************************************************************
#**
#**  File     :  /data/lua/modules/scenariotriggers.lua
#**
#**  Author(s):  John Comes
#**
#**  Summary  :  Generalized trigger functions for scenarios.
#**
#**  Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
#****************************************************************************
local ScenarioUtils = import('/lua/ScenarioUtilities.lua')

function CreateAreaTrigger(callbackFunction, rectangle, category, onceOnly, invert, aiBrain, number, requireBuilt)
    #LOG('*SCENARIO DEBUG: FORKING AREA TRIGGER THREAD')
    return ForkThread(AreaTriggerThread, callbackFunction, {rectangle}, category, onceOnly, invert, aiBrain, number, requireBuilt)
end

function CreateMultipleAreaTrigger(callbackFunction, rectangleTable, category, onceOnly, invert, aiBrain, number, requireBuilt)
    return ForkThread(AreaTriggerThread, callbackFunction, rectangleTable, category, onceOnly, invert, aiBrain, number, requireBuilt)
end

function AreaTriggerThread(callbackFunction, rectangleTable, category, onceOnly, invert, aiBrain, number, requireBuilt, name)
    local recTable = {}
    for k,v in rectangleTable do
        if type(v) == 'string' then
            table.insert(recTable,ScenarioUtils.AreaToRect(v))
        else
            table.insert(recTable, v)
        end
    end
    while true do
        local amount = 0
        local totalEntities = {}
        for k, v in recTable do
            local entities = GetUnitsInRect( v )
            if entities then
                for ke, ve in entities do
                    totalEntities[table.getn(totalEntities) + 1] = ve
                end
            end
        end
        local triggered = false
        local triggeringEntity
        local numEntities = table.getn(totalEntities)
        if numEntities > 0 then
            for k, v in totalEntities do
                local contains = EntityCategoryContains(category, v)
                if contains and (aiBrain and v:GetAIBrain() == aiBrain) and (not requireBuilt or (requireBuilt and not v:IsBeingBuilt())) then
                    amount = amount + 1
                    #If we want to trigger as soon as one of a type is in there, kick out immediately.
                    if not number then
                        triggeringEntity = v
                        triggered = true
                        break
                    #If we want to trigger on an amount, then add the entity into the triggeringEntity table
                    #so we can pass that table back to the callback function.
                    else
                        if not triggeringEntity then
                            triggeringEntity = {}
                        end
                        table.insert(triggeringEntity, v)
                    end
                end
            end
        end
        #Check to see if we have a triggering amount inside in the area.
        if number and ((amount >= number and not invert) or (amount < number and invert)) then
            triggered = true
        end
        #TRIGGER IF:
        #You don't want a specific amount and the correct unit category entered
        #You don't want a specific amount, there are no longer the category inside and you wanted the test inverted
        #You want a specific amount and we have enough.
        if ( triggered and not invert and not number) or (not triggered and invert and not number) or (triggered and number) then
            if name then
                callbackFunction(TriggerManager, name, triggeringEntity)
            else
                callbackFunction(triggeringEntity)
            end
            if onceOnly then
                return
            end
        end
        WaitTicks(1)
    end
end



function CreateThreatTriggerAroundUnit(callbackFunction, aiBrain, unit, rings, onceOnly, value, greater)
    return ForkThread(ThreatTriggerAroundUnitThread, callbackFunction, aiBrain, unit, rings, onceOnly, value, greater)
end

function ThreatTriggerAroundUnitThread(callbackFunction, aiBrain, unit, rings, onceOnly, value, greater, name)
    while not unit:IsDead() do
        local threat = aiBrain:GetThreatAtPosition(unit:GetPosition(), rings, false)
        if greater and threat >= value then
            if name then
                callbackFunction(TriggerManager, name)
            else
                callbackFunction()
            end
            if onceOnly then
                return
            end
        elseif not greater and threat <= value then
            if name then
                callbackFunction(TriggerManager, name)
            else
                callbackFunction()
            end
            if onceOnly then
                return
            end
        end
        WaitSeconds(0.5)
    end
end


function CreateThreatTriggerAroundPosition(callbackFunction, aiBrain, posVector, rings, onceOnly, value, greater)
    return ForkThread(ThreatTriggerAroundPositionThread, callbackFunction, aiBrain, posVector, rings, onceOnly, value, greater)
end

function ThreatTriggerAroundPositionThread(callbackFunction, aiBrain, posVector, rings, onceOnly, value, greater, name)
    if type(posVector) == 'string' then
        posVector = ScenarioUtils.MarkerToPosition(posVector)
    end
    while true do
        local threat = aiBrain:GetThreatAtPosition(posVector, rings, false)
        if greater and threat >= value then
            if name then
                callbackFunction(TriggerManager, name)
            else
                callbackFunction()
            end
            if onceOnly then
                return
            end
        elseif not greater and threat <= value then
            if name then
                callbackFunction(TriggerManager, name)
            else
                callbackFunction()
            end
            if onceOnly then
                return
            end
        end
        WaitSeconds(0.5)
    end
end



function CreateTimerTrigger(callbackFunction, seconds, displayBool)
    return ForkThread(TimerTriggerThread, callbackFunction, seconds, nil, displayBool)
end

function TimerTriggerThread(callbackFunction, seconds, name, displayBool)
    if displayBool then
        local currentSeconds = seconds
        while currentSeconds > 0 do
            if not Sync.ObjectiveTimer then
                Sync.ObjectiveTimer = currentSeconds
            end
            currentSeconds = currentSeconds - 1
            WaitSeconds(1)
        end
        Sync.ObjectiveTimer = 0
    else
        WaitSeconds(seconds)
    end
    if name then
        callbackFunction(TriggerManager, name)
    else
        callbackFunction()
    end
end



function CreateGroupDeathTrigger(callbackFunction, group)
    return ForkThread(GroupDeathTriggerThread, callbackFunction, group)
end

function GroupDeathTriggerThread(callbackFunction, group, name)
    local allDead = false
    while not allDead do
        allDead = true
        for k, v in group do
            if not v:IsDead() then
                allDead = false
                break
            end
        end
        if allDead then
            if name then
                callbackFunction(TriggerManager, name)
            else
                callbackFunction()
            end
        end
        WaitSeconds(0.5)
    end
end

function CreateSubGroupDeathTrigger(callbackFunction, group, num)
    return ForkThread(SubGroupDeathTriggerThread, callbackFunction, group, num)
end

function SubGroupDeathTriggerThread(callbackFunction, group, num, name)
    local numDead = 0
    while(numDead < num) do
        numDead = 0
        for k, v in group do
            if(v:IsDead()) then
                numDead = numDead + 1
            end
            if(numDead == num) then
                if name then
                    callbackFunction(TriggerManager, name)
                else
                    callbackFunction()
                end
                break
            end
        end
        WaitSeconds(0.5)
    end
end

function CreateUnitBuiltTrigger(callbackFunction, unit, category)
    unit:AddOnUnitBuiltCallback(callbackFunction, category)
end

function CreateUnitDeathTrigger(callbackFunction, unit)
    unit:AddOnKilledCallback(callbackFunction)
end

function CreateUnitDestroyedTrigger(callbackFunction, unit)
    unit:AddOnReclaimedCallback(callbackFunction)
    unit:AddOnCapturedCallback(callbackFunction)
    unit:AddOnKilledCallback(callbackFunction)
end

function CreateUnitPercentageBuiltTrigger(callbackFunction, aiBrain, category, percent)
    aiBrain:AddUnitBuiltPercentageCallback(callbackFunction, category, percent)
end

function CreateUnitLoadTrigger(callbackFunction, unit)
    unit:AddOnLoadCallback(callbackFunction)
end

function CreateUnitUnloadTrigger(callbackFunction, unit)
    unit:AddOnUnloadCallback(callbackFunction)
end

function CreateUnitVeterancyTrigger(callbackFunction, unit)
    unit:AddOnVeteranCallback(callbackFunction)
end

function RemoveUnitTrigger(unit, callbackFunction)
    unit:RemoveCallback(callbackFunction)
end

function CreateArmyStatTrigger(callbackFunction, aiBrain, name, triggerTable)
    local spec = {
        Name = name,
        CallbackFunction = callbackFunction,
    }
    for num, trigger in aiBrain.TriggerList do
        if name == trigger.Name then
            error('*TRIGGER ERROR: Must use unique names for new triggers- Supplied name: '..trigger.Name, 2)
            return
        end
    end
    for num, triggerData in triggerTable do
        if string.find (triggerData.StatType, "Economy_") then
            aiBrain:GetArmyStat(triggerData.StatType, 0.0)
        else
            aiBrain:GetArmyStat(triggerData.StatType, 0)
        end
        if triggerData.Category then
            aiBrain:SetArmyStatsTrigger(triggerData.StatType, name, triggerData.CompareType, triggerData.Value, triggerData.Category)
        else
            aiBrain:SetArmyStatsTrigger(triggerData.StatType, name, triggerData.CompareType, triggerData.Value)
        end
    end
    table.insert(aiBrain.TriggerList, spec)
end

function CreateArmyIntelTrigger(callbackFunction, aiBrain, reconType, blip, value, category, onceOnly, targetAIBrain)
    local spec = {
        CallbackFunction = callbackFunction,
        Type = reconType,
        Category = category,
        Blip = blip,
        Value = value,
        OnceOnly = onceOnly,
        TargetAIBrain = targetAIBrain,
    }
    aiBrain:SetupArmyIntelTrigger(spec)
end

function CreateArmyUnitCategoryVeterancyTrigger(callbackFunction, aiBrain, category, level)
    local spec = {
        CallbackFunction = callbackFunction,
        Category = category,
        Level = level,
    }
    aiBrain:SetupBrainVeterancyTrigger(spec)
end

function CreateUnitDamagedTrigger( callbackFunction, unit, amount, repeatNum )
    unit:AddOnDamagedCallback(callbackFunction, amount, repeatNum)
end

function CreateUnitDistanceTrigger( callbackFunction, unitOne, unitTwo, distance )
    ForkThread( UnitDistanceTriggerThread, callbackFunction, unitOne, unitTwo, distance )
end

function UnitDistanceTriggerThread( callbackFunction, unitOne, unitTwo, distance )
    while not ( VDist3( unitOne:GetPosition(), unitTwo:GetPosition() ) < distance  ) do
        WaitSeconds(0.5)
    end
    callbackFunction()
end

function CreateUnitToPositionDistanceTrigger( callbackFunction, unit, marker, distance )
    ForkThread(UnitToPositionDistanceTriggerThread, callbackFunction, unit, marker, distance)
end

function UnitToPositionDistanceTriggerThread( cb, unit, marker, distance, name )
    if type(marker) == 'string' then
        marker = ScenarioUtils.MarkerToPosition(marker)
    end
    local fired = false
    while not fired do
        if unit:IsDead() then
            return false
        else
            local position = unit:GetPosition()
            local value = VDist2( position[1], position[3], marker[1], marker[3])
            if value <= distance then
                fired = true
                if name then
                    cb(TriggerManager, name, unit)
                    return
                else
                    cb(unit)
                    return
                end
            end
        end
        WaitSeconds(.5)
    end
end

function CreateUnitNearTypeTrigger( callbackFunction, unit, brain, category, distance )
    return ForkThread( CreateUnitNearTypeTriggerThread, callbackFunction, unit, brain, category, distance )
end

function CreateUnitNearTypeTriggerThread( callbackFunction, unit, brain, category, distance, name )
    local fired = false
    while not fired do
        if unit:IsDead() then
            return false
        else
            local position = unit:GetPosition()
            for k,catUnit in brain:GetListOfUnits(category, false) do
                if (VDist3( position, catUnit:GetPosition() ) < distance) and not catUnit:IsBeingBuilt() then
                    fired = true
                    if name then
                        callbackFunction(TriggerManager, name, unit, catUnit)
                    else
                        callbackFunction(unit, catUnit)
                    end
                end
            end
        end
        WaitSeconds(.5)
    end
end

function CreateUnitReclaimedTrigger( cb, unit )
    unit:AddOnReclaimedCallback(cb)
end

function CreateUnitStartReclaimTrigger( cb, unit )
    unit:AddOnStartReclaimCallback(cb)
end

function CreateUnitStopReclaimTrigger( cb, unit )
    unit:AddOnStopReclaimCallback(cb)
end

function CreateUnitCapturedTrigger( cbOldUnit, cbNewUnit, unit )
    unit:AddOnCapturedCallback(cbOldUnit, cbNewUnit)
end

function CreateUnitStartCaptureTrigger( cb, unit )
    unit:AddOnStartCaptureCallback(cb)
end

function CreateUnitStopCaptureTrigger( cb, unit )
    unit:AddOnStopCaptureCallback(cb)
end

function CreateUnitStartBeingCapturedTrigger( cb, unit )
    unit:AddOnStartBeingCapturedCallback(cb)
end

function CreateUnitStopBeingCapturedTrigger( cb, unit )
    unit:AddOnStopBeingCapturedCallback(cb)
end

function CreateUnitFailedBeingCapturedTrigger( cb, unit )
    unit:AddOnFailedBeingCapturedCallback(cb)
end

function CreateUnitFailedCaptureTrigger( cb, unit )
    unit:AddOnFailedCaptureCallback(cb)
end

function VariableBoolCheckThread(cb, varName, value, name)
    if value then
        while not ScenarioInfo.VarTable[varName] do
            WaitSeconds(.5)
        end
        cb(TriggerManager, name)
    else
        while ScenarioInfo.VarTable[varName] do
            WaitSeconds(.5)
        end
        cb(TriggerManager, name)
    end
end

function MissionNumberTriggerThread(cb, value, name)
    if value then
        while not ScenarioInfo.VarTable['Mission Number'] == value do
            WaitSeconds(.5)
        end
        cb(TriggerManager, name)
    end
end
