--*****************************************************************************
--* File: lua/modules/ui/dialogs/console.lua
--* Author: Chris Blackwell
--* Summary: command console
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Group = import('/lua/modules/maui/group.lua').Group
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local ItemList = import('/lua/modules/maui/itemlist.lua').ItemList
local Scrollbar = import('/lua/modules/maui/scrollbar.lua').Scrollbar
local Border = import('/lua/modules/maui/border.lua').Border
local Edit = import('/lua/modules/maui/edit.lua').Edit
local Control = import('/lua/modules/maui/control.lua').Control

local commandDeque = {}
local maxCommandDequeSize = 10
local currentCommandDequeIndex = 0
local currentText = ""
local consoleFontName = UIUtil.fixedFont
local consoleFontSize = 12

local function InsertCommand(text)
    table.insert(commandDeque, text)
    if table.getn(commandDeque) > maxCommandDequeSize then
        table.remove(commandDeque, 1)
    end
    currentCommandDequeIndex = table.getn(commandDeque)
end

function CreateDialog()
    local mainFrame = GetFrame(0)

    local parent = Group(mainFrame, "consoleParent")
    LayoutHelpers.PercentIn(parent, mainFrame, .33, .05, .95, .95)
    parent.Depth:Set(UIUtil.consoleDepth)
    parent:SetName("console")

    local background = Bitmap(parent)
    background:SetSolidColor(UIUtil.consoleBGColor())
    LayoutHelpers.FillParent(background, parent)

    local border = Border(background)
    LayoutHelpers.FillParent(border, background)
    border:SetTextures( UIUtil.UIFile('/widgets/gen_brd_vert.dds'),
                        UIUtil.UIFile('/widgets/gen_brd_horz.dds'),
                        UIUtil.UIFile('/widgets/gen_brd_ul.dds'),
                        UIUtil.UIFile('/widgets/gen_brd_ur.dds'),
                        UIUtil.UIFile('/widgets/gen_brd_ll.dds'),
                        UIUtil.UIFile('/widgets/gen_brd_lr.dds'))

    local edit = Edit(border)
    LayoutHelpers.FromBottomIn(edit, parent, .02)
    LayoutHelpers.FromLeftIn(edit, parent, .05)
    LayoutHelpers.FromRightIn(edit, parent, .05)
    edit:SetForegroundColor(UIUtil.consoleFGColor())
    edit:SetBackgroundColor(UIUtil.consoleTextBGColor())
    edit:SetHighlightForegroundColor("black")
    edit:SetHighlightBackgroundColor(UIUtil.consoleTextBGColor())
    edit:SetFont(consoleFontName, consoleFontSize)
    edit.Height:Set(function() return math.floor(edit:GetFontHeight()) end)

    local consoleOutput = ItemList(border)
    LayoutHelpers.Above(consoleOutput, edit, 10)
    LayoutHelpers.FromTopIn(consoleOutput, parent, .02)
    consoleOutput.Right:Set(edit.Right)
    consoleOutput:SetColors(UIUtil.consoleFGColor(), UIUtil.consoleTextBGColor(), UIUtil.consoleFGColor(), UIUtil.consoleTextBGColor()) -- we don't really want selection here so don't differentiate colors
    consoleOutput:SetFont(consoleFontName, consoleFontSize)

    UIUtil.CreateVertScrollbarFor(consoleOutput)


    -- set up output control to recieve console spew
    local outputHandler = AddConsoleOutputReciever(function(text)
        consoleOutput:AddItem(text)
        local itemCount = consoleOutput:GetItemCount()
        if itemCount > 300 then  -- 300 seems like a good buffer size
            consoleOutput:DeleteItem(0)
        end
        consoleOutput:SetSelection(itemCount - 1)
        consoleOutput:ScrollToBottom()
    end)

    -- if this gets destroyed we need to unregister and destroy the output handler
    parent.OnDestroy = function(self)
        RemoveConsoleOutputReciever(outputHandler)
    end

    -- handle edit control
    local conFuncsList = false
    local raiseConFuncList = true
    edit.OnTextChanged = function(self, newText, oldText)
        if raiseConFuncList then
            if conFuncsList then
                conFuncsList:Destroy()
                conFuncsList = false
            end
            local matches = ConTextMatches(newText)
            local numMatches = table.getn(matches)
            if (numMatches > 0) then
                conFuncsList = ItemList(consoleOutput)
                conFuncsList:SetFont(consoleFontName, consoleFontSize)
                LayoutHelpers.Above(conFuncsList, edit)
                conFuncsList.Right:Set(function() return edit.Right() - 32 end)
                conFuncsList.Height:Set(function()
                    return math.min(consoleOutput.Height(), conFuncsList:GetRowHeight() * numMatches)
                end)
                for i,v in matches do
                    conFuncsList:AddItem(v)
                end
                UIUtil.CreateVertScrollbarFor(conFuncsList)
                conFuncsList:SetSelection(conFuncsList:GetItemCount() - 1)
                conFuncsList:ScrollToBottom()

                conFuncsList.OnDoubleClick = function(self, row)
                    edit:SetText(conFuncsList:GetItem(row))
                    edit:AcquireFocus()
                end
            end
         end
    end

    edit.OnEnterPressed = function(self, text)
        ConExecuteSave(text)
        InsertCommand(text)
    end

    edit.OnNonTextKeyPressed = function(self, keycode)
        local function ChangeConFuncSelection(direction)
            local selection = math.max(math.min(conFuncsList:GetSelection() + direction, conFuncsList:GetItemCount() - 1), 0)
            conFuncsList:SetSelection(selection)
            raiseConFuncList = false
            edit:SetText(conFuncsList:GetItem(selection))
            raiseConFuncList = true
        end

        if keycode == UIUtil.VK_UP then
            if conFuncsList then
                ChangeConFuncSelection(-1)
            else
                -- store off the current text if this is the first press
                local commandDequeSize = table.getn(commandDeque)
                if commandDequeSize > 0 then
                    if currentCommandDequeIndex == commandDequeSize then currentText = edit:GetText() end
                    edit:SetText(commandDeque[currentCommandDequeIndex])
                    currentCommandDequeIndex = currentCommandDequeIndex - 1
                    if currentCommandDequeIndex <= 0 then currentCommandDequeIndex = 1 end
                end
            end
        elseif keycode == UIUtil.VK_DOWN then
            if conFuncsList then
                ChangeConFuncSelection(1)
            else
                currentCommandDequeIndex = currentCommandDequeIndex + 1
                if currentCommandDequeIndex > table.getn(commandDeque) then
                    currentCommandDequeIndex = table.getn(commandDeque)
                    edit:SetText(currentText)
                else
                    edit:SetText(commandDeque[currentCommandDequeIndex])
                end
            end
        end
    end

    edit.OnCharPressed = function(self, charcode)
        -- close console on tilde key (apostrophe, or ascii 96)
        if charcode == 96 then
            edit:AbandonKeyboardFocus()
            parent:Hide()
            return true
        elseif charcode == UIUtil.VK_TAB then
            if conFuncsList then
                edit:SetText(conFuncsList:GetItem(conFuncsList:GetSelection()))
                return true
            else
                return false
            end
        else
            return false
        end
    end

    -- override Show behavior to acquire keyboard focus
    parent.Show = function(self)
        edit:AcquireFocus()
        Control.Show(self)
    end

    return parent
end