--*****************************************************************************
--* File: lua/modules/ui/dialogs/mapselect.lua
--* Author: Chris Blackwell
--* Summary: Dialog to facilitate map selection
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local ItemList = import('/lua/modules/maui/itemlist.lua').ItemList
local Scrollbar = import('/lua/modules/maui/scrollbar.lua').Scrollbar
local Text = import('/lua/modules/maui/text.lua').Text
local MultiLineText = import('/lua/modules/maui/multilinetext.lua').MultiLineText
local Button = import('/lua/modules/maui/button.lua').Button
local Group = import('/lua/modules/maui/group.lua').Group
local MenuCommon = import('/lua/modules/ui/menus/menucommon.lua')
local MapPreview = import('/lua/modules/ui/controls/mappreview.lua').MapPreview
local MainMenu = import('/lua/modules/ui/menus/main.lua')
local MapUtil = import('/lua/modules/ui/maputil.lua')
local Combo = import('/lua/modules/ui/controls/combo.lua').Combo

local scenarios = MapUtil.EnumerateScenarios()
local selectedScenario = false
local description = false
local descText = false
local posGroup = false
local mapList = false
local filters = {}
local filterTitle = false
local mapsize = false
local mapplayers = false
local mapInfo = false
local selectButton = false

local currentFilters = { 
    ['sup_players'] = 0, 
    ['map_size'] = 0, 
}

local scenarioKeymap = {}

local mapFilters = {
    {
        FilterName = "<LOC MAPSEL_0009>Supported Players",
        FilterKey = 'sup_players',
        Options = {
            {text = "<LOC MAPSEL_0010>All", key = 0},
            {text = "<LOC MAPSEL_0011>2", key = 2},
            {text = "<LOC MAPSEL_0012>3", key = 3},
            {text = "<LOC MAPSEL_0013>4", key = 4},
            {text = "<LOC MAPSEL_0014>5", key = 5},
            {text = "<LOC MAPSEL_0015>6", key = 6},
            {text = "<LOC MAPSEL_0016>7", key = 7},
            {text = "<LOC MAPSEL_0017>8", key = 8},
        }
    },
    {
        FilterName = "<LOC MAPSEL_0024>Map Size",
        FilterKey = 'map_size',
        Options = {
            {text = "<LOC MAPSEL_0025>All", key = 0},
            {text = "<LOC MAPSEL_0026>5km", key = 256},
            {text = "<LOC MAPSEL_0027>10km", key = 512},
            {text = "<LOC MAPSEL_0028>20km", key = 1024},
            {text = "<LOC MAPSEL_0029>40km", key = 2048},
            {text = "<LOC MAPSEL_0030>81km", key = 4096},
        }
    },
}

-- Create a filter dropdown and title from the table above
function CreateFilter(parent, filterData)
    local group = Group(parent)
    group.Depth:Set(function() return parent.Depth() + 10 end)
    group.Width:Set(300)
    
    local tempname = filterData.FilterName
    group.title = UIUtil.CreateText(group, tempname, 16, UIUtil.bodyFont)
    LayoutHelpers.AtLeftTopIn(group.title, group)
    
    group.combo = Combo(group, 14, 10)
    LayoutHelpers.AtVerticalCenterIn(group.combo, group.title)
    group.combo.Right:Set(group.Right)
    group.combo.Width:Set(100)
    
    local itemArray = {}
    group.combo.keyMap = {}
    for index, val in filterData.Options do
        itemArray[index] = val.text
        group.combo.keyMap[index] = val.key
    end
    group.combo.Key = filterData.FilterKey
    group.combo:AddItems(itemArray, 1)
    
    group.combo.OnClick = function(self, index, text)
        currentFilters[self.Key] = self.keyMap[index]
        PopulateMapList()
    end

    group.combo.UpdateValue = function(key)
    end
    
    group.Height:Set(group.title.Height())
    
    return group
end

local function ShowMapPositions(mapCtrl, scenario)
    if posGroup then
        posGroup:Destroy()
        posGroup = false
    end

    if not scenario.size then
        LOG("MapSelect: Can't show map positions as size field isn't in scenario yet (must be resaved with new editor!)")
        return
    end

    posGroup = Group(mapCtrl)
    LayoutHelpers.FillParent(posGroup, mapCtrl)

    local startPos = MapUtil.GetStartPositions(scenario)

    local cHeight = posGroup:Height()
    local cWidth = posGroup:Width()

    local mWidth = scenario.size[1]
    local mHeight = scenario.size[2]

    for army, pos in startPos do
        local marker = Bitmap(posGroup, UIUtil.UIFile('/dialogs/mapselect02/commander.dds'))
        LayoutHelpers.AtLeftTopIn(marker, posGroup, (pos[1] / mWidth) * cWidth, (pos[2] / mHeight) * cHeight)
    end
end


-- selectBehavior and exitBehavior are functions that will be run when the select or exit buttons are pressed.
-- selectBehavior will recieve the selectedScenario table in the format:
--      Name - scenario name
--      File - scenario file name
--      MapFile - scenario map file name
--      ScenarioInfo - c++ ScenarioInfo object
-- This method is a bit different in that it returns a pointer to itself
function CreateDialog(selectBehavior, exitBehavior, over, defaultScenarioName, numPlayers)
-- control layout
    local parent = nil
    local background = nil
    if over then
        parent = UIUtil.CreateScreenGroup(over, "<LOC MAPSEL_0001>Map Select ScreenGroup")
        parent.Depth:Set(GetFrame(over:GetRootFrame():GetTargetHead()):GetTopmostDepth() + 1)
    else
        parent = UIUtil.CreateScreenGroup(GetFrame(0), "<LOC MAPSEL_0002>Map Select ScreenGroup")
        background = MenuCommon.SetupBackground(GetFrame(0))
    end

    -- don't parent background to screen group so it doesn't get destroyed until we leave the menus

    local exitButton = nil
    if background then
        exitButton = MenuCommon.CreateExitMenuButton(parent, background, "<LOC _Back>")
    end

    --TODO this panel needs to be moved to dialogs
    local panel = Bitmap(parent, UIUtil.UIFile('/dialogs/mapselect02/panel_bmp.dds'))
    LayoutHelpers.AtCenterIn(panel, parent)

    local worldCover = Bitmap(parent)
    local curColor = 17
    worldCover:SetSolidColor('00000000')
    worldCover:SetNeedsFrameUpdate(true)
    worldCover.OnFrame = function(self, elapsedTime)
        curColor = curColor + (elapsedTime * 200)
        local hexColor = 0
        if curColor > 150 then
            curColor = 150
            hexColor = STR_itox(curColor)
            worldCover:SetNeedsFrameUpdate(false)
        else
            hexColor = STR_itox(curColor)
        end
        worldCover:SetSolidColor(tostring(hexColor)..'000000')
    end
    worldCover.Depth:Set(function() return panel.Depth() - 1 end)
    LayoutHelpers.FillParent(worldCover, GetFrame(0))

    local cancelButton = UIUtil.CreateButtonStd(panel, '/widgets/small', "<LOC _Cancel>", 16, 0, 0, "UI_Menu_Cancel_02")
    LayoutHelpers.AtLeftTopIn(cancelButton, panel, 430, 560)
    
    selectButton = UIUtil.CreateButtonStd(panel, '/widgets/small', "<LOC MAPSEL_0004>Select Map", 16)
    LayoutHelpers.RightOf(selectButton, cancelButton, 15)
    selectButton.Depth:Set(function() return panel.Depth() + 10 end) --TODO what is this getting under when it's in over state?

    mapList = ItemList(panel)
    mapList:SetFont(UIUtil.bodyFont, 16)
    mapList:SetColors(UIUtil.fontColor, "00000000", "FF000000",  UIUtil.highlightColor)
    mapList.Width:Set(328)
    mapList.Height:Set(250)
    LayoutHelpers.AtLeftTopIn(mapList, panel, 50, 100)
    mapList.Depth:Set(function() return panel.Depth() + 10 end) --TODO what is this getting under when it's in over state?

    UIUtil.CreateVertScrollbarFor(mapList)

    local preview = MapPreview(panel)
    preview.Width:Set(290)
    preview.Height:Set(288)
    LayoutHelpers.AtLeftTopIn(preview, panel, 453, 90)
    
    local previewOverlay = Bitmap(preview, UIUtil.UIFile('/dialogs/mapselect02/map-panel-glow_bmp.dds'))
    LayoutHelpers.AtCenterIn(previewOverlay, preview)

    local nopreviewtext = UIUtil.CreateText(panel, "<LOC _No_Preview>No Preview", 24)
    LayoutHelpers.AtCenterIn(nopreviewtext, preview)
    nopreviewtext:Hide()

    description = ItemList(panel)
    description:SetFont(UIUtil.bodyFont, 16)
    description:SetColors(UIUtil.fontColor, "00000000", "FF000000",  UIUtil.highlightColor)
    description.Width:Set(328)
    description.Height:Set(162)
    LayoutHelpers.AtLeftTopIn(description, panel, 50, 368)
    UIUtil.CreateVertScrollbarFor(description)

    mapInfo = UIUtil.CreateText(panel, "<LOC sel_map_0000>Map Info", 18)
    LayoutHelpers.AtLeftTopIn(mapInfo, panel, 450, 392)
    
    mapsize = UIUtil.CreateText(panel, "<LOC sel_map_0001>Map Size: ", 16, UIUtil.bodyFont)
    LayoutHelpers.Below(mapsize, mapInfo)
    
    mapplayers = UIUtil.CreateText(panel, "<LOC sel_map_0002>Max Players: ", 16, UIUtil.bodyFont)
    LayoutHelpers.Below(mapplayers, mapsize, 7)
    
    filterTitle = UIUtil.CreateText(panel, "<LOC sel_map_0003>Filters", 18)
    LayoutHelpers.Below(filterTitle, mapplayers, 7)
    
    for i, filterData in mapFilters do
        local index = i
        filters[index] = CreateFilter(filterTitle, filterData)
        if index == 1 then
            LayoutHelpers.Below(filters[1], filterTitle, 3)
        else
            LayoutHelpers.Below(filters[index], filters[index-1], 7)
        end
    end

-- initialize controls
    PopulateMapList()
    
--  control behvaior
    if exitButton then
        exitButton.OnClick = function(self)
            exitBehavior()
        end
    end

    selectButton.OnClick = function(self, modifiers)
        LOG(repr(table.getsize(selectedScenario.Configurations.standard.teams[1].armies), numPlayers), "<<<<<<<<<<<<<<<<")
        if table.getsize(selectedScenario.Configurations.standard.teams[1].armies) < numPlayers then
            UIUtil.ShowInfoDialog(panel, LOCF("<LOC sel_map_????>You must select a map that supports at least %d players", numPlayers), "<LOC PROFILE_0011>OK")
            return
        else
            selectBehavior(selectedScenario)
        end
    end

    cancelButton.OnClick = function(self, modifiers)
        exitBehavior()
    end

    mapList.OnClick = function(self, row, noSound)
        local errors = false
        mapList:SetSelection(row)
        local scen = scenarios[scenarioKeymap[row+1]]
        selectedScenario = scen
        local file = scen.map
        if DiskGetFileInfo(file) then
            PrefetchMap(file)
            local map = LoadMapPreview(file)
            preview:Show()
            nopreviewtext:Hide()
            preview:SetMap(scen.preview,map)
            ShowMapPositions(preview,scen)
            if map then
                SetDescription(scen)
                map:Destroy()
            else
                description:DeleteAllItems()
                description:AddItem(LOC("<LOC MAPSEL_0000>No description available."))
            end
        else
            WARN("No scenario map file defined")
            preview:Hide()
            nopreviewtext:Show()
            description:DeleteAllItems()
            description:AddItem(LOC("<LOC MAPSEL_0000>No description available."))
            mapplayers:SetText(LOCF("<LOC map_select_0002>NO START SPOTS DEFINED"))
            mapsize:SetText(LOCF("<LOC map_select_0003>NO MAP SIZE INFORMATION"))
            selectButton:Disable()
        end
        local sound = Sound({Cue = "UI_Skirmish_Map_Select", Bank = "Interface",})
        if not noSound then
            PlaySound(sound)
        end
    end

    -- set list to first item or default
    defaultRow = 0
    if defaultScenarioName then
        for i, scenario in scenarios do
            if scenario.file == defaultScenarioName then
                defaultRow = i - 1
                break
            end
        end
    end
    mapList:OnClick(defaultRow, true)
    mapList:ShowItem(defaultRow)

    return parent
end

function SetDescription(scen)
    local errors = false
    description:DeleteAllItems()
    if scen.size then
        mapsize:SetText(LOCF("<LOC map_select_0000>Map Size: %dkm x %dkm", scen.size[1]/50, scen.size[2]/50))
    else
        mapsize:SetText(LOCF("<LOC map_select_0004>NO MAP SIZE INFORMATION"))
        errors = true
    end
    if scen.Configurations.standard.teams[1].armies then
        local maxplayers = table.getsize(scen.Configurations.standard.teams[1].armies)
        mapplayers:SetText(LOCF("<LOC map_select_0001>Max Players: %d", maxplayers))
    else
        mapplayers:SetText(LOCF("<LOC map_select_0005>NO START SPOTS DEFINED"))
        errors = true
    end
    if scen.name then
        description:AddItem(scen.name) 
    else
        description:AddItem(LOC("<LOC map_select_0006>No Scenario Name"))
        errors = true
    end
    description:AddItem("")
    if scen.description then
        local textBoxWidth = description.Width()
        local wrapped = import('/lua/modules/maui/text.lua').WrapText(LOC(scen.description), textBoxWidth, 
            function(curText) return description:GetStringAdvance(curText) end)
        for i, line in wrapped do
            description:AddItem(line)
        end 
    else
        description:AddItem(LOC("<LOC map_select_0007>No Scenario Description"))
        errors = true
    end
    if errors then
        selectButton:Disable()
    else
        selectButton:Enable()
    end
end

function PopulateMapList()    
    mapList:DeleteAllItems()
    local tempMaps = {}
    local count = 1
    for i,sceninfo in scenarios do
        local validMap = true
        local index = i
        if currentFilters.sup_players != 0 then
            if table.getsize(sceninfo.Configurations.standard.teams[1].armies) < currentFilters.sup_players then
                validMap = false
            end
        end
        if currentFilters.map_size != 0 then
            if sceninfo.size[1] != currentFilters.map_size then
                validMap = false
            end
        end
        if validMap then
            table.insert(tempMaps, sceninfo)
            scenarioKeymap[count] = index
            count = count + 1 
        end
    end
    
    for i,sceninfo in tempMaps do
        mapList:AddItem(sceninfo.name)
    end
end