--*****************************************************************************
--* File: lua/modules/ui/dialogs/modmanager.lua
--* Author: Chris Blackwell
--* Summary: Allows you to choose mods
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local Checkbox = import('/lua/modules/maui/checkbox.lua').Checkbox
local ItemList = import('/lua/modules/maui/itemlist.lua').ItemList
local Scrollbar = import('/lua/modules/maui/scrollbar.lua').Scrollbar
local Text = import('/lua/modules/maui/text.lua').Text
local MultiLineText = import('/lua/modules/maui/multilinetext.lua').MultiLineText
local Button = import('/lua/modules/maui/button.lua').Button
local Group = import('/lua/modules/maui/group.lua').Group
local MenuCommon = import('/lua/modules/ui/menus/menucommon.lua')
local MainMenu = import('/lua/modules/ui/menus/main.lua')
local Mods = import('/lua/system/mods.lua')

function CreateDialog(over, inLobby, exitBehavior)

    ---------------------------------------------------------------------------
    -- basic layout and operation of dialog
    ---------------------------------------------------------------------------
    local parent = nil
    local background = nil
    if over then
        parent = over
    else
        parent = UIUtil.CreateScreenGroup(GetFrame(0), "Map Select ScreenGroup")
        background = MenuCommon.SetupBackground(GetFrame(0))
    end

    -- don't parent background to screen group so it doesn't get destroyed until we leave the menus
    local exitButton = nil
    if background then
        exitButton = MenuCommon.CreateExitMenuButton(parent, background, "<LOC _Back>")
    end

    local panel = Bitmap(parent, UIUtil.UIFile('/dialogs/options/panel-front-end_bmp.dds'))
    LayoutHelpers.AtCenterIn(panel, parent)
    
    local title = UIUtil.CreateText(panel, LOC("<LOC _Mod_Manager>Mod Manager"), 24)
    LayoutHelpers.AtHorizontalCenterIn(title, panel)
    LayoutHelpers.AtTopIn(title, panel, 35)
    
    if over then
        panel.Depth:Set(GetFrame(over:GetRootFrame():GetTargetHead()):GetTopmostDepth() + 1)
    end
    
    local worldCover = UIUtil.CreateWorldCover(panel)
    
    local dlgLabel = UIUtil.CreateText(panel, "<LOC uimod_0001>Click to select or deselect", 16, UIUtil.bodyFont)
    LayoutHelpers.AtTopIn(dlgLabel, panel, 400)
    LayoutHelpers.AtHorizontalCenterIn(dlgLabel, panel)

    ---------------------------------------------------------------------------
    -- Mod list control
    ---------------------------------------------------------------------------
    local numElementsPerPage = 4
    
    local scrollGroup = Group(panel)
    scrollGroup.Width:Set(500)
    scrollGroup.Height:Set(300)
    
    LayoutHelpers.AtLeftTopIn(scrollGroup, panel, 40, 80)
    UIUtil.CreateVertScrollbarFor(scrollGroup)
    
    scrollGroup.controlList = {}
    scrollGroup.top = 1
    
    -- called when the scrollbar for the control requires data to size itself
    -- GetScrollValues must return 4 values in this order:
    -- rangeMin, rangeMax, visibleMin, visibleMax
    -- aixs can be "Vert" or "Horz"
    scrollGroup.GetScrollValues = function(self, axis)
        return 1, table.getn(self.controlList), self.top, math.min(self.top + numElementsPerPage - 1, table.getn(scrollGroup.controlList))
    end

    -- called when the scrollbar wants to scroll a specific number of lines (negative indicates scroll up)
    scrollGroup.ScrollLines = function(self, axis, delta)
        self:ScrollSetTop(axis, self.top + math.floor(delta))
    end

    -- called when the scrollbar wants to scroll a specific number of pages (negative indicates scroll up)
    scrollGroup.ScrollPages = function(self, axis, delta)
        self:ScrollSetTop(axis, self.top + math.floor(delta) * numElementsPerPage)
    end

    -- called when the scrollbar wants to set a new visible top line
    scrollGroup.ScrollSetTop = function(self, axis, top)
        top = math.floor(top)
        if top == self.top then return end
        self.top = math.max(math.min(table.getn(self.controlList) - numElementsPerPage + 1 , top), 1)
        self:CalcVisible()
    end

    -- called to determine if the control is scrollable on a particular access. Must return true or false.
    scrollGroup.IsScrollable = function(self, axis)
        return true
    end
    
    -- determines what controls should be visible or not, and hide/show appropriately
    scrollGroup.CalcVisible = function(self)
        local top = self.top
        local bottom = self.top + numElementsPerPage
        for index, control in ipairs(self.controlList) do
            if index < top or index >= bottom then
                control:Hide()
            else
                control:Show()
                control.Left:Set(self.Left)
                local lIndex = index
                local lControl = control
                control.Top:Set(function() return self.Top() + ((lIndex - top) * lControl.Height()) end)
            end
        end
    end
    
    ---------------------------------------------------------------------------
    -- Mod list element
    ---------------------------------------------------------------------------
    local function CreateListElement(parent, modInfo)
        local bg = Bitmap(parent)
        bg.Height:Set(function() return parent.Height() / numElementsPerPage  end)
        bg.Width:Set(parent.Width)
        bg.modInfo = modInfo
        
        local icon = Bitmap(bg, UIUtil.UIFile(modInfo.icon))
        LayoutHelpers.AtLeftIn(icon, bg, 5)
        LayoutHelpers.AtBottomIn(icon, bg)
        
        local name = UIUtil.CreateText(bg, modInfo.name, 12, UIUtil.bodyFont)
        LayoutHelpers.AtLeftTopIn(name, bg, 5, 5)
        name:SetDropShadow(true)
        
        local activecheck = Checkbox(bg, 
            UIUtil.UIFile('/widgets/radio02/rad_btn_disabled.dds'), 
            UIUtil.UIFile('/widgets/radio02/rad_btn_enabled.dds'), 
            UIUtil.UIFile('/widgets/radio02/rad_btn_disabled.dds'), 
            UIUtil.UIFile('/widgets/radio02/rad_btn_enabled.dds'), 
            UIUtil.UIFile('/widgets/radio02/rad_btn_disabled.dds'), 
            UIUtil.UIFile('/widgets/radio02/rad_btn_enabled.dds'))
        LayoutHelpers.AtRightIn(activecheck, bg, 5)
        activecheck.Top:Set(name.Top)
        activecheck:DisableHitTest()
        
        local active = UIUtil.CreateText(bg, "", 12, UIUtil.bodyFont)
        LayoutHelpers.LeftOf(active, activecheck)
        active.Top:Set(name.Top)
        active:SetDropShadow(true)
        
        local desc = MultiLineText(bg, UIUtil.bodyFont, 12, UIUtil.fontColor)
        desc.Left:Set(icon.Right)
        desc.Top:Set(function() return name.Bottom() + 5 end)
        desc.Height:Set(icon.Height)
        desc.Width:Set(400)
        desc:SetText(modInfo.description)

        icon:DisableHitTest()
        name:DisableHitTest()
        desc:DisableHitTest()

        if modInfo.active then
            activecheck:SetCheck(true)
            active:SetText(LOC("<LOC uimod_0002>Active"))
            active:SetColor("cyan")
            bg:SetSolidColor("FF008080")
            bg.active = true
        else
            activecheck:SetCheck(false)
            active:SetText(LOC("<LOC uimod_0003>Disabled"))
            active:SetColor("red")
            bg:SetSolidColor(UIUtil.panelColor())
            bg.active = false
        end

        bg:Hide()
        
        bg.HandleEvent = function(self, event)
            if event.Type == 'ButtonPress' or event.Type == 'ButtonDClick' then
                if self.active then
                    activecheck:SetCheck(false)
                    active:SetText(LOC("<LOC uimod_0004>Disabled"))
                    active:SetColor("red")
                    self:SetSolidColor(UIUtil.panelColor())
                    self.active = false
                else
                    activecheck:SetCheck(true)
                    active:SetText(LOC("<LOC uimod_0005>Active"))
                    active:SetColor("cyan")
                    self:SetSolidColor("FF008080")
                    self.active = true
                end
            end
        end
        
        return bg
    end
    
    ---------------------------------------------------------------------------
    -- Mod list element
    ---------------------------------------------------------------------------
    Mods.InitModInfo() -- force reload of mod info to pick up changes on disk
    for id, info in sortedpairs(Mods.AllMods()) do
        if info.selectable then
            table.insert(scrollGroup.controlList, CreateListElement(scrollGroup, info))
        end
    end

    scrollGroup:CalcVisible()

    ---------------------------------------------------------------------------
    -- OK and cancel button behaviors
    ---------------------------------------------------------------------------
    local function KillDialog(cancel)
        if not cancel then
            local activeMods = {}
            
            for index, control in scrollGroup.controlList do
                if control.active then
                    table.insert(activeMods, control.modInfo.id)
                end
            end
            Mods.RequestActiveMods(activeMods)
        end
        
        if over then
            panel:Destroy()
        else
            parent:Destroy()
        end
        if exitBehavior then
            exitBehavior()
        end
    end

    local okBtn = UIUtil.CreateButtonStd(panel, "/dialogs/standard_btn/standard", "<LOC _Ok>", 14, 2)
    LayoutHelpers.AtLeftTopIn(okBtn, panel, 155, 420)
    okBtn.OnClick = function(self, modifiers)
        KillDialog(false)
    end

    local cancelBtn = UIUtil.CreateButtonStd(panel, "/dialogs/standard_btn/standard", "<LOC _Cancel>", 14, 2)
    LayoutHelpers.AtRightTopIn(cancelBtn, panel, 155, 420)
    cancelBtn.OnClick = function(self, modifiers)
        KillDialog(true)
    end

    if exitButton then
        exitButton.OnClick = function(self, modifiers)
            KillDialog(true)
        end
    end
end