--*****************************************************************************
--* File: lua/modules/ui/dialogs/profile.lua
--* Author: Chris Blackwell
--* Summary: manages user profiles
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************


local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local Button = import('/lua/modules/maui/button.lua').Button
local Group = import('/lua/modules/maui/group.lua').Group
local Edit = import('/lua/modules/maui/edit.lua').Edit
local ItemList = import('/lua/modules/maui/itemlist.lua').ItemList
local Tooltip = import('/lua/modules/ui/game/tooltip.lua')
local Prefs = import('/lua/modules/ui/prefs.lua')

local function SetupEditField(control)
    control:SetForegroundColor(UIUtil.fontColor)
    control:SetBackgroundColor("00569FFF")
    control:SetHighlightForegroundColor(UIUtil.highlightColor)
    control:SetHighlightBackgroundColor("880085EF")
    control:SetFont(UIUtil.bodyFont, 20)
    control.Height:Set(function() return control:GetFontHeight() end)
    control:AcquireFocus()
    return control
end

local tempProfileCurrent = GetPreference("profile.current")
local tempProfileTable = GetPreference("profile.profiles")

function CreateDialog(exitBehavior)
    local function AnyProfilesDefined(parent)
        if not Prefs.ProfilesExist() then
            UIUtil.ShowInfoDialog(parent, "<LOC PROFILE_0008>You must create at least one profile", "<LOC PROFILE_0009>OK")
            return false
        end
        
        return true
    end

    local panel = Bitmap(GetFrame(0), UIUtil.UIFile('/dialogs/profile02/panel_bmp.dds'))
    LayoutHelpers.AtCenterIn(panel, GetFrame(0))
    panel.Depth:Set(100000)
    
    local worldCover = UIUtil.CreateWorldCover(panel)

    local titleText = UIUtil.CreateText(panel, "<LOC PROFILE_0001>Profile Manager", 24)
    LayoutHelpers.AtTopIn(titleText, panel, 40)
    LayoutHelpers.AtHorizontalCenterIn(titleText, panel)
    
    local deleteButton = UIUtil.CreateButtonStd(panel, '/widgets/small', "<LOC PROFILE_0006>Delete", 14)
    LayoutHelpers.AtLeftTopIn(deleteButton, panel, 45, 340)
    Tooltip.AddButtonTooltip(deleteButton, "Profile_delete")
    
    local optionsButton = UIUtil.CreateButtonStd(panel, '/widgets/small', "<LOC PROFILE_0012>Options", 14)
    LayoutHelpers.RightOf(optionsButton, deleteButton)
    Tooltip.AddButtonTooltip(optionsButton, "mainmenu_options")
    optionsButton.OnClick = function(self, modifiers)
        if not AnyProfilesDefined(panel) then return end
        import('/lua/modules/ui/dialogs/options.lua').CreateDialog(panel)
    end
    
    local createButton = UIUtil.CreateButtonStd(panel, '/widgets/small', "<LOC PROFILE_0004>Create", 14)
    LayoutHelpers.RightOf(createButton, optionsButton)
    Tooltip.AddButtonTooltip(createButton, "Profile_create")
    
    local cancelButton = UIUtil.CreateButtonStd(panel, '/widgets/small', "<LOC PROFILE_0005>Cancel", 14)
    LayoutHelpers.AtTopIn(cancelButton, panel, 380)
    cancelButton.Right:Set(function() return (panel.Width() / 2) + panel.Left() end)
    Tooltip.AddButtonTooltip(cancelButton, "Profile_cancel")
        
    local okButton = UIUtil.CreateButtonStd(panel, '/widgets/small', "<LOC PROFILE_0007>OK", 14)
    LayoutHelpers.AtTopIn(okButton, panel, 380)
    okButton.Left:Set(function() return (panel.Width() / 2) + panel.Left() end)
    Tooltip.AddButtonTooltip(okButton, "Profile_ok")

    local profileListBG = Bitmap(panel)
    profileListBG:SetSolidColor("00569FFF")
    LayoutHelpers.AtLeftTopIn(profileListBG, panel, 52, 95)
    profileListBG.Width:Set(406)
    profileListBG.Height:Set(230)

    local profileList = ItemList(profileListBG)
    LayoutHelpers.FillParent(profileList, profileListBG)
    profileList:SetFont("Arial Bold", 16)
    profileList:SetColors(UIUtil.fontColor, "00000000", "ff000000",  UIUtil.highlightColor)
    UIUtil.CreateVertScrollbarFor(profileList)

    function UpdateProfileList()
        profileList:DeleteAllItems()
        local profiles = GetPreference("profile.profiles")
        if not profiles then return end
        for key, value in profiles do
            profileList:AddItem(value.Name)
        end
        local current = GetPreference("profile.current")
        if current then
            profileList:SetSelection(current - 1)
        end
    end

    profileList.OnClick = function(self, row)
        if row < 0 then return end
        profileList:SetSelection(row)
        SetPreference("profile.current", row + 1)
        local profiles = GetPreference("profile.profiles")
    end

    createButton.OnClick = function(self)
        CreationDialog(panel)
    end

    cancelButton.OnClick = function(self)
        SetPreference("profile.current", tempProfileCurrent)
        SetPreference("profile.profiles", tempProfileTable)
        if not AnyProfilesDefined(panel) then return end
        panel:Destroy()
        if exitBehavior then
            exitBehavior()
        end
    end

    deleteButton.OnClick = function(self)
        UIUtil.QuickDialog(panel, "<LOC PROFILE_0000>Are you sure you want to delete this profile?", "<LOC _Yes>", function()
            local current = GetPreference("profile.current")
            if not current then return end
            local profiles = GetPreference("profile.profiles")
            if not profiles then return end
            table.remove(profiles, current)
            SetPreference("profile.profiles", profiles)
            SetPreference("profile.current", 1)
            SavePreferences()
            UpdateProfileList()
       end, "<LOC _No>", nil)
    end

    okButton.OnClick = function(self)
        if not AnyProfilesDefined(panel) then return end
--TODO - send "profile changed" message to engine
        panel:Destroy()
        if exitBehavior then
            exitBehavior()
        end
    end

    -- if there are any profiles, populate the list and select the current profile
    UpdateProfileList()

    local profiles = GetPreference("profile.profiles")
    local current = GetPreference("profile.current")
    if profiles and current then
        profileList:SetSelection(current - 1)
        if profiles[current] == nil then
            SetPreference("profile.current", 0) -- if current profile is damaged, reset to 0
        end
    end

    return panel
end

function CreationDialog(parent)
    local bg = Bitmap(parent, UIUtil.UIFile('/dialogs/dialog/panel_bmp.dds'))
    LayoutHelpers.AtCenterIn(bg, parent)
    bg.Depth:Set(function() return parent.Depth() + 10 end)
    
    local worldCover = UIUtil.CreateWorldCover(bg)
    
    local cancelButton = UIUtil.CreateButtonStd(bg, '/widgets/small', "<LOC PROFILE_0005>Cancel", 14, 3)
    LayoutHelpers.AtLeftTopIn(cancelButton, bg, 50, 115)
    
    local okButton = UIUtil.CreateButtonStd(bg, '/widgets/small', "<LOC PROFILE_0007>OK", 14, 3)
    LayoutHelpers.RightOf(okButton, cancelButton, 30)
    
    local nameEdit = Edit(bg)
    nameEdit.Top:Set(function() return bg.Top() + 50 end)
    LayoutHelpers.AtHorizontalCenterIn(nameEdit, bg)
    nameEdit.Width:Set(350)
    SetupEditField(nameEdit)
    nameEdit:AcquireFocus()    
    nameEdit.OnCharPressed = function(self, charcode)
        local count = string.len(self:GetText())
        if count > 20 then
            return true
        else
            return false
        end
    end
    
    local function DestroyMe()
        bg:Destroy()
    end
    
    okButton.OnClick = function(self)
        local name = nameEdit:GetText()
        if name == "" then
            UIUtil.ShowInfoDialog(bg, "<LOC PROFILE_0010>Please fill in a profile name", "<LOC PROFILE_0011>OK")
            return
        end

        if not Prefs.CreateProfile(name) then
            UIUtil.ShowInfoDialog(bg, "<LOC PROFILE_0013>That name is already in use", "<LOC PROFILE_0011>OK")
            return
        end

        UpdateProfileList()
        DestroyMe()
    end 

    cancelButton.OnClick = function(self)
        DestroyMe()
    end 
end
