--*****************************************************************************
--* File: lua/modules/ui/dialogs/score.lua
--* Author: Chris Blackwell
--* Summary: Post game score dialog
--*
--* Copyright  :005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Group = import('/lua/modules/maui/group.lua').Group
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local Button = import('/lua/modules/maui/button.lua').Button
local Text = import('/lua/modules/maui/text.lua').Text
local MultiLineText = import('/lua/modules/maui/multilinetext.lua').MultiLineText
local Checkbox = import('/lua/modules/maui/checkbox.lua').Checkbox
local RadioGroup = import('/lua/modules/maui/mauiutil.lua').RadioGroup
local Options = import('/lua/modules/ui/game/options.lua')
local Tooltip = import('/lua/modules/ui/game/tooltip.lua')
local ItemList = import('/lua/modules/maui/itemlist.lua').ItemList

local dialog = false
local currentPage = false
local curGrid = false
local curHistogram = false
local curInfo = false
local elapsedTime = false
local curType = false
local curSortCol = false
local curSortDescending = true
local savedGameTime = false

local tabs = {
    {
        title = "<LOC SCORE_0030>Campaign",
        button = "campaign",
        scoreKey = 'campaign',
        -- TODO: This is here to get loc tags in to database, will make more sense in line when campaign score is done
        text = {
            "<LOC SCORE_0031>Restart",
            "<LOC SCORE_0032>Load",
            "<LOC SCORE_0033>Continue",
            "<LOC SCORE_0034>Replay",
            "<LOC SCORE_0035>Load Replay",
            "<LOC SCORE_0045>Sort By Team",
        }
    },
    {
        title = "<LOC SCORE_0000>General",
        button = "general",
        scoreKey = 'general',
        columns = {
          {title = "<LOC SCORE_0001>Score", scoreKey = 'score',},
          {title = "<LOC SCORE_0002>Kills", scoreKey = 'kills',},
          {title = "<LOC SCORE_0003>Built", scoreKey = 'built',},
          {title = "<LOC SCORE_0004>Losses", scoreKey = 'lost',},
          {title = "<LOC SCORE_0005>Mass", scoreKey = 'mass',},
          {title = "<LOC SCORE_0006>Energy", scoreKey = 'energy',},  
        },
        types = {
            {title = "<LOC SCORE_0007>Count", scoreKey = 'count',},
            {title = "<LOC SCORE_0008>Mass Value", scoreKey = 'mass',},
            {title = "<LOC SCORE_0009>Energy Value", scoreKey = 'energy',},
        },
        grid = true,
    },
    {
        title = "<LOC SCORE_0010>Units",
        button = "units",
        scoreKey = 'units',
        columns = {
            {title = "<LOC SCORE_0011>CDR", scoreKey = 'cdr',},
            {title = "<LOC SCORE_0012>Land", scoreKey = 'land',},
            {title = "<LOC SCORE_0013>Naval", scoreKey = 'naval',},
            {title = "<LOC SCORE_0014>Air", scoreKey = 'air',},
            {title = "<LOC SCORE_0015>Structures", scoreKey = 'structures',},
            {title = "<LOC SCORE_0016>Experimental", scoreKey = 'experimental',},
        },
        types = {
            {title = "<LOC SCORE_0017>Kills", scoreKey = 'kills',},
            {title = "<LOC SCORE_0018>Built", scoreKey = 'built',},
            {title = "<LOC SCORE_0019>Losses", scoreKey = 'lost',},                       
        },
        grid = true,
    },
    {
        title = "<LOC SCORE_0020>Resources",
        button = "resources",
        scoreKey = 'resources',
        columns = {
            {title = "<LOC SCORE_0021>Mass Collected", scoreKey = 'massin',},
            {title = "<LOC SCORE_0022>Mass Spent", scoreKey = 'massout',},
            {title = "<LOC SCORE_0023>Mass Wasted", scoreKey = 'massover',},
            {title = "<LOC SCORE_0024>Energy Collected", scoreKey = 'energyin',},
            {title = "<LOC SCORE_0025>Energy Spent", scoreKey = 'energyout',},
            {title = "<LOC SCORE_0026>Energy Wasted", scoreKey = 'energyover',},
        },
        types = {
            {title = "<LOC SCORE_0027>Total", scoreKey = 'total',},
            {title = "<LOC SCORE_0028>Rate", scoreKey = 'rate',},
        },
        grid = true,
    },
}

-- determine which data to show based on current page, data type and column
local function UpdateDisplay()
    -- show elapsed time
    elapsedTime:SetText(GetGameTime())

    -- check for campaign, handled differently
    if currentPage.tabData.button == "campaign" then
        return    
    end

    -- set the current score page key
    local curScoreKey = currentPage.tabData.scoreKey
    
    -- set up a small table to sort by the current sort value
    local sortOrder = {}
    for index, info in ipairs(curInfo) do
        sortOrder[index] = {}
        sortOrder[index].infoIndex = index
        sortOrder[index].sortKey = curInfo.scoreData.current[index][curScoreKey][curSortCol][curType] or curInfo.scoreData.current[index][curScoreKey][curSortCol]
    end

    table.sort(sortOrder, function(first, second)
        if curSortDescending then
            return (first.sortKey > second.sortKey)
        else
            return (first.sortKey < second.sortKey)
        end
    end)
    
    if curGrid then
        for index, gridRow in ipairs(curGrid) do
            local curPlayerIndex = sortOrder[index].infoIndex
            local info = curInfo[curPlayerIndex]
            gridRow.color:SetSolidColor(info.color)
            gridRow.factionIcon:SetTexture(UIUtil.SkinnableFile(UIUtil.GetFactionIcon(info.faction)))
            gridRow.playerName:SetText(info.name)
            gridRow.teamName:SetText(info.teamName)
            
            for curCol, colData in currentPage.tabData.columns do
                -- check if data sub-type otherwise use parent value
                curGrid[index].cols[curCol]:SetText(tostring(math.floor(
                    curInfo.scoreData.current[curPlayerIndex][curScoreKey][colData.scoreKey][curType] or 
                    curInfo.scoreData.current[curPlayerIndex][curScoreKey][colData.scoreKey])))
            end
        end
    end
end

local function SortByColumn(colKey)
    if curSortCol == colKey then
        curSortDescending = not curSortDescending
    else
        curSortCol = colKey
        curSortDescending = true
    end

    UpdateDisplay()
end

local function SetDataType(dataType)
    curType = dataType

    -- TODO coordinate buttons? make them a radio group for text, and not for histogram?    
    UpdateDisplay()
end

function UpdateData()
    
    -- clear curInfo
    curInfo = {}
    
    -- first get army info and place it in the 
    local armiesTable = GetArmiesTable().armiesTable

    -- popluate data in to cur info
    for index, armyInfo in armiesTable do 
        -- set basic info from armies table
        curInfo[index] = {}
        curInfo[index].name = armyInfo.nickname
        curInfo[index].faction = armyInfo.faction
        curInfo[index].color = armyInfo.color
        curInfo[index].teamName = index --TODO we need to get team data in here
    end

    -- set score data
    curInfo.scoreData = import('/lua/modules/ui/game/scoreaccum.lua').scoreData
end

function CreateDialog(victory, showCampaign, operationVictoryTable)
    if dialog then return end

    local worldView = import('/lua/modules/ui/game/borders.lua').GetMapGroup()

    if not UIUtil.GetNetworkBool() then
        Options.PauseGame(true)
    end
    
    -- create the dialog
    dialog = Group(worldView, "scoreDialogParentGroup")
    dialog:SetRenderPass(UIUtil.UIRP_PostGlow)  -- just in case our parent is the map
    LayoutHelpers.AtCenterIn(dialog, worldView)
    dialog.Depth:Set(GetFrame(0):GetTopmostDepth())
    
    local bg = Bitmap(dialog, UIUtil.SkinnableFile('/dialogs/score-victory-defeat/panel_bmp.dds'))
    LayoutHelpers.AtCenterIn(bg, worldView)
    
    dialog.Width:Set(bg.Width)
    dialog.Height:Set(bg.Height)
    
    local worldCover = Bitmap(bg, UIUtil.UIFile("/menus02/background-paint02_bmp.dds"))
    worldCover.Depth:Set(function() return bg.Depth() - 1 end)
    LayoutHelpers.FillParent(worldCover, GetFrame(0))
    
    local title = UIUtil.CreateText(bg, "", 18, UIUtil.titleFont)
    LayoutHelpers.RelativeTo(title, bg, UIUtil.SkinnableFile('/dialogs/score-victory-defeat/score-victory-defeat_layout.lua'), 'l_titles', 'panel_bmp')

    -- set controls that are global to the dialog
    local quitButton = UIUtil.CreateButtonStd(bg, '/dialogs/standard_btn/standard', "<LOC _Quit>", 11)
    LayoutHelpers.RelativeTo(quitButton, bg, UIUtil.SkinnableFile('/dialogs/score-victory-defeat/score-victory-defeat_layout.lua'), 'l_cancel_btn', 'panel_bmp')
    quitButton.OnClick = function(self, modifiers)
        if UIUtil.BETA_FLAG then
            ExitApplication()
        else
            if operationVictoryTable then
                import('/lua/modules/ui/campaign/campaignmanager.lua').SetAutoContinueOpStatus(operationVictoryTable.success)
            else
                import('/lua/modules/ui/campaign/campaignmanager.lua').ClearAutoContinueOp()
            end
            ExitGame()
        end
    end
    quitButton.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "PostScore_Quit", nil, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
        Button.HandleEvent(self, event)
    end
    
    local elapsedTimeLabel = UIUtil.CreateText(bg, "<LOC SCORE_0029>Game Time:", 14, UIUtil.titleFont)
    LayoutHelpers.RelativeTo(elapsedTimeLabel, bg, UIUtil.SkinnableFile('/dialogs/score-victory-defeat/score-victory-defeat_layout.lua'), 'l_elapsed_time', 'panel_bmp')
    
    elapsedTime = UIUtil.CreateText(bg, "", 14, UIUtil.titleFont)
    LayoutHelpers.RightOf(elapsedTime, elapsedTimeLabel, 5)
  
    local replayButton = UIUtil.CreateButtonStd(bg, '/dialogs/standard_btn/standard', "<LOC _Replay>", 11)
    LayoutHelpers.RelativeTo(replayButton, bg, UIUtil.SkinnableFile('/dialogs/score-victory-defeat/score-victory-defeat_layout.lua'), 'l_replay_btn_up', 'panel_bmp')
    replayButton:Disable()  --TODO remove when replay enabled!
    replayButton.OnClick = function(self, modifiers)
    end
    replayButton.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "PostScore_Replay", nil, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
        Button.HandleEvent(self, event)
    end
    
    -- when a new page is selected, create the page and deal with the tab correctly
    function SetNewPage(tabControl)
        -- kill any other page
        if currentPage then
            currentPage.tabBitmap:Destroy()
            currentPage.tabButton:Show()
            currentPage:Destroy()
        end

        -- store the tab data for this tab for easy access
        local tabData = tabControl.tabData
        
        -- page is a big as group to make placement easy, and destruction of the page easy
        currentPage = Group(bg, "currentScorePageGroup")
        currentPage.tabData = tabData
        LayoutHelpers.FillParent(currentPage, bg)
                
        -- show the "selected" state of the tab, which hides the button and shows a bitmap
        currentPage.tabButton = tabControl
        currentPage.tabButton:Hide()
        currentPage.tabBitmap = Bitmap(bg, UIUtil.SkinnableFile('/dialogs/tab_btn/general_btn_selected.dds'))
        currentPage.tabBitmap.Depth:Set(function() return bg.Depth() + 100 end)
        LayoutHelpers.AtCenterIn(currentPage.tabBitmap, currentPage.tabButton)
        local tabLabel = UIUtil.CreateText(currentPage.tabBitmap, tabData.title, 11, UIUtil.titleFont)
        LayoutHelpers.AtCenterIn(tabLabel, currentPage.tabBitmap)
        
        -- if there's a grid to be shown, set it up along with histogram
        -- Note: currently, only campaign score page won't have a grid
        if tabData.grid then
            -- create sub tabs to display grid or hisotogram
            local gridTab = Checkbox(currentPage
                , UIUtil.SkinnableFile('/dialogs/score_btn/grid_btn_up.dds')
                , UIUtil.SkinnableFile('/dialogs/score_btn/chart_btn_up.dds')
                , UIUtil.SkinnableFile('/dialogs/score_btn/grid_btn_down.dds')
                , UIUtil.SkinnableFile('/dialogs/score_btn/chart_btn_down.dds')
                , UIUtil.SkinnableFile('/dialogs/score_btn/grid_btn_dis.dds')
                , UIUtil.SkinnableFile('/dialogs/score_btn/chart_btn_dis.dds'))
            LayoutHelpers.RelativeTo(gridTab, bg, UIUtil.SkinnableFile('/dialogs/score-victory-defeat/score-victory-defeat_layout.lua'), 'l_chart_btn_up', 'panel_bmp')
            local tabLabel = UIUtil.CreateText(gridTab, "<LOC _Grid>", 11, UIUtil.titleFont)
            LayoutHelpers.AtCenterIn(tabLabel, gridTab)
            gridTab:Disable()

            -- create the grid group/bg
            local gridGroup = Group(currentPage, "scoreGridGroup")
            -- no layout info for these sub pages, so position manually
            LayoutHelpers.AtLeftTopIn(gridGroup, currentPage, 43, 102)
            gridGroup.Width:Set(743)
            gridGroup.Height:Set(257)
            gridTab:SetCheck(true)

            local gridBG = Bitmap(gridGroup, UIUtil.SkinnableFile('/dialogs/score-victory-defeat/totals-back_bmp.dds'))
            LayoutHelpers.RelativeTo(gridBG, bg, UIUtil.SkinnableFile('/dialogs/score-victory-defeat/score-victory-defeat_layout.lua'), 'totals-back_bmp', 'panel_bmp')

            -- create the histogram group/bg
            local histogramGroup = Group(currentPage, "scoreHistogramGroup")
            LayoutHelpers.FillParent(histogramGroup, gridGroup)
            histogramGroup:Hide()

            local histogramBG = Bitmap(histogramGroup, UIUtil.SkinnableFile('/dialogs/score-victory-defeat/histograph-back_bmp.dds'))
            LayoutHelpers.RelativeTo(histogramBG, bg, UIUtil.SkinnableFile('/dialogs/score-victory-defeat/score-victory-defeat_layout.lua'), 'histograph-back_bmp', 'panel_bmp')

            -- set up behavior or histogram/group tabs
            gridTab.OnClick = function(self, modifiers)
                if not self:IsChecked() then
                    histogramGroup:Hide()
                    gridGroup:Show()
                    gridTab:SetCheck(true)
                    histogramTab:SetCheck(false)
                end
            end
            gridTab.HandleEvent = function(self, event)
                if event.Type == 'MouseEnter' then
                    Tooltip.CreateMouseoverDisplay(self, "PostScore_Grid", nil, true)
                elseif event.Type == 'MouseExit' then
                    Tooltip.DestroyMouseoverDisplay()
                end
                Checkbox.HandleEvent(self, event)
            end
                        
            -- set up labels (no layout info yet, so store x values in a table)
            -- note the numbers are colmun num, for easy access
            local labelXPos = {
                icon = 5,
                player = 38,
                team = 259,
                [1] = 414,
                [2] = 508,
                [3] = 602,
                [4] = 696,
                [5] = 790,
                [6] = 884,
            }
            
            local playerText = MultiLineText(gridBG, UIUtil.titleFont, 16, UIUtil.fontColor)
            playerText.Width:Set(250)
            playerText.Height:Set(35)
            LayoutHelpers.AtLeftTopIn(playerText, gridBG, labelXPos.icon, 12)   -- note this is at icon as there is none in the label
            playerText:SetText(LOC("<LOC _Player>"))
            
            local teamText = MultiLineText(gridBG, UIUtil.titleFont, 16, UIUtil.fontColor)
            teamText.Width:Set(80)
            teamText.Height:Set(35)
            LayoutHelpers.AtLeftTopIn(teamText, gridBG, labelXPos.team, 12)
            teamText:SetText(LOC("<LOC _Team>"))
            
            -- set up sorting buttons
            local sortButtons = {}
            for index, colName in tabData.columns do
                sortButtons[index] = {}
                sortButtons[index].checkbox = Checkbox(gridBG
                    , UIUtil.SkinnableFile('/dialogs/rect-box_btn/rect-box-d_btn_up.dds')
                    , UIUtil.SkinnableFile('/dialogs/rect-box_btn/rect-box-s_btn_up.dds')
                    , UIUtil.SkinnableFile('/dialogs/rect-box_btn/rect-box-d_btn_over.dds')
                    , UIUtil.SkinnableFile('/dialogs/rect-box_btn/rect-box-s_btn_over.dds')
                    , UIUtil.SkinnableFile('/dialogs/rect-box_btn/rect-box-d_btn_dis.dds')
                    , UIUtil.SkinnableFile('/dialogs/rect-box_btn/rect-box-s_btn_dis.dds'))
                LayoutHelpers.AtLeftTopIn(sortButtons[index].checkbox, gridBG, labelXPos[index] - 30, -3)
                sortButtons[index].checkbox.toolTip = colName.scoreKey

                sortButtons[index].label = MultiLineText(sortButtons[index].checkbox, UIUtil.titleFont, 12, UIUtil.fontColor)
                LayoutHelpers.FillParentFixedBorder(sortButtons[index].label, sortButtons[index].checkbox, 15)
                local wi, hi = GetTextureDimensions(UIUtil.UIFile('/dialogs/rect-box_btn/rect-box-d_btn_up.dds'))
                sortButtons[index].label.Width:Set(wi - 5)
                sortButtons[index].label.Height:Set(hi - 5)
                sortButtons[index].label:SetText(LOC(colName.title))
                sortButtons[index].label:DisableHitTest()

                sortButtons[index].checkbox._sortCol = colName.scoreKey
            end
            
            local sortGroup = RadioGroup(sortButtons)
            sortGroup.OnClick = function(self, index, item)
                SortByColumn(item.checkbox._sortCol)
            end
            sortGroup:SetCheckboxEventHandler(function(self, event) 
                if event.Type == 'MouseEnter' then
                    Tooltip.CreateMouseoverDisplay(self, "PostScore_"..self.toolTip, nil, true)
                elseif event.Type == 'MouseExit' then
                    Tooltip.DestroyMouseoverDisplay()
                end
            end)
            
            --default sort col
            -- TODO should this be set elsewhere?
            curSortCol = tabData.columns[1].scoreKey
            
            -- clear out existing grid so we can have fresh data
            curGrid = {}
            
            -- set up a row for each player (well each army at least)
            -- layouts may have to change based on team?
            local armiesInfo = GetArmiesTable()

            local prev = gridBG
            for index, armyInfo in armiesInfo.armiesTable do
                local armyBg = Bitmap(gridGroup, UIUtil.SkinnableFile('/dialogs/score-victory-defeat/player-back_bmp.dds'))
                LayoutHelpers.Below(armyBg, prev, -5)
                prev = armyBg
                
                curGrid[index] = {}
                
                curGrid[index].color = Bitmap(armyBg)
                curGrid[index].color.Width:Set(20)
                curGrid[index].color.Height:Set(20)
                LayoutHelpers.AtLeftTopIn(curGrid[index].color, armyBg, labelXPos.icon, 6)

                curGrid[index].factionIcon = Bitmap(curGrid[index].color)
                LayoutHelpers.FillParent(curGrid[index].factionIcon, curGrid[index].color)

                curGrid[index].playerName = UIUtil.CreateText(armyBg, "", 16, UIUtil.bodyFont)
                LayoutHelpers.AtLeftTopIn(curGrid[index].playerName, armyBg, labelXPos.player, 6)
                
                curGrid[index].teamName = UIUtil.CreateText(armyBg, "", 16, UIUtil.bodyFont)
                LayoutHelpers.AtLeftTopIn(curGrid[index].teamName, armyBg, labelXPos.team, 6)
                
                curGrid[index].cols = {}
                for col = 1, 6 do
                    curGrid[index].cols[col] = UIUtil.CreateText(armyBg, "", 16, UIUtil.bodyFont)
                    LayoutHelpers.AtLeftTopIn(curGrid[index].cols[col], armyBg, labelXPos[col], 6)
                end
            end

            -- set up data type buttons
            local typeButtons = {}
            for index, dataType in tabData.types do
                typeButtons[index] = {}
                typeButtons[index].checkbox = Checkbox(gridGroup
                    , UIUtil.SkinnableFile('/dialogs/rect-box_btn/rect-box-d_btn_up.dds')
                    , UIUtil.SkinnableFile('/dialogs/rect-box_btn/rect-box-s_btn_up.dds')
                    , UIUtil.SkinnableFile('/dialogs/rect-box_btn/rect-box-d_btn_over.dds')
                    , UIUtil.SkinnableFile('/dialogs/rect-box_btn/rect-box-s_btn_over.dds')
                    , UIUtil.SkinnableFile('/dialogs/rect-box_btn/rect-box-d_btn_dis.dds')
                    , UIUtil.SkinnableFile('/dialogs/rect-box_btn/rect-box-s_btn_dis.dds'))
                LayoutHelpers.RelativeTo(typeButtons[index].checkbox, bg, 
                    UIUtil.SkinnableFile('/dialogs/score-victory-defeat/score-victory-defeat_layout.lua'), 
                    'l_btn0'..index, 'panel_bmp')
                typeButtons[index].checkbox.toolTip = dataType.scoreKey
    
                typeButtons[index].label = MultiLineText(typeButtons[index].checkbox, UIUtil.titleFont, 12, UIUtil.fontColor)
                LayoutHelpers.FillParentFixedBorder(typeButtons[index].label, typeButtons[index].checkbox, 15)
                typeButtons[index].label:SetText(LOC(dataType.title))
                typeButtons[index].label:DisableHitTest()
                
                typeButtons[index].checkbox._dataType = dataType.scoreKey
                
            end

            local typesGroup = RadioGroup(typeButtons)
            typesGroup.OnClick = function(self, index, item)
                SetDataType(item.checkbox._dataType)
            end
            typesGroup:SetCheckboxEventHandler(function(self, event) 
                if event.Type == 'MouseEnter' then
                    Tooltip.CreateMouseoverDisplay(self, "PostScore_"..self.toolTip, nil, true)
                elseif event.Type == 'MouseExit' then
                    Tooltip.DestroyMouseoverDisplay()
                end
            end)

            -- TODO default type? also when these become checkboxes, need to depress type
            -- maybe call SetDataType, but have an updateDisplay function so we don't call UpdateDisplay while creating
            curType = tabData.types[1].scoreKey
            
            -- now create histogram display
        elseif tabData.button == "campaign" then
            -- Set up campaign display
            local campaignBG = Bitmap(currentPage)
            campaignBG:SetSolidColor("black")
            LayoutHelpers.AtLeftTopIn(campaignBG, currentPage, 38, 79)
            campaignBG.Width:Set(756)
            campaignBG.Height:Set(286)

            local primaryObjectivesLabel = UIUtil.CreateText(campaignBG, "<LOC SCORE_0037>Primary Objectives", 14, UIUtil.titleFont)
            LayoutHelpers.AtLeftTopIn(primaryObjectivesLabel, campaignBG, 0, 5)
            local primaryObjectives = ItemList(campaignBG)
            primaryObjectives:SetFont(UIUtil.bodyFont, 14)
            primaryObjectives.Width:Set(320)
            primaryObjectives.Height:Set(70)
            LayoutHelpers.Below(primaryObjectives, primaryObjectivesLabel)
            UIUtil.CreateVertScrollbarFor(primaryObjectives)

            local secondaryObjectivesLabel = UIUtil.CreateText(campaignBG, "<LOC SCORE_0040>Secondary Objectives", 14, UIUtil.titleFont)
            LayoutHelpers.Below(secondaryObjectivesLabel, primaryObjectives, 5)
            local secondaryObjectives = ItemList(campaignBG)
            secondaryObjectives:SetFont(UIUtil.bodyFont, 14)
            secondaryObjectives.Width:Set(primaryObjectives.Width)
            secondaryObjectives.Height:Set(primaryObjectives.Height)
            LayoutHelpers.Below(secondaryObjectives, secondaryObjectivesLabel)
            UIUtil.CreateVertScrollbarFor(secondaryObjectives)

            local bonusObjectivesLabel = UIUtil.CreateText(campaignBG, "<LOC SCORE_0041>Bonus Objectives", 14, UIUtil.titleFont)
            LayoutHelpers.Below(bonusObjectivesLabel, secondaryObjectives, 5)
            local bonusObjectives = ItemList(campaignBG)
            bonusObjectives:SetFont(UIUtil.bodyFont, 14)
            bonusObjectives.Width:Set(primaryObjectives.Width)
            bonusObjectives.Height:Set(primaryObjectives.Height)
            LayoutHelpers.Below(bonusObjectives, bonusObjectivesLabel)
            local sb = UIUtil.CreateVertScrollbarFor(bonusObjectives)
            
            local infoGroup = Group(campaignBG)
            infoGroup.Left:Set(sb.Right)
            infoGroup.Top:Set(campaignBG.Top)
            infoGroup.Bottom:Set(campaignBG.Bottom)
            infoGroup.Right:Set(campaignBG.Right)

            local debriefLabel = UIUtil.CreateText(infoGroup, "<LOC SCORE_0044>Debriefing", 14, UIUtil.titleFont)
            LayoutHelpers.AtLeftTopIn(debriefLabel, infoGroup, 0, 15)
            
            local debriefText = UIUtil.CreateTextBox(infoGroup)
            debriefText.Top:Set(debriefLabel.Bottom)
            debriefText.Bottom:Set(infoGroup.Bottom)
            debriefText.Width:Set(function() return math.floor(infoGroup.Width() / 2) end)
            LayoutHelpers.Below(debriefText, debriefLabel)

            local obTable = import('/lua/modules/ui/game/objectives2.lua').GetCurrentObjectiveTable()
            if obTable then
                for key, objective in obTable do
                    local compStr
                    if objective.Status then
                        if objective.Status == 'complete' then
                            compStr = "<LOC SCORE_0038>Accomplished"
                        else
                            compStr = "<LOC SCORE_0039>Failed"
                        end
                    else
                        compStr = "<LOC SCORE_0054>Incomplete"
                    end
                    
                    local targetList
                    if objective.ObjData.type == 'primary' then
                        targetList = primaryObjectives
                    elseif objective.ObjData.type == 'secondary' then
                        targetList = secondaryObjectives
                    else
                        targetList = bonusObjectives
                    end

                    targetList:AddItem(LOC(objective.ObjData.title) .. " (" .. LOC(compStr) .. ")")
                end
            end
            
            local successKey 
            if operationVictoryTable.success then
                successKey = 'success'
            else
                successKey = 'failure'
            end
            
            local debriefString = import('/lua/modules/ui/campaign/campaigndebriefingtext.lua').campaignDebriefingText[operationVictoryTable.opKey][successKey]
            UIUtil.SetTextBoxText(debriefText, debriefString)

            -- show medal
            local medalBmps = import('/lua/modules/ui/campaign/campaignmanager.lua').GetMedalBitmaps(
                  operationVictoryTable.opKey
                , operationVictoryTable.difficulty
                , operationVictoryTable.allPrimary
                , operationVictoryTable.allSecondary
                , operationVictoryTable.allBonus)
            
            local medalLabel = UIUtil.CreateText(infoGroup, "<LOC SCORE_0057>Operation Medal", 14, UIUtil.titleFont)
            LayoutHelpers.RightOf(medalLabel, debriefText, 40)
            if medalBmps then
                local medalBottom = Bitmap(infoGroup)
                medalBottom:SetTexture(medalBmps.mission)
                LayoutHelpers.CenteredBelow(medalBottom, medalLabel)
                local medalMiddle = Bitmap(medalBottom)
                medalMiddle:SetTexture(medalBmps.difficulty)
                LayoutHelpers.AtCenterIn(medalMiddle, medalBottom)
                local medalTop = Bitmap(medalMiddle)
                medalTop:SetTexture(medalBmps.award)
                LayoutHelpers.AtCenterIn(medalTop, medalMiddle)
            end
            
            local opScoreLabel = UIUtil.CreateText(infoGroup, "<LOC SCORE_0043>Operation Score", 14, UIUtil.titleFont)
            LayoutHelpers.CenteredBelow(opScoreLabel, medalLabel, 40)
            
            local opScore = UIUtil.CreateText(infoGroup, tostring(curInfo.scoreData.current[GetArmiesTable().focusArmy].general.score), 14, UIUtil.titleFont)
            LayoutHelpers.CenteredBelow(opScore, opScoreLabel, 5)
        end

        local victoryString
        if operationVictoryTable then
            if operationVictoryTable.success then
                victoryString = "<LOC SCORE_0055>Operation Successful"
            else
                victoryString = "<LOC SCORE_0056>Operation Failed"
            end
        else
            if victory != nil then
                if victory then
                    victoryString = "<LOC SCORE_0046>Victory"
                else
                    victoryString = "<LOC SCORE_0047>Defeat"
                end
            else
                victoryString = "<LOC _Score>"
            end

        end

        -- sets the score dialog box title to the appropriate string
        title:SetText(LOC(victoryString))

        UpdateDisplay()
    end

    -- set up top level tabs
    local prev = false
    local defaultTab = false
    for index, value in tabs do
        -- don't show campaign button normally
        if value.button == "campaign" and not showCampaign then
            continue
        end
        
        local curButton = UIUtil.CreateButtonStd(bg, '/dialogs/tab_btn/general', value.title, 11)

        if prev then
            LayoutHelpers.RightOf(curButton, prev)
        else
            LayoutHelpers.RelativeTo(curButton, bg, UIUtil.SkinnableFile('/dialogs/score-victory-defeat/score-victory-defeat_layout.lua'), 'l_general_btn', 'panel_bmp')
            defaultTab = curButton
        end
        prev = curButton

        curButton.OnClick = function(self, modifiers)
            SetNewPage(self)
        end
        local tempKey = value.scoreKey
        curButton.HandleEvent = function(self, event)
            if event.Type == 'MouseEnter' then
                Tooltip.CreateMouseoverDisplay(self, "PostScore_" .. tempKey, nil, true)
            elseif event.Type == 'MouseExit' then
                Tooltip.DestroyMouseoverDisplay()
            end
            Button.HandleEvent(self, event)
        end
        
        curButton.tabData = value
    end

    UpdateData()
    SetNewPage(defaultTab)
end

