--*****************************************************************************
--* File: lua/modules/ui/game/construction.lua
--* Author: Chris Blackwell
--* Summary: Construction management UI
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Group = import('/lua/modules/maui/group.lua').Group
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local Grid = import('/lua/modules/maui/grid.lua').Grid
local Checkbox = import('/lua/modules/maui/checkbox.lua').Checkbox
local Button = import('/lua/modules/maui/button.lua').Button
local GameCommon = import('/lua/modules/ui/game/gamecommon.lua')
local RadioGroup = import('/lua/modules/maui/mauiutil.lua').RadioGroup
local Queue = import('/lua/modules/ui/game/queue.lua')
local Tooltip = import('/lua/modules/ui/game/tooltip.lua')

local unitGridPages = {
    RULEUTL_Basic = {Order = 0, Label = "<LOC CONSTRUCT_0000>T1"},
    RULEUTL_Advanced = {Order = 1, Label = "<LOC CONSTRUCT_0001>T2"},
    RULEUTL_Secret = {Order = 2, Label = "<LOC CONSTRUCT_0002>T3"},
    RULEUTL_Experimental = {Order = 3, Label = "<LOC CONSTRUCT_0003>Exp"},
    RULEUTL_Munition = {Order = 4, Label = "<LOC CONSTRUCT_0004>Munition"},   -- note that this doesn't exist yet
}

-- these are external controls used for positioning, so don't add them to our local control table
local controlClusterGroup = false
local mfdControl = false
local ordersControl = false
local queueControl = false
local techTabControlTable = {}
local filteredFactories = {}
local filteredTech1Units = {}
local filteredTech2Units = {}
local filteredTech3Units = {}
local filteredTech4Units = {}
local filteredEnhancements = {}
local filteredTechTable = {}
local techLevelSelected = filteredTechTable[1]
local tempBuildableCatagories = false
local currentSelection = false
local tech1Tab = false
local tech2Tab = false
local tech3Tab = false
local tech4Tab = false
local tech5Tab = false
local techEnhTab = false
local buildingEnhancements = false
local mouseoverDisplay = false
local enhancementBool = false
local layoutVar = false

local controls = {
    unitGrid = false,
    constructionGroup = false,
    scrollUnitsFwdButton = false,
    scrollUnitsBackButton = false,
    unitsBGCapMin = false,
    unitsBGMiddle = false,
    unitsBGCapMax = false,
    techRadioGroup = false,
    scrollBG = false,
    enhanceButton = false,
}


local whatIfBuilder = nil
local whatIfBlueprintID = nil
local selectedwhatIfBuilder = nil
local selectedwhatIfBlueprintID = nil

function GetTableSize(table)
    index = 0
    if table then
        for i, v in table do
            index = index + 1
        end
    end
    return index
end

function ClearWhatIfModifier()
    whatIfBuilder = nil
    whatIfBlueprintID = nil
    selectedwhatIfBuilder = nil
    selectedwhatIfBlueprintID = nil
end

function GetCurrentWhatIfModifier()
    local consumeEnergy = 0
    local consumeMass = 0
    local storeEnergy = 0
    local storeMass = 0

    if whatIfBuilder != nil and whatIfBlueprintID != nil then
        local targetBP = __blueprints[whatIfBlueprintID]
        local time, energy, mass = import('/lua/modules/game.lua').GetConstructEconomyModel(whatIfBuilder, targetBP.Economy)
        if time > 0 then
            consumeEnergy = -energy / time
            consumeMass = -mass / time
        end
        storeEnergy = targetBP.Economy.StorageEnergy or 0
        storeMass = targetBP.Economy.StorageMass or 0
    elseif selectedwhatIfBuilder != nil and selectedwhatIfBlueprintID != nil then
        local targetBP = __blueprints[selectedwhatIfBlueprintID]
        local time, energy, mass = import('/lua/modules/game.lua').GetConstructEconomyModel(selectedwhatIfBuilder, targetBP.Economy)
        if time > 0 then
            consumeEnergy = -energy / time
            consumeMass = -mass / time
        end
        storeEnergy = targetBP.Economy.StorageEnergy or 0
        storeMass = targetBP.Economy.StorageMass or 0
    end

    return { consumeEnergy,
             consumeMass,
             storeEnergy,
             storeMass }
end

local function CreateUnitsGrid(parent)
    controls.unitGrid = Grid(parent, 62, 62)
    controls.unitGrid:SetName("Unit Grid")
end

local function DoBottomLayout()
    ------------------------------------------------------------------------
    -- set up containing group
    if not controls.constructionGroup then
        controls.constructionGroup = Group(controlClusterGroup, "controlClusterGroup")
    end
    controls.constructionGroup.Top:Set(controlClusterGroup.Top)
    controls.constructionGroup.Bottom:Set(controlClusterGroup.Bottom)
    controls.constructionGroup.Left:Set(mfdControl.Right)
    controls.constructionGroup.Right:Set(ordersControl.Left)

    ------------------------------------------------------------------------
    -- set up unit select
    if not controls.unitsBGCapMin then
        controls.unitsBGCapMin = Bitmap(controls.constructionGroup)
    end
    controls.unitsBGCapMin:SetTexture(UIUtil.SkinnableFile('/game/units_bmp/unit-02_bmp_l.dds'))
    LayoutHelpers.AtTopIn(controls.unitsBGCapMin, controls.constructionGroup, 1)
    LayoutHelpers.AtLeftIn(controls.unitsBGCapMin, controls.constructionGroup, -2)
    LayoutHelpers.ResetBottom(controls.unitsBGCapMin)
    LayoutHelpers.ResetRight(controls.unitsBGCapMin)

    if not controls.unitsBGCapMax then
        controls.unitsBGCapMax = Bitmap(controls.constructionGroup)
    end
    controls.unitsBGCapMax:SetTexture(UIUtil.SkinnableFile('/game/units_bmp/unit-02_bmp_r.dds'))
    LayoutHelpers.AtTopIn(controls.unitsBGCapMax, controls.constructionGroup, 1)
    LayoutHelpers.AtRightIn(controls.unitsBGCapMax, controls.constructionGroup, 2)
    LayoutHelpers.ResetLeft(controls.unitsBGCapMax)
    LayoutHelpers.ResetBottom(controls.unitsBGCapMax)

    if not controls.unitsBGMiddle then
        controls.unitsBGMiddle = Bitmap(controls.constructionGroup)
    end
    controls.unitsBGMiddle:SetTexture(UIUtil.SkinnableFile('/game/units_bmp/unit-02_bmp_m.dds'))
    controls.unitsBGMiddle.Left:Set(controls.unitsBGCapMin.Right)
    controls.unitsBGMiddle.Right:Set(controls.unitsBGCapMax.Left)
    controls.unitsBGMiddle.Top:Set(controls.unitsBGCapMin.Top)
    controls.unitsBGMiddle:SetTiled(true)
    LayoutHelpers.ResetWidth(controls.unitsBGMiddle)
    LayoutHelpers.ResetBottom(controls.unitsBGMiddle)
    controls.unitsBGMiddle.Height:Set(controls.unitsBGMiddle.BitmapHeight)

    if not tech1Tab then
        tech1Tab = Group(controlClusterGroup, "tech1TabGroup")
    end
    tech1Tab.Top:Set(0)
    tech1Tab.Left:Set(0)
    tech1Tab.Right:Set(1)
    tech1Tab.Bottom:Set(1)
    if not tech2Tab then
        tech2Tab = Group(controlClusterGroup, "tech2TabGroup")
    end
    tech2Tab.Top:Set(0)
    tech2Tab.Left:Set(0)
    tech2Tab.Right:Set(1)
    tech2Tab.Bottom:Set(1)
    if not tech3Tab then
        tech3Tab = Group(controlClusterGroup, "tech3TabGroup")
    end
    tech3Tab.Top:Set(0)
    tech3Tab.Left:Set(0)
    tech3Tab.Right:Set(1)
    tech3Tab.Bottom:Set(1)
    if not tech4Tab then
        tech4Tab = Group(controlClusterGroup, "tech4TabGroup")
    end
    tech4Tab.Top:Set(0)
    tech4Tab.Left:Set(0)
    tech4Tab.Right:Set(1)
    tech4Tab.Bottom:Set(1)
    if not tech1Tab.checkbox then
        tech1Tab.checkbox = Checkbox(tech1Tab)
    end
    tech1Tab.checkbox:SetNewTextures(
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab_btn_up.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab_btn_over.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab_btn_down.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab_btn_down.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab_btn_dis.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab_btn_dis.dds'))
    LayoutHelpers.RelativeTo(tech1Tab.checkbox, controls.unitsBGCapMin,
        UIUtil.SkinnableFile('/game/units_bmp/units_bmp_layout.lua'),
        "l_techtab1", "unit_bmp_l")
    tech1Tab.checkbox.Depth:Set(function() return controls.unitsBGMiddle.Depth() + 100 end)
    tech1Tab.checkbox:Disable()

    if not tech1Tab.icon then
        tech1Tab.icon = Bitmap(tech1Tab, UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-level-1_icon_bmp.dds'))
    end
    LayoutHelpers.RelativeTo(tech1Tab.icon, controls.unitsBGCapMin,
        UIUtil.SkinnableFile('/game/units_bmp/units_bmp_layout.lua'),
        "l_techtab-icon1", "unit_bmp_l")
    tech1Tab.icon.Depth:Set(function() return controls.unitsBGMiddle.Depth() + 101 end)
    tech1Tab.icon:DisableHitTest()

    if not tech2Tab.checkbox then
        tech2Tab.checkbox = Checkbox(tech2Tab)
    end
    tech2Tab.checkbox:SetNewTextures(
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab_btn_up.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab_btn_over.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab_btn_down.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab_btn_down.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab_btn_dis.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab_btn_dis.dds'))
    LayoutHelpers.RelativeTo(tech2Tab.checkbox, controls.unitsBGCapMin,
    UIUtil.SkinnableFile('/game/units_bmp/units_bmp_layout.lua'),
    "l_techtab2", "unit_bmp_l")
    tech2Tab.checkbox.Depth:Set(function() return tech1Tab.checkbox.Depth() + 1 end)
    tech2Tab.checkbox:Disable()

    if not tech2Tab.icon then
        tech2Tab.icon = Bitmap(tech2Tab, UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-level-2_icon_bmp.dds'))
    end
    LayoutHelpers.RelativeTo(tech2Tab.icon, controls.unitsBGCapMin,
    UIUtil.SkinnableFile('/game/units_bmp/units_bmp_layout.lua'),
    "l_techtab-icon2", "unit_bmp_l")
    tech2Tab.icon.Depth:Set(function() return tech1Tab.checkbox.Depth() + 20 end)
    tech2Tab.icon:DisableHitTest()

    if not tech3Tab.checkbox then
        tech3Tab.checkbox = Checkbox(tech2Tab)
    end
    tech3Tab.checkbox:SetNewTextures(
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab_btn_up.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab_btn_over.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab_btn_down.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab_btn_down.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab_btn_dis.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab_btn_dis.dds'))
    LayoutHelpers.RelativeTo(tech3Tab.checkbox, controls.unitsBGCapMin,
        UIUtil.SkinnableFile('/game/units_bmp/units_bmp_layout.lua'),
        "l_techtab3", "unit_bmp_l")
    tech3Tab.checkbox.Depth:Set(function() return tech2Tab.checkbox.Depth() + 1 end)
    tech3Tab.checkbox:Disable()

    if not tech3Tab.icon then
        tech3Tab.icon = Bitmap(tech3Tab, UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-level-3_icon_bmp.dds'))
    end
    LayoutHelpers.RelativeTo(tech3Tab.icon, controls.unitsBGCapMin,
        UIUtil.SkinnableFile('/game/units_bmp/units_bmp_layout.lua'),
        "l_techtab-icon3", "unit_bmp_l")
    tech3Tab.icon.Depth:Set(function() return tech2Tab.checkbox.Depth() + 20 end)
    tech3Tab.icon:DisableHitTest()

    if not tech4Tab.checkbox then
        tech4Tab.checkbox = Checkbox(tech2Tab)
    end
    tech4Tab.checkbox:SetNewTextures(
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab_btn_up.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab_btn_over.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab_btn_down.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab_btn_down.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab_btn_dis.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab_btn_dis.dds'))
    LayoutHelpers.RelativeTo(tech4Tab.checkbox, controls.unitsBGCapMin,
        UIUtil.SkinnableFile('/game/units_bmp/units_bmp_layout.lua'),
        "l_techtab4", "unit_bmp_l")
    tech4Tab.checkbox.Depth:Set(function() return tech3Tab.checkbox.Depth() + 1 end)
    tech4Tab.checkbox:Disable()

    if not tech4Tab.icon then
        tech4Tab.icon = Bitmap(tech4Tab, UIUtil.SkinnableFile('/game/tech-tabs_btn/experimental_icon_bmp.dds'))
    end
    LayoutHelpers.RelativeTo(tech4Tab.icon, controls.unitsBGCapMin,
        UIUtil.SkinnableFile('/game/units_bmp/units_bmp_layout.lua'),
        "l_techtab-icon4", "unit_bmp_l")
    tech4Tab.icon.Depth:Set(function() return tech1Tab.checkbox.Depth() + 20 end)
    tech4Tab.icon:DisableHitTest()

    if not controls.unitGrid then
        CreateUnitsGrid(controls.unitsBGMiddle)
    end
    
    tech2disabledOverGroup = Group(controlClusterGroup)
    LayoutHelpers.FillParent(tech2disabledOverGroup, tech2Tab.checkbox) 
    tech2disabledOverGroup.Depth:Set(function() return tech2Tab.checkbox.Depth() + 5 end)
    tech2disabledOverGroup.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "tech_level_2_cons_tab_dis", nil, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
    end 
    
    tech3disabledOverGroup = Group(controlClusterGroup)
    LayoutHelpers.FillParent(tech3disabledOverGroup, tech3Tab.checkbox) 
    tech3disabledOverGroup.Depth:Set(function() return tech3Tab.checkbox.Depth() + 5 end)
    tech3disabledOverGroup.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "tech_level_3_cons_tab_dis", nil, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
    end 
    
    tech4disabledOverGroup = Group(controlClusterGroup)
    LayoutHelpers.FillParent(tech4disabledOverGroup, tech4Tab.checkbox) 
    tech4disabledOverGroup.Depth:Set(function() return tech4Tab.checkbox.Depth() + 5 end)
    tech4disabledOverGroup.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "tech_level_4_cons_tab_dis", nil, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
    end 
    
    --Create a blank label for the radio functionality to work
    if not tech1Tab.label then
        tech1Tab.label = UIUtil.CreateText(tech1Tab, "", 16, UIUtil.bodyFont)
    end
    tech1Tab.label.Top:Set(0)
    tech1Tab.label.Left:Set(0)
    tech1Tab.label.Right:Set(1)
    tech1Tab.label.Bottom:Set(1)
    if not tech2Tab.label then
        tech2Tab.label = UIUtil.CreateText(tech2Tab, "", 16, UIUtil.bodyFont)
    end
    tech2Tab.label.Top:Set(0)
    tech2Tab.label.Left:Set(0)
    tech2Tab.label.Right:Set(1)
    tech2Tab.label.Bottom:Set(1)
    if not tech3Tab.label then
        tech3Tab.label = UIUtil.CreateText(tech3Tab, "", 16, UIUtil.bodyFont)
    end
    tech3Tab.label.Top:Set(0)
    tech3Tab.label.Left:Set(0)
    tech3Tab.label.Right:Set(1)
    tech3Tab.label.Bottom:Set(1)
    if not tech4Tab.label then
        tech4Tab.label = UIUtil.CreateText(tech4Tab, "", 16, UIUtil.bodyFont)
    end
    tech4Tab.label.Top:Set(0)
    tech4Tab.label.Left:Set(0)
    tech4Tab.label.Right:Set(1)
    tech4Tab.label.Bottom:Set(1)

    techTabControlTable[1] = tech1Tab
    techTabControlTable[2] = tech2Tab
    techTabControlTable[3] = tech3Tab
    techTabControlTable[4] = tech4Tab

    if not controls.techRadioGroup then
        controls.techRadioGroup = RadioGroup(techTabControlTable, 1)
    end
    controls.techRadioGroup.OnChoose = function(self, index, item)
        techLevelSelected = filteredTechTable[index]
        buildingEnhancements = false
        PopulateGridButtons()
    end
    controls.techRadioGroup:SetCheckboxEventHandler(function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "tech_level_"..self._radioGroupIndex.."_cons_tab")
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
    end)

    if controls.scrollUnitsBackButton then controls.scrollUnitsBackButton:Destroy() end
    controls.scrollUnitsBackButton = UIUtil.CreateButton(controls.unitsBGCapMin
        , '/game/lg-vert_scr/arrow-up_scr_up.dds'
        , '/game/lg-vert_scr/arrow-up_scr_down.dds'
        , '/game/lg-vert_scr/arrow-up_scr_over.dds'
        , '/game/lg-vert_scr/arrow-up_scr_dis.dds')
    controls.scrollUnitsBackButton.Left:Set(function() return controls.unitsBGCapMin.Left() + 68 end)
    controls.scrollUnitsBackButton.Top:Set(function() return controls.unitsBGCapMin.Top() + 66 end)
    controls.scrollUnitsBackButton.Depth:Set(function() return controls.unitsBGCapMin.Depth() + 200 end)
    controls.scrollUnitsBackButton.OnClick = function(self)
        controls.unitGrid:ScrollLines("Vert", -1)
    end
    controls.scrollUnitsBackButton:Disable()

    if controls.scrollUnitsFwdButton then controls.scrollUnitsFwdButton:Destroy() end
    controls.scrollUnitsFwdButton = UIUtil.CreateButton(controls.unitsBGCapMin
        , '/game/lg-vert_scr/arrow-down_scr_up.dds'
        , '/game/lg-vert_scr/arrow-down_scr_down.dds'
        , '/game/lg-vert_scr/arrow-down_scr_over.dds'
        , '/game/lg-vert_scr/arrow-down_scr_dis.dds')
    LayoutHelpers.Below(controls.scrollUnitsFwdButton, controls.scrollUnitsBackButton, -5)
    controls.scrollUnitsFwdButton.Depth:Set(function() return controls.scrollUnitsBackButton.Depth() + 1 end)
    controls.scrollUnitsFwdButton.OnClick = function(self)
        controls.unitGrid:ScrollLines("Vert", 1)
    end
    controls.scrollUnitsFwdButton:Disable()

    if not controls.enhanceButton then
        controls.enhanceButton = Button(controls.unitsBGCapMin,
            UIUtil.UIFile('/dialogs/enhance-panel_btn/infinite_btn_up.dds'),
            UIUtil.UIFile('/dialogs/enhance-panel_btn/infinite_btn_down.dds'),
            UIUtil.UIFile('/dialogs/enhance-panel_btn/infinite_btn_over.dds'),
            UIUtil.UIFile('/dialogs/enhance-panel_btn/infinite_btn_dis.dds'))
    end
    controls.enhanceButton.Bottom:Set(function() return controls.scrollUnitsBackButton.Top() + 0 end)
    LayoutHelpers.AtHorizontalCenterIn(controls.enhanceButton, controls.scrollUnitsBackButton)
    LayoutHelpers.ResetTop(controls.enhanceButton)
    controls.enhanceButton.OnClick = function(self, modifiers)
        import('/lua/modules/ui/game/enhancements.lua').ToggleDialog(filteredEnhancements, currentSelection[1])
    end
    controls.enhanceButton.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "enhance")
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
        Button.HandleEvent(self, event)
    end

    controls.unitGrid.Left:Set(function() return controls.unitsBGMiddle.Left() + 20 end)
    controls.unitGrid.Right:Set(function() return controls.unitsBGCapMax.Right() - 10 end)
    controls.unitGrid.Height:Set(function() return 64 * 2 end)
    LayoutHelpers.AtVerticalCenterIn(controls.unitGrid, controls.unitsBGMiddle, -5)
    LayoutHelpers.ResetWidth(controls.unitGrid)
end

local function DoLeftLayout()
    ------------------------------------------------------------------------
    -- set up containing group
    if not controls.constructionGroup then
        controls.constructionGroup = Group(controlClusterGroup, "constructionGroup")
    end
    controls.constructionGroup.Top:Set(ordersControl.Bottom)
    controls.constructionGroup.Bottom:Set(function() return controlClusterGroup.Bottom() - 1 end)
    controls.constructionGroup.Left:Set(function() return controlClusterGroup.Left() + 5 end)
    controls.constructionGroup.Right:Set(controlClusterGroup.Right)

    ------------------------------------------------------------------------
    -- set up unit select
    if not controls.unitsBGCapMin then
        controls.unitsBGCapMin = Bitmap(controls.constructionGroup)
    end
    controls.unitsBGCapMin:SetTexture(UIUtil.SkinnableFile('/game/units-vertical_bmp/unit-02_bmp_t.dds'))
    LayoutHelpers.AtTopIn(controls.unitsBGCapMin, controls.constructionGroup)
    LayoutHelpers.AtLeftIn(controls.unitsBGCapMin, controls.constructionGroup)
    LayoutHelpers.ResetRight(controls.unitsBGCapMin)
    LayoutHelpers.ResetBottom(controls.unitsBGCapMin)

    if not controls.unitsBGCapMax then
        controls.unitsBGCapMax = Bitmap(controls.constructionGroup)
    end
    controls.unitsBGCapMax:SetTexture(UIUtil.SkinnableFile('/game/units-vertical_bmp/unit-02_bmp_b.dds'))
    LayoutHelpers.AtLeftIn(controls.unitsBGCapMax, controls.constructionGroup)
    controls.unitsBGCapMax.Bottom:Set(function() return queueControl.Top() - 25 end)
    LayoutHelpers.ResetTop(controls.unitsBGCapMax)
    LayoutHelpers.ResetRight(controls.unitsBGCapMax)

    if not controls.unitsBGMiddle then
        controls.unitsBGMiddle = Bitmap(controls.constructionGroup)
    end
    controls.unitsBGMiddle:SetTexture(UIUtil.SkinnableFile('/game/units-vertical_bmp/unit-02_bmp_m.dds'))
    LayoutHelpers.AtLeftIn(controls.unitsBGMiddle, controls.constructionGroup)
    controls.unitsBGMiddle.Top:Set(controls.unitsBGCapMin.Bottom)
    controls.unitsBGMiddle.Bottom:Set(controls.unitsBGCapMax.Top)
    LayoutHelpers.ResetHeight(controls.unitsBGMiddle)
    LayoutHelpers.ResetRight(controls.unitsBGMiddle)
    controls.unitsBGMiddle.Width:Set(controls.unitsBGMiddle.BitmapWidth)
    controls.unitsBGMiddle:SetTiled(true)

    if not controls.unitGrid then
        CreateUnitsGrid(controls.unitsBGMiddle)
    end

    LayoutHelpers.AtLeftIn(controls.unitGrid, controls.unitsBGMiddle, 37)
    LayoutHelpers.AtRightIn(controls.unitGrid, controls.unitsBGMiddle, 0)
    controls.unitGrid.Top:Set(function() return controls.unitsBGMiddle.Top() - 18 end)
    controls.unitGrid.Bottom:Set(function() return controls.unitsBGMiddle.Bottom() + 3 end)
    LayoutHelpers.ResetWidth(controls.unitGrid)
    LayoutHelpers.ResetHeight(controls.unitGrid)

    if not tech1Tab then
        tech1Tab = Group(controlClusterGroup, "tech1TabGroup")
    end
    tech1Tab.Top:Set(0)
    tech1Tab.Left:Set(0)
    tech1Tab.Right:Set(1)
    tech1Tab.Bottom:Set(1)
    if not tech2Tab then
        tech2Tab = Group(controlClusterGroup, "tech2TabGroup")
    end
    tech2Tab.Top:Set(0)
    tech2Tab.Left:Set(0)
    tech2Tab.Right:Set(1)
    tech2Tab.Bottom:Set(1)
    if not tech3Tab then
        tech3Tab = Group(controlClusterGroup, "tech3TabGroup")
    end
    tech3Tab.Top:Set(0)
    tech3Tab.Left:Set(0)
    tech3Tab.Right:Set(1)
    tech3Tab.Bottom:Set(1)
    if not tech4Tab then
        tech4Tab = Group(controlClusterGroup, "tech4TabGroup")
    end
    tech4Tab.Top:Set(0)
    tech4Tab.Left:Set(0)
    tech4Tab.Right:Set(1)
    tech4Tab.Bottom:Set(1)

    if not tech1Tab.checkbox then
        tech1Tab.checkbox = Checkbox(tech2Tab)
    end
    tech1Tab.checkbox:SetNewTextures(
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_up.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_over.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_down.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_down.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_dis.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_dis.dds'))
    LayoutHelpers.RelativeTo(tech1Tab.checkbox, controls.unitsBGCapMin,
        UIUtil.SkinnableFile('/game/units-vertical_bmp/units-vertical_bmp_layout.lua'),
        "l_techtab1", "unit_bmp_m")
    tech1Tab.checkbox.Depth:Set(function() return controls.unitsBGMiddle.Depth() + 100 end)
    tech1Tab.checkbox:Disable()

    if not tech1Tab.icon then
        tech1Tab.icon = Bitmap(tech1Tab, UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-level-1_icon_bmp.dds'))
    end
    LayoutHelpers.RelativeTo(tech1Tab.icon, controls.unitsBGCapMin,
        UIUtil.SkinnableFile('/game/units-vertical_bmp/units-vertical_bmp_layout.lua'),
        "l_techtab-icon1", "unit_bmp_m")
    tech1Tab.icon.Depth:Set(function() return controls.unitsBGMiddle.Depth() + 101 end)
    tech1Tab.icon:DisableHitTest()

    if not tech2Tab.checkbox then
        tech2Tab.checkbox = Checkbox(tech2Tab)
    end
    tech2Tab.checkbox:SetNewTextures(
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_up.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_over.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_down.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_down.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_dis.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_dis.dds'))
    LayoutHelpers.RelativeTo(tech2Tab.checkbox, controls.unitsBGCapMin,
        UIUtil.SkinnableFile('/game/units-vertical_bmp/units-vertical_bmp_layout.lua'),
        "l_techtab2", "unit_bmp_m")
    tech2Tab.checkbox.Depth:Set(function() return tech1Tab.checkbox.Depth() + 1 end)
    tech2Tab.checkbox:Disable()

    if not tech2Tab.icon then
        tech2Tab.icon = Bitmap(tech2Tab, UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-level-2_icon_bmp.dds'))
    end
    LayoutHelpers.RelativeTo(tech2Tab.icon, controls.unitsBGCapMin,
        UIUtil.SkinnableFile('/game/units-vertical_bmp/units-vertical_bmp_layout.lua'),
        "l_techtab-icon2", "unit_bmp_m")
    tech2Tab.icon.Depth:Set(function() return tech1Tab.checkbox.Depth() + 20 end)
    tech2Tab.icon:DisableHitTest()

    if not tech3Tab.checkbox then
        tech3Tab.checkbox = Checkbox(tech2Tab)
    end
    tech3Tab.checkbox:SetNewTextures(
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_up.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_over.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_down.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_down.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_dis.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_dis.dds'))
    LayoutHelpers.RelativeTo(tech3Tab.checkbox, controls.unitsBGCapMin,
        UIUtil.SkinnableFile('/game/units-vertical_bmp/units-vertical_bmp_layout.lua'),
        "l_techtab3", "unit_bmp_m")
    tech3Tab.checkbox.Depth:Set(function() return tech2Tab.checkbox.Depth() + 1 end)
    tech3Tab.checkbox:Disable()

    if not tech3Tab.icon then
        tech3Tab.icon = Bitmap(tech3Tab, UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-level-3_icon_bmp.dds'))
    end
    LayoutHelpers.RelativeTo(tech3Tab.icon, controls.unitsBGCapMin,
        UIUtil.SkinnableFile('/game/units-vertical_bmp/units-vertical_bmp_layout.lua'),
        "l_techtab-icon3", "unit_bmp_m")
    tech3Tab.icon.Depth:Set(function() return tech2Tab.checkbox.Depth() + 20 end)
    tech3Tab.icon:DisableHitTest()

    if not tech4Tab.checkbox then
        tech4Tab.checkbox = Checkbox(tech2Tab)
    end
    tech4Tab.checkbox:SetNewTextures(
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_up.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_over.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_down.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_down.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_dis.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_dis.dds'))
    LayoutHelpers.RelativeTo(tech4Tab.checkbox, controls.unitsBGCapMin,
        UIUtil.SkinnableFile('/game/units-vertical_bmp/units-vertical_bmp_layout.lua'),
        "l_techtab4", "unit_bmp_m")
    tech4Tab.checkbox.Depth:Set(function() return tech3Tab.checkbox.Depth() + 1 end)
    tech4Tab.checkbox:Disable()

    if not tech4Tab.icon then
        tech4Tab.icon = Bitmap(tech4Tab, UIUtil.SkinnableFile('/game/tech-tabs_btn/experimental_icon_bmp.dds'))
    end
    LayoutHelpers.RelativeTo(tech4Tab.icon, controls.unitsBGCapMin,
        UIUtil.SkinnableFile('/game/units-vertical_bmp/units-vertical_bmp_layout.lua'),
        "l_techtab-icon4", "unit_bmp_m")
    tech4Tab.icon.Depth:Set(function() return tech1Tab.checkbox.Depth() + 20 end)
    tech4Tab.icon:DisableHitTest()

    if not controls.unitGrid then
        CreateUnitsGrid(controls.unitsBGMiddle)
    end
    
    tech2disabledOverGroup = Group(controlClusterGroup)
    LayoutHelpers.FillParent(tech2disabledOverGroup, tech2Tab.checkbox) 
    tech2disabledOverGroup.Depth:Set(function() return tech2Tab.checkbox.Depth() + 5 end)
    tech2disabledOverGroup.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "tech_level_2_cons_tab_dis", nil, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
    end 
    
    tech3disabledOverGroup = Group(controlClusterGroup)
    LayoutHelpers.FillParent(tech3disabledOverGroup, tech3Tab.checkbox) 
    tech3disabledOverGroup.Depth:Set(function() return tech3Tab.checkbox.Depth() + 5 end)
    tech3disabledOverGroup.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "tech_level_3_cons_tab_dis", nil, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
    end 
    
    tech4disabledOverGroup = Group(controlClusterGroup)
    LayoutHelpers.FillParent(tech4disabledOverGroup, tech4Tab.checkbox) 
    tech4disabledOverGroup.Depth:Set(function() return tech4Tab.checkbox.Depth() + 5 end)
    tech4disabledOverGroup.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "tech_level_4_cons_tab_dis", nil, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
    end 
    
    --Create a blank label for the radio functionality to work
    if not tech1Tab.label then
        tech1Tab.label = UIUtil.CreateText(tech1Tab, "", 16, UIUtil.bodyFont)
    end
    tech1Tab.label.Top:Set(0)
    tech1Tab.label.Left:Set(0)
    tech1Tab.label.Right:Set(1)
    tech1Tab.label.Bottom:Set(1)
    if not tech2Tab.label then
        tech2Tab.label = UIUtil.CreateText(tech2Tab, "", 16, UIUtil.bodyFont)
    end
    tech2Tab.label.Top:Set(0)
    tech2Tab.label.Left:Set(0)
    tech2Tab.label.Right:Set(1)
    tech2Tab.label.Bottom:Set(1)
    if not tech3Tab.label then
        tech3Tab.label = UIUtil.CreateText(tech3Tab, "", 16, UIUtil.bodyFont)
    end
    tech3Tab.label.Top:Set(0)
    tech3Tab.label.Left:Set(0)
    tech3Tab.label.Right:Set(1)
    tech3Tab.label.Bottom:Set(1)
    if not tech4Tab.label then
        tech4Tab.label = UIUtil.CreateText(tech4Tab, "", 16, UIUtil.bodyFont)
    end
    tech4Tab.label.Top:Set(0)
    tech4Tab.label.Left:Set(0)
    tech4Tab.label.Right:Set(1)
    tech4Tab.label.Bottom:Set(1)

    techTabControlTable[1] = tech1Tab
    techTabControlTable[2] = tech2Tab
    techTabControlTable[3] = tech3Tab
    techTabControlTable[4] = tech4Tab

    if not controls.techRadioGroup then
        controls.techRadioGroup = RadioGroup(techTabControlTable, 1)
    end
    controls.techRadioGroup.OnChoose = function(self, index, item)
        techLevelSelected = filteredTechTable[index]
        buildingEnhancements = false
        PopulateGridButtons()
    end
    controls.techRadioGroup:SetCheckboxEventHandler(function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "tech_level_"..self._radioGroupIndex.."_cons_tab")
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
    end)

    if controls.scrollUnitsBackButton then controls.scrollUnitsBackButton:Destroy() end
    controls.scrollUnitsBackButton = UIUtil.CreateButton(controls.unitsBGCapMin
        , '/game/lg-vert_scr/arrow-up_scr_up.dds'
        , '/game/lg-vert_scr/arrow-up_scr_down.dds'
        , '/game/lg-vert_scr/arrow-up_scr_over.dds'
        , '/game/lg-vert_scr/arrow-up_scr_dis.dds')
    controls.scrollUnitsBackButton.Left:Set(function() return controls.unitsBGCapMin.Left() + 5 end)
    controls.scrollUnitsBackButton.Top:Set(function() return controls.unitsBGCapMin.Top() + 110 end)
    controls.scrollUnitsBackButton.Depth:Set(function() return controls.unitsBGCapMin.Depth() + 200 end)
    controls.scrollUnitsBackButton.OnClick = function(self)
        controls.unitGrid:ScrollLines("Vert", -1)
    end
    controls.scrollUnitsBackButton:Disable()

    if controls.scrollUnitsFwdButton then controls.scrollUnitsFwdButton:Destroy() end
    controls.scrollUnitsFwdButton = UIUtil.CreateButton(controls.unitsBGCapMin
        , '/game/lg-vert_scr/arrow-down_scr_up.dds'
        , '/game/lg-vert_scr/arrow-down_scr_down.dds'
        , '/game/lg-vert_scr/arrow-down_scr_over.dds'
        , '/game/lg-vert_scr/arrow-down_scr_dis.dds')
    LayoutHelpers.Below(controls.scrollUnitsFwdButton, controls.scrollUnitsBackButton, -5)
    controls.scrollUnitsFwdButton.Depth:Set(function() return controls.scrollUnitsBackButton.Depth() + 1 end)
    controls.scrollUnitsFwdButton.OnClick = function(self)
        controls.unitGrid:ScrollLines("Vert", 1)
    end
    controls.scrollUnitsFwdButton:Disable()

    if not controls.enhanceButton then
        controls.enhanceButton = Button(controls.unitsBGCapMin,
            UIUtil.UIFile('/dialogs/enhance-panel_btn/infinite_btn_up.dds'),
            UIUtil.UIFile('/dialogs/enhance-panel_btn/infinite_btn_down.dds'),
            UIUtil.UIFile('/dialogs/enhance-panel_btn/infinite_btn_over.dds'),
            UIUtil.UIFile('/dialogs/enhance-panel_btn/infinite_btn_dis.dds'))
    end
    controls.enhanceButton.Top:Set(controls.scrollUnitsFwdButton.Bottom)
    controls.enhanceButton.Left:Set(function() return controls.scrollUnitsFwdButton.Left() + 0 end)
    controls.enhanceButton.Height:Set(64)
    controls.enhanceButton.Width:Set(36)
    LayoutHelpers.ResetBottom(controls.enhanceButton)
    controls.enhanceButton.OnClick = function(self, modifiers)
        import('/lua/modules/ui/game/enhancements.lua').ToggleDialog(filteredEnhancements, currentSelection[1])
    end
    controls.enhanceButton.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "enhance")
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
        Button.HandleEvent(self, event)
    end

end


local function DoRightLayout()
    ------------------------------------------------------------------------
    -- set up containing group
    if not controls.constructionGroup then
        controls.constructionGroup = Group(controlClusterGroup, "controlClusterGroup")
    end
    controls.constructionGroup.Top:Set(ordersControl.Bottom)
    controls.constructionGroup.Bottom:Set(function() return controlClusterGroup.Bottom() - 1 end)
    controls.constructionGroup.Left:Set(function() return controlClusterGroup.Left() + 5 end)
    controls.constructionGroup.Right:Set(controlClusterGroup.Right)

    ------------------------------------------------------------------------
    -- set up unit select
    if not controls.unitsBGCapMin then
        controls.unitsBGCapMin = Bitmap(controls.constructionGroup)
    end
    controls.unitsBGCapMin:SetTexture(UIUtil.SkinnableFile('/game/units-vertical_bmp/unit-02_bmp_t.dds'))
    LayoutHelpers.AtTopIn(controls.unitsBGCapMin, controls.constructionGroup)
    LayoutHelpers.AtLeftIn(controls.unitsBGCapMin, controls.constructionGroup)
    LayoutHelpers.ResetRight(controls.unitsBGCapMin)
    LayoutHelpers.ResetBottom(controls.unitsBGCapMin)

    if not controls.unitsBGCapMax then
        controls.unitsBGCapMax = Bitmap(controls.constructionGroup)
    end
    controls.unitsBGCapMax:SetTexture(UIUtil.SkinnableFile('/game/units-vertical_bmp/unit-02_bmp_b.dds'))
    LayoutHelpers.AtLeftIn(controls.unitsBGCapMax, controls.constructionGroup)
    controls.unitsBGCapMax.Bottom:Set(function() return queueControl.Top() - 25 end)
    LayoutHelpers.ResetTop(controls.unitsBGCapMax)
    LayoutHelpers.ResetRight(controls.unitsBGCapMax)

    if not controls.unitsBGMiddle then
        controls.unitsBGMiddle = Bitmap(controls.constructionGroup)
    end
    controls.unitsBGMiddle:SetTexture(UIUtil.SkinnableFile('/game/units-vertical_bmp/unit-02_bmp_m.dds'))
    LayoutHelpers.AtLeftIn(controls.unitsBGMiddle, controls.constructionGroup)
    controls.unitsBGMiddle.Top:Set(controls.unitsBGCapMin.Bottom)
    controls.unitsBGMiddle.Bottom:Set(controls.unitsBGCapMax.Top)
    LayoutHelpers.ResetHeight(controls.unitsBGMiddle)
    LayoutHelpers.ResetRight(controls.unitsBGMiddle)
    controls.unitsBGMiddle.Width:Set(controls.unitsBGMiddle.BitmapWidth)
    controls.unitsBGMiddle:SetTiled(true)

    if not controls.unitGrid then
        CreateUnitsGrid(controls.unitsBGMiddle)
    end

    LayoutHelpers.AtLeftIn(controls.unitGrid, controls.unitsBGMiddle, 37)
    LayoutHelpers.AtRightIn(controls.unitGrid, controls.unitsBGMiddle, 0)
    controls.unitGrid.Top:Set(function() return controls.unitsBGMiddle.Top() - 18 end)
    controls.unitGrid.Bottom:Set(function() return controls.unitsBGMiddle.Bottom() + 3 end)
    LayoutHelpers.ResetWidth(controls.unitGrid)
    LayoutHelpers.ResetHeight(controls.unitGrid)

    if not tech1Tab then
        tech1Tab = Group(controlClusterGroup, "tech1TabGroup")
    end
    tech1Tab.Top:Set(0)
    tech1Tab.Left:Set(0)
    tech1Tab.Right:Set(1)
    tech1Tab.Bottom:Set(1)
    if not tech2Tab then
        tech2Tab = Group(controlClusterGroup, "tech2TabGroup")
    end
    tech2Tab.Top:Set(0)
    tech2Tab.Left:Set(0)
    tech2Tab.Right:Set(1)
    tech2Tab.Bottom:Set(1)
    if not tech3Tab then
        tech3Tab = Group(controlClusterGroup, "tech3TabGroup")
    end
    tech3Tab.Top:Set(0)
    tech3Tab.Left:Set(0)
    tech3Tab.Right:Set(1)
    tech3Tab.Bottom:Set(1)
    if not tech4Tab then
        tech4Tab = Group(controlClusterGroup, "tech4TabGroup")
    end
    tech4Tab.Top:Set(0)
    tech4Tab.Left:Set(0)
    tech4Tab.Right:Set(1)
    tech4Tab.Bottom:Set(1)

    if not tech1Tab.checkbox then
        tech1Tab.checkbox = Checkbox(tech2Tab)
    end
    tech1Tab.checkbox:SetNewTextures(
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_up.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_over.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_down.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_down.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_dis.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_dis.dds'))
    LayoutHelpers.RelativeTo(tech1Tab.checkbox, controls.unitsBGCapMin,
        UIUtil.SkinnableFile('/game/units-vertical_bmp/units-vertical_bmp_layout.lua'),
        "l_techtab1", "unit_bmp_m")
    tech1Tab.checkbox.Depth:Set(function() return controls.unitsBGMiddle.Depth() + 100 end)
    tech1Tab.checkbox:Disable()

    if not tech1Tab.icon then
        tech1Tab.icon = Bitmap(tech1Tab, UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-level-1_icon_bmp.dds'))
    end
    LayoutHelpers.RelativeTo(tech1Tab.icon, controls.unitsBGCapMin,
        UIUtil.SkinnableFile('/game/units-vertical_bmp/units-vertical_bmp_layout.lua'),
        "l_techtab-icon1", "unit_bmp_m")
    tech1Tab.icon.Depth:Set(function() return controls.unitsBGMiddle.Depth() + 101 end)
    tech1Tab.icon:DisableHitTest()

    if not tech2Tab.checkbox then
        tech2Tab.checkbox = Checkbox(tech2Tab)
    end
    tech2Tab.checkbox:SetNewTextures(
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_up.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_over.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_down.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_down.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_dis.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_dis.dds'))
    LayoutHelpers.RelativeTo(tech2Tab.checkbox, controls.unitsBGCapMin,
        UIUtil.SkinnableFile('/game/units-vertical_bmp/units-vertical_bmp_layout.lua'),
        "l_techtab2", "unit_bmp_m")
    tech2Tab.checkbox.Depth:Set(function() return tech1Tab.checkbox.Depth() + 1 end)
    tech2Tab.checkbox:Disable()

    if not tech2Tab.icon then
        tech2Tab.icon = Bitmap(tech2Tab, UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-level-2_icon_bmp.dds'))
    end
    LayoutHelpers.RelativeTo(tech2Tab.icon, controls.unitsBGCapMin,
        UIUtil.SkinnableFile('/game/units-vertical_bmp/units-vertical_bmp_layout.lua'),
        "l_techtab-icon2", "unit_bmp_m")
    tech2Tab.icon.Depth:Set(function() return tech1Tab.checkbox.Depth() + 20 end)
    tech2Tab.icon:DisableHitTest()

    if not tech3Tab.checkbox then
        tech3Tab.checkbox = Checkbox(tech2Tab)
    end
    tech3Tab.checkbox:SetNewTextures(
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_up.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_over.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_down.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_down.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_dis.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_dis.dds'))
    LayoutHelpers.RelativeTo(tech3Tab.checkbox, controls.unitsBGCapMin,
        UIUtil.SkinnableFile('/game/units-vertical_bmp/units-vertical_bmp_layout.lua'),
        "l_techtab3", "unit_bmp_m")
    tech3Tab.checkbox.Depth:Set(function() return tech2Tab.checkbox.Depth() + 1 end)
    tech3Tab.checkbox:Disable()

    if not tech3Tab.icon then
        tech3Tab.icon = Bitmap(tech3Tab, UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-level-3_icon_bmp.dds'))
    end
    LayoutHelpers.RelativeTo(tech3Tab.icon, controls.unitsBGCapMin,
        UIUtil.SkinnableFile('/game/units-vertical_bmp/units-vertical_bmp_layout.lua'),
        "l_techtab-icon3", "unit_bmp_m")
    tech3Tab.icon.Depth:Set(function() return tech2Tab.checkbox.Depth() + 20 end)
    tech3Tab.icon:DisableHitTest()

    if not tech4Tab.checkbox then
        tech4Tab.checkbox = Checkbox(tech2Tab)
    end
    tech4Tab.checkbox:SetNewTextures(
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_up.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_over.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_down.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_down.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_dis.dds'),
            UIUtil.SkinnableFile('/game/tech-tabs_btn/tech-tab02_btn_dis.dds'))
    LayoutHelpers.RelativeTo(tech4Tab.checkbox, controls.unitsBGCapMin,
        UIUtil.SkinnableFile('/game/units-vertical_bmp/units-vertical_bmp_layout.lua'),
        "l_techtab4", "unit_bmp_m")
    tech4Tab.checkbox.Depth:Set(function() return tech3Tab.checkbox.Depth() + 1 end)
    tech4Tab.checkbox:Disable()

    if not tech4Tab.icon then
        tech4Tab.icon = Bitmap(tech4Tab, UIUtil.SkinnableFile('/game/tech-tabs_btn/experimental_icon_bmp.dds'))
    end
    LayoutHelpers.RelativeTo(tech4Tab.icon, controls.unitsBGCapMin,
        UIUtil.SkinnableFile('/game/units-vertical_bmp/units-vertical_bmp_layout.lua'),
        "l_techtab-icon4", "unit_bmp_m")
    tech4Tab.icon.Depth:Set(function() return tech1Tab.checkbox.Depth() + 20 end)
    tech4Tab.icon:DisableHitTest()

    if not controls.unitGrid then
        CreateUnitsGrid(controls.unitsBGMiddle)
    end
    
    tech2disabledOverGroup = Group(controlClusterGroup)
    LayoutHelpers.FillParent(tech2disabledOverGroup, tech2Tab.checkbox) 
    tech2disabledOverGroup.Depth:Set(function() return tech2Tab.checkbox.Depth() + 5 end)
    tech2disabledOverGroup.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "tech_level_2_cons_tab_dis", nil, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
    end 
    
    tech3disabledOverGroup = Group(controlClusterGroup)
    LayoutHelpers.FillParent(tech3disabledOverGroup, tech3Tab.checkbox) 
    tech3disabledOverGroup.Depth:Set(function() return tech3Tab.checkbox.Depth() + 5 end)
    tech3disabledOverGroup.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "tech_level_3_cons_tab_dis", nil, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
    end 
    
    tech4disabledOverGroup = Group(controlClusterGroup)
    LayoutHelpers.FillParent(tech4disabledOverGroup, tech4Tab.checkbox) 
    tech4disabledOverGroup.Depth:Set(function() return tech4Tab.checkbox.Depth() + 5 end)
    tech4disabledOverGroup.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "tech_level_4_cons_tab_dis", nil, true)
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
    end 
    
    --Create a blank label for the radio functionality to work
    if not tech1Tab.label then
        tech1Tab.label = UIUtil.CreateText(tech1Tab, "", 16, UIUtil.bodyFont)
    end
    tech1Tab.label.Top:Set(0)
    tech1Tab.label.Left:Set(0)
    tech1Tab.label.Right:Set(1)
    tech1Tab.label.Bottom:Set(1)
    if not tech2Tab.label then
        tech2Tab.label = UIUtil.CreateText(tech2Tab, "", 16, UIUtil.bodyFont)
    end
    tech2Tab.label.Top:Set(0)
    tech2Tab.label.Left:Set(0)
    tech2Tab.label.Right:Set(1)
    tech2Tab.label.Bottom:Set(1)
    if not tech3Tab.label then
        tech3Tab.label = UIUtil.CreateText(tech3Tab, "", 16, UIUtil.bodyFont)
    end
    tech3Tab.label.Top:Set(0)
    tech3Tab.label.Left:Set(0)
    tech3Tab.label.Right:Set(1)
    tech3Tab.label.Bottom:Set(1)
    if not tech4Tab.label then
        tech4Tab.label = UIUtil.CreateText(tech4Tab, "", 16, UIUtil.bodyFont)
    end
    tech4Tab.label.Top:Set(0)
    tech4Tab.label.Left:Set(0)
    tech4Tab.label.Right:Set(1)
    tech4Tab.label.Bottom:Set(1)

    techTabControlTable[1] = tech1Tab
    techTabControlTable[2] = tech2Tab
    techTabControlTable[3] = tech3Tab
    techTabControlTable[4] = tech4Tab

    if not controls.techRadioGroup then
        controls.techRadioGroup = RadioGroup(techTabControlTable, 1)
    end
    controls.techRadioGroup.OnChoose = function(self, index, item)
        techLevelSelected = filteredTechTable[index]
        buildingEnhancements = false
        PopulateGridButtons()
    end
    controls.techRadioGroup:SetCheckboxEventHandler(function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "tech_level_"..self._radioGroupIndex.."_cons_tab")
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
    end)


    if controls.scrollUnitsBackButton then controls.scrollUnitsBackButton:Destroy() end
    controls.scrollUnitsBackButton = UIUtil.CreateButton(controls.unitsBGCapMin
        , '/game/lg-vert_scr/arrow-up_scr_up.dds'
        , '/game/lg-vert_scr/arrow-up_scr_down.dds'
        , '/game/lg-vert_scr/arrow-up_scr_over.dds'
        , '/game/lg-vert_scr/arrow-up_scr_dis.dds')
    controls.scrollUnitsBackButton.Left:Set(function() return controls.unitsBGCapMin.Left() + 5 end)
    controls.scrollUnitsBackButton.Top:Set(function() return controls.unitsBGCapMin.Top() + 110 end)
    controls.scrollUnitsBackButton.Depth:Set(function() return controls.unitsBGCapMin.Depth() + 200 end)
    controls.scrollUnitsBackButton.OnClick = function(self)
        controls.unitGrid:ScrollLines("Vert", -1)
    end
    controls.scrollUnitsBackButton:Disable()

    if controls.scrollUnitsFwdButton then controls.scrollUnitsFwdButton:Destroy() end
    controls.scrollUnitsFwdButton = UIUtil.CreateButton(controls.unitsBGCapMin
        , '/game/lg-vert_scr/arrow-down_scr_up.dds'
        , '/game/lg-vert_scr/arrow-down_scr_down.dds'
        , '/game/lg-vert_scr/arrow-down_scr_over.dds'
        , '/game/lg-vert_scr/arrow-down_scr_dis.dds')
    LayoutHelpers.Below(controls.scrollUnitsFwdButton, controls.scrollUnitsBackButton, -5)
    controls.scrollUnitsFwdButton.Depth:Set(function() return controls.scrollUnitsBackButton.Depth() + 1 end)
    controls.scrollUnitsFwdButton.OnClick = function(self)
        controls.unitGrid:ScrollLines("Vert", 1)
    end
    controls.scrollUnitsFwdButton:Disable()

    if not controls.enhanceButton then
        controls.enhanceButton = Button(controls.unitsBGCapMin,
            UIUtil.UIFile('/dialogs/enhance-panel_btn/infinite_btn_up.dds'),
            UIUtil.UIFile('/dialogs/enhance-panel_btn/infinite_btn_down.dds'),
            UIUtil.UIFile('/dialogs/enhance-panel_btn/infinite_btn_over.dds'),
            UIUtil.UIFile('/dialogs/enhance-panel_btn/infinite_btn_dis.dds'))
    end
    controls.enhanceButton.Top:Set(controls.scrollUnitsFwdButton.Bottom)
    controls.enhanceButton.Left:Set(function() return controls.scrollUnitsFwdButton.Left() + 0 end)
    controls.enhanceButton.Height:Set(64)
    controls.enhanceButton.Width:Set(36)
    LayoutHelpers.ResetBottom(controls.enhanceButton)
    controls.enhanceButton.OnClick = function(self, modifiers)
        import('/lua/modules/ui/game/enhancements.lua').ToggleDialog(filteredEnhancements, currentSelection[1])
    end
    controls.enhanceButton.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "enhance")
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
        Button.HandleEvent(self, event)
    end

end

function SetLayout(layout)
    layoutVar = layout
    if layout == 'bottom' then
        DoBottomLayout()
    elseif layout == 'left' then
        DoLeftLayout()
    elseif layout == 'right' then
        DoRightLayout()
    end
    controls.scrollUnitsBackButton:UseAlphaHitTest(false)
    controls.scrollUnitsFwdButton:UseAlphaHitTest(false)
    import('/lua/modules/ui/game/unitviewDetail.lua').SetLayout(layout,controls.unitsBGMiddle)
end

function SetupConstructionControl(parent, inMFDControl, inOrdersControl, inQueueControl)
    mfdControl = inMFDControl
    ordersControl = inOrdersControl
    controlClusterGroup = parent
    queueControl = inQueueControl

--[[
    tech1Tab.checkbox.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "tech_level_1_cons_tab")
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
    end
    tech2Tab.checkbox.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "tech_level_2_cons_tab")
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
        Checkbox.HandleEvent(self, event)
    end
    tech3Tab.checkbox.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "tech_level_3_cons_tab")
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
        Checkbox.HandleEvent(self, event)
    end
    tech4Tab.checkbox.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            Tooltip.CreateMouseoverDisplay(self, "experimental_cons_tab")
        elseif event.Type == 'MouseExit' then
            Tooltip.DestroyMouseoverDisplay()
        end
        Checkbox.HandleEvent(self, event)
    end
--]]

    SetLayout(UIUtil.currentLayout)
    import('/lua/modules/ui/game/unitviewDetail.lua').SetupUnitViewLayout(parent,controls.unitsBGMiddle)

    return controls.constructionGroup
end

function OnQueueChanged(newQueue)
    Queue.OnQueueChanged(newQueue)
end

function CreateEnhancementButton(parent, enhancementName, enhancement)
    local tempBmpname = ""
    local tempEntityID = currentSelection[1]:GetBlueprint()

    if enhancement.Slot == "Back" then
        tempBmpName = tempEntityID.BlueprintId .. "-b-" .. enhancement.Icon
    elseif enhancement.Slot == "LCH" then
        tempBmpName = tempEntityID.BlueprintId .. "-la-" .. enhancement.Icon
    elseif enhancement.Slot == "RCH" then
        tempBmpName = tempEntityID.BlueprintId .. "-ra-" .. enhancement.Icon
    elseif enhancement.Slot == "Shield" then
        tempBmpName = tempEntityID.BlueprintId .. "-ch"
    end

    if string.sub(tempEntityID.BlueprintId, 2, 2) == 'a' then
        tempBmpName = '/dialogs/enhance-icons-aeon-' .. tempEntityID.BlueprintId .. '/'.. tempBmpName
    elseif string.sub(tempEntityID.BlueprintId, 2, 2) == 'e' then
        tempBmpName = '/dialogs/enhance-icons-uef-' .. tempEntityID.BlueprintId .. '/' .. tempBmpName
    elseif string.sub(tempEntityID.BlueprintId, 2, 2) == 'r' then
        tempBmpName = '/dialogs/enhance-icons-cybran-' .. tempEntityID.BlueprintId .. '/' .. tempBmpName
    end

    local bg = Bitmap(parent)   --todo put land/air/sea background here
    bg:SetSolidColor('black')
    bg.Width:Set(GameCommon.iconBmpWidth)
    bg.Height:Set(GameCommon.iconBmpHeight)
    local button = Button(bg,
        UIUtil.UIFile(tempBmpName .. '_btn_up.dds'),
        UIUtil.UIFile(tempBmpName .. '_btn_over.dds'),
        UIUtil.UIFile(tempBmpName .. '_btn_down.dds'),
        UIUtil.UIFile(tempBmpName .. '_btn_sel.dds'),
        'UI_Menu_MouseDown_Sml')
    --LayoutHelpers.FillParent(button, bg)
    button.Top:Set(bg.Top)
    button.Left:Set(function() return bg.Left() + 1 end)

    button.OnClick = function(self, modifiers)
        if modifiers.Left then
            if modifiers.Shift == true then
                IssueCommand("UNITCOMMAND_Enhance", enhancementName, false)
            else
                IssueCommand("UNITCOMMAND_Enhance", enhancementName, true)
            end
        end
    end

    button.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            import('/lua/modules/ui/game/unitviewDetail.lua').ShowEnhancement(enhancement, tempEntityID.BlueprintId, enhancement.Icon, currentSelection[1])
        end
        if event.Type == 'MouseExit' then
            import('/lua/modules/ui/game/unitviewDetail.lua').Hide()
        end
        Button.HandleEvent(self, event)
    end

    return bg
end

local UPPER_GLOW_THRESHHOLD = .5
local LOWER_GLOW_THRESHHOLD = .1
local GLOW_SPEED = 2

-- not standard so use our own button create code
function CreateBuildableButton(parent, blueprintId, blueprint, unitList)
    local iconName, upIconName, downIconName, overIconName = GameCommon.GetCachedUnitIconFileNames(blueprint)
    local bg = Bitmap(parent)   --todo put land/air/sea background here
    bg:SetSolidColor('00000000')
    bg.Width:Set(GameCommon.iconBmpWidth)
    bg.Height:Set(GameCommon.iconBmpHeight)
    local button = Button(bg, upIconName, downIconName, overIconName, upIconName, 'UI_Opt_Mini_Button_Click', 'UI_Opt_Mini_Button_Over')
    local selectedUnits = unitList
    --LayoutHelpers.FillParent(button, bg)
    button.Top:Set(bg.Top)
    button.Left:Set(function() return bg.Left() + 1 end)
    #button.Width:Set(GameCommon.iconWidth)
    #button.Height:Set(GameCommon.iconHeight)
    button._blueprintId = blueprintId
    
    button.Glow = Bitmap(button)
    button.Glow:SetTexture(UIUtil.UIFile('/game/units_bmp/glow.dds'))
    #button.Glow:SetSolidColor('FF00FF00')
    #button.Glow.Height:Set(64)
    #button.Glow.Width:Set(64)
    LayoutHelpers.AtCenterIn(button.Glow, button)
    button.Glow:SetAlpha(0)
    button.Incrimenting = true
    button.Glow.OnFrame = function(self, elapsedTime)
        local curAlpha = self:GetAlpha()
        if self.Incrimenting then
            curAlpha = curAlpha + (elapsedTime * GLOW_SPEED)
            if curAlpha > UPPER_GLOW_THRESHHOLD then
                curAlpha = UPPER_GLOW_THRESHHOLD
                self.Incrimenting = false
            end
        else
            curAlpha = curAlpha - (elapsedTime * GLOW_SPEED)
            if curAlpha < LOWER_GLOW_THRESHHOLD then
                curAlpha = LOWER_GLOW_THRESHHOLD
                self.Incrimenting = true
            end
        end
        self:SetAlpha(curAlpha)
    end

    button.OnClick = function(self, modifiers)
        local blueprint = __blueprints[self._blueprintId]
        local count = 1
        local performUpgrade = false

        selectedwhatIfBuilder = whatIfBuilder
        selectedwhatIfBlueprintID = whatIfBlueprintID

        if modifiers.Left then
            if modifiers.Ctrl == true and modifiers.Shift == nil then
                count = 50
            elseif modifiers.Shift == true and modifiers.Ctrl == nil then
                count = 5
            elseif modifiers.Shift == true and modifiers.Ctrl == true then
                count = 500
            end

            -- see if we are issuing an upgrade order
            if blueprint.General.UpgradesFrom == 'none' then
                performUpgrade = false
            else
                for i,v in selectedUnits do
                    if v then   -- it's possible that your unit will have died by the time this gets to it
                        local unitBp = v:GetBlueprint()
                        if blueprint.General.UpgradesFrom == unitBp.BlueprintId then
                            performUpgrade = true
                        elseif blueprint.General.UpgradesFrom == unitBp.General.UpgradesTo then
                            performUpgrade = true
                        end
                    end
                end
            end

            if performUpgrade then
                -- if the item to build is an upgrade.. then issue the upgrade command
                local clear = true
                if blueprint.Physics.MotionType == 'RULEUMT_None' or modifiers.Shift then
                    clear = false
                end
                IssueBlueprintCommand("UNITCOMMAND_Upgrade", self._blueprintId, 1, clear)
            else
                if blueprint.Physics.MotionType == 'RULEUMT_None' or EntityCategoryContains(categories.NEEDMOBILEBUILD, self._blueprintId) then
                    -- stationary means it needs to be placed, so go in to build mobile mode
                    import('/lua/modules/ui/game/commandmode.lua').StartCommandMode("build", self._blueprintId)
                else
                    -- if the item to build can move, it must be built by a factory
                    --TODO -what about mobile factories?
                    IssueBlueprintCommand("UNITCOMMAND_BuildFactory", self._blueprintId, count)
                end
            end
        end
    end

    button.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            self.Glow:SetNeedsFrameUpdate(true)
            self.Glow:SetAlpha(LOWER_GLOW_THRESHHOLD)
            local blueprint = __blueprints[self._blueprintId]
            local builder = unitList[1]
            whatIfBuilder = builder
            whatIfBlueprintID = self._blueprintId
            Tooltip.CreateMouseoverDisplay(self, blueprint.Description)
            import('/lua/modules/ui/game/unitviewDetail.lua').Show(blueprint, builder, self._blueprintId)
        end
        if event.Type == 'MouseExit' then
            self.Glow:SetNeedsFrameUpdate(false)
            self.Glow:SetAlpha(0)
            whatIfBuilder = nil
            whatIfBlueprintID = nil
            Tooltip.DestroyMouseoverDisplay()
            import('/lua/modules/ui/game/unitviewDetail.lua').Hide()
        end
        Button.HandleEvent(self, event)
    end

    return bg
end

-- not standard so use our own button create code
function CreateAttachedEntityButton(parent, blueprintId, blueprint, userUnit)
    local iconName, upIconName, downIconName, overIconName = GameCommon.GetCachedUnitIconFileNames(blueprint)
    local selectedIcon = '/textures/ui/common/icons/units/default_vehicle_icon.dds' -- Tempory icons
    local disabledIcon = '/textures/ui/common/icons/units/default_icon.dds'         -- Tempory icons
    local bg = Bitmap(parent)   --todo put land/air/sea background here
    local disabledButton = userUnit:HasUnloadCommandQueuedUp()
    bg:SetSolidColor('black')
    #bg.Width:Set(GameCommon.iconBmpWidth)
    #bg.Height:Set(GameCommon.iconBmpHeight)
    local button
    if disabledButton then
        button = Button(bg, disabledIcon, disabledIcon, disabledIcon, disabledIcon, 'UI_Menu_MouseDown_Sml')
        button:SetUpAltButtons( disabledIcon, disabledIcon, disabledIcon, disabledIcon )
    else
        button = Button(bg, upIconName, downIconName, overIconName, upIconName, 'UI_Menu_MouseDown_Sml')
        button:SetUpAltButtons( selectedIcon, selectedIcon, selectedIcon, selectedIcon )
    end
    --LayoutHelpers.FillParent(button, bg)
    button.Top:Set(bg.Top)
    button.Left:Set(function() return bg.Left() + 1 end)
    button._blueprintId = blueprintId

    button.OnClick = function(self, modifiers)
        if not disabledButton then
            if modifiers.Left then
                -- Toggling selection of the entity
                button:OnAltToggle()

                -- Add or Remove the entity to the session selection
                if button.mAltToggledFlag then
                    AddToSessionExtraSelectList(userUnit)
                else
                    RemoveFromSessionExtraSelectList(userUnit)
                end
            end
        end
    end

    button.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            local blueprint = __blueprints[self._blueprintId]
            import('/lua/modules/ui/game/unitviewDetail.lua').Show(blueprint)
        end
        if event.Type == 'MouseExit' then
            import('/lua/modules/ui/game/unitviewDetail.lua').Hide()
        end
        Button.HandleEvent(self, event)
    end

    return bg
end

-- not standard so use our own button create code
function CreateAssistingEntityButton(parent, blueprintId, blueprint, userUnit)
    local iconName = blueprint.RuntimeData.IconFileName
    local bg = Bitmap(parent)   --todo put land/air/sea background here
    bg:SetSolidColor('black')
    #bg.Width:Set(GameCommon.iconBmpWidth)
    #bg.Height:Set(GameCommon.iconBmpHeight)

    local button = Button(bg, iconName, iconName, iconName, iconName, 'UI_Menu_MouseDown_Sml')
    --LayoutHelpers.FillParent(button, bg)
    button.Top:Set(bg.Top)
    button.Left:Set(function() return bg.Left() + 1 end)
    button._blueprintId = blueprintId
    local canAssist = userUnit:IsAutoMode()

    local text
    if canAssist then
        text = UIUtil.CreateText(button, "<LOC CONSTRUCT_0005>Assisting", 12, "Arial")
    else
        text = UIUtil.CreateText(button, '', 12, "Arial")
    end

    LayoutHelpers.AtLeftTopIn(text, bg)
    bg.text = text

    button.OnClick = function(self, modifiers)
        if modifiers.Left then
            SelectUnits( { userUnit } )
        end

        if modifiers.Right then
            if canAssist then
                canAssist = false
                userUnit:ProcessInfo('SetAutoMode', 'false')
                bg.text:SetText('')
            else
                canAssist = true
                userUnit:ProcessInfo('SetAutoMode', 'true')
                bg.text:SetText('Assisting')
            end
        end
    end


    button.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            local blueprint = __blueprints[self._blueprintId]
            import('/lua/modules/ui/game/unitviewDetail.lua').Show(blueprint, true)
        end
        if event.Type == 'MouseExit' then
            import('/lua/modules/ui/game/unitviewDetail.lua').Hide()
        end
        Button.HandleEvent(self, event)
    end

    return bg
end

-- EntityCategoryFilterDown(categories.TECH1, buildableCategories)

function EnhancementPopulate(slot)
    local filteredEnhancements = {}
    for i, v in filteredTechTable[5] do
        if v['Slot'] == slot then
            filteredEnhancements[i] = v
        end
    end
    techLevelSelected = filteredEnhancements
    buildingEnhancements = true
    PopulateGridButtons(slot)
end

function ToggleEnhancementWindow(state, enhancementWindow)
    if state then
        if UIUtil.currentLayout == 'bottom' then
            controls.unitsBGCapMin.Left:Set(enhancementWindow.Right)
        end
    else
        if UIUtil.currentLayout == 'bottom' then
            LayoutHelpers.AtLeftIn(controls.unitsBGCapMin, controls.constructionGroup, -2)
            LayoutHelpers.ResetRight(controls.unitsBGCapMin)
        end
    end
end

function PopulateGridButtons(inSlot)
    controls.unitGrid:DeleteAndDestroyAll(true)
    ClearSessionExtraSelectList()

    if not buildingEnhancements and enhancementBool then
        controls.enhanceButton:Enable()
    else
        controls.enhanceButton:Disable()
    end

    local totalCount = 0

    -- Check for setting up the transport content list
    if EntityCategoryEmpty(tempBuildableCategories) and not buildingEnhancements then
        local userUnitList = GetAttachedUnitsList(currentSelection)
        totalCount = table.getn(userUnitList)
        if totalCount > 0 then
            local visCols, visRows = controls.unitGrid:GetVisible()
            controls.unitGrid:AppendCols(visCols, true)
            controls.unitGrid:AppendRows(math.ceil(table.getn(userUnitList) / visCols), true)

            for i,v in userUnitList do
                    local blueprintID = v:GetBlueprint().BlueprintId
                    local blueprint = __blueprints[blueprintID]
                    local button = CreateAttachedEntityButton(controls.unitGrid, blueprintID, blueprint, v)
                    local col = math.mod(i - 1, visCols) + 1 -- note the sub/add due to one based arrays
                    local row = math.ceil(i / visCols)
                    controls.unitGrid:SetItem(button, col, row, true)
            end
        end
    -- Check for setting up the buildable categories list
    else
        local assistList = GetAssistingUnitsList(currentSelection)
        local entityCount = GetTableSize(techLevelSelected)
        totalCount = entityCount + table.getn(assistList)
        local visCols, visRows = controls.unitGrid:GetVisible()
        controls.unitGrid:AppendCols(visCols, true)
        controls.unitGrid:AppendRows(math.ceil(totalCount / visCols), true)

        if buildingEnhancements then
            local tempEntityID = currentSelection[1]:GetEntityId()
            local existingEnhancements = import('/lua/modules/ui/game/enhancements.lua').GetEnhancements(tempEntityID)
            local index = 1
            local restrictedList = import('/lua/modules/ui/game/enhancements.lua').GetRestricted()
            for i, v in techLevelSelected do
                local isRestricted = false
                if restrictedList then
                    for h, x in restrictedList do
                        if x == i then
                            isRestricted = true
                            break
                        end
                    end
                end
                if not isRestricted then
                    if existingEnhancements == nil then
                        if v['Prerequisite'] == nil then
                            local button = CreateEnhancementButton(controls.unitGrid, i, v)
                            local col = math.mod(index - 1, visCols) + 1 -- note the sub/add due to one based arrays
                            local row = math.ceil(index / visCols)
                            controls.unitGrid:SetItem(button, col, row, true)
                            index = index + 1
                        end
                    else
                        if existingEnhancements[inSlot] != nil then
                            if existingEnhancements[inSlot] == v["Prerequisite"] then
                                local button = CreateEnhancementButton(controls.unitGrid, i, v)
                                local col = math.mod(index - 1, visCols) + 1 -- note the sub/add due to one based arrays
                                local row = math.ceil(index / visCols)
                                controls.unitGrid:SetItem(button, col, row, true)
                                index = index + 1
                            end
                        else
                            if v['Prerequisite'] == nil then
                                local button = CreateEnhancementButton(controls.unitGrid, i, v)
                                local col = math.mod(index - 1, visCols) + 1 -- note the sub/add due to one based arrays
                                local row = math.ceil(index / visCols)
                                controls.unitGrid:SetItem(button, col, row, true)
                                index = index + 1
                            end
                        end
                    end
                end
            end
        else
            if totalCount> 0 then
                for i,v in techLevelSelected do
                    local blueprint = __blueprints[v]
                    local button = CreateBuildableButton(controls.unitGrid, v, blueprint, currentSelection)
                    local col = math.mod(i - 1, visCols) + 1 -- note the sub/add due to one based arrays
                    local row = math.ceil(i / visCols)
                    controls.unitGrid:SetItem(button, col, row, true)
                end
                for i,v in assistList do
                    local index = entityCount + i
                    local blueprintID = v:GetBlueprint().BlueprintId
                    local blueprint = __blueprints[blueprintID]
                    local button = CreateAssistingEntityButton(controls.unitGrid, blueprintID, blueprint, v)
                    local col = math.mod(index - 1, visCols) + 1 -- note the sub/add due to one based arrays
                    local row = math.ceil(index / visCols)
                    controls.unitGrid:SetItem(button, col, row, true)
                end
            end
        end
    end
    controls.unitGrid:EndBatch()

    -- Determine the check status of the "infinite loop" button
    local onlyFactories = true
    currentInfiniteQueueCheckStatus = false

    for i,v in currentSelection do
        if v:IsRepeatQueue() then
            currentInfiniteQueueCheckStatus = true
        end
        if not v:IsInCategory('FACTORY') then
            onlyFactories = false
        end
    end

    if onlyFactories then
        Queue.SetInfiniteQueueToggle(true)
        Queue.SetInfinityCheck(currentInfiniteQueueCheckStatus)
    else
        Queue.SetInfinityCheck(false)
    end

    local visCols, visRows = controls.unitGrid:GetVisible()
    if (visCols * visRows) >= totalCount then
        controls.scrollUnitsBackButton:Disable()
        controls.scrollUnitsFwdButton:Disable()
    else
        controls.scrollUnitsBackButton:Enable()
        controls.scrollUnitsFwdButton:Enable()
    end
end


function SetAvailableGridButtons(buildableCategories, unitList, entityList)
    buildingEnhancements = false
    tempBuildableCategories = buildableCategories
    currentSelection = unitList
    local buttonToSet = false
    local buildingUnitBP = false



    -- remove upgrades from entity list
    collapseArray(entityList, function(entity) return (__blueprints[entity].General.Category == 'Upgrade') end)
    filteredTech1Units = false
    filteredTech2Units = false
    filteredTech3Units = false
    filteredTech4Units = false
    filteredFactories = {}
    filteredEnhancements = false

    filteredTechTable[1] = false
    filteredTechTable[2] = false
    filteredTechTable[3] = false
    filteredTechTable[4] = false
    filteredTechTable[5] = false

    local index = 1
    for i, v in entityList do
        local blueprint = __blueprints[v]
        for h, x in blueprint.Categories do
            if x == 'CONSTRUCTIONSORTDOWN' then
                filteredFactories[index] = v
                table.remove(entityList, i)
            end
        end
    end

    filteredTech1Units = EntityCategoryFilterDown(categories.TECH1, entityList)
    filteredTech2Units = EntityCategoryFilterDown(categories.TECH2, entityList)
    filteredTech3Units = EntityCategoryFilterDown(categories.TECH3, entityList)
    filteredTech4Units = EntityCategoryFilterDown(categories.EXPERIMENTAL, entityList)

    for i, v in filteredFactories do
        local blueprint = __blueprints[v]
        for h, x in blueprint.Categories do
            if x == 'TECH2' then
                table.insert(filteredTech1Units, v)
            elseif x == 'TECH3' then
                table.insert(filteredTech2Units, v)
            end
        end
    end
    
    if table.getn(currentSelection) == 1 then
        buildingUnitBP = currentSelection[1]:GetBlueprint()

        if buildingUnitBP["Enhancements"] then
            filteredEnhancements = buildingUnitBP["Enhancements"]
        end
    end
    
    local function SortFunc(unit1, unit2)
        local bp1 = __blueprints[unit1]
        local bp2 = __blueprints[unit2]
        if bp1.StrategicIconSortPriority <= bp2.StrategicIconSortPriority then
            return false
        else
            return true
        end
    end
    
    if filteredTech1Units then
        table.sort(filteredTech1Units, SortFunc)
    end
    if filteredTech2Units then
        table.sort(filteredTech2Units, SortFunc)
    end
    if filteredTech3Units then
        table.sort(filteredTech3Units, SortFunc)
    end
    if filteredTech4Units then
        table.sort(filteredTech4Units, SortFunc)
    end
    if filteredEnhancements then
        table.sort(filteredEnhancements, SortFunc)
    end
    
    filteredTechTable[1] = filteredTech1Units
    filteredTechTable[2] = filteredTech2Units
    filteredTechTable[3] = filteredTech3Units
    filteredTechTable[4] = filteredTech4Units
    filteredTechTable[5] = filteredEnhancements
    
    
    if filteredEnhancements then
        --import('/lua/modules/ui/game/orders.lua').ToggleEnhancementButton(filteredEnhancements, currentSelection[1])
        enhancementBool = true
    else
        --import('/lua/modules/ui/game/orders.lua').ToggleEnhancementButton(false)
        enhancementBool = false
    end

    if table.getn(filteredTech1Units) > 0 then
        tech1Tab.checkbox:Enable()
        techLevelSelected = filteredTechTable[1]
        buttonToSet = 1
        buildingEnhancements = false
    else
        tech1Tab.checkbox:Disable()
    end
    if table.getn(filteredTech2Units) > 0 then
        tech2disabledOverGroup:Hide()
        if table.getn(filteredTech2Units) == 1 then
            if (filteredTech2Units[1].General.UpgradesFrom != nil and filteredTech2Units[1].General.UpgradesFrom != "") then
                tech2Tab.checkbox:Enable()
                techLevelSelected = filteredTechTable[2]
                buttonToSet = 2
                buildingEnhancements = false
            else
                tech2Tab.checkbox:Enable()
                if table.getn(filteredTech1Units) == 0 then
                    buttonToSet = 2
                    techLevelSelected = filteredTechTable[2]
                else
                    buttonToSet = 1
                end
            end
        else
            tech2Tab.checkbox:Enable()
            techLevelSelected = filteredTechTable[2]
            buttonToSet = 2
            buildingEnhancements = false
        end
    else
        tech2disabledOverGroup:Show()
        tech2Tab.checkbox:Disable()
    end
    if table.getn(filteredTech3Units) > 0 then
        tech3disabledOverGroup:Hide()
        if table.getn(filteredTech3Units) == 1 then
            if (filteredTech3Units[1].General.UpgradesFrom != nil and filteredTech3Units[1].General.UpgradesFrom != "") then
                tech3Tab.checkbox:Enable()
                techLevelSelected = filteredTechTable[3]
                buttonToSet = 3
                buildingEnhancements = false
            else
                tech3Tab.checkbox:Enable()
                if table.getn(filteredTech2Units) == 0 then
                    buttonToSet = 3
                    techLevelSelected = filteredTechTable[3]
                else
                    buttonToSet = 2
                end
            end
        else
            tech3Tab.checkbox:Enable()
            techLevelSelected = filteredTechTable[3]
            buttonToSet = 3
            buildingEnhancements = false
        end
    else
        tech3disabledOverGroup:Show()
        tech3Tab.checkbox:Disable()
    end
    if table.getn(filteredTech4Units) > 0 then
        tech4disabledOverGroup:Hide()
        tech4Tab.checkbox:Enable()
    else
        tech4disabledOverGroup:Show()
        tech4Tab.checkbox:Disable()
    end


    if buttonToSet == 1 then
        tech2Tab.checkbox:SetCheck(false)
        tech3Tab.checkbox:SetCheck(false)
        tech4Tab.checkbox:SetCheck(false)
        tech1Tab.checkbox:SetCheck(true)
        controls.techRadioGroup.current = buttonToSet
    elseif buttonToSet == 2 then
        tech1Tab.checkbox:SetCheck(false)
        tech3Tab.checkbox:SetCheck(false)
        tech4Tab.checkbox:SetCheck(false)
        tech2Tab.checkbox:SetCheck(true)
        controls.techRadioGroup.current = buttonToSet
    elseif buttonToSet == 3 then
        tech2Tab.checkbox:SetCheck(false)
        tech1Tab.checkbox:SetCheck(false)
        tech4Tab.checkbox:SetCheck(false)
        tech3Tab.checkbox:SetCheck(true)
        controls.techRadioGroup.current = buttonToSet
    end

    PopulateGridButtons()
end
