--*****************************************************************************
--* File: lua/modules/ui/game/gamemain.lua
--* Author: Chris Blackwell
--* Summary: Entry point for the in game UI
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Group = import('/lua/modules/maui/group.lua').Group
local WldUIProvider = import('/lua/modules/ui/game/wlduiprovider.lua').WldUIProvider
local GameCommon = import('/lua/modules/ui/game/gamecommon.lua')
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local StatusBar = import('/lua/modules/maui/statusbar.lua').StatusBar

local gameParent = false
local controlClusterGroup = false
local statusClusterGroup = false
local mapGroup = false
local mfdControl = false
local ordersControl = false
local queueControl = false

isPaused = false
local isReplay = false

function GetReplayState()
    return isReplay
end

-- query this to see if the UI is hidden
gameUIHidden = false

function SetLayout(layout)
    import('/lua/modules/ui/game/borders.lua').SetLayout(layout)
    import('/lua/modules/ui/game/multifunction.lua').SetLayout(layout)
    if isReplay then
        import('/lua/modules/ui/game/replay.lua').SetLayout(layout)
    else
        import('/lua/modules/ui/game/orders.lua').SetLayout(layout)
    end
    import('/lua/modules/ui/game/unitview.lua').SetLayout(layout)
    import('/lua/modules/ui/game/objectives2.lua').SetLayout(layout)
    import('/lua/modules/ui/game/construction.lua').SetLayout(layout)
    import('/lua/modules/ui/game/enhancements.lua').SetLayout(layout)
    import('/lua/modules/ui/game/selection.lua').SetLayout(layout)
    import('/lua/modules/ui/game/queue.lua').SetLayout(layout)
    import('/lua/modules/ui/game/chat.lua').SetLayout(layout)
end

function OnFirstUpdate()
    local avatars = GetArmyAvatars()
    if avatars and avatars[1]:IsInCategory("COMMAND") then
        local armiesInfo = GetArmiesTable()
        local focusArmy = armiesInfo.focusArmy
        local playerName = armiesInfo.armiesTable[focusArmy].nickname
        UISelectAndZoomTo( avatars[1] )
        ConExecute('RenameUnit ' .. playerName)
        import('/lua/modules/UserMusic.lua').StartPeaceMusic()
    end
    import('/lua/modules/ui/game/score.lua').CreateScoreOverlay()
end

function CreateUI(isReplay)
    -- set up our layout change function
    UIUtil.changeLayoutFunction = SetLayout

    GameCommon.InitializeUnitIconBitmapNames()

    gameParent = UIUtil.CreateScreenGroup(GetFrame(0), "GameMain ScreenGroup")

    --TODO world scroll is a stupid implementation, and needs to die, this should be in the engine
    local worldScroll = import('/lua/modules/ui/game/worldscroll.lua')
    local topBorder

    controlClusterGroup, statusClusterGroup, mapGroup, topBorder =  import('/lua/modules/ui/game/borders.lua').SetupBorderControl(gameParent)

    topBorder:SetNeedsFrameUpdate(true)
    topBorder.OnFrame = function(self, deltaTime)
        topBorder:SetNeedsFrameUpdate(false)
        OnFirstUpdate()
    end

    import('/lua/modules/ui/game/worldview.lua').CreateMainWorldView(gameParent, mapGroup)

    local massGroup, energyGroup = import('/lua/modules/ui/game/economy.lua').CreateEconomyBar(statusClusterGroup)
    import('/lua/modules/ui/game/options.lua').CreateOptionsPanel(statusClusterGroup, massGroup, energyGroup)

    mfdControl = import('/lua/modules/ui/game/multifunction.lua').SetupMultifunctionControl(controlClusterGroup)
    if isReplay then
        ordersControl = import('/lua/modules/ui/game/replay.lua').SetupReplayControl(controlClusterGroup, mfdControl)
    else
        ordersControl = import('/lua/modules/ui/game/orders.lua').SetupOrdersControl(controlClusterGroup, mfdControl)
    end
    queueControl = import('/lua/modules/ui/game/queue.lua').SetupQueueControl(controlClusterGroup, mfdControl, ordersControl)
    import('/lua/modules/ui/game/construction.lua').SetupConstructionControl(controlClusterGroup, mfdControl, ordersControl, queueControl)
    import('/lua/modules/ui/game/selection.lua').SetupSelectionBox(controlClusterGroup, mfdControl, ordersControl, queueControl)
    import('/lua/modules/ui/game/unitview.lua').SetupUnitViewLayout(controlClusterGroup)
    import('/lua/modules/ui/game/chat.lua').SetupChatLayout(mapGroup)
    import('/lua/modules/ui/game/avatars.lua').CreateAvatarIcons(mapGroup)
    import('/lua/modules/ui/game/missiontext.lua').CreateMissionText(mapGroup, massGroup, energyGroup)
    import('/lua/modules/ui/game/objectives2.lua').SetupObjectivesLayout(mapGroup)
    import('/lua/modules/ui/game/objectiveDetail.lua').SetupObjectiveDetail(mapGroup)
    import('/lua/modules/ui/game/helptext.lua').CreateHelpText(mapGroup)
    import('/lua/modules/ui/game/timer.lua').CreateTimerDialog(mapGroup)
    import('/lua/modules/ui/game/pause.lua').CreatePauseUI(mapGroup)
    import('/lua/modules/ui/game/transmissionlog.lua').CreateTransmissionLog(mapGroup)
    import('/lua/modules/ui/game/consoleecho.lua').CreateConsoleEcho(mapGroup)
    import('/lua/modules/ui/game/enhancements.lua').CreateDialog(UIUtil.currentLayout, mfdControl, gameParent, mapGroup)

    import('/lua/modules/ui/game/diplomacy.lua').CreateDiplomacyControl()

    worldScroll.CreateWorldScrollers()

    if GetNumRootFrames() > 1 then
        import('/lua/modules/ui/game/multihead.lua').CreateSecondView()
    end
end

local provider = false

local function LoadDialog(parent)
    local dialog = Group(parent, "quickDialogGroup")
    LayoutHelpers.AtCenterIn(dialog, parent)
    dialog.Depth:Set(GetFrame(parent:GetRootFrame():GetTargetHead()):GetTopmostDepth() + 1)

    local background = Bitmap(dialog, UIUtil.SkinnableFile('/load/background-portal_bmp.dds'))
    LayoutHelpers.FillParent(background, parent)
    background.Depth:Set(function() return dialog.Depth() - 1 end)
    
    local panel = Bitmap(dialog, UIUtil.SkinnableFile('/load/panel_bmp.dds'))
    dialog.Width:Set(panel.Width)
    dialog.Height:Set(panel.Height)
    LayoutHelpers.FillParent(panel, dialog)
    
    dialog._statusbar = StatusBar(panel, 0, 100, false, false, UIUtil.SkinnableFile('/load/load-bar-bg_bmp.dds'), UIUtil.SkinnableFile('/load/load-bar_bmp.dds'), false)
    LayoutHelpers.AtLeftTopIn(dialog._statusbar, dialog, 76, 149)
    
    dialog._statusbarneg = StatusBar(panel, 0, 100, false, true, UIUtil.SkinnableFile('/load/load-bar-bg_bmp.dds'), UIUtil.SkinnableFile('/load/load-bar_bmp.dds'), false)
    LayoutHelpers.AtLeftTopIn(dialog._statusbarneg, dialog, 69, 149)
    dialog._statusbarneg:Hide()
    
    dialog._textLine = UIUtil.CreateText(dialog._statusbarneg, "", 16, UIUtil.titleFont)
    LayoutHelpers.AtCenterIn(dialog._textLine, dialog, -15)
         
    return dialog
end

function CreateWldUIProvider()

    provider = WldUIProvider()

    local loadingDialog = false
    local waitingDialog = false
    local frame1Logo = false

    local lastTime = 0

    provider.StartLoadingDialog = function(self)
        if not loadingDialog then
            loadingDialog = LoadDialog(GetFrame(0))
            if GetNumRootFrames() > 1 then
                local frame1 = GetFrame(1)
                local frame1Logo = Bitmap(frame1, UIUtil.UIFile('/marketing/splash.dds'))
                LayoutHelpers.FillParent(frame1Logo, frame1)
            end
--TEMP cheap and cheasy animation until UpdateLoadingDialog actually gets called            
            lastTime = 0
            ForkThread(
                function()
                    while loadingDialog do
                        if loadingDialog._statusbarneg:IsHidden() then
                            loadingDialog._statusbar:SetValue(lastTime)
                        else
                            loadingDialog._statusbarneg:SetValue(100 - lastTime)
                        end                         
                        lastTime = lastTime + 3
                        if lastTime > 100 then 
                            lastTime = 0
                            if loadingDialog._statusbarneg:IsHidden() then
                                loadingDialog._statusbarneg:Show()
                                loadingDialog._statusbar:Hide()
                                loadingDialog._statusbarneg:SetValue(100)
                            else
                                loadingDialog._statusbarneg:Hide()
                                loadingDialog._statusbar:Show()
                                loadingDialog._statusbar:SetValue(0)
                            end
                        end
                        WaitFrames(1)
                    end
                end
            )
        end
    end

    provider.UpdateLoadingDialog = function(self, elapsedTime)
        if loadingDialog then
            loadingDialog._statusbar:SetValue(math.min(elapsedTime, 1))
        end
    end

    provider.StopLoadingDialog = function(self)
        if loadingDialog then
            loadingDialog:Destroy()
            loadingDialog = false
        end
    end

    provider.StartWaitingDialog = function(self)
        if not waitingDialog then waitingDialog = UIUtil.ShowInfoDialog(GetFrame(0), "<LOC gamemain_0001>Waiting For Other Players...") end
    end

    provider.UpdateWaitingDialog = function(self, elapsedTime)
        -- currently no function, but could animate waiting dialog
    end

    provider.StopWaitingDialog = function(self)
        if waitingDialog then
            waitingDialog:Destroy()
            waitingDialog = false
        end
    end

    provider.CreateGameInterface = function(self, inIsReplay)
        isReplay = inIsReplay
        if frame1Logo then
            frame1Logo:Destroy()
            frame1Logo = false
        end
        CreateUI(isReplay)
    end

    provider.DestroyGameInterface = function(self)
        if gameParent then gameParent:Destroy() end
    end
end

-- This function is called whenever the set of currently selected units changes
-- See /lua/modules/unit.lua for more information on the lua unit object
--      oldSelection: What the selection was before
--      newSelection: What the selection is now
--      added: Which units were added to the old selection
--      removed: Which units where removed from the old selection
function OnSelectionChanged(oldSelection, newSelection, added, removed)
    local availableOrders,  buildableCategories = GetUnitCommandData(newSelection)

    if not gameUIHidden then
        import('/lua/modules/ui/game/queue.lua').CheckForOrderQueue(newSelection, newSelection)
        if not isReplay then
            import('/lua/modules/ui/game/orders.lua').SetAvailableOrders(availableOrders, newSelection)
        end
        -- todo change the current command mode if no longer available? or set to nil?
        import('/lua/modules/ui/game/selection.lua').SelectionManager(buildableCategories,newSelection)
        import('/lua/modules/ui/game/enhancements.lua').ToggleDialog(nil, nil, true)
    end

    import('/lua/modules/ui/game/selection.lua').PlaySelectionSound(added)
end

function OnQueueChanged(newQueue)
    import('/lua/modules/ui/game/construction.lua').OnQueueChanged(newQueue)
end

# Called after the Sim has confirmed the game is indeed paused. This will happen
# on everyone's machine in a network game.
function OnPause(pausedBy, timeoutsRemaining)
    PauseSound("World",true)
    PauseSound("Music",true)
    local isOwner = false
    if pausedBy == SessionGetLocalCommandSource() then
        isOwner = true
    end
    isPaused = true
    import('/lua/modules/ui/game/missiontext.lua').PauseTransmission()
    import('/lua/modules/ui/game/pause.lua').OnPause(pausedBy, timeoutsRemaining, isOwner)
    import('/lua/modules/ui/game/options.lua').SetPauseButton(true, isOwner)
end

# Called after the Sim has confirmed that the game has resumed.
function OnResume()
    PauseSound("World",false)
    PauseSound("Music",false)
    isPaused = false
    import('/lua/modules/ui/game/missiontext.lua').ResumeTransmission()
    import('/lua/modules/ui/game/pause.lua').OnResume()
    import('/lua/modules/ui/game/options.lua').SetPauseButton(false)
end

# Called immediately when the user hits the pause button. This only ever gets
# called on the machine that initiated the pause (i.e. other network players
# won't call this)
function OnUserPause(pause)
end

local _beatFunctions = {}

function AddBeatFunction(fn)
    table.insert(_beatFunctions, fn)
end

function RemoveBeatFunction(fn)
    for i,v in _beatFunctions do
        if v == fn then
            table.remove(_beatFunctions, i)
            break
        end
    end
end

-- this function is called whenever the sim beats
function OnBeat()
    for i,v in _beatFunctions do
        if v then v() end
    end
end

function GetStatusCluster()
    return statusClusterGroup
end

function GetControlCluster()
    return controlClusterGroup
end

function GetGameParent()
    return gameParent
end

function HideGameUI(movieBool)
    if gameParent then
        if gameUIHidden then
            controlClusterGroup:Show()
            statusClusterGroup:Show()
            import('/lua/modules/ui/game/worldview.lua').Contract()
            import('/lua/modules/ui/game/borders.lua').HideBorder(false)
            import('/lua/modules/ui/game/tracking.lua').Expand()
            import('/lua/modules/ui/game/unitview.lua').Expand()
            import('/lua/modules/ui/game/avatars.lua').Expand()
            import('/lua/modules/ui/game/economy.lua').Expand()
            import('/lua/modules/ui/game/pause.lua').Expand()
            gameUIHidden = false
        else
            controlClusterGroup:Hide()
            statusClusterGroup:Hide()
            import('/lua/modules/ui/game/worldview.lua').Expand()
            import('/lua/modules/ui/game/borders.lua').HideBorder(true)
            import('/lua/modules/ui/game/tracking.lua').Contract()
            import('/lua/modules/ui/game/unitview.lua').Contract()
            import('/lua/modules/ui/game/avatars.lua').Contract()
            import('/lua/modules/ui/game/economy.lua').Contract()
            import('/lua/modules/ui/game/pause.lua').Contract()
            gameUIHidden = true
        end
    end
end

# Given a userunit that is adjacent to a given blueprint, does it yield a
# bonus? Used by the UI to draw extra info
function OnDetectAdjacencyBonus(userUnit, otherBp)
    # fixme: todo
    return true
end

function OnFocusArmyUnitDamaged(unit)
    import('/lua/modules/UserMusic.lua').NotifyBattle()
end

