--*****************************************************************************
--* File: lua/modules/ui/game/objectives2.lua
--*
--* Copyright  2006 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

# The objectives table should be an array whose order mirrors the order in which you wish the objectives to display
# The format of each entry is as follows:
# {
#    type = 'primary'                                        -- values are 'primary', 'secondary' and 'bonus'
#    complete = 'incomplete'                                 -- status of objective - 'complete' 'incomplete' 'failed'
#    title = 'Short Description'                             -- text shown in the list of objectives
#    description = 'Long Description'                        -- text shown in the extended description box
#    targets = { blipId1, Vector(10,10,10), blipId2, ... }   -- objective is a list of target unit(s) and/or location(s)
# }

local DEBUG_OBJECTIVES = false

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Group = import('/lua/modules/maui/group.lua').Group
local Button = import('/lua/modules/maui/button.lua').Button
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local GameCommon = import('/lua/modules/ui/game/gamecommon.lua')
local MultiLineText = import('/lua/modules/maui/multilinetext.lua').MultiLineText
local MissionText = import('/lua/modules/ui/game/missiontext.lua')

Widgets = false
local savedParent = false
local pendingObjectives = false

local newObjectiveSound = Sound({Cue = "UI_Objective_Window_Open", Bank = "Interface",})
local objectiveClickSound = Sound({Cue = "UI_Objective_Description", Bank = "Interface",})

function GetCurrentObjectiveTable()
    return Widgets
end

function FlashObjective(widget)
    local layout = UIUtil.SkinnableFile('/dialogs/objective-unit/objective-unit_layout.lua')
    local bg = Bitmap(widget.targetButton)

    bg:SetSolidColor( 'ff00ff00' )
    bg.Top:Set( widget.targetButton.Top )
    bg.Left:Set( widget.targetButton.Left )
    bg.Bottom:Set( widget.targetButton.Bottom )
    bg.Right:Set( widget.targetButton.Right )

    bg.Depth:Set( function() return widget.targetButton.Depth() - 1 end )
    bg:SetNeedsFrameUpdate(true)
    widget.targetButton:SetAlpha(0.75)
    bg.time = 0.0
    bg.OnFrame = function(self,delta)
        self.time = self.time + math.pi * 0.05
        local alpha = MATH_Lerp(math.sin(self.time), -1.0, 1.0, 0.0, 0.5)
        self:SetAlpha(alpha)
    end

    widget.flashingBG = bg
end

function SlideInObjective(widget)
    # We slide in the targetButton, not the frame. The frame is laid out relative
    # to the target button.
    widget.targetButton:Show()
    widget.frameSm:Show()
    widget.targetButton:SetNeedsFrameUpdate(true)
    local left = widget.targetButton.Left()

    local dx = -60.0

    widget.targetButton.OnFrame = function(self,delta)
        dx = dx + delta * 120.0
        if dx >= 0.0 then
            widget.animating = false
            widget.targetButton:SetNeedsFrameUpdate(false)
            widget.targetButton.Left:Set( left )
            FlashObjective(widget)
        else
            widget.targetButton.Left:Set( function() return left + dx end )
        end
    end
end

function OpenDescription(widget)
    widget.isOpen = true
    import('/lua/modules/maui/effecthelpers.lua').FadeIn(widget.frame,0.5,0.0,1.0)
end

function AddObjectives(objTable)
    if not Widgets then
        pendingObjectives = table.deepcopy(objTable)
        return
    end

    local newWidgets = {}

    for _,tag in table.keys(objTable) do
        local objective = objTable[tag]

        if not Widgets[objective.tag] then
            if DEBUG_OBJECTIVES then
                LOG('Creating Objective: ',objective.tag, '   (',objective.title,')')
            end

            Widgets[objective.tag] = {}
            local Widget = Widgets[objective.tag]
            Widget.ObjData = table.deepcopy(objective)
            Widget.StartTime = GetGameTimeSeconds()

            # Setup target button and action image for all objectives, even
            # for bonus objectives that won't be displayed (objectiveDetail
            # window will use them).
            Widget.targetButton = Button(savedParent)
            Widget.targetButton:SetNewTextures( GetTargetImages(Widget) )
            Widget.targetButton:SetTexture(Widget.targetButton.mNormal)
            Widget.targetButton.tag = objective.tag

            if objective.actionImage then
                local actionImage = UIUtil.SkinnableFile(objective.actionImage)
                Widget.actionButton = Button(Widget.targetButton,actionImage,actionImage,actionImage,actionImage)
                Widget.actionButton:DisableHitTest()
            end

            if Widget.ObjData.type != 'bonus' then
                Widget.Active = true

                Widget.targetButton.OnClick = function(self,modifiers)
                    if Widget.flashingBG then
                        Widget.flashingBG:Destroy()
                        Widget.flashingBG = nil
                        self:SetAlpha(1.0)
                    end

                    if Widget.isOpen or modifiers.Right then
                        local targets = Widget.ObjData.targets
                        if targets then
                            local max = table.getn(targets)
                            local desiredTarget = math.mod( Widget.TargetFocus or 0, table.getn(targets )) + 1

                            for idx,target in targets do
                                if idx == desiredTarget then
                                    if target.Type == 'Position' then
                                        local rect = Rect(target.Value[1] - 10,
                                                          target.Value[3] - 10,
                                                          target.Value[1] + 10,
                                                          target.Value[3] + 10)
                                        GetCamera("WorldCamera"):MoveToRegion( rect, 1.0 )
                                    elseif target.Type == 'Area' then
                                        GetCamera("WorldCamera"):MoveToRegion( target.Value, 1.0 )
                                    end
                                    Widget.TargetFocus = idx
                                end
                            end
                        end
                    end

                    if not Widget.isOpen then
                        PlaySound(objectiveClickSound)
                        OpenDescription(Widget)

                        for k,v in Widgets do
                            if v != Widget then
                                v.isOpen = false
                            end
                        end
                        SetLayout(UIUtil.currentLayout)
                    end
                end


                # frame for when it's expanded
                Widget.frame = Bitmap(savedParent)
                Widget.frame:SetTexture(UIUtil.UIFile('/dialogs/objective-unit/panel02_bmp.dds'))

                # frame for when it's contracted
                Widget.frameSm = Bitmap(savedParent)
                Widget.frameSm:SetTexture(UIUtil.UIFile('/dialogs/objective-unit/panel-icon-sm_bmp.dds'))
                Widget.frameSm:DisableHitTest()

                local closeFrame = UIUtil.CreateButtonStd(Widget.frame, '/dialogs/close_btn/close')
                closeFrame.Right:Set(function() return Widget.frame.Right() - 28 end)
                closeFrame.Top:Set(function() return Widget.frame.Top() + 24 end)
                closeFrame.OnClick = function(self,modifiers)
                    Widget.isOpen = false
                    Widget.frame:Hide()
                    Widget.frameSm:Show()
                    SetLayout(UIUtil.currentLayout)
                end

                Widget.title = UIUtil.CreateText(Widget.frame, "", 12, UIUtil.titleFont)
                Widget.title:SetColor( UIUtil.dialogCaptionColor )
                Widget.title:SetDropShadow(true)

                Widget.progress = UIUtil.CreateText(Widget.frame, "", 12, UIUtil.titleFont)
                Widget.progress:SetColor( UIUtil.dialogColumnColor )
                Widget.progress:SetDropShadow(true)

                Widget.description = MultiLineText(Widget.frame, UIUtil.bodyFont, 14, UIUtil.bodyColor)
                Widget.description:SetDropShadow(true)

                Widget.frame:DisableHitTest(true)
                closeFrame:EnableHitTest()
            else
                Widget.Active = false
                Widget.targetButton:DisableHitTest()
                Widget.targetButton:Hide()
                if Widget.actionImage then
                    Widget.actionImage:DisableHitTest()
                    Widget.actionImage:Hide()
                end
            end

            table.insert( newWidgets, Widget )
        end
    end

    if DEBUG_OBJECTIVES then
        LOG('Objectives after add: ')
        for k,v in Widgets do
            LOG('k = ',k)
        end
    end

    SetLayout(UIUtil.currentLayout)

    ForkThread(
        function()
            if DEBUG_OBJECTIVES then
                LOG('new widgets = ')
            end
            for k,vtemp in newWidgets do
                local w = vtemp
                # It's possible the objective got completed immediately so we
                # have to doublecheck that it still exists.
                if Widgets[w.ObjData.tag] and Widgets[w.ObjData.tag].Active then
                    if DEBUG_OBJECTIVES then
                        LOG(k, ' tag =',w.ObjData.tag)
                    end
                    w.animating = true
                    w.frameSm:Hide()
                    w.targetButton:Hide()
                end
            end

            # slide them in, one by one
            for k,v in newWidgets do
                # It's possible the objective got completed immediately so we
                # have to doublecheck that it still exists.
                if Widgets[v.ObjData.tag] and Widgets[v.ObjData.tag].Active then
                    PlaySound(newObjectiveSound)
                    SlideInObjective(v)
                    WaitSeconds(2)
                end
            end
        end
    )
end

function GetTargetImages(widget)
    # look for an image to display
    for k,v in widget.ObjData.targets do
        local target = v
        if v.BlueprintId then
            local blueprint = __blueprints[v.BlueprintId]
            return GameCommon.GetCachedUnitIconFileNames(blueprint)
        elseif v.Type == 'Area' then
            iconName = UIUtil.UIFile('/game/target-area/target-area_bmp.dds')
            return iconName, iconName, iconName, iconName
        end
    end

    local questionMark = UIUtil.UIFile('/dialogs/objective-unit/help-lg-graphics_bmp.dds')
    return questionMark, questionMark, questionMark, questionMark
end

function UpdateTarget(widget,data)
    if data.Value then
        for k,v in widget.ObjData.targets do
            if v.TargetTag == data.TargetTag then
                widget.ObjData.targets[k] = data
            end
        end
    else
        # No position value, remove target
        for k,v in widget.ObjData.targets do
            if v.TargetTag == data.TargetTag then
                table.remove( widget.ObjData.targets, k )
                break
            end
        end
    end

    if widget.targetButton then
        widget.targetButton:SetNewTextures( GetTargetImages(widget) )
        if widget.targetButton.mMouseOver then
            widget.targetButton:SetTexture(widget.targetButton.mHighlight)
        else
            widget.targetButton:SetTexture(widget.targetButton.mNormal)
        end
    end
end

function DisableObjective(tag)
    if Widgets[tag].Active then
        Widgets[tag].targetButton:Hide()
        Widgets[tag].frame:Hide()
        Widgets[tag].frameSm:Hide()
        if Widgets[tag].actionButton then
            Widgets[tag].actionButton:Hide()
        end
        Widgets[tag].Active = false
    end
end

function UpdateObjectivesTable(updateTable)
    if DEBUG_OBJECTIVES then
        LOG('updateTable = ',repr(updateTable))
    end
    for index,update in updateTable do
        if Widgets[update.tag] then
            if DEBUG_OBJECTIVES then
                LOG('updating ',Widgets[update.tag].ObjData.title,' ',update.tag)
            end
            # apply update
            if update.updateField == 'target' then
                if DEBUG_OBJECTIVES then
                    LOG('   target = ', repr(update.updateData))
                end
                local objTargets = Widgets[update.tag].ObjData.targets
                local found = false
                for k,v in objTargets do
                    if v.TargetTag == update.updateData.TargetTag then
                        found = true
                    end
                end

                if not found then
                    table.insert( objTargets, update.updateData )
                end

                UpdateTarget(Widgets[update.tag],update.updateData)
            elseif update.updateField == 'complete' then
                if DEBUG_OBJECTIVES then
                    LOG('   complete = ',repr(update.updateData))
                end
                if update.updateData == 'complete' then
                    DisableObjective(update.tag)

                    # string is used by objectiveDetail (not displayed)
                    Widgets[update.tag].Status = 'complete'
                    Widgets[update.tag].EndTime = GetGameTimeSeconds()

                    if DEBUG_OBJECTIVES then
                        LOG('widgets ',update.tag,' complete')
                    end
                elseif update.updateData == 'failed' then
                    DisableObjective(update.tag)

                    # string is used by objectiveDetail (not displayed)
                    Widgets[update.tag].Status = 'failed'
                    Widgets[update.tag].EndTime = GetGameTimeSeconds()

                    if DEBUG_OBJECTIVES then
                        LOG('widgets ',update.tag,' failed')
                    end
                end
            elseif update.updateField == 'title' then
                if DEBUG_OBJECTIVES then
                    LOG('   title = ',repr(update.updateData))
                end
                Widgets[update.tag].ObjData.title = update.updateData
            elseif update.updateField == 'description' then
                if DEBUG_OBJECTIVES then
                    LOG('   description = ',repr(update.updateData))
                end
                Widgets[update.tag].ObjData.description = update.updateData
            elseif update.updateField == 'progress' then
                if DEBUG_OBJECTIVES then
                    LOG('   progress = ',repr(update.updateData))
                end
                Widgets[update.tag].ObjData.progress = update.updateData

                if Widgets[update.tag].progress then
                    if update.updateData and update.updateData != "" then
                        Widgets[update.tag].progress:SetText( LOC(update.updateData) )
                        Widgets[update.tag].progress:Show()
                    else
                        Widgets[update.tag].progress:Hide()
                    end
                end
            else
                WARN("Unknown objective update field: ",update.updateField)
            end
        end
    end

    SetLayout(UIUtil.currentLayout)

    if DEBUG_OBJECTIVES then
        LOG('Widget keys after update = ')
        for k,v in Widgets do
            LOG('key = ',k)
        end
    end
end

function DoBottomLayout()
    if not Widgets then
        return
    end

    local prev = nil
    local layout = UIUtil.SkinnableFile('/dialogs/objective-unit/objective-unit_layout.lua')
    local layoutData = import(layout()).layout
    for _,key in table.keys(Widgets) do
        local widget = Widgets[key]

        if not widget.animating and widget.Active then
            if prev then
                LayoutHelpers.Below( widget.targetButton, prev, 18 )
            else
                 widget.targetButton.Left:Set( function() return savedParent.Left() + 24 end )
                 widget.targetButton.Top:Set(
                    function()
                        if not MissionText.controls.movieGroupBG or MissionText.controls.movieGroupBG:IsHidden() then
                            return savedParent.Top() + 24
                        else
                            return MissionText.controls.movieGroupBG.Bottom() + 4
                        end
                    end
                )
            end

            widget.targetButton.Width:Set(48)
            widget.targetButton.Height:Set(48)
            widget.targetButton.Depth:Set( function() return widget.frameSm.Depth() + 3 end )

            widget.frame.Left:Set( function() return widget.targetButton.Right() - 12 end )
            widget.frame.Top:Set( function() return widget.targetButton.Top() - 24 end )
            widget.frame.Depth:Set( function() return savedParent.Depth() + 10 end )

            # Make the frame maintain the same top left as the targetbutton
            widget.frameSm.Left:Set( function() return widget.targetButton.Left() - 27 end )
            widget.frameSm.Top:Set( function() return widget.targetButton.Top() - 26 end )
            widget.frameSm.Depth:Set( function() return savedParent.Depth() + 10 end )

            if widget.actionButton then
                LayoutHelpers.AtLeftTopIn( widget.actionButton, widget.targetButton, 0, 0 )
                widget.actionButton.Depth:Set( function() return widget.targetButton.Depth() + 1 end )
                widget.actionButton.Width:Set(24)
                widget.actionButton.Height:Set(24)
            end

            LayoutHelpers.Reset(widget.title)
            widget.title.Top:Set(0)
            widget.title.Bottom:Set(0)
            widget.title.Left:Set(0)
            widget.title.Right:Set(0)

            LayoutHelpers.RelativeTo( widget.title, widget.frame, layout, 'l_title-text', 'panel02_bmp' )
            LayoutHelpers.DimensionsRelativeTo( widget.title, layout, 'l_title-text' )
            widget.title.Depth:Set( function() return widget.frame.Depth() + 1 end )
            widget.title:SetText( LOC(widget.ObjData.title) )

            LayoutHelpers.RelativeTo( widget.progress, widget.frame, layout, 'l_org_text', 'panel02_bmp', -4, 0 )
            LayoutHelpers.DimensionsRelativeTo( widget.progress, layout, 'l_org_text')
            widget.progress.Depth:Set( function() return widget.frame.Depth() + 1 end )
            widget.progress:SetText( LOC(widget.ObjData.progress) )

            LayoutHelpers.RelativeTo( widget.description, widget.frame, layout, 'l_body-text', 'panel02_bmp' )
            LayoutHelpers.DimensionsRelativeTo( widget.description, layout, 'l_body-text' )
            widget.description:SetText( LOC(widget.ObjData.description) or "" )

            if widget.isOpen then
                widget.frame:Show()
            else
                widget.frame:Hide()
            end

            prev = widget.targetButton
        end
    end
end

function DoRightLayout()
    DoBottomLayout()
end

function DoLeftLayout()
    DoBottomLayout()
end

function SetLayout(layout)
    if layout == 'bottom' then
        DoBottomLayout()
    elseif layout == 'left' then
        DoLeftLayout()
    elseif layout == 'right' then
        DoRightLayout()
    end
end

function SetupObjectivesLayout(mapGroup)
    savedParent = mapGroup
    Widgets = {}

    if pendingObjectives then
        AddObjectives(pendingObjectives)
    end

    # This sucks, but I can't think of a better way to dynamically position
    # the objectives. We'll create a group parented to the MissionText's
    # Movie group control. When the parent gets hidden, so does our control.
    # We then listen to the OnHide event to redo layout.
    local group = Group(MissionText.controls.movieGroup)
    group.Left:Set(MissionText.controls.movieGroup.Left)
    group.Top:Set(MissionText.controls.movieGroup.Top)
    group.Width:Set(MissionText.controls.movieGroup.Width)
    group.Height:Set(MissionText.controls.movieGroup.Height)
    group.OnHide = function(self)
        SetLayout(UIUtil.currentLayout)
    end
end
