--*****************************************************************************
--* File: lua/modules/ui/game/options.lua
--* Author: Chris Blackwell
--* Summary: Creates the options panel and manages the options dialog
--*
--* Copyright  2006 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local Button = import('/lua/modules/maui/button.lua').Button
local Checkbox = import('/lua/modules/maui/checkbox.lua').Checkbox
local Group = import('/lua/modules/maui/group.lua').Group
local TooltipInfo = import('/lua/modules/ui/help/tooltips.lua')
local Prefs = import('/lua/modules/ui/prefs.lua')
local Tooltip = import('/lua/modules/ui/game/tooltip.lua')
local Keymapping = import('/lua/keymap/defaultKeyMap.lua').defaultKeyMap
local GameMain = import('/lua/modules/ui/game/gamemain.lua')

local pauseGlow = false

local mouseoverDisplay = false
local pauseButton = false

local alpha = 0
local ascending = true

local pauseString = "Pause"
local objectivesButton = false
local scoreButton = false
local inboxButton = false
local diplomacyButton = false

local pauseBool = false

function SetPauseButton(state, isOwner)
    pauseButton:SetCheck(state, true)
    if state then
        pauseString = "Play"
    else
        pauseString = "Pause"
    end
    if mouseoverDisplay then
        if pauseBool then
            mouseoverDisplay:SetText(pauseString)
        end
    end
    if state and not isOwner then
        pauseButton:Disable()
    else
        pauseButton:Enable()
    end
end

function PauseGame(state)
    if state != nil then
        if state then
            SessionRequestPause()
        else
            SessionResume()
        end
    else
        if GameMain.isPaused then
            SessionResume()
        else
            SessionRequestPause()
        end
    end
end

function CreateMouseoverDisplay(parent, ID, delay)
    if mouseoverDisplay then
        mouseoverDisplay:Destroy()
        mouseoverDisplay = false
    end

    if not Prefs.GetOption('tooltips') then return end

    local createDelay = delay or 0
    local totalTime = 0
    local alpha = 0.0
    local text = TooltipInfo['Tooltips'][ID]['title'] or ID
    if TooltipInfo['Tooltips'][ID]['keyID'] and TooltipInfo['Tooltips'][ID]['keyID'] != "" then
        for i, v in Keymapping do
            if v.stringkey == TooltipInfo['Tooltips'][ID]['keyID'] then
                text = LOCF("%s (%s)", text, i)
                break
            end
        end
    end
    mouseoverDisplay = Tooltip.CreateToolTip(parent, text)
    mouseoverDisplay.Top:Set(parent.Top)
    LayoutHelpers.AtHorizontalCenterIn(mouseoverDisplay, parent)
    mouseoverDisplay:SetAlpha(alpha, true)
    mouseoverDisplay:SetNeedsFrameUpdate(true)
    mouseoverDisplay.OnFrame = function(self, deltaTime)
        if totalTime > createDelay then
            if parent then
                if alpha < 1 then
                    mouseoverDisplay:SetAlpha(alpha, true)
                    alpha = alpha + (deltaTime * 2)
                else
                    mouseoverDisplay:SetNeedsFrameUpdate(false)
                end
            else
                WARN("NO PARENT SPECIFIED FOR TOOLTIP")
            end
        end
        totalTime = totalTime + deltaTime
    end
end

function SetGlowAlpha(control, deltaTime)
    if ascending then
        alpha = alpha + (deltaTime * 2)
    else
        alpha = alpha - (deltaTime * 2)
    end
    if alpha > .4 then
        alpha = .4
        ascending = false
    end
    if alpha < 0 then
        alpha = 0.0
        ascending = true
    end
    pauseGlow:SetAlpha(alpha)
end

function DestroyMouseoverDisplay()
    mouseoverDisplay:Destroy()
    mouseoverDisplay = false
end

function CreateOptionsPanel(parent, massGroup, energyGroup)
    local bg = Bitmap(parent, UIUtil.UIFile('/game/resources/center02_bmp_m.dds'))
    LayoutHelpers.AtTopIn(bg, parent, -2)
    LayoutHelpers.AtHorizontalCenterIn(bg, parent)

    local leftBg = Bitmap(parent, UIUtil.UIFile('/game/resources/center_bmp_l.dds'))
    leftBg.Top:Set(function() return bg.Top() + 7 end)
    leftBg.Right:Set(function() return bg.Left() end)
    leftBg.Left:Set(function() return massGroup.Right() - 17 end)
    leftBg.Width:Set(function() return leftBg.Right() - leftBg.Left() end)
    leftBg.Depth:Set(50)
    leftBg:SetTiled(true)

    local rightBg = Bitmap(parent, UIUtil.UIFile('/game/resources/center_bmp_r.dds'))
    rightBg.Top:Set(function() return bg.Top() + 7 end)
    rightBg.Left:Set(function() return bg.Right() end)
    rightBg.Right:Set(function() return energyGroup.Left() + 15 end)
    rightBg.Width:Set(function() return rightBg.Right() - rightBg.Left() end)
    rightBg.Depth:Set(50)
    rightBg:SetTiled(true)

    local mbGroup = Group(bg, "mbGroup")
    mbGroup.Width:Set(42)
    mbGroup.Height:Set(35)
    mbGroup.Left:Set(function() return leftBg.Right() + 3 end)
    mbGroup.Top:Set(function() return bg.Top() + 8 end)
    local menuButton = UIUtil.CreateButtonStd(mbGroup, '/game/options_btn/menu', "", 12, 0, 0, "NO_SOUND", "UI_Options_Rollover")
    LayoutHelpers.AtCenterIn(menuButton, mbGroup)
    menuButton.OnClick = function(self, modifiers)
        import('/lua/modules/ui/dialogs/exitdialog.lua').CreateDialog()
    end
    menuButton.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            CreateMouseoverDisplay(self, "exit_menu")
        elseif event.Type == 'MouseExit' then
            DestroyMouseoverDisplay()
        end
        Button.HandleEvent(self, event)
    end

    local obGroup = Group(bg, "obGroup")
    obGroup.Width:Set(42)
    obGroup.Height:Set(35)
    obGroup.Left:Set(function() return mbGroup.Right() + 3 end)
    obGroup.Top:Set(mbGroup.Top)
    objectivesButton = UIUtil.CreateButtonStd(obGroup, '/game/options_btn/objectives', "", 12, 0, 0, "NO_SOUND", "UI_Options_Rollover")
    LayoutHelpers.AtCenterIn(objectivesButton, obGroup)
    objectivesButton.OnClick = function(self, modifiers)
        import('/lua/modules/ui/game/objectiveDetail.lua').ToggleDisplay()
    end
    objectivesButton.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            CreateMouseoverDisplay(self, "objectives")
        elseif event.Type == 'MouseExit' then
            DestroyMouseoverDisplay()
        end
        Button.HandleEvent(self, event)
    end

    local sbGroup = Group(bg, "sbGroup")
    sbGroup.Width:Set(42)
    sbGroup.Height:Set(35)
    sbGroup.Left:Set(function() return obGroup.Right() + 4 end)
    sbGroup.Top:Set(mbGroup.Top)
    scoreButton = UIUtil.CreateButtonStd(sbGroup, '/game/options_btn/score', "", 12, 0, 0, "NO_SOUND", "UI_Options_Rollover")
    LayoutHelpers.AtCenterIn(scoreButton, sbGroup)
    scoreButton.OnClick = function(self, modifiers)
        import('/lua/modules/ui/game/score.lua').ToggleScoreControl()
    end
    scoreButton.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            CreateMouseoverDisplay(self, "score")
        elseif event.Type == 'MouseExit' then
            DestroyMouseoverDisplay()
        end
        Button.HandleEvent(self, event)
    end

    local ibGroup = Group(bg, "ibGroup")
    ibGroup.Width:Set(42)
    ibGroup.Height:Set(35)
    ibGroup.Left:Set(function() return sbGroup.Right() + 4 end)
    ibGroup.Top:Set(mbGroup.Top)
    inboxButton = UIUtil.CreateButtonStd(ibGroup, '/game/options_btn/inbox', "", 12, 0, 0, "NO_SOUND", "UI_Options_Rollover")
    LayoutHelpers.AtCenterIn(inboxButton, ibGroup)
    inboxButton.OnClick = function(self, modifiers)
        import('/lua/modules/ui/game/transmissionlog.lua').ToggleTransmissionLog()
    end
    inboxButton.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            CreateMouseoverDisplay(self, "inbox")
        elseif event.Type == 'MouseExit' then
            DestroyMouseoverDisplay()
        end
        Button.HandleEvent(self, event)
    end

    local dbGroup = Group(bg, "dbGroup")
    dbGroup.Width:Set(42)
    dbGroup.Height:Set(35)
    dbGroup.Left:Set(function() return ibGroup.Right() + 4 end)
    dbGroup.Top:Set(mbGroup.Top)
    diplomacyButton = UIUtil.CreateButtonStd(dbGroup, '/game/options_btn/diplomacy', "", 12, 0, 0, "NO_SOUND", "UI_Options_Rollover")
    LayoutHelpers.AtCenterIn(diplomacyButton, dbGroup)
    diplomacyButton.OnClick = function(self, modifiers)
        import('/lua/modules/ui/game/diplomacy.lua').ToggleDialog()
    end
    diplomacyButton.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            CreateMouseoverDisplay(self, "diplomacy")
        elseif event.Type == 'MouseExit' then
            DestroyMouseoverDisplay()
        end
        Button.HandleEvent(self, event)
    end
    if UIUtil.CampaignMode then
        diplomacyButton:Disable()
    end

    local pauseGroup = Group(bg, "pauseGroup")
    pauseGroup.Width:Set(42)
    pauseGroup.Height:Set(35)
    pauseGroup.Left:Set(function() return dbGroup.Right() + 3 end)
    pauseGroup.Top:Set(mbGroup.Top)
    pauseButton = Checkbox(pauseGroup,
        UIUtil.UIFile('/game/options_btn/pause_btn_up.dds'),
        UIUtil.UIFile('/game/options_btn/play_btn_up.dds'),
        UIUtil.UIFile('/game/options_btn/pause_btn_over.dds'),
        UIUtil.UIFile('/game/options_btn/play_btn_over.dds'),
        UIUtil.UIFile('/game/options_btn/pause_btn_dis.dds'),
        UIUtil.UIFile('/game/options_btn/play_btn_dis.dds'),
        "NO_SOUND", "UI_Options_Rollover")
    LayoutHelpers.AtCenterIn(pauseButton, pauseGroup)
    pauseButton.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            CreateMouseoverDisplay(self, "options_"..pauseString)
            pauseBool = true
        elseif event.Type == 'MouseExit' then
            DestroyMouseoverDisplay()
            pauseBool = false
        elseif event.Type == 'ButtonPress' or event.Type == 'ButtonDClick' then
            if self._checkState == 'unchecked' then
                SessionRequestPause()
                self._checkState = 'checked'
            else
                SessionResume()
                self._checkState = 'unchecked'
            end
        end
        return true
    end
    if not pauseGlow then
        pauseGlow = Bitmap(pauseButton, UIUtil.UIFile('/game/options_btn/glow_bmp.dds'))
    end
    LayoutHelpers.AtCenterIn(pauseGlow, pauseButton)
    pauseGlow:SetAlpha(0)
    pauseGlow:DisableHitTest()
    pauseGlow.OnFrame = function(self, deltaTime)
        SetGlowAlpha(self, deltaTime)
    end
end

function TogglePauseIndicator(state)
    if state then
        pauseGlow:SetNeedsFrameUpdate(true)
    else
        if pauseGlow then
            pauseGlow:SetAlpha(0)
            pauseGlow:SetNeedsFrameUpdate(false)
        end
    end
end

function ToggleScore()
    scoreButton.OnClick()
end

function ToggleObjectives()
    objectivesButton.OnClick()
end

function ToggleTransmissions()
    inboxButton.OnClick()
end

function ToggleDiplomacy()
    diplomacyButton.OnClick()
end