--*****************************************************************************
--* File: lua/modules/ui/game/orders.lua
--* Author: Chris Blackwell
--* Summary: Unit orders UI
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Group = import('/lua/modules/maui/group.lua').Group
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local Grid = import('/lua/modules/maui/grid.lua').Grid
local Checkbox = import('/lua/modules/maui/checkbox.lua').Checkbox
local GameCommon = import('/lua/modules/ui/game/gamecommon.lua')
local Button = import('/lua/modules/maui/button.lua').Button
local Tooltip = import('/lua/modules/ui/game/tooltip.lua')
local TooltipInfo = import('/lua/modules/ui/help/tooltips.lua')
local Prefs = import('/lua/modules/ui/prefs.lua')
local Keymapping = import('/lua/keymap/defaultKeyMap.lua').defaultKeyMap

local controls =
{
    mouseoverDisplay = false,
    orderButtonGrid = false,
    bg = false,
    orderGlow = false,
}

-- positioning controls, don't belong to file
local controlClusterGroup = false
local mfdControl = false
local layoutVar = false
local glowThread = false

-- these variables control the number of slots available for orders
-- though they are fixed, the code is written so they could easily be made soft
local numSlots = 12
local firstAltSlot = 7
local vertRows = 3
local horzRows = 4
local vertCols = numSlots/vertRows
local horzCols = numSlots/horzRows

local function CreateOrderGlow(parent)
    controls.orderGlow = Bitmap(parent, UIUtil.UIFile('/game/orders/glow-02_bmp.dds'))
    LayoutHelpers.AtCenterIn(controls.orderGlow, parent)
    controls.orderGlow:SetAlpha(0.0)
    controls.orderGlow:DisableHitTest()
    controls.orderGlow:SetNeedsFrameUpdate(true)
    local alpha = 0.0
    local incriment = true
    controls.orderGlow.OnFrame = function(self, deltaTime)
        if incriment then
            alpha = alpha + (deltaTime * 2)
        else
            alpha = alpha - (deltaTime * 2)
        end
        if alpha < 0 then
            alpha = 0.0
            incriment = true
        end
        if alpha > .4 then
            alpha = .4
            incriment = false
        end
        controls.orderGlow:SetAlpha(alpha)
    end     
end

function CreateMouseoverDisplay(parent, ID)
    if controls.mouseoverDisplay then
        controls.mouseoverDisplay:Destroy()
        controls.mouseoverDisplay = false
    end
    
    if not Prefs.GetOption('tooltips') then return end
    
    local text = TooltipInfo['Tooltips'][ID]['title'] or ID
    local desc = TooltipInfo['Tooltips'][ID]['description'] or ID
    
    if TooltipInfo['Tooltips'][ID]['keyID'] and TooltipInfo['Tooltips'][ID]['keyID'] != "" then
        for i, v in Keymapping do
            if v.stringkey == TooltipInfo['Tooltips'][ID]['keyID'] then
                text = LOCF("%s (%s)", text, i)
                break
            end
        end
    end
    
    controls.mouseoverDisplay = Tooltip.CreateExtendedToolTip(parent, text, desc)
    local Frame = GetFrame(0)
    controls.mouseoverDisplay.Bottom:Set(parent.Top)
    if (parent.Left() + (parent.Width() / 2)) - (controls.mouseoverDisplay.Width() / 2) < 0 then
        controls.mouseoverDisplay.Left:Set(4)
    elseif (parent.Right() - (parent.Width() / 2)) + (controls.mouseoverDisplay.Width() / 2) > Frame.Right() then
        controls.mouseoverDisplay.Right:Set(function() return Frame.Right() - 4 end)
    else
        LayoutHelpers.AtHorizontalCenterIn(controls.mouseoverDisplay, parent)
    end
    
    local alpha = 0.0
    controls.mouseoverDisplay:SetAlpha(alpha, true)
    local mdThread = ForkThread(function()
        while alpha <= 1.0 do
            controls.mouseoverDisplay:SetAlpha(alpha, true)
            alpha = alpha + 0.1
            WaitSeconds(0.01)
        end
    end)

    controls.mouseoverDisplay.OnDestroy = function(self)
        KillThread(mdThread)  
    end
end

local function CreateOrderButtonGrid()
    controls.orderButtonGrid = Grid(controls.bg, GameCommon.iconWidth, GameCommon.iconHeight)
    controls.orderButtonGrid:SetName("Orders Grid")
	controls.orderButtonGrid:DeleteAll()
end

local function DoBottomLayout()
	if controls.mouseoverDisplay then
		controls.mouseoverDisplay:Destroy()
		controls.mouseoverDisplay = false
	end

    if not controls.bg then
        controls.bg = Bitmap(controlClusterGroup)
    end
    controls.bg:SetTexture(UIUtil.SkinnableFile('/game/orders_bmp/orders_bmp.dds'))
    LayoutHelpers.AtRightIn(controls.bg, controlClusterGroup, 6)
    LayoutHelpers.AtTopIn(controls.bg, controlClusterGroup, -3)
    LayoutHelpers.ResetLeft(controls.bg)

    if not controls.orderButtonGrid then
        CreateOrderButtonGrid()
    end
    controls.orderButtonGrid.Width:Set(GameCommon.iconWidth * horzCols)
    controls.orderButtonGrid.Height:Set(GameCommon.iconHeight * horzRows)
    LayoutHelpers.AtCenterIn(controls.orderButtonGrid, controls.bg)
    controls.orderButtonGrid:AppendRows(horzRows)
    controls.orderButtonGrid:AppendCols(horzCols)
end

local function DoLeftLayout()
	if controls.mouseoverDisplay then
		controls.mouseoverDisplay:Destroy()
		controls.mouseoverDisplay = false
	end

    if not controls.bg then
        controls.bg = Bitmap(controlClusterGroup)
    end
    
    controls.bg:SetTexture(UIUtil.SkinnableFile('/game/orders-vertical_bmp/orders-vertical_bmp.dds'))
    LayoutHelpers.Below(controls.bg, mfdControl)
    LayoutHelpers.ResetRight(controls.bg)

    if not controls.orderButtonGrid then
        CreateOrderButtonGrid()
    end
    controls.orderButtonGrid.Width:Set(GameCommon.iconWidth * vertCols)
    controls.orderButtonGrid.Height:Set(GameCommon.iconHeight * vertRows)
    LayoutHelpers.AtCenterIn(controls.orderButtonGrid, controls.bg)
    controls.orderButtonGrid:AppendRows(vertRows)
    controls.orderButtonGrid:AppendCols(vertCols)
end

local function DoRightLayout()
	if controls.mouseoverDisplay then
		controls.mouseoverDisplay:Destroy()
        controls.mouseoverDisplay = false
	end

    if not controls.bg then
        controls.bg = Bitmap(controlClusterGroup)
    end
    controls.bg:SetTexture(UIUtil.SkinnableFile('/game/orders-vertical_bmp/orders-vertical_bmp.dds'))
    LayoutHelpers.Below(controls.bg, mfdControl)
    LayoutHelpers.ResetRight(controls.bg)

    if not controls.orderButtonGrid then
        CreateOrderButtonGrid()
    end
    controls.orderButtonGrid.Width:Set(GameCommon.iconWidth * vertCols)
    controls.orderButtonGrid.Height:Set(GameCommon.iconHeight * vertRows)
    LayoutHelpers.AtCenterIn(controls.orderButtonGrid, controls.bg)
    controls.orderButtonGrid:AppendRows(vertRows)
    controls.orderButtonGrid:AppendCols(vertCols)
end

function SetLayout(layout)
    layoutVar = layout
    if layout == 'bottom' then
        DoBottomLayout()
    elseif layout == 'left' then
        DoLeftLayout()
    elseif layout == 'right' then
        DoRightLayout()
    end
end

-- local logic data
local orderCheckboxMap = false
local currentSelection = false

-- helper function to create order bitmaps
-- note, your bitmaps must be in /game/orders/ and have the standard button naming convention
local function GetOrderBitmapNames(bitmapId)
    if bitmapId == nil then
        LOG("Error - nil bitmap passed to GetOrderBitmapNames")
        bitmapId = "basic-empty"    --TODO do I really want to default it?
    end
    
    local button_prefix = "/game/orders/" .. bitmapId .. "_btn_"
    return UIUtil.SkinnableFile(button_prefix .. "up.dds")
        ,  UIUtil.SkinnableFile(button_prefix .. "up_sel.dds")
        ,  UIUtil.SkinnableFile(button_prefix .. "over.dds")
        ,  UIUtil.SkinnableFile(button_prefix .. "over_sel.dds")
        ,  UIUtil.SkinnableFile(button_prefix .. "dis.dds")
        ,  UIUtil.SkinnableFile(button_prefix .. "dis_sel.dds")
        , "UI_Action_MouseDown", "UI_Action_Rollover"   -- sets click and rollover cues
end

-- used by most orders, which start and stop a command mode, so they toggle on when pressed
-- and toggle off when done
local function StandardOrderBehavior(self, modifiers)
    -- if we're checked, end the current command mode, otherwise start it
    if self:IsChecked() then
        import('/lua/modules/ui/game/commandmode.lua').EndCommandMode(true)
    else
        import('/lua/modules/ui/game/commandmode.lua').StartCommandMode("order", self._order)
    end
end

-- used by orders that happen immediately and don't change the command mode (ie the stop button)
local function MomentaryOrderBehavior(self, modifiers)
    IssueCommand(GetUnitCommandFromCommandCap(self._order))
    self:SetCheck(false)
end

-- used by things that build weapons, etc
local function BuildOrderBehavior(self, modifiers)
    if modifiers.Left then
	    IssueCommand(GetUnitCommandFromCommandCap(self._order))
    elseif modifiers.Right then
	    self:ToggleCheck()
	    self._autoIndicator:SetHidden(not self:IsChecked())
        if self:IsChecked() then
            self._curHelpText = self._data.helpText .. " (Auto)"
        else
            self._curHelpText = self._data.helpText
        end
        if controls.mouseoverDisplay.text then
            controls.mouseoverDisplay.text:SetText(self._curHelpText)
        end
	    SetAutoMode(currentSelection, self:IsChecked())
    end
end

local function BuildInitFunction(control, unitList)
    local isAutoMode = GetIsAutoMode(unitList)
    control:SetCheck(isAutoMode)
    control.OnHide = function(self, hidden)
        return true -- supress hiding children as it screws up auto indicator vis when added to the grid
    end
    control._autoIndicator = Bitmap(control, UIUtil.SkinnableFile("/game/orders/autocast_bmp.dds"))
    control._autoIndicator._hiddenState = not isAutoMode
    LayoutHelpers.AtCenterIn(control._autoIndicator, control)
    control._autoIndicator:SetHidden(not isAutoMode)
    if isAutoMode then
        control._curHelpText = control._data.helpText .. " (Auto)"
    else
        control._curHelpText = control._data.helpText
    end
end

-- pause button specific behvior
-- TODO pause button will be moved to construction manager
local function PauseOrderBehavior(self, modifiers)
	Checkbox.OnClick(self)
	SetPaused(currentSelection, self:IsChecked())
end

local function PauseInitFunction(control, unitList)
    control:SetCheck(GetIsPaused(unitList))
end

function TogglePauseState()
    local pauseState = GetIsPaused(currentSelection)
    SetPaused(currentSelection, not pauseState)
end

-- generic script button specific behvior
local function ScriptButtonOrderBehavior(self, modifiers)
	ToggleScriptBit(currentSelection, self._data.extraInfo, self:IsChecked())
	Checkbox.OnClick(self)
    if controls.mouseoverDisplay.text then
        controls.mouseoverDisplay.text:SetText(self._curHelpText)
    end
end

local function ScriptButtonInitFunction(control, unitList)
    control:SetCheck(GetScriptBit(unitList, control._data.extraInfo))
end

-- retaliate button specific behvior
local retaliateStateInfo = {
    [-1] = {bitmap = 'stand-ground',    helpText = "mode_mixed",},
    [0] = { bitmap = 'return-fire',     helpText = "mode_return_fire",},
    [1] = { bitmap = 'hold-fire',       helpText = "mode_hold",},
}

local function RetaliateOrderBehavior(self, modifiers)
	ToggleFireState(currentSelection, self._toggleState)
	
	self._toggleState = self._toggleState + 1
	if self._toggleState >= 3 then
	    self._toggleState = 0
	end
	
    if not retaliateStateInfo[self._toggleState] then
        LOG("Error: orders.lua - invalid toggle state: ", tostring(self._toggleState))
    end
    self:SetNewTextures(GetOrderBitmapNames(retaliateStateInfo[self._toggleState].bitmap))
    self._curHelpText = retaliateStateInfo[self._toggleState].helpText
    if controls.mouseoverDisplay.text then
        controls.mouseoverDisplay.text:SetText(self._curHelpText)
    end
end

local function RetaliateInitFunction(control, unitList)
    control._toggleState = GetFireState(unitList)
    if not retaliateStateInfo[control._toggleState] then
        LOG("Error: orders.lua - invalid toggle state: ", tostring(self._toggleState))
    end
    control:SetNewTextures(GetOrderBitmapNames(retaliateStateInfo[control._toggleState].bitmap))
    control._curHelpText = retaliateStateInfo[control._toggleState].helpText
end

function CycleRetaliateStateUp()
    local currentFireState = GetFireState(currentSelection)
    if currentFireState > 2 then
        currentFireState = 0
    end
    ToggleFireState(currentSelection, currentFireState)
end

local function pauseFunc()
    import('/lua/modules/ui/game/queue.lua').EnablePauseToggle()
end

-- sets up an orderInfo for each order that comes in
-- preferredSlot is custom data that is used to determine what slot the order occupies
-- initialStateFunc is a function that gets called once the control is created and allows you to set the initial state of the button
--      the function should have this declaration: function(checkbox, unitList)
-- extraInfo is used for storing any extra information required in setting up the button
local standardOrdersTable = {
    -- Common rules
    RULEUCC_Move = {                helpText = "move",          bitmapId = 'move',                  preferredSlot = 1,  behavior = StandardOrderBehavior,},
    RULEUCC_Attack = {              helpText = "attack",        bitmapId = 'attack',                preferredSlot = 2,  behavior = StandardOrderBehavior, },
    RULEUCC_Patrol = {              helpText = "patrol",        bitmapId = 'patrol',                preferredSlot = 3,  behavior = StandardOrderBehavior, },
    RULEUCC_Stop = {                helpText = "stop",          bitmapId = 'stop',                  preferredSlot = 4,  behavior = MomentaryOrderBehavior, },
    RULEUCC_Guard = {               helpText = "assist",        bitmapId = 'guard',                 preferredSlot = 5,  behavior = StandardOrderBehavior, },
    RULEUCC_RetaliateToggle = {     helpText = "mode",          bitmapId = 'stand-ground',          preferredSlot = 6,  behavior = RetaliateOrderBehavior,      initialStateFunc = RetaliateInitFunction, },

    -- Unit specific rules
    RULEUCC_SiloBuildTactical = {   helpText = "build_tactical",bitmapId = 'silo-build-tactical',   preferredSlot = 7,  behavior = BuildOrderBehavior,          initialStateFunc = BuildInitFunction,},
    RULEUCC_SiloBuildNuke = {       helpText = "build_nuke",    bitmapId = 'silo-build-nuke',       preferredSlot = 7,  behavior = BuildOrderBehavior,          initialStateFunc = BuildInitFunction,},
    RULEUCC_Overcharge = {          helpText = "overcharge",    bitmapId = 'overcharge',            preferredSlot = 8,  behavior = StandardOrderBehavior,},
    RULEUCC_Transport = {           helpText = "transport",     bitmapId = 'unload',                preferredSlot = 8,  behavior = StandardOrderBehavior, },
    RULEUCC_Nuke = {                helpText = "fire_nuke",     bitmapId = 'nuke',                  preferredSlot = 9,  behavior = StandardOrderBehavior, },
    RULEUCC_Tactical = {            helpText = "fire_tactical", bitmapId = 'nuke',                  preferredSlot = 9,  behavior = StandardOrderBehavior, },
    RULEUCC_Teleport = {            helpText = "teleport",      bitmapId = 'teleport',              preferredSlot = 9,  behavior = StandardOrderBehavior, },
    RULEUCC_Ferry = {               helpText = "ferry",         bitmapId = 'ferry',                 preferredSlot = 9,  behavior = StandardOrderBehavior, },
    RULEUCC_Sacrifice = {           helpText = "sacrifice",     bitmapId = 'sacrifice',             preferredSlot = 9,  behavior = StandardOrderBehavior,},
    RULEUCC_Dive = {                helpText = "dive",          bitmapId = 'dive',                  preferredSlot = 10, behavior = MomentaryOrderBehavior,},   
    RULEUCC_Reclaim = {             helpText = "reclaim",       bitmapId = 'reclaim',               preferredSlot = 10, behavior = StandardOrderBehavior, },
    RULEUCC_Capture = {             helpText = "capture",       bitmapId = 'convert',               preferredSlot = 11, behavior = StandardOrderBehavior,},
    RULEUCC_Repair = {              helpText = "repair",        bitmapId = 'repair',                preferredSlot = 12, behavior = StandardOrderBehavior,},

    -- Unit toggle rules
    RULEUCC_ShieldToggle = {        helpText = "toggle_shield",     bitmapId = 'pause',             preferredSlot = 7,  behavior = ScriptButtonOrderBehavior,   initialStateFunc = ScriptButtonInitFunction, extraInfo = 0,},
    RULEUCC_WeaponToggle = {        helpText = "toggle_weapon",     bitmapId = 'pause',             preferredSlot = 7,  behavior = ScriptButtonOrderBehavior,   initialStateFunc = ScriptButtonInitFunction, extraInfo = 1,},    
    RULEUCC_JammingToggle = {       helpText = "toggle_jamming",    bitmapId = 'pause',             preferredSlot = 8,  behavior = ScriptButtonOrderBehavior,   initialStateFunc = ScriptButtonInitFunction, extraInfo = 2,},
    RULEUCC_IntelToggle = {         helpText = "toggle_intel",      bitmapId = 'pause',             preferredSlot = 8,  behavior = ScriptButtonOrderBehavior,   initialStateFunc = ScriptButtonInitFunction, extraInfo = 3,},
    RULEUCC_ProductionToggle = {    helpText = "toggle_production", bitmapId = 'pause',             preferredSlot = 9,  behavior = ScriptButtonOrderBehavior,   initialStateFunc = ScriptButtonInitFunction, extraInfo = 4,},
    RULEUCC_StealthToggle = {       helpText = "toggle_stealth",    bitmapId = 'pause',             preferredSlot = 9,  behavior = ScriptButtonOrderBehavior,   initialStateFunc = ScriptButtonInitFunction, extraInfo = 5,},
    RULEUCC_GenericToggle = {       helpText = "toggle_generic",    bitmapId = 'pause',             preferredSlot = 10, behavior = ScriptButtonOrderBehavior,   initialStateFunc = ScriptButtonInitFunction, extraInfo = 6,},
    RULEUCC_SpecialToggle = {       helpText = "toggle_special",    bitmapId = 'pause',             preferredSlot = 11, behavior = ScriptButtonOrderBehavior,   initialStateFunc = ScriptButtonInitFunction, extraInfo = 7,},
    RULEUCC_CloakToggle = {         helpText = "toggle_cloak",      bitmapId = 'pause',             preferredSlot = 11, behavior = ScriptButtonOrderBehavior,   initialStateFunc = ScriptButtonInitFunction, extraInfo = 8,},
}

local specialOrdersTable = {
    RULEUCC_Pause = {behavior = pauseFunc},
}


-- this is a used as a set
local commonOrders = {
    RULEUCC_Move = true,
    RULEUCC_Attack = true,
    RULEUCC_Patrol = true,
    RULEUCC_Stop = true,
    RULEUCC_Guard = true, 
    RULEUCC_RetaliateToggle = true, 
}

--[[
Add an order to a particular slot, destroys what's currently in the slot if anything
Returns checkbox if you need to add any data to the structure

The orderInfo format is:
{
    helpText = <string>,    -- 
    bitmapId = <string>,    -- the id used to construct the bitmap name (see GetOrderBitmapNames above)
    disabled = <bool>,      -- if true, button will start disabled
    behavior = <function>,  -- function(self, modifiers) this is the checkbox OnClick behavior
}

Since this is a table, if you need any more information, for instance, the command to be emmited from the OnClick
you can add it to the table and it will be ignored, so you're safe to put whatever info you need in to it. When the
OnClick callback is called, self._data will contain this info.
--]]

local function AddOrder(orderInfo, slot, batchMode)
    batchMode = batchMode or false
    
    local checkbox = Checkbox(controls.orderButtonGrid, GetOrderBitmapNames(orderInfo.bitmapId))

    -- set the info in to the data member for retrieval
    checkbox._data = orderInfo
    
    -- set disabled state if needed
    if orderInfo.disabled then
        checkbox:Disable()
    end
    
    -- set up initial help text
    checkbox._curHelpText = orderInfo.helpText

    -- set up click handler
    checkbox.OnClick = orderInfo.behavior

    -- set up tooltips
    checkbox.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            if controls.orderGlow then
                controls.orderGlow:Destroy()
                controls.orderGlow = false
            end                
            CreateMouseoverDisplay(self, self._curHelpText, 1)
            glowThread = CreateOrderGlow(self)
        elseif event.Type == 'MouseExit' then
            if controls.mouseoverDisplay then
                controls.mouseoverDisplay:Destroy()
                controls.mouseoverDisplay = false
            end
            if controls.orderGlow then
                controls.orderGlow:Destroy()
                controls.orderGlow = false
                KillThread(glowThread)
            end
        end
        Checkbox.HandleEvent(self, event)
    end

    -- calculate row and column, remove old item, add new checkbox
    local cols, rows = controls.orderButtonGrid:GetDimensions()
    local row = math.ceil(slot / cols)
    local col = math.mod(slot - 1, cols) + 1
    controls.orderButtonGrid:DestroyItem(col, row, batchMode)
    controls.orderButtonGrid:SetItem(checkbox, col, row, batchMode)    

    return checkbox
end

-- creates the buttons for the common orders, and then disables them if they aren't in the order set
local function CreateCommonOrders(availableOrders)
    for key in commonOrders do
        local orderInfo = standardOrdersTable[key]
        orderInfo.disabled = true

        local orderCheckbox = AddOrder(orderInfo, orderInfo.preferredSlot, true)
        orderCheckbox._order = key
        orderCheckbox._toggleState = 0

        if orderInfo.initialStateFunc then
            orderInfo.initialStateFunc(orderCheckbox, currentSelection)
        end
      
        orderCheckboxMap[key] = orderCheckbox
    end
    
    for index, availOrder in availableOrders do
        if not standardOrdersTable[availOrder] then continue end   -- skip any orders we don't have in our table
        if commonOrders[availOrder] then
            local ck = orderCheckboxMap[availOrder]
            ck:Enable()
        end
    end
end

-- creates the buttons for the alt orders, placing them as possible
local function CreateAltOrders(availableOrders)
--TODO? it would indeed be easier if the alt orders slot was in the blueprint, but for now try
--to determine where they go by using preferred slots
    -- determine what slots to put alt orders
    -- we first want a table of slots we want to fill, and what orders want to fill them
    local desiredSlot = {}
    for index, availOrder in availableOrders do
        if standardOrdersTable[availOrder] then 
            local preferredSlot = standardOrdersTable[availOrder].preferredSlot
            if not desiredSlot[preferredSlot] then
                desiredSlot[preferredSlot] = {}
            end
            table.insert(desiredSlot[preferredSlot], availOrder)
        else
            if specialOrdersTable[availOrder] != nil then
                specialOrdersTable[availOrder].behavior()
            end
        end
    end

    -- now go through that table and determine what doesn't fit and look for slots that are empty
    -- since this is only alt orders, just deal with slots 7-12
    local orderInSlot = {}
    
    -- go through first time and add all the first entries to their preferred slot
    for slot = firstAltSlot,numSlots do
        if desiredSlot[slot] then
            orderInSlot[slot] = desiredSlot[slot][1]
        end
    end

    -- now put any additional entries wherever they will fit
    for slot = firstAltSlot,numSlots do
        if desiredSlot[slot] and table.getn(desiredSlot[slot]) > 1 then
            for index, item in desiredSlot[slot] do
                if index > 1 then
                    local foundFreeSlot = false
                    for newSlot = firstAltSlot, numSlots do
                        if not orderInSlot[newSlot] then
                            orderInSlot[newSlot] = item
                            foundFreeSlot = true
                            break
                        end
                    end
                    if not foundFreeSlot then
                        WARN("No free slot for order: " .. item)
                        -- could break here, but don't, then you'll know how many extra orders you have
                    end
                end
            end
        end
    end

    -- now map it the other direction so it's order to slot
    local slotForOrder = {}
    for slot, order in orderInSlot do
        slotForOrder[order] = slot
    end
    
    -- create the alt order buttons
    for index, availOrder in availableOrders do
        if not standardOrdersTable[availOrder] then continue end   -- skip any orders we don't have in our table
        if not commonOrders[availOrder] then
            local orderInfo = standardOrdersTable[availOrder]
            orderInfo.disabled = false
            local orderCheckbox = AddOrder(orderInfo, slotForOrder[availOrder], true)

            orderCheckbox._order = availOrder

            if orderInfo.initialStateFunc then
                orderInfo.initialStateFunc(orderCheckbox, currentSelection)
            end

            orderCheckboxMap[availOrder] = orderCheckbox
        end
    end
end

-- called by gamemain when new orders are available, 
function SetAvailableOrders(availableOrders, newSelection)
    -- save new selection
    currentSelection = newSelection
    
    -- clear existing orders
    orderCheckboxMap = {}
    controls.orderButtonGrid:DestroyAllItems(true)
    
    CreateCommonOrders(availableOrders)
    CreateAltOrders(availableOrders)
    
    controls.orderButtonGrid:EndBatch()
end

-- called from gamemain to create control
function SetupOrdersControl(parent, mfd)
    controlClusterGroup = parent
    mfdControl = mfd

    SetLayout(UIUtil.currentLayout)

    -- created greyed out orders on setup
    CreateCommonOrders({})
    
    -- setup command mode behaviors
    import('/lua/modules/ui/game/commandmode.lua').AddStartBehavior(
        function(commandMode, data)
            local orderCheckbox = orderCheckboxMap[data]
            if orderCheckbox then
                orderCheckbox:SetCheck(true)
            end
        end
    )
    import('/lua/modules/ui/game/commandmode.lua').AddEndBehavior(
        function(commandMode, data)
            local orderCheckbox = orderCheckboxMap[data]
            if orderCheckbox then
                orderCheckbox:SetCheck(false)
            end
        end
    )
    
    return controls.bg
end
