--*****************************************************************************
--* File: lua/modules/ui/game/selection.lua
--* Author: Chris Blackwell
--* Summary: Unit selection manager UI
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Group = import('/lua/modules/maui/group.lua').Group
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local Grid = import('/lua/modules/maui/grid.lua').Grid
local Checkbox = import('/lua/modules/maui/checkbox.lua').Checkbox
local Button = import('/lua/modules/maui/button.lua').Button
local GameCommon = import('/lua/modules/ui/game/gamecommon.lua')
local RadioGroup = import('/lua/modules/maui/mauiutil.lua').RadioGroup
local Tooltip = import('/lua/modules/ui/game/tooltip.lua')

local mfdControl = false
local ordersControl = false
local queueControl = false
local controlClusterGroup = false
local currentSelection = false

local controls = {
    panel = false,
    panelTopLeft = false,
    panelTop = false,
    panelTopRight = false,
    panelLeft = false,
    panelRight = false,
    panelBottomLeft = false,
    panelBottom = false,
    panelBottomRight = false,
    iconGrid = false,
    panelLogo = false,
}    

function PlaySelectionSound(newSelection)
    for k, v in newSelection do
        local bp = v:GetBlueprint()
        if bp.Audio.UISelection then
            PlaySound(bp.Audio.UISelection)
            return
        end
    end
end

local selectionSets = {}
local selectionSetCallbacks = {}

-- add a function to get called when a selection set changes
-- with parameters function(name, unitArray, applied)
-- name: name of the selection set
-- unitArray: the actual array of units
-- applied: true if the set was applied, false if it was just created
function RegisterSelectionSetCallback(func)
    -- see if this function is already in here
    for i,v in selectionSetCallbacks do
        if v == func then
            return
        end
    end

    table.insert(selectionSetCallbacks(func))
end

function WithdrawSelectionSetCallback(func)
    for i,v in selectionSetCallbacks do
        if v == func then
            table.remove(selectionSetCallbacks, i)
            return
        end
    end
end

-- add a selection set based on the current selection
function AddCurrentSelectionSet(name)
    AddSelectionSet(name, GetSelectedUnits())
end

-- add a selection set based on an array of units
-- if selectedUnits is nil, clears the selection set
function AddSelectionSet(name, unitArray)
    -- remove units from the selection set if it already exists
    if selectionSets[name] then
        for index, unit in selectionSets[name] do
            unit:RemoveSelectionSet(name)
        end
    end
    
    selectionSets[name] = unitArray

    -- add new units to selection set (unitArray could be nil, so check first)
    if selectionSets[name] then
        for index, unit in selectionSets[name] do
            unit:AddSelectionSet(name)
        end
    end
    
    for i,v in selectionSetCallbacks do
        v(name, unitArray, false)
    end
end

-- select a specified selection set in the session
function ApplySelectionSet(name)

    # get a filtered list of only valid units back from the function
    selectionSets[name] = SelectUnits(selectionSets[name])
    
    # if we are out of units. just set our set to nil
    if table.getn(selectionSets[name]) == 0 then
        selectionSets[name] = nil
    else        
        for i,v in selectionSetCallbacks do
            v(name, selectionSets[name], true)
        end
    end
end

function SetupSelectionBox(parent, inMFDControl, inOrdersControl, inQueueControl)
    mfdControl = inMFDControl
    ordersControl = inOrdersControl
    queueControl = inQueueControl
    controlClusterGroup = parent
    
    SetLayout(UIUtil.currentLayout)
end

function SelectionManager(selection, newSelection)
    import('/lua/modules/ui/game/unitviewDetail.lua').Hide()
    currentSelection = newSelection
    local tempBuildableUnits = EntityCategoryGetUnitList(selection)
    if table.getsize(newSelection) == 1 then
        local bp = currentSelection[1]:GetBlueprint()
        local attachedList = GetAttachedUnitsList(newSelection)
        if table.getsize(tempBuildableUnits) > 0 or bp.Enhancements != nil then
            import('/lua/modules/ui/game/construction.lua').SetAvailableGridButtons(selection, newSelection, tempBuildableUnits)
            controls.panelTopLeft:Hide()
        elseif table.getsize(attachedList) > 0 and not EntityCategoryContains(categories.FACTORY, newSelection[1]) then
            controls.panelTopLeft:Show()
            SetAttachedIcons(attachedList) 
        else
            ClearQueueGrid()
            controls.panelTopLeft:Show()
        end
    else
        controls.panelTopLeft:Show()      
        --Sort out units by type
        local sortedUnits = {}
        for i, v in currentSelection do
            local blueprintID = v:GetBlueprint().BlueprintId
            if table.getn(sortedUnits) > 0 then
                local newEntry = true
                for h, x in sortedUnits do
                    if x[1] == blueprintID then
                        x[2] = x[2] + 1
                        table.insert(x[3], v)
                        newEntry = false
                    end
                end
                if newEntry then
                    table.insert(sortedUnits, {blueprintID, 1, {v}})
                end
            else
                table.insert(sortedUnits, {blueprintID, 1, {v}})
            end
        end
        local newSortedUnits = {}
        local categoryList = {categories.COMMAND, categories.EXPERIMENTAL, categories.TECH3, categories.TECH2, categories.TECH1,}
        for i, v in categoryList do
            for h, x in sortedUnits do
                if EntityCategoryContains(v, x[3][1]) then
                    table.insert(newSortedUnits, x)
                end
            end
        end
        if table.getsize(sortedUnits) == 1 then 
            if table.getsize(tempBuildableUnits) > 0 then
                import('/lua/modules/ui/game/construction.lua').SetAvailableGridButtons(selection,newSelection, tempBuildableUnits)
                controls.panelTopLeft:Hide()
            else
                SetSelectionIcons(newSortedUnits) 
            end
        else
            SetSelectionIcons(newSortedUnits) 
        end
    end
end

function SetLayout(layout)
    if layout == 'bottom' then
        DoBottomLayout()
    elseif layout == 'left' then
        DoLeftLayout()
    elseif layout == 'right' then
        DoRightLayout()
    end
    if not controls.panelTop then
        controls.panelTop = Bitmap(controls.panelTopLeft, UIUtil.UIFile('/game/generic_brd/generic_brd_horz_um.dds'))
        controls.panelTop:SetSolidColor('FF000000')
    end
    controls.panelTop.Left:Set(controls.panelTopLeft.Right)
    controls.panelTop.Right:Set(controls.panelTopRight.Left)
    controls.panelTop.Top:Set(controls.panelTopLeft.Top)
    
    if not controls.panelBottom then
        controls.panelBottom = Bitmap(controls.panelTopLeft, UIUtil.UIFile('/game/generic_brd/generic_brd_horz_lm.dds'))
        controls.panelBottom:SetSolidColor('FF000000')
    end
    controls.panelBottom.Left:Set(controls.panelBottomLeft.Right)
    controls.panelBottom.Right:Set(controls.panelBottomRight.Left)
    controls.panelBottom.Bottom:Set(controls.panelBottomLeft.Top)
    
    if not controls.panelLeft then
        controls.panelLeft = Bitmap(controls.panelTopLeft, UIUtil.UIFile('/game/generic_brd/generic_brd_vert_l.dds'))
        controls.panelLeft:SetSolidColor('FF000000')
    end
    controls.panelLeft.Top:Set(controls.panelTopLeft.Bottom)
    controls.panelLeft.Left:Set(controls.panelTopLeft.Left)
    controls.panelLeft.Bottom:Set(controls.panelBottomLeft.Top)
    
    if not controls.panelRight then
        controls.panelRight = Bitmap(controls.panelTopLeft, UIUtil.UIFile('/game/generic_brd/generic_brd_vert_r.dds'))
        controls.panelRight:SetSolidColor('FF000000')
    end
    controls.panelRight.Top:Set(controls.panelTopRight.Bottom)
    controls.panelRight.Right:Set(controls.panelTopRight.Right)
    controls.panelRight.Bottom:Set(controls.panelBottomRight.Top)
    
    if not controls.panel then
        controls.panel = Bitmap(controls.panelTopLeft, UIUtil.UIFile('/game/generic_brd/generic_brd_m.dds'))
        controls.panel:SetSolidColor('FF000000')
    end
    
    controls.panel.Left:Set(controls.panelTopLeft.Right)
    controls.panel.Right:Set(controls.panelTopRight.Left)
    controls.panel.Top:Set(controls.panelTopLeft.Bottom)
    controls.panel.Bottom:Set(controls.panelBottomLeft.Top)

    if not controls.iconGrid then
        controls.iconGrid = Grid(controls.panelTopLeft, 62, 62)
        controls.iconGrid:SetName("Selection Grid")
    end
    controls.iconGrid.Left:Set(function() return controls.panel.Left() - 30 end)
    controls.iconGrid.Top:Set(function() return controls.panel.Top() - 30 end)
    controls.iconGrid.Right:Set(controls.panel.Right)
    controls.iconGrid.Bottom:Set(controls.panel.Bottom)
end

function DoLeftLayout()
    if not controls.panelTopLeft then
        controls.panelTopLeft = Bitmap(controlClusterGroup, UIUtil.UIFile('/game/generic_brd/generic_brd_ul.dds'))
        controls.panelTopLeft:SetSolidColor('FF000000')
    end
    controls.panelTopLeft.Top:Set(function() return ordersControl.Bottom() + 2 end)
    controls.panelTopLeft.Left:Set(function() return controlClusterGroup.Left() - 2 end)
    controls.panelTopLeft.Depth:Set(500)
        
    if not controls.panelTopRight then
        controls.panelTopRight = Bitmap(controls.panelTopLeft, UIUtil.UIFile('/game/generic_brd/generic_brd_ur.dds'))
        controls.panelTopRight:SetSolidColor('FF000000')
    end
    controls.panelTopRight.Right:Set(controlClusterGroup.Right)
    controls.panelTopRight.Top:Set(controls.panelTopLeft.Top)
    
    if not controls.panelBottomLeft then
        controls.panelBottomLeft = Bitmap(controls.panelTopLeft, UIUtil.UIFile('/game/generic_brd/generic_brd_ll.dds'))
        controls.panelBottomLeft:SetSolidColor('FF000000')
    end
    controls.panelBottomLeft.Left:Set(controls.panelTopLeft.Left)
    controls.panelBottomLeft.Bottom:Set(function() return controlClusterGroup.Bottom() - 2 end)
    
    if not controls.panelBottomRight then
        controls.panelBottomRight = Bitmap(controls.panelTopLeft, UIUtil.UIFile('/game/generic_brd/generic_brd_lr.dds'))
        controls.panelBottomRight:SetSolidColor('FF000000')
    end
    controls.panelBottomRight.Right:Set(controls.panelTopRight.Right)
    controls.panelBottomRight.Bottom:Set(controls.panelBottomLeft.Bottom)
end

function DoRightLayout()
    if not controls.panelTopLeft then
        controls.panelTopLeft = Bitmap(controlClusterGroup, UIUtil.UIFile('/game/generic_brd/generic_brd_ul.dds'))
        controls.panelTopLeft:SetSolidColor('FF000000')
    end
    controls.panelTopLeft.Top:Set(function() return ordersControl.Bottom() + 2 end)
    controls.panelTopLeft.Left:Set(function() return controlClusterGroup.Left() - 2 end)
    controls.panelTopLeft.Depth:Set(500)
        
    if not controls.panelTopRight then
        controls.panelTopRight = Bitmap(controls.panelTopLeft, UIUtil.UIFile('/game/generic_brd/generic_brd_ur.dds'))
        controls.panelTopRight:SetSolidColor('FF000000')
    end
    controls.panelTopRight.Right:Set(controlClusterGroup.Right)
    controls.panelTopRight.Top:Set(controls.panelTopLeft.Top)
    
    if not controls.panelBottomLeft then
        controls.panelBottomLeft = Bitmap(controls.panelTopLeft, UIUtil.UIFile('/game/generic_brd/generic_brd_ll.dds'))
        controls.panelBottomLeft:SetSolidColor('FF000000')
    end
    controls.panelBottomLeft.Left:Set(controls.panelTopLeft.Left)
    controls.panelBottomLeft.Bottom:Set(function() return controlClusterGroup.Bottom() - 2 end)
    
    if not controls.panelBottomRight then
        controls.panelBottomRight = Bitmap(controls.panelTopLeft, UIUtil.UIFile('/game/generic_brd/generic_brd_lr.dds'))
        controls.panelBottomRight:SetSolidColor('FF000000')
    end
    controls.panelBottomRight.Right:Set(controls.panelTopRight.Right)
    controls.panelBottomRight.Bottom:Set(controls.panelBottomLeft.Bottom)
end

function DoBottomLayout()
    if not controls.panelTopLeft then
        controls.panelTopLeft = Bitmap(controlClusterGroup, UIUtil.UIFile('/game/generic_brd/generic_brd_ul.dds'))
        controls.panelTopLeft:SetSolidColor('FF000000')
    end
    controls.panelTopLeft.Top:Set(function() return mfdControl.Top() + 2 end)
    controls.panelTopLeft.Left:Set(function() return mfdControl.Right() - 2 end)
    controls.panelTopLeft.Depth:Set(500)
    
    if not controls.panelTopRight then
        controls.panelTopRight = Bitmap(controls.panelTopLeft, UIUtil.UIFile('/game/generic_brd/generic_brd_ur.dds'))
        controls.panelTopRight:SetSolidColor('FF000000')
    end
    controls.panelTopRight.Right:Set(ordersControl.Left)
    controls.panelTopRight.Top:Set(controls.panelTopLeft.Top)
    
    if not controls.panelBottomLeft then
        controls.panelBottomLeft = Bitmap(controls.panelTopLeft, UIUtil.UIFile('/game/generic_brd/generic_brd_ll.dds'))
        controls.panelBottomLeft:SetSolidColor('FF000000')
    end
    controls.panelBottomLeft.Left:Set(controls.panelTopLeft.Left)
    controls.panelBottomLeft.Bottom:Set(function() return mfdControl.Bottom() - 2 end)
    
    if not controls.panelBottomRight then
        controls.panelBottomRight = Bitmap(controls.panelTopLeft, UIUtil.UIFile('/game/generic_brd/generic_brd_lr.dds'))
        controls.panelBottomRight:SetSolidColor('FF000000')
    end
    controls.panelBottomRight.Right:Set(ordersControl.Left)
    controls.panelBottomRight.Bottom:Set(controls.panelBottomLeft.Bottom)
end   
            
function SetSelectionIcons(sortedUnitTable)
    ClearQueueGrid()
    local numTypes = table.getn(sortedUnitTable)
    local numIcons = 0
    if numTypes then
        local iconBPID = nil
        local cols, rows = controls.iconGrid:GetDimensions()
        numIcons = table.getn(sortedUnitTable)
        
        PopulateQueueGrid(numIcons)
        
        local i = 1
        for index, v in sortedUnitTable do
            local icon = controls.iconGrid:GetItem(i, 1)
            if not icon then
                WARN("Selection Manager: SetSelectionIcons attempt to get item ", i, " but it's not there.")
            end
            local blueprint = __blueprints[v[1]]
            local iconName, upIconName, downIconName, overIconName = GameCommon.GetCachedUnitIconFileNames(blueprint)
            icon.unitIcon:SetTexture(upIconName)
            icon.text:SetText(v[2])
            icon._position = i
            icon._unitType = v[1]
            icon._units = v[3]
            i = i + 1
            
            icon.HandleEvent = function(self, event)
                if event.Type == 'ButtonPress' or event.Type == 'ButtonDClick' then
                    if event.Modifiers.Left then
                        SelectUnits(self._units)
                    else
                        local ind = 1
                        local newUnitSelectTable = {}
                        for i, v in sortedUnitTable do
                            if self._position != i then
                                for y, z in v[3] do
                                    newUnitSelectTable[ind] = z
                                    ind = ind + 1
                                end
                            end
                        end
                        SelectUnits(newUnitSelectTable)
                    end
                elseif event.Type == 'MouseEnter' then
                    local blueprint = __blueprints[self._unitType]
                    import('/lua/modules/ui/game/unitviewDetail.lua').Show(blueprint, nil, self._unitType)
                elseif event.Type == 'MouseExit' then
                    import('/lua/modules/ui/game/unitviewDetail.lua').Hide()
                end
                Bitmap.HandleEvent(self, event)
            end
        end
    end
        
    controls.iconGrid:EndBatch()
end

function SetAttachedIcons(attachedUnits)
    ClearQueueGrid()
    local numTypes = table.getn(attachedUnits)
    local numIcons = 0
    if numTypes then
        local iconBPID = nil
        local cols, rows = controls.iconGrid:GetDimensions()
        numIcons = table.getn(attachedUnits)
        
        PopulateQueueGrid(numIcons, true)
        
        local i = 1
        for index, v in attachedUnits do
            local blueprintID = v:GetBlueprint().BlueprintId
            local blueprint = __blueprints[blueprintID]
            local icon = controls.iconGrid:GetItem(i, 1)
            local userUnit = v
            if not icon then
                WARN("Selection Manager: SetSelectionIcons attempt to get item ", i, " but it's not there.")
            end
            local iconName, upIconName, downIconName, overIconName = GameCommon.GetCachedUnitIconFileNames(blueprint)
            icon.unitIcon:SetTexture(upIconName)
            icon.unitIcon:SetNewTextures(upIconName, downIconName, overIconName, iconName)
            icon.unitIcon:SetUpAltButtons( downIconName, downIconName, downIconName, downIconName )
            icon.unitIcon._unitType = blueprintID
            i = i + 1
            
            icon.unitIcon.OnClick = function(self, modifiers)
                if modifiers.Left then
                    -- Toggling selection of the entity
                    self:OnAltToggle()
    
                    -- Add or Remove the entity to the session selection
                    if self.mAltToggledFlag then
                        AddToSessionExtraSelectList(userUnit)
                    else
                        RemoveFromSessionExtraSelectList(userUnit)
                    end
                end
            end
            icon.unitIcon.HandleEvent = function(self, event)
                if event.Type == 'MouseEnter' then
                    local blueprint = __blueprints[self._unitType]
                    import('/lua/modules/ui/game/unitviewDetail.lua').Show(blueprint, nil, self._unitType)
                elseif event.Type == 'MouseExit' then
                    import('/lua/modules/ui/game/unitviewDetail.lua').Hide()
                end
                Button.HandleEvent(self, event)
            end
        end
    end
    controls.iconGrid:EndBatch()
end

function CreateQueueButton(parent)
    local bg = Bitmap(parent) -- TODO unit specific BG here
    bg:SetSolidColor('black')
    #bg.Width:Set(GameCommon.iconBmpWidth)
    #bg.Height:Set(GameCommon.iconBmpHeight)
    local icon = Button(bg)
    LayoutHelpers.AtLeftTopIn(icon, bg)
    #LayoutHelpers.FillParent(icon, bg)
    bg.unitIcon = icon
    bg:Hide()

    return bg
end

function CreateQueueIcon(parent)
    local bg = Bitmap(parent) -- TODO unit specific BG here
    bg:SetSolidColor('black')
    #bg.Width:Set(GameCommon.iconBmpWidth)
    #bg.Height:Set(GameCommon.iconBmpHeight)
    local icon = Bitmap(bg)
    LayoutHelpers.AtLeftTopIn(icon, bg)
    #LayoutHelpers.FillParent(icon, bg)
    local text = UIUtil.CreateText(icon, "", 18, UIUtil.bodyFont)
    LayoutHelpers.AtLeftTopIn(text, icon, 5, 5)
    text:SetDropShadow(true)
    text:DisableHitTest()
    text:SetColor('white')
    bg.text = text
    bg.unitIcon = icon
    bg:Hide()

    return bg
end

function PopulateQueueGrid(numIcons, transportBool)
    if numIcons == 0 then
        ClearQueueGrid()
        return
    end
    local cols, rows = controls.iconGrid:GetDimensions()
    if transportBool then
        if numIcons > cols then
            controls.iconGrid:AppendCols(numIcons - cols, true)
            for i = cols + 1, numIcons do
                controls.iconGrid:SetItem(CreateQueueButton(controls.iconGrid), i, 1, true)
            end
        elseif numIcons < cols then
            for i = cols, numIcons + 1, -1 do
                controls.iconGrid:DestroyItem(i, 1, true)
                controls.iconGrid:DeleteCol(i, true)
            end
        end
    else
        if numIcons > cols then
            controls.iconGrid:AppendCols(numIcons - cols, true)
            for i = cols + 1, numIcons do
                controls.iconGrid:SetItem(CreateQueueIcon(controls.iconGrid), i, 1, true)
            end
        elseif numIcons < cols then
            for i = cols, numIcons + 1, -1 do
                controls.iconGrid:DestroyItem(i, 1, true)
                controls.iconGrid:DeleteCol(i, true)
            end
        end
    end
end

function ClearQueueGrid()
    controls.iconGrid:DeleteAndDestroyAll(true)
    controls.iconGrid:AppendRows(1, true)
    controls.iconGrid:EndBatch()
end