--*****************************************************************************
--* File: lua/modules/ui/game/tooltip.lua
--* Author: Ted Snook
--* Summary: Tool Tips
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local Group = import('/lua/modules/maui/group.lua').Group
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local TooltipInfo = import('/lua/modules/ui/help/tooltips.lua')
local Prefs = import('/lua/modules/ui/prefs.lua')
local Button = import('/lua/modules/maui/button.lua').Button
local Edit = import('/lua/modules/maui/edit.lua').Edit
local Keymapping = import('/lua/keymap/defaultKeyMap.lua').defaultKeyMap

local mouseoverDisplay = false
local createThread = false

function CreateMouseoverDisplay(parent, ID, delay, extendedBool, hotkeyID)
    if mouseoverDisplay then
        mouseoverDisplay:Destroy()
        mouseoverDisplay = false
    end
        
    if not Prefs.GetOption('tooltips') then return end
    local createDelay = 0
    if delay and Prefs.GetOption('tooltip_delay') then
        createDelay = math.max(delay, Prefs.GetOption('tooltip_delay'))
    else
        createDelay = Prefs.GetOption('tooltip_delay') or 0
    end
    local totalTime = 0
    local alpha = 0.0
    local text = ""
    local body = ""
    if TooltipInfo['Tooltips'][ID] then
        text = LOC(TooltipInfo['Tooltips'][ID]['title'])
        body = LOC(TooltipInfo['Tooltips'][ID]['description'])
        if TooltipInfo['Tooltips'][ID]['keyID'] and TooltipInfo['Tooltips'][ID]['keyID'] != "" then
            for i, v in Keymapping do
                if v.stringkey == TooltipInfo['Tooltips'][ID]['keyID'] then
                    text = LOCF("%s (%s)", text, i)
                    break
                end
            end
        end
    else
        if extendedBool then
            WARN("No tooltip in table for key: "..ID)
        end
        text = ID
        body = "No Description"
    end
    if extendedBool then
        mouseoverDisplay = CreateExtendedToolTip(parent, text, body)
    else
        mouseoverDisplay = CreateToolTip(parent, text)
    end
    if extendedBool then
        local Frame = GetFrame(0)
        if parent.Top() - mouseoverDisplay.Height() < 0 then
            mouseoverDisplay.Top:Set(function() return parent.Bottom() + 10 end)
        else
            mouseoverDisplay.Bottom:Set(parent.Top)
        end
        if (parent.Left() + (parent.Width() / 2)) - (mouseoverDisplay.Width() / 2) < 0 then
            mouseoverDisplay.Left:Set(parent.Right)
        elseif (parent.Right() - (parent.Width() / 2)) + (mouseoverDisplay.Width() / 2) > Frame.Right() then
            mouseoverDisplay.Right:Set(parent.Left)
        else
            LayoutHelpers.AtHorizontalCenterIn(mouseoverDisplay, parent)
        end
    else
        local Frame = GetFrame(0)
        mouseoverDisplay.Bottom:Set(parent.Top)
        if (parent.Left() + (parent.Width() / 2)) - (mouseoverDisplay.Width() / 2) < 0 then
            mouseoverDisplay.Left:Set(4)
        elseif (parent.Right() - (parent.Width() / 2)) + (mouseoverDisplay.Width() / 2) > Frame.Right() then
            mouseoverDisplay.Right:Set(function() return Frame.Right() - 4 end)
        else
            LayoutHelpers.AtHorizontalCenterIn(mouseoverDisplay, parent)
        end
    end
    mouseoverDisplay:SetAlpha(alpha, true)
    mouseoverDisplay:SetNeedsFrameUpdate(true)
    mouseoverDisplay.OnFrame = function(self, deltaTime)
        if totalTime > createDelay then
            if parent then
                if alpha < 1 then
                    mouseoverDisplay:SetAlpha(alpha, true)
                    alpha = alpha + (deltaTime * 4)
                else
                    mouseoverDisplay:SetNeedsFrameUpdate(false)
                end
            else
                WARN("NO PARENT SPECIFIED FOR TOOLTIP")
            end
        end
        totalTime = totalTime + deltaTime
    end
end

function DestroyMouseoverDisplay()
    if createThread then
        KillThread(createThread)
    end
    if mouseoverDisplay then
        mouseoverDisplay:Destroy()
        mouseoverDisplay = false
    end
end

function CreateToolTip(parent, text)
    local tooltip = UIUtil.CreateText(parent, text, 12, UIUtil.bodyFont)
    tooltip.Depth:Set(function() return parent.Depth() + 10000 end)

    tooltip.bg = Bitmap(tooltip)
    tooltip.bg:SetSolidColor('FF004040')
    tooltip.bg.Depth:Set(function() return tooltip.Depth() - 1 end)
    tooltip.bg.Top:Set(tooltip.Top)
    tooltip.bg.Bottom:Set(tooltip.Bottom)
    tooltip.bg.Left:Set(function() return tooltip.Left() - 2 end)
    tooltip.bg.Right:Set(function() return tooltip.Right() + 2 end)
    
    tooltip.border = Bitmap(tooltip)
    tooltip.border:SetSolidColor('dd000000')
    tooltip.border.Depth:Set(function() return tooltip.bg.Depth() - 1 end)
    tooltip.border.Left:Set(function() return tooltip.bg.Left() - 2 end)
    tooltip.border.Top:Set(function() return tooltip.bg.Top() - 2 end)
    tooltip.border.Right:Set(function() return tooltip.bg.Right() + 2 end)
    tooltip.border.Bottom:Set(function() return tooltip.bg.Bottom() + 2 end)
    
    tooltip:DisableHitTest(true)
    
    return tooltip
end

function CreateExtendedToolTip(parent, text, desc)
    if text != "" or desc != "" then
        local tooltip = Group(parent)
        tooltip.Depth:Set(function() return parent.Depth() + 10000 end)
        tooltip.Width:Set(150)
        
        if text != "" and text != nil then
            tooltip.title = UIUtil.CreateText(tooltip, text, 14, UIUtil.bodyFont)
            tooltip.title.Top:Set(tooltip.Top)
            tooltip.title.Left:Set(tooltip.Left)
                    
            tooltip.bg = Bitmap(tooltip)
            tooltip.bg:SetSolidColor('FF004040')
            tooltip.bg.Depth:Set(function() return tooltip.title.Depth() - 1 end)
            tooltip.bg.Top:Set(tooltip.title.Top)
            tooltip.bg.Bottom:Set(tooltip.title.Bottom)
            tooltip.bg.Left:Set(function() return tooltip.Left() - 2 end)
            tooltip.bg.Right:Set(function() return tooltip.Right() + 2 end)
        end
        
        tooltip.desc = {}
        local tempTable = false
        
        if desc != "" and desc != nil then
            tooltip.desc[1] = UIUtil.CreateText(tooltip, "", 12, UIUtil.bodyFont)
            tooltip.desc[1].Width:Set(tooltip.Width)
            if text == "" and text != nil then
                tooltip.desc[1].Top:Set(tooltip.Top)
                tooltip.desc[1].Left:Set(tooltip.Left)
            else
                LayoutHelpers.Below(tooltip.desc[1], tooltip.title)
            end
            
            local textBoxWidth = tooltip.Width()
            tempTable = import('/lua/modules/maui/text.lua').WrapText(LOC(desc), textBoxWidth,
            function(text)
                return  tooltip.desc[1]:GetStringAdvance(text)
            end)
            
            for i=1, table.getn(tempTable) do
                if i == 1 then
                    tooltip.desc[i]:SetText(tempTable[i])
                else
                    local index = i
                    tooltip.desc[i] = UIUtil.CreateText(tooltip, tempTable[i], 12, UIUtil.bodyFont)
                    tooltip.desc[i].Width:Set(tooltip.desc[1].Width)
                    LayoutHelpers.Below(tooltip.desc[index], tooltip.desc[index-1])
                end
                tooltip.desc[i]:SetColor('FFCCCCCC')
            end
            
            tooltip.extbg = Bitmap(tooltip)
            tooltip.extbg:SetSolidColor('FF000202')
            tooltip.extbg.Depth:Set(function() return tooltip.desc[1].Depth() - 1 end)
            tooltip.extbg.Top:Set(tooltip.desc[1].Top)
            tooltip.extbg.Left:Set(function() return tooltip.Left() - 2 end)
            tooltip.extbg.Right:Set(function() return tooltip.Right() + 2 end)
            tooltip.extbg.Bottom:Set(tooltip.desc[table.getn(tempTable)].Bottom)
        end
        
        
        tooltip.extborder = Bitmap(tooltip)
        tooltip.extborder:SetSolidColor('FF000202')
        if text != "" and text != nil then
            tooltip.extborder.Depth:Set(function() return tooltip.bg.Depth() - 1 end)
            tooltip.extborder.Top:Set(function() return tooltip.bg.Top() - 2 end)
            tooltip.extborder.Left:Set(function() return tooltip.bg.Left() - 2 end)
            tooltip.extborder.Right:Set(function() return tooltip.bg.Right() + 2 end)
        else
            tooltip.extborder.Depth:Set(function() return tooltip.extbg.Depth() - 1 end)
            tooltip.extborder.Top:Set(function() return tooltip.extbg.Top() - 2 end)
            tooltip.extborder.Left:Set(function() return tooltip.extbg.Left() - 2 end)
            tooltip.extborder.Right:Set(function() return tooltip.extbg.Right() + 2 end)
        end
        if desc != "" and desc != nil then
            tooltip.extborder.Bottom:Set(function() return tooltip.extbg.Bottom() + 2 end)
        else
            tooltip.extborder.Bottom:Set(function() return tooltip.bg.Bottom() + 2 end)
        end
        
        tooltip:DisableHitTest(true)
        
        tooltip.Width:Set(function() return math.max(tooltip.title.Width(), 150) end)
        
        if text == "" and text != nil then
            tooltip.Height:Set(function() return (tooltip.desc[1].Height() * table.getn(tempTable)) end)
        elseif desc == "" and desc != nil then
            tooltip.Height:Set(function() return tooltip.title.Height() end)
            tooltip.Width:Set(function() return tooltip.title.Width() end)
        else
            tooltip.Height:Set(function() return tooltip.title.Height() + (tooltip.desc[1].Height() * table.getn(tempTable)) end)
        end
        
        
        return tooltip
    else
        WARN("Tooltip error! Text and description are both empty!  This should not happen.")
    end
end

-- helpers functions to make is simple to add tooltips
function AddButtonTooltip(control, tooltipID, delay)
    local tempdelay = delay or 1
    control.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            CreateMouseoverDisplay(self, tooltipID, tempdelay, true)
        elseif event.Type == 'MouseExit' then
            DestroyMouseoverDisplay()
        end
        Button.HandleEvent(self, event)
    end
end 

function AddEditTooltip(control, tooltipID)
    control.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            CreateMouseoverDisplay(self, tooltipID, 1, true)
        elseif event.Type == 'MouseExit' then
            DestroyMouseoverDisplay()
        end
        Edit.HandleEvent(self, event)
    end
end

function AddControlTooltip(control, tooltipID)
    control.HandleEvent = function(self, event)
        if event.Type == 'MouseEnter' then
            CreateMouseoverDisplay(self, tooltipID, 1, true)
        elseif event.Type == 'MouseExit' then
            DestroyMouseoverDisplay()
        end
    end
end