--*****************************************************************************
--* File: lua/modules/ui/lobby/autolobby.lua
--* Author: Sam Demulling
--* Summary: Autolaunching games from GPGNet.  This is intentionally designed
--* to have no user options as GPGNet is setting them for the player.
--*
--* Copyright  2006 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('/lua/modules/ui/uiutil.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local Group = import('/lua/modules/maui/group.lua').Group
local Edit = import('/lua/modules/maui/edit.lua').Edit
local Text = import('/lua/modules/maui/text.lua').Text
local MenuCommon = import('/lua/modules/ui/menus/menucommon.lua')
local MapPreview = import('/lua/modules/ui/controls/mappreview.lua').MapPreview
local MultiLineText = import('/lua/modules/maui/multilinetext.lua').MultiLineText
local Prefs = import('/lua/modules/ui/prefs.lua')
local OnlineProvider = import('/lua/modules/multiplayer/onlineprovider.lua')
local Combo = import('/lua/modules/ui/controls/combo.lua').Combo
local MapUtil = import('/lua/modules/ui/maputil.lua')
local LobbyComm = import('/lua/modules/ui/lobby/lobbyComm.lua')
local teamOpts = import('/lua/modules/ui/lobby/lobbyOptions.lua').teamOptions
local globalOpts = import('/lua/modules/ui/lobby/lobbyOptions.lua').globalOpts
local gameColors = import('/lua/modules/ui/gameColors.lua').GameColors

local connectdialog = false
local parent = false
local keepAliveThread = false
local localPlayerName = false
local requiredPlayers = false

local connectingDialog = false
local connectionFailedDialog = false

local localPlayerID = false
local gameInfo = {
    GameOptions = {},
    PlayerOptions = {},
    Observers = {},
}

local Strings = LobbyComm.Strings

lobbyComm = false

-- Create a new unconnected lobby.
function CreateLobby(protocol, localPort, desiredPlayerName, localPlayerUID, natTraversalProvider)
    if not parent then parent = UIUtil.CreateScreenGroup(GetFrame(0), "CreateLobby ScreenGroup") end
    -- don't parent background to screen group so it doesn't get destroyed until we leave the menus
    local background = MenuCommon.SetupBackground(GetFrame(0))
    local function OnAbort()
		MenuCommon.MenuCleanup()
        parent:Destroy()
        parent = false
        ExitApplication()
    end
    connectdialog = UIUtil.ShowInfoDialog(parent, Strings.TryingToConnect, Strings.AbortConnect, OnAbort)

    InitLobbyComm(protocol, localPort, desiredPlayerName, localPlayerUID, natTraversalProvider)

    localPlayerName = lobbyComm:GetLocalPlayerName()
end


-- create the lobby as a host
function HostGame(gameName, scenarioFileName, singlePlayer)
    CreateUI()
    lobbyComm.desiredScenario = scenarioFileName
    lobbyComm:HostGame()
end

-- join an already existing lobby
function JoinGame(address, asObserver, playerName, uid)
    CreateUI()

    lobbyComm:JoinGame(address, playerName, uid);
end

function ConnectToPeer(boxedAddress,port,name,boxedID)
    lobbyComm:ConnectToPeer(boxedAddress,port,name,boxedID)
end

function DisconnectFromPeer(boxedID)
    lobbyComm:DisconnectFromPeer(boxedID)
end

function HostTryAddPlayer( senderID, requestedPlayerName, requestedFaction )
    local newSlot = -1

    for i = 1, LobbyComm.maxPlayerSlots do
        if gameInfo.PlayerOptions[i] == nil then
            newSlot = i
            break
        end
    end

    if newSlot == -1 or newSlot > requiredPlayers then
        return
    end

    local playerName = lobbyComm:MakeValidPlayerName(senderID,requestedPlayerName)

    gameInfo.PlayerOptions[newSlot] = LobbyComm.GetDefaultPlayerOptions(playerName)
    gameInfo.PlayerOptions[newSlot].Human = true
    gameInfo.PlayerOptions[newSlot].OwnerID = senderID
    gameInfo.PlayerOptions[newSlot].Faction = requestedFaction

    # figure out a reasonable default color
    for colorIndex,colorVal in gameColors.PlayerColors do
        if IsColorFree(colorIndex) then
            gameInfo.PlayerOptions[newSlot].PlayerColor = colorIndex
            break
        end
    end

    lobbyComm:SendData(
        senderID,
        {
            Type = 'SlotAssigned',
            Slot = newSlot,
            Options = gameInfo.PlayerOptions[newSlot],
        }
    )
end

function GetDesiredFaction()
    if HasCommandLineArg("/aeon") then
        return 2
    elseif HasCommandLineArg("/cybran") then
        return 3
    end
    return 1
end

-- create UI won't typically be called directly by another module
function CreateUI()

	if (connectdialog != false) then
		MenuCommon.MenuCleanup()
		connectdialog:Destroy()
		connectdialog = false
	end

    requiredPlayers = 2
    if (HasCommandLineArg("/players")) then
        local args = GetCommandLineArg("/players", 2)
        requiredPlayers = tonumber(args[1])
        LOG("requiredPlayers was set to: "..requiredPlayers)
    end

-- control layout
    if not parent then parent = UIUtil.CreateScreenGroup(GetFrame(0), "Lobby CreateUI ScreenGroup") end

    local background = MenuCommon.SetupBackground(GetFrame(0))
    --local exitButton = MenuCommon.CreateExitMenuButton(parent, background, "<LOC _Exit>")

    ---------------------------------------------------------------------------
    -- set up map panel
    ---------------------------------------------------------------------------
    local controlGroup = Group(parent, "controlGroup")
    LayoutHelpers.AtCenterIn(controlGroup, parent)
    controlGroup.Width:Set(970)
    controlGroup.Height:Set(670)

    local function OnCancel()
       ExitApplication()
    end
    UIUtil.ShowInfoDialog(controlGroup, "<LOC lobui_0201>Setting up automatch...", "<LOC _Cancel>", OnCancel)
end

function UpdateGame()
    if not lobbyComm:IsHost() then
        return
    end

    if gameInfo.ScenarioFile and (gameInfo.ScenarioFile != "") then
        local scenarioInfo = MapUtil.LoadScenario(gameInfo.ScenarioFile)
        if scenarioInfo and scenarioInfo.map and (scenarioInfo.map != "") then
            OnlineProvider.ChangeMap(scenarioInfo.map)
        end
    end

    if GetPlayerCount() == requiredPlayers then
        if not GetPlayersNotReady() then
            LOG("Host launching game.")
            lobbyComm:BroadcastData( { Type = 'Launch', GameInfo = gameInfo } )
            lobbyComm:LaunchGame(gameInfo)
        else
            LOG('Players not ready: ',repr(GetPlayersNotReady()))
        end
    end
end


# LobbyComm Callbacks
function InitLobbyComm(protocol, localPort, desiredPlayerName, localPlayerUID, natTraversalProvider)
    local LobCreateFunc = import('/lua/modules/ui/lobby/lobbyComm.lua').CreateLobbyComm
    lobbyComm = LobCreateFunc(protocol, localPort, desiredPlayerName, localPlayerUID, natTraversalProvider)

    local function CleanupAndExit()
        OnlineProvider.UnregisterChat()
        if keepAliveThread then
            KillThread(keepAliveThread)
        end
        lobbyComm:Destroy()
        ExitApplication()
    end

    lobbyComm.Connecting = function(self)
        connectingDialog = UIUtil.ShowInfoDialog(controlGroup, Strings.Connecting, "<LOC _Cancel>", CleanupAndExit)
    end

    lobbyComm.ConnectionFailed = function(self, reason)
        if connectingDialog then
            connectingDialog:Destroy()
        end

        connectionFailedDialog = UIUtil.ShowInfoDialog(controlGroup, LOCF(Strings.ConnectionFailed, reason), "<LOC _OK>", CleanupAndExit)
    end

    lobbyComm.LaunchFailed = function(self,reasonKey)
        if connectingDialog then
            connectingDialog:Destroy()
        end

        local failedDlg = UIUtil.ShowInfoDialog(controlGroup, LOCF(Strings.LaunchFailed,LOC(Strings[reasonKey])), "<LOC _OK>", CleanupAndExit)
    end

    lobbyComm.Ejected = function(self)
        if connectingDialog then
            connectingDialog:Destroy()
        end

        local failedDlg = UIUtil.ShowInfoDialog(controlGroup, Strings.Ejected, CleanupAndExit)
    end

    lobbyComm.ConnectionToHostEstablished = function(self,myID,theHostID)
        if connectingDialog then
            connectingDialog:Destroy()
        end
        hostID = theHostID
        localPlayerID = myID

        # Ok, I'm connected to the host. Now request to become a player
        lobbyComm:SendData( hostID, { Type = 'AddPlayer', RequestedPlayerName = localPlayerName, RequestedFaction = GetDesiredFaction(), } )
    end

    lobbyComm.DataReceived = function(self,data)
        LOG('DATA RECEIVED: ', repr(data))

        if lobbyComm:IsHost() then
            # Host Messages
            if data.Type == 'AddPlayer' then
                HostTryAddPlayer( data.SenderID, data.RequestedPlayerName, data.RequestedFaction )
                UpdateGame()
            elseif data.Type == 'PlayerOption' then
                gameInfo.PlayerOptions[data.Slot][data.Key] = data.Value
                UpdateGame()
            end
        else
            # Non-Host Messages
            if data.Type == 'Launch' then
                lobbyComm:LaunchGame(data.GameInfo)
            end

            if data.Type == 'SlotAssigned' then
                if data.Options.OwnerID == localPlayerID and data.Options.Human then
                    SetPlayerOption(data.Slot,"Ready",true)
                end
            end
        end
    end

    lobbyComm.SystemMessage = function(self, text)
        LOG("System: ",text)
    end

    lobbyComm.GameLaunched = function(self)
        OnlineProvider.Launching()
        OnlineProvider.UnregisterChat()
        if keepAliveThread then
            KillThread(keepAliveThread)
        end
        parent:Destroy()
        parent = false
        MenuCommon.MenuCleanup()
        lobbyComm:Destroy()
        lobbyComm = false
    end

    lobbyComm.Hosting = function(self)
        localPlayerID = lobbyComm:GetLocalPlayerID()
        hostID = localPlayerID

        # Give myself the first slot
        gameInfo.PlayerOptions[1] = LobbyComm.GetDefaultPlayerOptions(localPlayerName)
        gameInfo.PlayerOptions[1].OwnerID = localPlayerID
        gameInfo.PlayerOptions[1].Human = true
        gameInfo.PlayerOptions[1].Faction = GetDesiredFaction()
        gameInfo.PlayerOptions[1].Ready = true

        -- set default lobby values
        for index, option in teamOpts do
            local defValue = option.default
            SetGameOption(option.key,option.values[defValue].key)
        end

        for index, option in globalOpts do
            local defValue = option.default
            SetGameOption(option.key,option.values[defValue].key)
        end

        if self.desiredScenario and self.desiredScenario != "" then
            Prefs.SetToCurrentProfile('LastScenario', self.desiredScenario)
            SetGameOption('ScenarioFile',self.desiredScenario)
        end

        UpdateGame()

        keepAliveThread = ForkThread(
            # Eject players who haven't sent a heartbeat in a while
            function()
                while true and lobbyComm do
                    local peers = lobbyComm:GetPeers()
                    for k,peer in peers do
                        if peer.quiet > LobbyComm.quietTimeout then
                            local slot = FindSlotForID(peer.id)
                            lobbyComm:SystemMessage(LOCF(Strings.TimedOut,gameInfo.PlayerOptions[slot].PlayerName))
                            lobbyComm:EjectPlayer(peer.id)
                        end
                    end
                    WaitSeconds(1)
                end
            end
        )
    end

    lobbyComm.PeerDisconnected = function(self,peerName,peerID)
        local slot = FindSlotForID(peerID)
        gameInfo.PlayerOptions[slot] = nil
    end

    lobbyComm.GameConfigRequested = function(self)
        error('This should not have been called in autolobby.')
    end
end

function SetPlayerOption(slot, key, val)
    #gameInfo.PlayerOptions[slot][key] = val
    lobbyComm:BroadcastData(
        {
            Type = 'PlayerOption',
            Key = key,
            Value = val,
            Slot = slot,
        }
    )
end

function SetGameOption(key, val)
    if lobbyComm:IsHost() then
        gameInfo.GameOptions[key] = val
        UpdateGame()
    else
        WARN('Attempt to set game option by a non-host')
        return
    end
    lobbyComm:BroadcastData(
        {
            Type = 'GameOption',
            Key = key,
            Value = val,
        }
    )
end

function FindSlotForID(id)
    for k,player in gameInfo.PlayerOptions do
        if player.OwnerID == id then
            return k
        end
    end
    return nil
end

function IsLocallyOwned(slot)
    LOG('islocal ',slot,' .. ', localPlayerID)
    return (gameInfo.PlayerOptions[slot].OwnerID == localPlayerID)
end

function IsColorFree(colorIndex)
    for id,player in gameInfo.PlayerOptions do
        if player.PlayerColor == colorIndex then
            return false
        end
    end

    return true
end

function GetPlayerCount()
    local numPlayers = 0
    for k,player in gameInfo.PlayerOptions do
        if player.Team >= 0 then
            numPlayers = numPlayers + 1
        end
    end
    return numPlayers
end

function GetPlayersNotReady()
    local notReady = false
    for k,v in gameInfo.PlayerOptions do
        if v.Human and not v.Ready then
            if not notReady then
                notReady = {}
            end
            table.insert(notReady,v.PlayerName)
        end
    end
    return notReady
end
