--*****************************************************************************
--* File: lua/modules/ui/uimain.lua
--* Author: Chris Blackwell
--* Summary: The entry point for UI scripting
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local UIUtil = import('uiutil.lua')
local UIFile = UIUtil.UIFile
local Prefs = import('/lua/modules/ui/prefs.lua')
local Text = import('/lua/modules/maui/text.lua').Text
local OnlineProvider = import('/lua/modules/multiplayer/onlineprovider.lua')

local console = false
local alreadySetup = false

--* Initialize the UI states, this is always called on startup
function SetupUI()
    -- this function only needs to run once to set up the globals, so don't do it again if not needed
    if alreadySetup then return end
    alreadySetup = true

    UIUtil.currentLayout = Prefs.GetFromCurrentProfile("layout") or "bottom"
    local skin = Prefs.GetFromCurrentProfile("skin")
    UIUtil.SetCurrentSkin(skin or "uef")    -- default to uef

    UIUtil.consoleDepth = 5000000 -- no UI element should depth higher than this!

    local c = UIUtil.CreateCursor()
    SetCursor( c )
end

-- THE FOLLOWING FUNCTIONS SHOULD NOT BE CALLED FROM LUA CODE
-- THEY ARE CALLED FROM THE ENGINE AND EXPECT A DIFFERENT LUA STATE
function StartSplashScreen()
    console = false
    import('/lua/modules/ui/splash/splash.lua').CreateUI()
end

function StartFrontEndUI()
    console = false
    -- if there is an auto continue state, then launch op immediately
    if UIUtil.BETA_FLAG then
        import('/lua/modules/ui/menus/main.lua').CreateUI()
    else
        local autoContinueOp = import('/lua/modules/ui/campaign/campaignmanager.lua').GetAutoContinueOp()
        if autoContinueOp then
            import('/lua/modules/ui/campaign/campaignmanager.lua').AutoLaunchOperation()
        else
            import('/lua/modules/ui/menus/main.lua').CreateUI()
        end
    end

    if GetNumRootFrames() > 1 then
        import('/lua/modules/ui/game/multihead.lua').ShowLogoInHead1()
    end
end

function IsAutoLaunch()
    return HasCommandLineArg("/autolaunch")
end

function StartLobbyUI(protocol, port, playerName, uid, natTraversalProvider)
	OnlineProvider.SetOnline()
    local lobby = false
    if (IsAutoLaunch()) then
        lobby = import('/lua/modules/ui/lobby/autolobby.lua')
    else
        lobby = import('/lua/modules/ui/lobby/lobby.lua')
    end
    lobby.CreateLobby(protocol, port, playerName, uid, natTraversalProvider)
end

function HostGame(gameName, mapName)
	local lobby = false
    if (IsAutoLaunch()) then
        lobby = import('/lua/modules/ui/lobby/autolobby.lua')
    else
        lobby = import('/lua/modules/ui/lobby/lobby.lua')
    end
	lobby.HostGame(gameName, mapName, false)
end

function JoinGame(address, playerName, uid)
	local lobby = false
    if (IsAutoLaunch()) then
        lobby = import('/lua/modules/ui/lobby/autolobby.lua')
    else
        lobby = import('/lua/modules/ui/lobby/lobby.lua')
    end
	lobby.JoinGame(address, false, playerName, uid)
end

function ConnectToPeer(boxedAddress, port, name, boxedID)
    local lobby = false
    if (IsAutoLaunch()) then
        lobby = import('/lua/modules/ui/lobby/autolobby.lua')
    else
        lobby = import('/lua/modules/ui/lobby/lobby.lua')
    end
    lobby.ConnectToPeer(boxedAddress,port,name,boxedID)
end

function DisconnectFromPeer(boxedID)
    local lobby = false
    if (IsAutoLaunch()) then
        lobby = import('/lua/modules/ui/lobby/autolobby.lua')
    else
        lobby = import('/lua/modules/ui/lobby/lobby.lua')
    end
    lobby.DisconnectFromPeer(boxedID)
end

--Used by command line to host a game
function StartHostLobbyUI(protocol, port, playerName, gameName, mapName, natTraversalProvider)
    LOG("GPGNet StartHostLobbyUI")
    local lobby = false
    if (IsAutoLaunch()) then
        lobby = import('/lua/modules/ui/lobby/autolobby.lua')
    else
        lobby = import('/lua/modules/ui/lobby/lobby.lua')
    end
    lobby.CreateLobby(protocol, port, playerName, nil, natTraversalProvider)
    lobby.HostGame(gameName, mapName, false)
end

--Used by command line to join a game
function StartJoinLobbyUI(protocol, address, playerName, natTraversalProvider)
    local lobby = false
    if (IsAutoLaunch()) then
        lobby = import('/lua/modules/ui/lobby/autolobby.lua')
    else
        lobby = import('/lua/modules/ui/lobby/lobby.lua')
    end
    local port = 0
    lobby.CreateLobby(protocol, port, playerName, nil, natTraversalProvider)
    lobby.JoinGame(address, false)
end

function StartGameUI()
    console = false
    import('/lua/modules/ui/game/gamemain.lua').CreateWldUIProvider()
end
-- END SHOULD NOT BE CALLED FROM LUA CODE

-- toggle the console window (create if needed)
function ToggleConsole()
    if console then
        if console:IsHidden() then
            console:Show()
        else
            console:Hide()
        end
    else
        console = import('/lua/modules/ui/dialogs/console.lua').CreateDialog()
        console:Show()
    end
end

--* The following scripts are alternate entry points to the UI from the engine
--* Typically these are dialog popup type calls

-- context sensitive exit dialog
function ShowEscapeDialog()
    import('/lua/modules/ui/dialogs/exitdialog.lua').CreateDialog()
end

-- network disconnection/boot dialog
local prevDisconnectModule
function UpdateDisconnectDialog()
    local module = import('/lua/modules/ui/dialogs/disconnect.lua')
    if prevDisconnectModule and prevDisconnectModule != module then
        pcall(prevDisconnectModule.DestroyDialog)
    end
    prevDisconnectModule = module
    module.Update()
end

-- network desync info
function ShowDesyncDialog(beatNumber, strings)
    import('/lua/modules/ui/dialogs/desync.lua').UpdateDialog(beatNumber, strings)
end

--* The following functions are invoked by the engine to show some text at a certain location
--TODO keep a table of cursor texts to have multiple available?
local cursorText = false

function StartCursorText(x, y, text, color, time, flash)
    if cursorText and cursorText._inTimer then return end -- if cursor text has a time, don't stomp it
    if cursorText then cursorText:Destroy() end
    cursorText = Text(GetFrame(0))
    cursorText:SetText(text)
    cursorText:SetColor(color)
    cursorText:SetFont(UIUtil.bodyFont, 18)
    cursorText:SetDropShadow(true)
    cursorText.Left:Set(x + 16)
    cursorText.Top:Set(y)

    cursorText:SetNeedsFrameUpdate(true)

    cursorText.OnFrame = function(self, elapsedTime)
        if flash then
            if not cursorText._flashTime then
                cursorText._flashTime = 0
            else
                cursorText._flashTime = cursorText._flashTime + elapsedTime
            end
            if cursorText._flashTime > 0.25 then -- 1/4 second flashes
                cursorText:SetHidden(not cursorText:IsHidden())
                cursorText._flashTime = 0
            end
        end

        if not cursorText._time then
            cursorText._time = 0
        else
            cursorText._time = cursorText._time + elapsedTime
        end
        if time > 0 then
            cursorText._inTimer = true
            if time < (cursorText._time) then
                cursorText:Destroy()
                cursorText = nil
            end
        end
    end
end

function StopCursorText()
    if cursorText and cursorText._inTimer then return end -- if cursor text has a time, don't stomp it
    if cursorText then cursorText:Destroy() end
end