--*****************************************************************************
--* File: lua/modules/ui/uiutil.lua
--* Author: Chris Blackwell
--* Summary: Various utility functions to make UI scripts easier and more consistent
--*
--* Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
--*****************************************************************************

local LazyVar = import('/lua/modules/lazyvar.lua')
local LayoutHelpers = import('/lua/modules/maui/layouthelpers.lua')
local Group = import('/lua/modules/maui/group.lua').Group
local Text = import('/lua/modules/maui/text.lua').Text
local MultiLineText = import('/lua/modules/maui/multilinetext.lua').MultiLineText
local Options = import('/lua/modules/ui/game/options.lua')
local Button = import('/lua/modules/maui/button.lua').Button
local Checkbox = import('/lua/modules/maui/Checkbox.lua').Checkbox
local Scrollbar = import('/lua/modules/maui/scrollbar.lua').Scrollbar
local Bitmap = import('/lua/modules/maui/bitmap.lua').Bitmap
local Cursor = import('/lua/modules/maui/cursor.lua').Cursor
local Prefs = import('/lua/modules/ui/prefs.lua')
local Border = import('/lua/modules/maui/border.lua').Border
local ItemList = import('/lua/modules/maui/itemlist.lua').ItemList

--* Handy global variables to assist skinning
buttonFont = import('/lua/modules/lazyvar.lua').Create()            -- default font used for button faces
dialogButtonFont = import('/lua/modules/lazyvar.lua').Create()      -- default font used for dialog button faces
bodyFont = import('/lua/modules/lazyvar.lua').Create()              -- font used for all other text
fixedFont = import('/lua/modules/lazyvar.lua').Create()             -- font used for fixed width characters
titleFont = import('/lua/modules/lazyvar.lua').Create()             -- font used for titles and labels
fontColor = import('/lua/modules/lazyvar.lua').Create()             -- common font color
fontOverColor = import('/lua/modules/lazyvar.lua').Create()             -- common font color
fontDownColor = import('/lua/modules/lazyvar.lua').Create()             -- common font color
bodyColor = import('/lua/modules/lazyvar.lua').Create()             -- common color for dialog body text
dialogCaptionColor = import('/lua/modules/lazyvar.lua').Create()    -- common color for dialog titles
dialogColumnColor = import('/lua/modules/lazyvar.lua').Create()     -- common color for column headers in a dialog
dialogButtonColor = import('/lua/modules/lazyvar.lua').Create()     -- common color for buttons in a dialog
highlightColor = import('/lua/modules/lazyvar.lua').Create()        -- text highlight color
disabledColor = import('/lua/modules/lazyvar.lua').Create()         -- text disabled color
panelColor = import('/lua/modules/lazyvar.lua').Create()            -- default color when drawing a panel
transparentPanelColor = import('/lua/modules/lazyvar.lua').Create() -- default color when drawing a transparent panel
consoleBGColor = import('/lua/modules/lazyvar.lua').Create()        -- console background color
consoleFGColor = import('/lua/modules/lazyvar.lua').Create()        -- console foreground color (text)
consoleTextBGColor = import('/lua/modules/lazyvar.lua').Create()    -- console text background color
layouts = nil                                                       -- table of layouts supported by this skin, not a lazy var as we don't need updates

--* other handy variables!
consoleDepth = false  -- in order to get the console to always be on top, assign this number and never go over

networkBool = import('/lua/modules/lazyvar.lua').Create()    -- boolean whether the game is local or networked

CampaignMode = false

function SetCampaignMode(state)
    CampaignMode = state
end

--* These values MUST NOT CHANGE! They syncronize with values in UIManager.h and are used to
--* specify a render pass
-- render before the world is rendered
UIRP_UnderWorld     = 1
-- reserved for world views
UIRP_World          = 2
-- render with glow (note, won't render when world isn't visible)
UIRP_Glow           = 3
-- render without glow
UIRP_PostGlow       = 4

--* useful key codes
VK_TAB = 9
VK_PRIOR = 33
VK_NEXT = 34
VK_UP = 38
VK_DOWN = 40
VK_F1 = 112
VK_F2 = 113
VK_F3 = 114
VK_F4 = 115
VK_F5 = 116
VK_F6 = 117
VK_F7 = 118
VK_F8 = 119
VK_F9 = 120
VK_F10 = 121
VK_F11 = 122
VK_F12 = 123

local currentSkin = import('/lua/modules/lazyvar.lua').Create()

currentLayout = false
changeLayoutFunction = false    -- set this function to get called with the new layout name when layout changes

-- This flag turns things on / off for the Beta version
-- Yes, it's easy to mod off but the content behind the
-- button is removed, so it doesn't really matter.
BETA_FLAG = true

--* layout control, sets current layout preference
function SetCurrentLayout(layout)
    if not layout then return end
    -- make sure this skin contains the layout, otherwise do nothing
    local foundLayout = false
    for index, complayout in layouts do
        if layout == complayout then
            foundLayout = true
            break
        end
    end
    if not foundLayout then return end

    currentLayout = layout
    Prefs.SetToCurrentProfile("layout", currentLayout)
    --TEMP - this is here because the grids get hosed when you switch layouts. The grids will be fixed
    -- but until then this will have to suffice
    SelectUnits(nil)
    if changeLayoutFunction then changeLayoutFunction(layout) end
end
function GetNetworkBool()
    local sessionClientsTable = GetSessionClients()
    local networkBool = false
    local sessionBool = true
    if sessionClientsTable != nil then
        if table.getn(sessionClientsTable) > 1 then
            networkBool = true
        end
    else
        sessionBool = false
    end
    return networkBool, sessionBool
end

function GetAnimationPrefs()
    return Prefs.GetOption("ui_animation")
end

--* skin control, sets the current skin table
function SetCurrentSkin(skin)
    local skins = import('/lua/skins/skins.lua').skins

    if skins[skin] == nil then
        LOG("Skin '" .. skin .. "'not found, setting to default")
        skin = "uef"
    end

    currentSkin:Set(skin)

    buttonFont:Set(skins[skin].buttonFont or skins['default'].buttonFont)
    dialogButtonFont:Set(skins[skin].dialogButtonFont or skins['default'].dialogButtonFont)
    bodyFont:Set(skins[skin].bodyFont or skins['default'].bodyFont)
    fixedFont:Set(skins[skin].fixedFont or skins['default'].fixedFont)
    titleFont:Set(skins[skin].titleFont or skins['default'].titleFont)
    fontColor:Set(skins[skin].fontColor or skins['default'].fontColor)
    bodyColor:Set(skins[skin].bodyColor or skins['default'].bodyColor)
    fontOverColor:Set(skins[skin].fontOverColor or skins['default'].fontOverColor)
    fontDownColor:Set(skins[skin].fontDownColor or skins['default'].fontDownColor)
    dialogCaptionColor:Set(skins[skin].dialogCaptionColor or skins['default'].dialogCaptionColor)
    dialogColumnColor:Set(skins[skin].dialogColumnColor or skins['default'].dialogColumnColor)
    dialogButtonColor:Set(skins[skin].dialogButtonColor or skins['default'].dialogButtonColor)
    highlightColor:Set(skins[skin].highlightColor or skins['default'].highlightColor)
    disabledColor:Set(skins[skin].disabledColor or skins['default'].disabledColor)
    panelColor:Set(skins[skin].panelColor or skins['default'].panelColor)
    transparentPanelColor:Set(skins[skin].transparentPanelColor or skins['default'].transparentPanelColor)
    consoleBGColor:Set(skins[skin].consoleBGColor or skins['default'].consoleBGColor)
    consoleFGColor:Set(skins[skin].consoleFGColor or skins['default'].consoleFGColor)
    consoleTextBGColor:Set(skins[skin].consoleTextBGColor or skins['default'].consoleTextBGColor)
    layouts = skins[skin].layouts or skins['default'].layouts

    if not currentLayout then
        SetCurrentLayout(layouts[1])
    else
        SetCurrentLayout(currentLayout)
    end

    Prefs.SetToCurrentProfile("skin", skin)
end

--* cycle through all available skins
function RotateSkin(direction)
    local skins = import('/lua/skins/skins.lua').skins

    -- build an array of skin names
    local skinNames = {}
    for skin in skins do
        table.insert(skinNames, skin)
    end

    local dir
    if direction == '+' then
        dir = 1
    else
        dir = -1
    end

    -- Find the next skin from our current skin, skipping default, as it's not really a skin!
    -- note that if the skin table is updated while running, the order of the table might change
    -- so your cycle may be different. No big deal, just be aware it's a side effect.
    local numSkins = table.getn(skinNames)
    for index, skinName in skinNames do
        if skinName == currentSkin() then
            local nextSkinIndex = index + dir
            if nextSkinIndex > numSkins then nextSkinIndex = 1 end
            if nextSkinIndex < 1 then nextSkinIndex = numSkins end
            if skinNames[nextSkinIndex] == 'default' then   -- skip default entry as it's not really a skin
                nextSkinIndex = nextSkinIndex + dir
                if nextSkinIndex > numSkins then nextSkinIndex = 1 end
                if nextSkinIndex < 1 then nextSkinIndex = numSkins end
            end
            SetCurrentSkin(skinNames[nextSkinIndex])
            break
        end
    end
end

--* cycle through all available layouts
function RotateLayout(direction)
    local dir
    if direction == '+' then
        dir = 1
    else
        dir = -1
    end

    local numLayouts = table.getn(layouts)
    for index, layoutName in layouts do
        if layoutName == currentLayout then
            local nextLayoutIndex = index + dir
            if nextLayoutIndex > numLayouts then nextLayoutIndex = 1 end
            if nextLayoutIndex < 1 then nextLayoutIndex = numLayouts end
            SetCurrentLayout(layouts[nextLayoutIndex])
            break
        end
    end
end

--* given a path and name relative to the skin path, returns the full path based on the current skin
function UIFile(filespec)
    local skins = import('/lua/skins/skins.lua').skins
    local visitingSkin = currentSkin()
    local currentPath = skins[visitingSkin].texturesPath

    while visitingSkin do
        local curFile = currentPath .. filespec
        if DiskGetFileInfo(curFile) then
            return curFile
        else
            visitingSkin = skins[visitingSkin].default
            if visitingSkin then currentPath = skins[visitingSkin].texturesPath end
        end
    end

    LOG("Warning: Unable to find file ", filespec)
    -- pass out the final string anyway so resource loader can gracefully fil
    return filespec
end

--* return the filename as a lazy var function to allow triggering of OnDirty
function SkinnableFile(filespec)
    return function()
        return UIFile(filespec)
    end
end

--* each UI screen needs something to be responsible for parenting all its controls so
--* placement and destruction can occur. This creates a group which fills the screen.
function CreateScreenGroup(root, debugName)
    local screenGroup = Group(root, debugName or "screenGroup")
    LayoutHelpers.FillParent(screenGroup, root)
    return screenGroup
end

--* Get cursor information for a given cursor ID
function GetCursor(id)
    local skins = import('/lua/skins/skins.lua').skins
    local cursors = skins[currentSkin()].cursors or skins['default'].cursors
    if not cursors[id] then
        LOG("Requested cursor not found: " .. id)
    end
    return cursors[id][1], cursors[id][2], cursors[id][3], cursors[id][4], cursors[id][5]
end

--* create the one cursor used by the game
function CreateCursor()
    local cursor = Cursor(GetCursor('DEFAULT'))
    return cursor
end

--* return a text object with the appropriate font set
function CreateText(parent, label, pointSize, font)
    label = LOC(label) or LOC("<LOC uiutil_0000>[no text]")
    font = font or buttonFont
    local text = Text(parent, "Text: " .. label)
    text:SetFont(font, pointSize)
    text:SetColor(fontColor)
    text:SetText(label)
    return text
end


--* return a button set up with a text overlay and a click sound
function CreateButton(parent, up, down, over, disabled, label, pointSize, textOffsetVert, textOffsetHorz, clickCue, rolloverCue)
    textOffsetVert = textOffsetVert or 0
    textOffsetHorz = textOffsetHorz or 0
    if clickCue == "NO_SOUND" then
        clickCue = nil
    else
        clickCue = clickCue or "UI_Menu_MouseDown_Sml"
    end
    if rolloverCue == "NO_SOUND" then
        rolloverCue = nil
    else
        rolloverCue = rolloverCue or "UI_Menu_Rollover_Sml"
    end
    if type(up) == 'string' then
        up = SkinnableFile(up)
    end
    if type(down) == 'string' then
        down = SkinnableFile(down)
    end
    if type(over) == 'string' then
        over = SkinnableFile(over)
    end
    if type(disabled) == 'string' then
        disabled = SkinnableFile(disabled)
    end

    local button = Button(parent, up, down, over, disabled, clickCue, rolloverCue)
    button:UseAlphaHitTest(true)

    if label and pointSize then
        button.label = CreateText(button, label, pointSize)
        LayoutHelpers.AtCenterIn(button.label, button, textOffsetVert, textOffsetHorz)
        button.label:DisableHitTest()

        -- if text exists, set up to grey it out
        button.OnDisable = function(self)
            Button.OnDisable(self)
            button.label:SetColor(disabledColor)
        end

        button.OnEnable = function(self)
            Button.OnEnable(self)
            button.label:SetColor(fontColor)
        end
        button.OnRolloverEvent = function(self, event)
            if event == 'enter' then
                button.label:SetColor(fontOverColor)
            elseif event == 'exit' then
                button.label:SetColor(fontColor)
            elseif event == 'down' then
                button.label:SetColor(fontDownColor)
            end
        end
    end

    return button
end

function SetNewButtonTextures(button, up, down, over, disabled)
    -- if strings passed in, make them skinnables, otherwise assume they are already skinnables
    if type(up) == 'string' then
        up = SkinnableFile(up)
    end
    if type(down) == 'string' then
        down = SkinnableFile(down)
    end
    if type(over) == 'string' then
        over = SkinnableFile(over)
    end
    if type(disabled) == 'string' then
        disabled = SkinnableFile(disabled)
    end

    button:SetNewTextures(up, down, over, disabled)
end

--* create a button with standardized texture names
--* given a path and button name prefix, generates the four button asset file names according to the naming convention
function CreateButtonStd(parent, filename, label, pointSize, textOffsetVert, textOffsetHorz, clickCue, rolloverCue)
    return CreateButton(parent
        , filename .. "_btn_up.dds"
        , filename .. "_btn_down.dds"
        , filename .. "_btn_over.dds"
        , filename .. "_btn_dis.dds"
        , label
        , pointSize
        , textOffsetVert
        , textOffsetHorz
        , clickCue
        , rolloverCue
        )
end

function CreateCheckboxStd(parent, filename)
    return Checkbox( parent,
        SkinnableFile(filename .. '-d_btn_up.dds'),
        SkinnableFile(filename .. '-s_btn_up.dds'),
        SkinnableFile(filename .. '-d_btn_over.dds'),
        SkinnableFile(filename .. '-s_btn_over.dds'),
        SkinnableFile(filename .. '-d_btn_dis.dds'),
        SkinnableFile(filename .. '-s_btn_dis.dds'))
end

function CreateDialogButtonStd(parent, filename, label, pointSize, textOffsetVert, textOffsetHorz, clickCue, rolloverCue)
    local button = CreateButtonStd(parent,filename,label,pointSize,textOffsetVert,textOffsetHorz, clickCue, rolloverCue)
    button.label:SetFont( dialogButtonFont, pointSize )
    button.label:SetColor( dialogButtonColor )
    return button
end

function SetNewButtonStdTextures(button, filename)
    SetNewButtonTextures(button
        , filename .. "_btn_up.dds"
        , filename .. "_btn_down.dds"
        , filename .. "_btn_over.dds"
        , filename .. "_btn_dis.dds")
end

--* return the standard scrollbar
function CreateVertScrollbarFor(attachto)

    local scrollbar = Scrollbar(attachto, import('/lua/modules/maui/scrollbar.lua').ScrollAxis.Vert)
    scrollbar:SetTextures(  UIFile('/small-vert_scroll/back_scr_mid.dds')
                            ,UIFile('/small-vert_scroll/bar-mid_scr_over.dds')
                            ,UIFile('/small-vert_scroll/bar-top_scr_up.dds')
                            ,UIFile('/small-vert_scroll/bar-bot_scr_up.dds'))
                            
    local scrollUpButton = Button(  scrollbar
                                    , SkinnableFile('/small-vert_scroll/arrow-up_scr_up.dds')
                                    , SkinnableFile('/small-vert_scroll/arrow-up_scr_over.dds')
                                    , SkinnableFile('/small-vert_scroll/arrow-up_scr_down.dds')
                                    , SkinnableFile('/small-vert_scroll/arrow-up_scr_dis.dds')
                                    , "UI_Arrow_Click")

    local scrollDownButton = Button(  scrollbar
                                    , SkinnableFile('/small-vert_scroll/arrow-down_scr_up.dds')
                                    , SkinnableFile('/small-vert_scroll/arrow-down_scr_over.dds')
                                    , SkinnableFile('/small-vert_scroll/arrow-down_scr_down.dds')
                                    , SkinnableFile('/small-vert_scroll/arrow-down_scr_dis.dds')
                                    , "UI_Arrow_Click")

    scrollbar.Left:Set(attachto.Right)
    scrollbar.Top:Set(scrollUpButton.Bottom)
    scrollbar.Bottom:Set(scrollDownButton.Top)

    scrollUpButton.Left:Set(scrollbar.Left)
    scrollUpButton.Top:Set(attachto.Top)
    scrollDownButton.Left:Set(scrollbar.Left)
    scrollDownButton.Bottom:Set(attachto.Bottom)
    
    scrollbar.Right:Set(scrollUpButton.Right)
    
    scrollbar:AddButtons(scrollUpButton, scrollDownButton)
    scrollbar:SetScrollable(attachto)

    return scrollbar
end

function CreateHorzScrollbarFor(attachto)

    local scrollRightButton = Button(  attachto
                                    , SkinnableFile('/widgets/large-h_scr/arrow-right_scr_up.dds')
                                    , SkinnableFile('/widgets/large-h_scr/arrow-right_scr_down.dds')
                                    , SkinnableFile('/widgets/large-h_scr/arrow-right_scr_over.dds')
                                    , SkinnableFile('/widgets/large-h_scr/arrow-right_scr_dis.dds'))
    scrollRightButton.Right:Set(attachto.Right)
    scrollRightButton.Bottom:Set(attachto.Top)

    local scrollLeftButton = Button(  attachto
                                    , SkinnableFile('/widgets/large-h_scr/arrow-left_scr_up.dds')
                                    , SkinnableFile('/widgets/large-h_scr/arrow-left_scr_down.dds')
                                    , SkinnableFile('/widgets/large-h_scr/arrow-left_scr_over.dds')
                                    , SkinnableFile('/widgets/large-h_scr/arrow-left_scr_dis.dds'))
    scrollLeftButton.Left:Set(attachto.Left)
    scrollLeftButton.Bottom:Set(attachto.Top)

    local scrollbar = Scrollbar(attachto, import('/lua/modules/maui/scrollbar.lua').ScrollAxis.Horz)
    scrollbar:SetTextures(  UIFile('/widgets/back_scr/back_scr_mid.dds')
                            ,UIFile('/widgets/large-h_scr/bar-mid_scr_over.dds')
                            ,UIFile('/widgets/large-h_scr/bar-right_scr_over.dds')
                            ,UIFile('/widgets/large-h_scr/bar-left_scr_over.dds'))
    scrollbar.Left:Set(scrollLeftButton.Right)
    scrollbar.Right:Set(scrollRightButton.Left)
    scrollbar.Top:Set(scrollRightButton.Top)
    scrollbar.Bottom:Set(scrollRightButton.Bottom)

    scrollbar:AddButtons(scrollLeftButton, scrollRightButton)
    scrollbar:SetScrollable(attachto)

    return scrollbar
end

-- create and manage an info dialog
function QuickDialog(parent, dialogText, button1Text, button1Callback, button2Text, button2Callback, button3Text, button3Callback, destroyOnCallback)
    -- if there is a callback and destroy not specified, assume destroy
    if (destroyOnCallback == nil) and (button1Callback or button2Callback or button3Callback) then
        destroyOnCallback = true
    end
    local netBool, sessionBool = GetNetworkBool()
    if sessionBool then
        if not netBool then
            Options.PauseGame(true)
        end
    end
    
    local dialog = Group(parent, "quickDialogGroup")
    LayoutHelpers.AtCenterIn(dialog, parent)
    dialog.Depth:Set(GetFrame(parent:GetRootFrame():GetTargetHead()):GetTopmostDepth() + 1)
    local background = Bitmap(dialog, SkinnableFile('/dialogs/dialog/panel_bmp_m.dds'))
    dialog._background = background
    dialog.Width:Set(background.Width)
    dialog.Height:Set(background.Height)
    LayoutHelpers.FillParent(background, dialog)
    
    local textLine = {}
    textLine[1] = CreateText(dialog, "", 18, titleFont)
    textLine[1].Top:Set(background.Top)
    LayoutHelpers.AtHorizontalCenterIn(textLine[1], dialog)
    
    local textBoxWidth = (dialog.Width() - 60) 
    local tempTable = import('/lua/modules/maui/text.lua').WrapText(LOC(dialogText), textBoxWidth,
    function(text)
        return textLine[1]:GetStringAdvance(text)
    end)

    local tempLines = table.getn(tempTable)
    
    local prevControl = false
    for i, v in tempTable do
        if i == 1 then
            textLine[1]:SetText(v)
            prevControl = textLine[1]
        else
            textLine[i] = CreateText(dialog, v, 18, titleFont)
            LayoutHelpers.Below(textLine[i], prevControl)
            prevControl = textLine[i]
        end
    end
    
    background:SetTiled(true)
    background.Bottom:Set(textLine[tempLines].Bottom)
    
    local backgroundTop = Bitmap(dialog, SkinnableFile('/dialogs/dialog/panel_bmp_t.dds'))
    backgroundTop.Bottom:Set(background.Top)
    backgroundTop.Left:Set(background.Left)
    local backgroundBottom = Bitmap(dialog, SkinnableFile('/dialogs/dialog/panel_bmp_b.dds'))
    backgroundBottom.Top:Set(background.Bottom)
    backgroundBottom.Left:Set(background.Left)
    
    local worldCover = CreateWorldCover(dialog)
    
    local function MakeButton(text, callback)
        local button = CreateButtonStd( background
                                        , '/widgets/small'
                                        , text
                                        , 12
                                        , 0)
        if callback then
            button.OnClick = function(self)
                callback()
				if destroyOnCallback then
					dialog:Destroy()
				end
            end
        else
            button.OnClick = function(self)
                dialog:Destroy()
            end
        end
        return button
    end

    local button1 = false
    local button2 = false
    local button3 = false

    if button1Text then
        button1 = MakeButton(button1Text, button1Callback)
        LayoutHelpers.Below(button1, background, 22)
    end
    if button2Text then
        button2 = MakeButton(button2Text, button2Callback)
        LayoutHelpers.Below(button2, background, 22)
    end
    if button3Text then
        button3 = MakeButton(button3Text, button3Callback)
        LayoutHelpers.Below(button3, background, 22)
    end

    if button3 then
        -- center each button to one third of the dialog
        button1.Left:Set(function()
            return dialog.Left() + (((dialog.Width() / 3) - button1.Width()) / 2)
        end)
        button2.Left:Set(function()
            local thirdWidth = dialog.Width() / 3
            return dialog.Left() + thirdWidth + ((thirdWidth - button2.Width()) / 2)
        end)
        button3.Left:Set(function()
            local thirdWidth = dialog.Width() / 3
            return dialog.Left() + (thirdWidth * 2) + ((thirdWidth - button3.Width()) / 2)
        end)
    elseif button2 then
        -- center each button to half the dialog
        button1.Left:Set(function()
            return dialog.Left() + (((dialog.Width() / 2) - button1.Width()) / 2)
        end)
        button2.Left:Set(function()
            local halfWidth = dialog.Width() / 2
            return dialog.Left() + halfWidth + ((halfWidth - button2.Width()) / 2)
        end)
    elseif button1 then
        LayoutHelpers.AtHorizontalCenterIn(button1, dialog)
    end

    return dialog
end

function CreateWorldCover(parent)
    local worldCover = Bitmap(parent)
    local curColor = 17
    worldCover:SetSolidColor('00000000')
    worldCover:SetNeedsFrameUpdate(true)
    worldCover.OnFrame = function(self, elapsedTime)
        curColor = math.floor(curColor + (elapsedTime * 200))
        local hexColor = 0
        if curColor > 150 then
            curColor = 150
            hexColor = STR_itox(curColor)
            worldCover:SetNeedsFrameUpdate(false)
        else
            hexColor = STR_itox(curColor)
        end
        worldCover:SetSolidColor(tostring(hexColor)..'000000')
    end
    worldCover.Depth:Set(function() return parent.Depth() - 1 end)
    LayoutHelpers.FillParent(worldCover, GetFrame(0))
    return worldCover
end

function ShowInfoDialog(parent, dialogText, buttonText, buttonCallback, destroyOnCallback)
    return QuickDialog(parent, dialogText, buttonText, buttonCallback, nil, nil, nil, nil, destroyOnCallback)
end

-- create a table of sequential file names (useful for loading animations)
function CreateSequentialFilenameTable(root, ext, first, last, numPlaces)
    local retTable = {}
    local formatString = string.format("%%s%%0%dd.%%s", numPlaces)
    for index = first, last do
        retTable[(index - first) + 1] = SkinnableFile(string.format(formatString, root, index, ext))
    end
    return retTable
end

-- create a box which is controlled by its external borders, and gives access to the "client" area as well
function CreateBox(parent)
    local border = Border(parent)
    border:SetTextures(
        SkinnableFile('/game/generic_brd/generic_brd_vert_l.dds'),
        SkinnableFile('/game/generic_brd/generic_brd_horz_um.dds'),
        SkinnableFile('/game/generic_brd/generic_brd_ul.dds'),
        SkinnableFile('/game/generic_brd/generic_brd_ur.dds'),
        SkinnableFile('/game/generic_brd/generic_brd_ll.dds'),
        SkinnableFile('/game/generic_brd/generic_brd_lr.dds'))
    local clientArea = Bitmap(parent, SkinnableFile('/game/generic_brd/generic_brd_m.dds'))
    border:LayoutControlInside(clientArea)
    clientArea.Width:Set(function() return clientArea.Right() - clientArea.Left() end)
    clientArea.Height:Set(function() return clientAreat.Bottom() - clientArea.Top() end)
    return border, clientArea
end

-- create borders around a control, with optional background
function CreateBorder(parent, addBg)
    local border = Border(parent)
    border:SetTextures(
        SkinnableFile('/game/generic_brd/generic_brd_vert_l.dds'),
        SkinnableFile('/game/generic_brd/generic_brd_horz_um.dds'),
        SkinnableFile('/game/generic_brd/generic_brd_ul.dds'),
        SkinnableFile('/game/generic_brd/generic_brd_ur.dds'),
        SkinnableFile('/game/generic_brd/generic_brd_ll.dds'),
        SkinnableFile('/game/generic_brd/generic_brd_lr.dds'))
    border:LayoutAroundControl(parent)

    local bg = nil
    if addBg then
        bg = Bitmap(parent, SkinnableFile('/game/generic_brd/generic_brd_m.dds'))
        border:LayoutControlInside(bg)
        bg.Width:Set(function() return bg.Right() - bg.Left() end)
        bg.Height:Set(function() return bg.Bottom() - bg.Top() end)
    end

    return border, bg
end

local factionIcons = {
    "/widgets/faction-icons-alpha_bmp/uef_ico.dds",
    "/widgets/faction-icons-alpha_bmp/aeon_ico.dds",
    "/widgets/faction-icons-alpha_bmp/cybran_ico.dds"
}

function GetFactionIcon(factionIndex)
    return factionIcons[factionIndex + 1] or "/widgets/faction-icons-alpha_bmp/uef_ico.dds"
end

-- make sure you lay out text box before you attempt to set text
function CreateTextBox(parent)
    local box = ItemList(parent)
    box:SetFont(bodyFont, 14)
    box:SetColors(bodyColor, "black",  highlightColor, "white")
    CreateVertScrollbarFor(box)
    return box
end

function SetTextBoxText(textBox, text)
    textBox:DeleteAllItems()
    local wrapped = import('/lua/modules/maui/text.lua').WrapText(LOC(text), textBox.Width(), function(curText) return textBox:GetStringAdvance(curText) end)
    for i, line in wrapped do
        textBox:AddItem(line)
    end 
end

