#****************************************************************************
#**
#**  File     :  /cdimage/lua/modules/unit.lua
#**  Author(s):  John Comes, David Tomandl, Gordon Duclos
#**
#**  Summary  : The Unit lua module
#**
#**  Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
#****************************************************************************

local Entity = import('entity.lua').Entity
local WeaponUnpacker = import('unpacker.lua').WeaponUnpacker
local explosion = import('defaultexplosions.lua')
local EffectTemplate = import('EffectTemplates.lua')
local EffectUtilities = import('EffectUtilities.lua')
local Game = import('game.lua')
local utilities = import('utilities.lua')
local Shield = import('/lua/modules/shield.lua').Shield
local UnitShield = import('/lua/modules/shield.lua').UnitShield

Unit = Class(moho.unit_methods) {
    Weapons = {},

    FxScale = 1,
    FxDamageScale = 1,
    # FX Damage tables. A random damage effect table of emitters is choosen out of this table
    FxDamage1 = {EffectTemplate.DamageSmoke01,EffectTemplate.DamageSparks01},
    FxDamage2 = {EffectTemplate.DamageFireSmoke01,EffectTemplate.DamageSparks01},
    FxDamage3 = {EffectTemplate.DamageFire01,EffectTemplate.DamageSparks01},
    DamageEffectsOnBones = true,

    # This determines how severe the rocking motion while immobile in water is
    MaxRockSpeed = 1.5,

    # This will be true for all units being constructed as upgrades
    DisallowCollisions = false,

    # Destruction params
    HideUnitOnDestroy = true,
    PlayDestructionEffects = true,
    ShowUnitDestructionDebris = true,
    DestructionExplosionWaitDelayMin = 0,
    DestructionExplosionWaitDelayMax = 0.5,
    DeathThreadDestructionWaitTime = 0,
    DestructionPartsHighToss = {},
    DestructionPartsLowToss = {},
    DestructionPartsChassisToss = {},
    EconomyProductionInitiallyActive = true,


    #Unpackers:
    #This is only used for units that have an animation controlling more than one weapon
    Unpackers = nil,
#    Unpackers = {
#        MainWeaponsUnpacker = {
#            WeaponsUnpackerAnim = '/units/uas0103/uas0103_aopen01.sca',
#            WeaponsUnpackerRate = 1,
#            WeaponsUnpackerRepackDelay = 5,
#            WeaponsControlled = {'MainGun', 'MainGun2'},
#        },
#    },

    VeteranLevel = 0,


    ##########################################################################################
    ## INITIALIZATION
    ##########################################################################################

    OnPreCreate = function(self)
        #Entity.OnPreCreate(self)
        if not self.Trash then
            self.Trash = TrashBag()
        end
        self.EventCallbacks = {
                OnKilled = {},
                OnUnitBuilt = {},
                OnUnload = {},
                OnLoad = {},
                OnReclaimed = {},
                OnStartReclaim = {},
                OnStopReclaim = {},
                OnStopBeingBuilt = {},
                OnHorizontalStartMove = {},
                OnCaptured = {},
                OnCapturedNewUnit = {},
                OnDamaged = {},
                OnStartCapture = {},
                OnStopCapture = {},
                OnFailedCapture = {},
                OnStartBeingCaptured = {},
                OnStopBeingCaptured = {},
                OnFailedBeingCaptured = {},
                OnVeteran = {},
                ProjectileDamaged = {},
                SpecialToggleEnableFunction = false,
                SpecialToggleDisableFunction = false,
        }
    end,

    OnCreate = function(self)
        Entity.OnCreate(self)
        if self.Unpackers then
            for k, v in self.Unpackers do
                self.Trash:Add(v)
            end
        end
        # Turn off land bones if this unit has them.
        self:HideLandBones()
        # Set number of effects per damage depending on its volume
        local x, y, z = self:GetUnitSizes()
        local vol = x*y*z
        #print('Created ' .. self:GetBlueprint().Display.DisplayName .. ': Volume:' .. vol)
        local damageamounts = 1
        if vol >= 50 then
            damageamounts = 6
            self.FxDamageScale = 2
        elseif vol >= 25 then
            damageamounts = 4
        elseif vol >= 2 then
            damageamounts = 2
        end
        #print('Damage Amounts: ' .. damageamounts)
        self.FxDamage1Amount = self.FxDamage1Amount or damageamounts
        self.FxDamage2Amount = self.FxDamage2Amount or damageamounts
        self.FxDamage3Amount = self.FxDamage3Amount or damageamounts
        self.DamageEffectsBag = {
            {},
            {},
            {},
        }

        # Setup effect emitter bags
        self.MovementEffectsBag = {}
        self.IdleEffectsBag = {}
        self.TopSpeedEffectsBag = {}
        self.BeamExhaustEffectsBag = {}
        self.TransportBeamEffectsBag = {}
        self.BuildEffectsBag = TrashBag()
        self.ReclaimEffectsBag = TrashBag()
        self.OnBeingBuiltEffectsBag = TrashBag()
        self.CaptureEffectsBag = TrashBag()

        self.HasFuel = true

        local bpEcon = self:GetBlueprint().Economy

        self:SetConsumptionPerSecondEnergy(bpEcon.MaintenanceConsumptionPerSecondEnergy or 0)
        self:SetConsumptionPerSecondMass(bpEcon.MaintenanceConsumptionPerSecondMass or 0)
        self:SetProductionPerSecondEnergy(bpEcon.ProductionPerSecondEnergy or 0)
        self:SetProductionPerSecondMass(bpEcon.ProductionPerSecondMass or 0)

        if self.EconomyProductionInitiallyActive then
            #LOG('*DEBUG: SETTING PRODUCTION ACTIVE')
            self:SetProductionActive(true)
        end

        local bpVision = self:GetBlueprint().Intel.VisionRadius
        if bpVision then
            self:SetIntelRadius('Vision', bpVision)
        else
            self:SetIntelRadius('Vision', 0)
        end

        self:SetCanTakeDamage(true)

        self:SetCanBeKilled(true)

        local bpDeathAnim = self:GetBlueprint().Display.AnimationDeath

        if bpDeathAnim and table.getn(bpDeathAnim) > 0 then
            self.PlayDeathAnimation = true
        end

        # Used for keeping track of resource consumption
        self.MaintenanceConsumption = false
        self.ActiveConsumption = false
        self.EnergyModifier = 0
        self.MassModifier = 0
    end,


    ##########################################################################################
    ## MISC FUNCTIONS
    ##########################################################################################

    GetFootPrintSize = function(self)
        local fp = self:GetBlueprint().Footprint
        return math.max(fp.SizeX, fp.SizeZ)
    end,

    # Returns 4 numbers:
    #   skirt x0, skirt z0, skirt.x1, skirt.z1
    GetSkirtRect = function(self)
        local bp = self:GetBlueprint()
        local x, y, z = unpack(self:GetPosition())
        local fx = x - bp.Footprint.SizeX*.5
        local fz = z - bp.Footprint.SizeZ*.5
        local sx = fx + bp.Physics.SkirtOffsetX
        local sz = fz + bp.Physics.SkirtOffsetZ
        return sx, sz, sx+bp.Physics.SkirtSizeX, sz+bp.Physics.SkirtSizeZ
    end,

    GetUnitSizes = function(self)
        local bp = self:GetBlueprint()
        return bp.SizeX, bp.SizeY, bp.SizeZ
    end,

    GetRandomPoint = function(self, scalar)
        local sx, sy, sz = self:GetUnitSizes()
        sx = sx * scalar
        sy = sy * scalar
        sz = sz * scalar
        #print('SizeX: ' .. sx .. '   SizeY: ' .. sy .. '   SizeZ: ' .. sz)
        local x = Random()*sx - (sx*.5)
        local y = Random()*sy
        local z = Random()*sz - (sz*.5)
        #print('RandomX:' .. x .. ' RandomY:' .. y .. ' RandomZ:' .. z)
        return x,y,z
    end,

    ForkThread = function(self, fn, ...)
        if fn then
            local thread = ForkThread(fn, self, unpack(arg))
            self.Trash:Add(thread)
            return thread
        else
            return nil
        end
    end,

    LifeTimeThread = function(self)
        local bp = self:GetBlueprint().Defense.LifeTime
        if not bp then return end
        WaitSeconds(bp)
        self:Destroy()
    end,

    SetTargetPriorities = function(self, priTable)
        for i = 1, self:GetWeaponCount() do
            local wep = self:GetWeapon(i)
            wep:SetWeaponPriorities(priTable)
        end
    end,

    ##########################################################################################
    ## TOGGLES
    ##########################################################################################
    OnScriptBitSet = function(self, bit)
        if bit == 0 then # shield toggle
            self:EnableShield()
        elseif bit == 1 then # weapon toggle
            # do something
        elseif bit == 2 then # jamming toggle
            self:SetMaintenanceConsumptionInactive()
            self:DisableUnitIntel('Jammer')
        elseif bit == 3 then # intel toggle
            # do something
        elseif bit == 4 then # production toggle
            # do something
        elseif bit == 5 then # stealth toggle
            self:SetMaintenanceConsumptionInactive()
            self:DisableUnitIntel('RadarStealth')
            self:DisableUnitIntel('SonarStealth')
        elseif bit == 6 then # generic pause toggle
            self:SetPaused(true)
        elseif bit == 7 then # special toggle
            self:EnableSpecialToggle()
        elseif bit == 8 then # cloak toggle
            self:SetMaintenanceConsumptionInactive()
            self:DisableUnitIntel('Cloak')
        end
    end,

    OnScriptBitClear = function(self, bit)
        if bit == 0 then # shield toggle
            self:DisableShield()
        elseif bit == 1 then # weapon toggle
            # do something
        elseif bit == 2 then # jamming toggle
            self:SetMaintenanceConsumptionActive()
            self:EnableUnitIntel('Jammer')
        elseif bit == 3 then # intel toggle
            # do something
        elseif bit == 4 then # production toggle
            # do something
        elseif bit == 5 then # stealth toggle
            self:SetMaintenanceConsumptionActive()
            self:EnableUnitIntel('RadarStealth')
            self:EnableUnitIntel('SonarStealth')
        elseif bit == 6 then # generic pause toggle
            self:SetPaused(false)
        elseif bit == 7 then # special toggle
            self:DisableSpecialToggle()
        elseif bit == 8 then # cloak toggle
            self:SetMaintenanceConsumptionActive()
            self:EnableUnitIntel('Cloak')
        end
    end,

    OnPaused = function(self)
        self:DisableShield()
        self:DisableUnitIntel()
        self:PlayUnitAmbientSound( nil )
        for i = 1, self:GetWeaponCount() do
            local wep = self:GetWeapon(i)
            local bp = wep:GetBlueprint()
            if bp and bp.NoPause != true then
                wep:SetEnabled(false)
                wep:AimManipulatorSetEnabled(false)
            end
        end
        if not self.PausedIndicator then
            self.PausedIndicator = CreateAttachedEmitter(self,  0, self:GetArmy(), '/effects/emitters/light_paused_01_emit.bp'):OffsetEmitter(0, (self:GetBlueprint().SizeY) + 1, 0):ScaleEmitter(3)
            self.Trash:Add(self.PausedIndicator)
        end
    end,

    OnUnpaused = function(self)
        self:EnableShield()
        self:EnableUnitIntel()
        self:PlayUnitAmbientSound( 'ActiveLoop' )
        if self.PausedIndicator then
            self.PausedIndicator:Destroy()
            self.PausedIndicator = nil
        end
    end,

    EnableSpecialToggle = function(self)
        if self.EventCallbacks.SpecialToggleEnableFunction then
            self.EventCallbacks.SpecialToggleEnableFunction(self)
        end
    end,

    DisableSpecialToggle = function(self)
        if self.EventCallbacks.SpecialToggleDisableFunction then
            self.EventCallbacks.SpecialToggleDisableFunction(self)
        end
    end,

    AddSpecialToggleEnable = function(self, fn)
        if fn then
            self.EventCallbacks.SpecialToggleEnableFunction = fn
        end
    end,

    AddSpecialToggleDisable = function(self, fn)
        if fn then
            self.EventCallbacks.SpecialToggleDisableFunction = fn
        end
    end,


    ##########################################################################################
    ## MISC EVENTS
    ##########################################################################################


    OnProductionActive = function(self)
         #LOG('*DEBUG: PRODUCTION IS NOW ACTIVE')
    end,

    OnActive = function(self)
    end,

    OnInactive = function(self)
    end,


    OnStartCapture = function(self, target)
        self:DoOnStartCaptureCallbacks(target)
        self:StartCaptureEffects(target)
    end,

    OnStopCapture = function(self, target)
        self:DoOnStopCaptureCallbacks(target)
        self:StopCaptureEffects(target)
    end,

    StartCaptureEffects = function( self, target )
		self.CaptureEffectsBag:Add( self:ForkThread( self.CreateCaptureEffects, target ) )
    end,

    CreateCaptureEffects = function( self, target )
    end,

    StopCaptureEffects = function( self, target )
		self.CaptureEffectsBag:Destroy()
    end,

    OnFailedCapture = function(self, target)
        self:DoOnFailedCaptureCallbacks(target)
        self:StopCaptureEffects(target)
    end,

    OnStartBeingCaptured = function(self, captor)
        self:DoOnStartBeingCapturedCallbacks(captor)
    end,

    OnStopBeingCaptured = function(self, captor)
        self:DoOnStopBeingCapturedCallbacks(captor)
    end,

    OnFailedBeingCaptured = function(self, captor)
        self:DoOnFailedBeingCapturedCallbacks(captor)
    end,

    OnReclaimed = function(self, entity)
        self:DoOnReclaimedCallbacks(entity)
        self:Destroy()
    end,

    OnStartReclaim = function(self, target)
        self:DoOnStartReclaimCallbacks()
        self:StartReclaimEffects(target)
    end,

    OnStopReclaim = function(self, target)
        self:DoOnStopReclaimCallbacks()
		self:StopReclaimEffects(target)
    end,

    StartReclaimEffects = function( self, target )
		self.ReclaimEffectsBag:Add( self:ForkThread( self.CreateReclaimEffects, target ) )
    end,

    CreateReclaimEffects = function( self, target )
    end,

    StopReclaimEffects = function( self, target )
		self.ReclaimEffectsBag:Destroy()
    end,

    OnDecayed = function(self)
        self:Destroy()
    end,

    OnCaptured = function(self, captor)
        self:DoOnCapturedCallbacks(captor)
        local newUnitCallbacks = {}
        if self.EventCallbacks.OnCapturedNewUnit then
            newUnitCallbacks = self.EventCallbacks.OnCapturedNewUnit
        end
        local newUnit = ChangeUnitArmy(self, captor:GetArmy())
        # Because the old unit is lost we cannot call a member function for newUnit callbacks
        for k,cb in newUnitCallbacks do
            if cb then
                cb(newUnit, captor)
            end
        end
    end,


    ##########################################################################################
    ## ECONOMY
    ##########################################################################################

    OnConsumptionActive = function(self)
    end,

    OnConsumptionInActive = function(self)
    end,

    # We are splitting Consumption into two catagories:
    #   Maintenance -- for units that are usually "on": radar, mass extractors, etc.
    #   Active -- when upgrading, constructing, or something similar.
    #
    # It will be possible for both or neither of these consumption methods to be
    # in operation at the same time.  Here are the functions to turn them off and on.
    SetMaintenanceConsumptionActive = function(self)
        self.MaintenanceConsumption = true
        self:UpdateConsumptionValues()
    end,

    SetMaintenanceConsumptionInactive = function(self)
        self.MaintenanceConsumption = false
        self:UpdateConsumptionValues()
    end,

    SetActiveConsumptionActive = function(self)
        self.ActiveConsumption = true
        self:UpdateConsumptionValues()
    end,

    SetActiveConsumptionInactive = function(self)
        self.ActiveConsumption = false
        self:UpdateConsumptionValues()
    end,

    SetBuildTimeMultiplier = function(self, time_mult)
        self.BuildTimeMultiplier = time_mult
    end,

    InheritEnhancement = function(self, target)
        self.BuildEnhancement = target.BuildEnhancement
    end,

    ClearEnhancement = function(self)
        self.BuildEnhancement = nil
    end,

    #
    # Called when we start building a unit, turn on/off, get/lose bonuses, or on
    # any other change that might affect our build rate or resource use.
    #
    UpdateConsumptionValues = function(self)
        local myBlueprint = self:GetBlueprint()

        local energy_rate = 0
        local mass_rate = 0
        local build_rate = 0

        if self.ActiveConsumption then
            local focus = self:GetFocusUnit()
            local time = 1
            local mass = 0
            local energy = 0
            if self.BuildEnhancement then
                time, energy, mass = Game.GetConstructEconomyModel(self, self.BuildEnhancement)
            elseif focus then
                # bonuses are already factored in by GetBuildCosts
                time, energy, mass = self:GetBuildCosts(focus:GetBlueprint())
            end
            energy = energy * (self.EnergyBuildAdjMod or 1)
            if energy < 1 then
                energy = 1
            end
            mass = mass * (self.MassBuildAdjMod or 1)
            if mass < 1 then
                mass = 1
            end
            time = time * (self.BuildTimeMultiplier or 1)
            build_rate = 1 / time
            energy_rate = energy / time
            mass_rate = mass / time
        end

        if self.MaintenanceConsumption then
            local mai_energy = (myBlueprint.Economy.MaintenanceConsumptionPerSecondEnergy or self.EnergyMaintenanceConsumptionOverride) or 0
            local mai_mass = myBlueprint.Economy.MaintenanceConsumptionPerSecondMass or 0

            # apply bonuses
            mai_energy = mai_energy * (100 + self.EnergyModifier) * (self.EnergyMaintAdjMod or 1) * 0.01
            mai_mass = mai_mass * (100 + self.MassModifier) * (self.MassMaintAdjMod or 1) * 0.01

            energy_rate = energy_rate + mai_energy
            mass_rate = mass_rate + mai_mass
        end

        # apply minimum rates
        energy_rate = math.max(energy_rate, myBlueprint.Economy.MinConsumptionPerSecondEnergy or 0)
        mass_rate = math.max(mass_rate, myBlueprint.Economy.MinConsumptionPerSecondMass or 0)

        self:SetBuildRate(build_rate)
        self:SetConsumptionPerSecondEnergy(energy_rate)
        self:SetConsumptionPerSecondMass(mass_rate)

        if (energy_rate > 0) or (mass_rate > 0) then
            self:SetConsumptionActive(true)
        else
            self:SetConsumptionActive(false)
        end
    end,

    UpdateProductionValues = function(self)
        local bpEcon = self:GetBlueprint().Economy
        if not bpEcon then return end
        self:SetProductionPerSecondEnergy((bpEcon.ProductionPerSecondEnergy or 0) * (self.EnergyProdAdjMod or 1))
        self:SetProductionPerSecondMass((bpEcon.ProductionPerSecondMass or 0) * (self.MassProdAdjMod or 1))
    end,

    SetEnergyMaintenanceConsumptionOverride = function(self, override)
        self.EnergyMaintenanceConsumptionOverride = override or 0
    end,


    ##########################################################################################
    ## DAMAGE
    ##########################################################################################
    #Sets if the unit can take damage.  val = true means it can take damage.
    #val = false means it can't take damage
    SetCanTakeDamage = function(self, val)
        self.CanTakeDamage = val
    end,

    OnDamage = function(self, instigator, amount, vector, damageType)
        if self.CanTakeDamage then
            self:DoOnDamagedCallbacks(instigator)
            self:DoTakeDamage(instigator, amount, vector, damageType)
        end
    end,

    DoTakeDamage = function(self, instigator, amount, vector, damageType)
        local preAdjHealth = self:GetHealth()
        self:AdjustHealth(instigator, -amount)
        local health = self:GetHealth()
        if( health <= 0 ) then
            if( damageType == 'Reclaimed' ) then
                self:Destroy()
            else
                local excessDamageRatio = 0.0
                # Calculate the excess damage amount
                local excess = preAdjHealth - amount
                local maxHealth = self:GetMaxHealth()
                if(excess < 0 and maxHealth > 0) then
                    excessDamageRatio = -excess / maxHealth
                end
                self:Kill(instigator, damageType, excessDamageRatio)
            end
        end
    end,

    ManageDamageEffects = function(self, newHealth, oldHealth)
        #LOG('*DEBUG: ManageDamageEffects, New: ', repr(newHealth), ' Old: ', repr(oldHealth))
        local army = self:GetArmy()
        if newHealth < oldHealth then
            local totalBones = self:GetBoneCount()

            if newHealth >= 0.5 and newHealth < 0.75 then
                for i = 1, self.FxDamage1Amount do
                    local bone = Random(1, totalBones) - 1
                    local effects = self.FxDamage1[Random(1,table.getn(self.FxDamage1))]
                    if not effects then return end
                    for k, v in effects do
                        table.insert(self.DamageEffectsBag[1], CreateAttachedEmitter(self,bone,army, v):ScaleEmitter(self.FxDamageScale))
                    end
                end
            elseif newHealth >= 0.25 and newHealth < 0.5 then
                for i = 1, self.FxDamage2Amount do
                    local bone = Random(1, totalBones) - 1
                    local effects = self.FxDamage2[Random(1,table.getn(self.FxDamage2))]
                    if not effects then return end
                    for k, v in effects do
                        table.insert(self.DamageEffectsBag[2], CreateAttachedEmitter(self,bone,army, v):ScaleEmitter(self.FxDamageScale))
                    end
                end
            elseif newHealth == 0 then
                for i = 1, self.FxDamage3Amount do
                    local bone = Random(1, totalBones) - 1
                    local effects = self.FxDamage3[Random(1,table.getn(self.FxDamage3))]
                    if not effects then return end
                    for k, v in effects do
                        table.insert(self.DamageEffectsBag[3], CreateAttachedEmitter(self,bone,army, v):ScaleEmitter(self.FxDamageScale))
                    end
                end
            end
        else
            if newHealth <= 0.25 and newHealth > 0 then
                for k, v in self.DamageEffectsBag[3] do
                    v:Destroy()
                end
            elseif newHealth <= 0.5 and newHealth > 0.25 then
                for k, v in self.DamageEffectsBag[2] do
                    v:Destroy()
                end
            elseif newHealth <= 0.75 and newHealth > 0.5 then
                for k, v in self.DamageEffectsBag[1] do
                    v:Destroy()
                end
            end
        end
    end,

    OnHealthChanged = function(self, new, old)
        self:ManageDamageEffects(new, old)
    end,


    DestroyAllDamageEffects = function(self)
        for kb, vb in self.DamageEffectsBag do
            for ke, ve in vb do
                ve:Destroy()
            end
        end
    end,

    CheckCanBeKilled = function(self,other)
        return self.CanBeKilled
    end,

    # On killed: this function plays when the unit takes a mortal hit.  It plays all the default death effect
    # it also spawns the wreckage based upon how much it was overkilled.
    OnKilled = function(self, instigator, type, overkillRatio)
        local bp = self:GetBlueprint()
        if self:GetCurrentLayer() == 'Water' and bp.Physics.MotionType == 'RULEUMT_Hover' then
            self:PlayUnitSound('HoverKilledOnWater')
        end
        self:PlayUnitSound('Killed')

        #If factory, destory what I'm building if I die
        if EntityCategoryContains(categories.FACTORY, self) then        
            if self.UnitBeingBuilt and not self.UnitBeingBuilt:IsDead() and self.UnitBeingBuilt:GetFractionComplete() != 1 then
                self.UnitBeingBuilt:Destroy()
            end
        end

        if self.PlayDeathAnimation then
            self:ForkThread(self.PlayAnimationThread, 'AnimationDeath')
            self:SetCollisionShape('None')
        end
        self:OnKilledVO()
        self:DoOnKilledCallbacks()
        self:DestroyTopSpeedEffects()

        if self.UnitBeingTeleported and not self.UnitBeingTeleported:IsDead() then
            self.UnitBeingTeleported:Destroy()
            self.UnitBeingTeleported = nil
        end

        #Notify instigator that you killed me.
        if instigator and instigator:IsUnit() then
            instigator:OnKilledUnit(self)
        end
        if self.DeathWeaponEnabled != false then
            self:DoDeathWeapon()
        end
        self:ForkThread(self.DeathThread, overkillRatio )
    end,


    #Sets if the unit can be killed.  val = true means it can be killed.
    #val = false means it can't be killed
    SetCanBeKilled = function(self, val)
        self.CanBeKilled = val
    end,

    OnKilledUnit = function(self, unitKilled)
        self:CheckVeteranLevel()
    end,

    DoDeathWeapon = function(self)
        local bp = self:GetBlueprint()
        for k, v in bp.Weapon do
            if(v.Label == 'DeathWeapon') then
                if v.FireOnDeath == true then
                    self:SetWeaponEnabledByLabel('DeathWeapon', true)
                    self:GetWeaponByLabel('DeathWeapon'):Fire()
                else
                    self:ForkThread(self.DeathWeaponDamageThread, v.DamageRadius, v.Damage, v.DamageType, v.DamageFriendly)
                end
                break
            end
        end
    end,

    OnCollisionCheck = function(self, other, firingWeapon)
        if self.DisallowCollisions then
            return false
        end
        if EntityCategoryContains(categories.PROJECTILE, other) then
            if self:GetArmy() == other:GetArmy() then
                return other:GetCollideFriendly()
            end
        end
        return true
    end,


    OnCollisionCheckWeapon = function(self, firingWeapon)
        if self.DisallowCollisions then
            return false
        end
        local collide = firingWeapon:GetBlueprint().CollideFriendly
        if collide == false then
            if self:GetArmy() == firingWeapon.unit:GetArmy() then
                return false
            end
        end
        return true
    end,


    PlayAnimationThread = function(self, anim)
        local bp = self:GetBlueprint().Display[anim]
        if bp then
            local totWeight = 0
            for k, v in bp do
                if v.Weight then
                    totWeight = totWeight + v.Weight
                end
            end
            local val = 1
            local num = Random(0, totWeight)
            for k, v in bp do
                if v.Weight then
                    val = val + v.Weight
                end
                if num < val then
                    bp = v
                    break
                end
            end
            if bp.Mesh then
                self:SetMesh(bp.Mesh)
            end
            if bp.Animation then
                local sinkAnim = CreateAnimator(self)
                self:StopRocking()
                self.DeathAnimManip = sinkAnim
                sinkAnim:PlayAnim(bp.Animation)
                if bp.AnimationRateMax and bp.AnimationRateMin then
                    local rate = Random(bp.AnimationRateMin * 10, bp.AnimationRateMax * 10) / 10
                    sinkAnim:SetRate(rate)
                end
                self.Trash:Add(sinkAnim)
                WaitFor(sinkAnim)
            end
        end
    end,

    #
    # Create a unit's wrecked mesh blueprint from its regular mesh blueprint, by changing the shader and albedo
    #
    CreateWreckage = function( self, overkillRatio )

		# if overkill ratio is high, the wreck is vaporized! No wreackage for you!
		if overkillRatio then
			if overkillRatio > 1.0 then
				return
			end
		end

		# generate wreakage in place of the dead unit
        if self:GetBlueprint().Wreckage.WreckageLayers[self:GetCurrentLayer()] then
			self:CreateWreckageProp(overkillRatio)
        end
    end,

    CreateWreckageProp = function( self, overkillRatio )
		local bp = self:GetBlueprint()
		local wreck = bp.Wreckage.Blueprint
		#if wreck and ( wreck == '/props/DefaultWreckage/DefaultWreckage_prop.bp' ) then
		if wreck then
			#LOG('*DEBUG: Spawning Wreckage = ', repr(wreck), 'overkill = ',repr(overkillRatio))
			local pos = self:GetPosition()
			local mass = bp.Economy.BuildCostMass * (bp.Wreckage.MassMult or 0)
			local energy = bp.Economy.BuildCostEnergy * (bp.Wreckage.EnergyMult or 0)
			local time = (bp.Wreckage.ReclaimTimeMultiplier or 1)
			pos[2] = GetSurfaceHeight(pos[1], pos[3]) + GetTerrainTypeOffset(pos[1], pos[3])

			local prop = CreateProp( pos, wreck )
			prop:SetScale(bp.Display.UniformScale)
			prop:SetOrientation(self:GetOrientation(), true)
			prop:SetPropCollision('Box', bp.CollisionOffsetX, bp.CollisionOffsetY, bp.CollisionOffsetZ, bp.SizeX* 0.5, bp.SizeY* 0.5, bp.SizeZ * 0.5)
			prop:SetMaxReclaimValues(time, time, mass, energy)

			mass = mass - (mass * (overkillRatio or 1))
			energy = energy - (energy * (overkillRatio or 1))
			time = time - (time * (overkillRatio or 1))

			prop:SetReclaimValues(time, time, mass, energy)
			prop:SetMaxHealth(bp.Defense.Health)
			prop:SetHealth(self, bp.Defense.Health * (bp.Wreckage.HealthMult or 1))

			#FIXME: SetVizToNeurals('Intel') is correct here, so you can't see enemy wreckage appearing
			# under the fog. However the engine has a bug with prop intel that makes the wreckage
			# never appear at all, even when you drive up to it, so this is disabled for now.
			#prop:SetVizToNeutrals('Intel')

			prop:SetMesh(bp.Display.MeshBlueprintWrecked)
			prop.AssociatedBP = self:GetBlueprint().BlueprintId

			# Create some ambient wreckage smoke
			explosion.CreateWreckageEffects(self,prop)
		#elseif wreck then
		#	local pos = self:GetPosition()
		#	local mass = bp.Economy.BuildCostMass * (bp.Wreckage.MassMult or 0)
		#	local energy = bp.Economy.BuildCostEnergy * (bp.Wreckage.EnergyMult or 0)
		#	local time = (bp.Wreckage.ReclaimTimeMultiplier or 1)
		#	pos[2] = GetSurfaceHeight(pos[1], pos[3]) + GetTerrainTypeOffset(pos[1], pos[3])
        #
		#	local prop = CreateProp( pos, wreck )
		#	prop:SetScale(1.0)
		#	prop:SetOrientation(self:GetOrientation(), true)
		#	prop:SetPropCollision('Box', bp.CollisionOffsetX, bp.CollisionOffsetY, bp.CollisionOffsetZ, bp.SizeX* 0.5, bp.SizeY* 0.5, bp.SizeZ * 0.5)
		#	prop:SetMaxReclaimValues(time, time, mass, energy)
		#	LOG('Prop ', repr(prop))
		#	LOG('Prop Mesh ', wreck)
		#	LOG('Prop Mesh ', prop.GetBlueprint().Display.Mesh)
        #
		#	mass = mass - (mass * (overkillRatio or 1))
		#	energy = energy - (energy * (overkillRatio or 1))
		#	time = time - (time * (overkillRatio or 1))
        #
		#	prop:SetReclaimValues(time, time, mass, energy)
		#	prop:SetMaxHealth(bp.Defense.Health)
		#	prop:SetHealth(self, bp.Defense.Health * (bp.Wreckage.HealthMult or 1))
		#	
        #    prop:SetMesh(wreck.Display)
		#	prop.AssociatedBP = self:GetBlueprint().BlueprintId
		#			
		#	# Create some ambient wreckage smoke
		#	explosion.CreateWreckageEffects(self,prop)
		end
		
    end,

    CreateUnitDestructionDebris = function( self, high, low, chassis )
        #LOG('*DEBUG: CreateUnitDestructionDebris TOSSING HIGH = ', repr(high), ' LOW = ', repr(low), ' CHASSIS = ', repr(chassis))
        local HighDestructionParts = table.getn(self.DestructionPartsHighToss)
        local LowDestructionParts = table.getn(self.DestructionPartsLowToss)
        local ChassisDestructionParts = table.getn(self.DestructionPartsChassisToss)

        # Limit the number of parts that we throw out
        local HighPartLimit = HighDestructionParts
        local LowPartLimit = LowDestructionParts
        local ChassisPartLimit = ChassisDestructionParts

        # Create projectiles and accelerate them out and away from the unit
        if high and (HighDestructionParts > 0) then
            HighPartLimit = Random( 1, HighDestructionParts)
            for i = 1, HighPartLimit do
                self:ShowBone( self.DestructionPartsHighToss[i], false )
                boneProj = self:CreateProjectileAtBone('/effects/entities/DebrisBoneAttachHigh01/DebrisBoneAttachHigh01_proj.bp',self.DestructionPartsHighToss[i])
                self:AttachBoneToEntityBone(self.DestructionPartsHighToss[i],boneProj,-1,false)
                #explosion.CreateUnitDebrisEffects( self, self.DestructionPartsHighToss[i] )
            end
        end
        if low and (LowDestructionParts > 0) then
            LowPartLimit = Random( 1, LowDestructionParts)
            #LOG('*DEBUG: CreateUnitDestructionDebris Parts', repr(self.DestructionPartsLowToss))
            #LOG('*DEBUG: CreateUnitDestructionDebris LowPartLimit ', repr(LowPartLimit))
            for i = 1, LowPartLimit do
                self:ShowBone( self.DestructionPartsLowToss[i], false )
                boneProj = self:CreateProjectileAtBone('/effects/entities/DebrisBoneAttachLow01/DebrisBoneAttachLow01_proj.bp',self.DestructionPartsLowToss[i])
                self:AttachBoneToEntityBone(self.DestructionPartsLowToss[i],boneProj,-1,false)
                #explosion.CreateUnitDebrisEffects( self, self.DestructionPartsLowToss[i] )
            end
        end
        if chassis and (ChassisDestructionParts > 0) then
            ChassisPartLimit = Random( 1, ChassisDestructionParts)
            for i = 1, Random( 1, ChassisDestructionParts) do
                self:ShowBone( self.DestructionPartsChassisToss[i], false )
                boneProj = self:CreateProjectileAtBone('/effects/entities/DebrisBoneAttachChassis01/DebrisBoneAttachChassis01_proj.bp',self.DestructionPartsChassisToss[i])
                self:AttachBoneToEntityBone(self.DestructionPartsChassisToss[i],boneProj,-1,false)
                #explosion.CreateUnitDebrisEffects( self, self.DestructionPartsChassisToss[i] )
            end
        end
    end,

    CreateDestructionEffects = function( self, overKillRatio )
        explosion.CreateScalableUnitExplosion( self, overKillRatio )
    end,

    DeathWeaponDamageThread = function( self , damageRadius, damage, damageType, damageFriendly)
        WaitSeconds( 0.1 )
        DamageArea(self, self:GetPosition(), damageRadius or 1, damage or 1, damageType or 'Normal', damageFriendly or true)
    end,

    DeathThread = function( self, overkillRatio )
        #LOG('*DEBUG: OVERKILL RATIO = ', repr(overkillRatio))

        WaitSeconds( utilities.GetRandomFloat( self.DestructionExplosionWaitDelayMin, self.DestructionExplosionWaitDelayMax) )
        self:DestroyAllDamageEffects()

        if self.PlayDestructionEffects then
            self:CreateDestructionEffects( self, overkillRatio )
        end

        #MetaImpact( self, self:GetPosition(), 0.1, 0.5 )

        self:CreateWreckage( overkillRatio )

        if( self.HideUnitOnDestroy ) then
            self:HideBone(0, true)
        end

        # CURRENTLY DISABLED UNTIL DESTRUCTION
        # Create destruction debris out of the mesh, currently these projectiles look like crap,
        # since projectile rotation and terrain collision doesn't work that great. These are left in
        # hopes that this will look better in the future.. =)
        if( self.ShowUnitDestructionDebris and overkillRatio ) then
            if overkillRatio <= 1 then
                self.CreateUnitDestructionDebris( self, true, true, false )
            elseif overkillRatio <= 2 then
                self.CreateUnitDestructionDebris( self, true, true, false )
            elseif overkillRatio <= 3 then
                self.CreateUnitDestructionDebris( self, true, true, true )
            else #VAPORIZED
                self.CreateUnitDestructionDebris( self, true, true, true )
            end
        end

        #LOG('*DEBUG: DeathThread Destroying in ',  self.DeathThreadDestructionWaitTime )
        WaitSeconds(self.DeathThreadDestructionWaitTime)
        if self.DeathAnimManip then
            WaitFor(self.DeathAnimManip)
        end
        self:PlayUnitSound('Destroyed')
        self:Destroy()
    end,

    OnDestroy = function(self)
    
        #If factory, destory what I'm building if I die
        if EntityCategoryContains(categories.FACTORY, self) then        
            if self.UnitBeingBuilt and not self.UnitBeingBuilt:IsDead() and self.UnitBeingBuilt:GetFractionComplete() != 1 then
                self.UnitBeingBuilt:Destroy()
            end
        end
    
        # destroy everything added to the trash
        #LOG('*DEBUG: Unit:OnDestroy')
        self.Trash:Destroy()
        if self.BuildEffectsBag then
            self.BuildEffectsBag:Destroy()
        end
        if self.CaptureEffectsBag then
			self.CaptureEffectsBag:Destroy()
		end
        if self.ReclaimEffectsBag then
			self.ReclaimEffectsBag:Destroy()
        end
        if self.OnBeingBuiltEffectsBag then
            self.OnBeingBuiltEffectsBag:Destroy()
        end

        if self.TeleportDrain then
            RemoveEconomyEvent( self, self.TeleportDrain)
        end
        RemoveAllUnitEnhancements(self)
        ChangeState(self, self.DeadState)
    end,

    HideLandBones = function(self)
        #LOG( self:GetUnitId() .. " being built on layer '" .. self:GetCurrentLayer() .. "'" )
        #HIDE THE BONES FOR BUILDINGS BUILT ON LAND.
        if self.LandBuiltHiddenBones and self:GetCurrentLayer() == 'Land' then
            for k, v in self.LandBuiltHiddenBones do
                #LOG('*DEBUG: HIDING BONE = ', repr(v), ' BECAUSE IT IS BUILT ON LAND')
                if self:IsValidBone(v) then
                    self:HideBone(v, true)
                #else
                    #LOG('*WARNING: NOT HIDING BONE ', repr(v), ' BECAUSE IT IS INVALID ON UNIT ', repr(self:GetUnitId()))
                end
            end
        #else
            #LOG('*DEBUG: _NOT_ HIDING BONE = ', repr(v), ' BECAUSE IT IS BUILT ON ', self:GetCurrentLayer())
        end
    end,

    #GENERIC FUNCTION FOR SHOWING A TABLE OF BONES
    #TABLE = LIST OF BONES
    #CHILDREND = TRUE/FALSE TO SHOW CHILD BONES
    ShowBones = function(self, table, children)
        #LOG('*DEBUG: IN SHOWBONES TABLE = ', repr(table))
        for k, v in table do
            if self:IsValidBone(v) then
                self:ShowBone(v, children)
            else
                LOG('*WARNING: TRYING TO SHOW BONE ', repr(v), ' ON UNIT ',repr(self:GetUnitId()),' BUT IT DOES NOT EXIST IN THE MODEL. PLEASE CHECK YOUR SCRIPT IN THE BUILD PROGRESS BONES.')
            end
        end
    end,


    OnFerryPointSet = function(self)
        local bp = self:GetBlueprint().Audio
        if bp then
            local aiBrain = self:GetAIBrain()
            local factionIndex = aiBrain:GetFactionIndex()
            if factionIndex == 1 then
                if bp['FerryPointSetByUEF'] then
                    aiBrain:FerryPointSet(bp['FerryPointSetByUEF'])
                end
            elseif factionIndex == 2 then
                if bp['FerryPointSetByAeon'] then
                    aiBrain:FerryPointSet(bp['FerryPointSetByAeon'])
                end
            elseif factionIndex == 3 then
                if bp['FerryPointSetByCybran'] then
                    aiBrain:FerryPointSet(bp['FerryPointSetByCybran'])
                end
            end
        end
    end,

    OnDamageBy = function(self,index)
        local bp = self:GetBlueprint().Audio
        if bp then
            local aiBrain = self:GetAIBrain()
            local factionIndex = aiBrain:GetFactionIndex()
            if factionIndex == 1 then
                if bp['UnderAttackUEF'] then
                    aiBrain:UnderAttack(bp['UnderAttackUEF'])
                end
                if EntityCategoryContains(categories.STRUCTURE, self) then
                    if bp['BaseUnderAttackUEF'] then
                        aiBrain:BaseUnderAttack(bp['BaseUnderAttackUEF'])
                    end
                end
            elseif factionIndex == 2 then
                if bp['UnderAttackAeon'] then
                    aiBrain:UnderAttack(bp['UnderAttackAeon'])
                end
                if EntityCategoryContains(categories.STRUCTURE, self) then
                    if bp['BaseUnderAttackAeon'] then
                        aiBrain:BaseUnderAttack(bp['BaseUnderAttackAeon'])
                    end
                end
            elseif factionIndex == 3 then
                if bp['UnderAttackCybran'] then
                    aiBrain:UnderAttack(bp['UnderAttackCybran'])
                end
                if EntityCategoryContains(categories.STRUCTURE, self) then
                    if bp['BaseUnderAttackCybran'] then
                        aiBrain:BaseUnderAttack(bp['BaseUnderAttackCybran'])
                    end
                end
            end
        end
    end,

    OnNukeArmed = function(self)
        local bp = self:GetBlueprint().Audio
        if bp then
            local aiBrain = self:GetAIBrain()
            local factionIndex = aiBrain:GetFactionIndex()
            if factionIndex == 1 then
                if bp['NukeArmedUEF'] then
                    aiBrain:NukeArmed(bp['NukeArmedUEF'])
                end
            elseif factionIndex == 2 then
                if bp['NukeArmedAeon'] then
                    aiBrain:NukeArmed(bp['NukeArmedAeon'])
                end
            elseif factionIndex == 3 then
                if bp['NukeArmedCybran'] then
                    aiBrain:NukeArmed(bp['NukeArmedCybran'])
                end
            end
        end
    end,

    OnNukeLaunched = function(self)
        local bp = self:GetBlueprint().Audio
        local unitBrain = self:GetAIBrain()
        if bp then
            for num, aiBrain in ArmyBrains do
                local factionIndex = aiBrain:GetFactionIndex()
                if unitBrain == aiBrain then
                    if factionIndex == 1 then
                        if bp['NuclearLaunchInitiatedByUEF'] then
                            aiBrain:NuclearLaunchInitiated(bp['NuclearLaunchInitiatedByUEF'])
                        end
                    elseif factionIndex == 2 then
                        if bp['NuclearLaunchInitiatedByAeon'] then
                            aiBrain:NuclearLaunchInitiated(bp['NuclearLaunchInitiatedByAeon'])
                        end
                    elseif factionIndex == 2 then
                        if bp['NuclearLaunchInitiatedByCybran'] then
                            aiBrain:NuclearLaunchInitiated(bp['NuclearLaunchInitiatedByCybran'])
                        end
                    end
                else
                    if factionIndex == 1 then
                        if bp['NuclearLaunchDetectedUEF'] then
                            aiBrain:NuclearLaunchDetected(bp['NuclearLaunchDetectedUEF'])
                        end
                    elseif factionIndex == 2 then
                        if bp['NuclearLaunchDetectedAeon'] then
                            aiBrain:NuclearLaunchDetected(bp['NuclearLaunchDetectedAeon'])
                        end
                    elseif factionIndex == 3 then
                        if bp['NuclearLaunchDetectedCybran'] then
                            aiBrain:NuclearLaunchDetected(bp['NuclearLaunchDetectedCybran'])
                        end
                    end
                end
            end
        end
    end,

    OnKilledVO = function(self)
        local bp = self:GetBlueprint().Audio
        if bp then
            for num, aiBrain in ArmyBrains do
                local factionIndex = aiBrain:GetFactionIndex()
                if factionIndex == 1 then
                    if bp['ExperimentalUnitDestroyedUEF'] then
                        aiBrain:ExperimentalUnitDestroyed(bp['ExperimentalUnitDestroyedUEF'])
                    elseif bp['BattleshipDestroyedUEF'] then
                        aiBrain:BattleshipDestroyed(bp['BattleshipDestroyedUEF'])
                    end
                elseif factionIndex == 2 then
                    if bp['ExperimentalUnitDestroyedAeon'] then
                        aiBrain:ExperimentalUnitDestroyed(bp['ExperimentalUnitDestroyedAeon'])
                    elseif bp['BattleshipDestroyedAeon'] then
                        aiBrain:BattleshipDestroyed(bp['BattleshipDestroyedAeon'])
                    end
                elseif factionIndex == 3 then
                    if bp['ExperimentalUnitDestroyedCybran'] then
                        aiBrain:ExperimentalUnitDestroyed(bp['ExperimentalUnitDestroyedCybran'])
                    elseif bp['BattleshipDestroyedCybran'] then
                        aiBrain:BattleshipDestroyed(bp['BattleshipDestroyedCybran'])
                    end
                end
            end
        end
    end,

    SetAllWeaponsEnabled = function(self, enable)
        for i = 1, self:GetWeaponCount() do
            local wep = self:GetWeapon(i)
            wep:SetEnabled(enable)
            wep:AimManipulatorSetEnabled(enable)
        end
    end,

    SetWeaponEnabledByLabel = function(self, label, enable)
        local wep = self:GetWeaponByLabel(label)
        wep:SetEnabled(enable)
        wep:AimManipulatorSetEnabled(enable)
    end,

    GetWeaponManipulatorByLabel = function(self, label)
        local wep = self:GetWeaponByLabel(label)
        return wep:GetAimManipulator()
    end,

    GetWeaponByLabel = function(self, label)
        local wep
        for i = 1, self:GetWeaponCount() do
            wep = self:GetWeapon(i)
            if (wep:GetBlueprint().Label == label) then
                break
            end
        end
        return wep
    end,

    ResetWeaponByLabel = function(self, label)
        local wep = self:GetWeaponByLabel(label)
        wep:ResetTarget()
    end,

    SetDeathWeaponEnabled = function(self, enable)
        self.DeathWeaponEnabled = enable
    end,

    #############################################################################################
    ## CONSTRUCTING - BEING BUILT
    #############################################################################################
    OnBeingBuiltProgress = function(self, unit, oldProg, newProg)
    end,

    OnStartBeingBuilt = function(self, builder, layer)
        self:StartBeingBuiltEffects(builder, layer)
        local aiBrain = self:GetAIBrain()
        if table.getn(aiBrain.UnitBuiltTriggerList) > 0 then
            for k,v in aiBrain.UnitBuiltTriggerList do
                if EntityCategoryContains(v.Category, self) then
                    self:ForkThread(self.UnitBuiltPercentageCallbackThread, v.Percent, v.Callback)
                end
            end
        end
    end,

    UnitBuiltPercentageCallbackThread = function(self, percent, callback)
        while not self:IsDead() and self:GetHealthPercent() < percent do
            WaitSeconds(1)
        end
        local aiBrain = self:GetAIBrain()
        for k,v in aiBrain.UnitBuiltTriggerList do
            if v.Callback == callback then
                callback(self)
                aiBrain.UnitBuiltTriggerList[k] = nil
            end
        end
    end,

    OnStopBeingBuilt = function(self, builder, layer)

        local bp = self:GetBlueprint()

        self:SetupIntel()

        self:StopBeingBuiltEffects(builder, layer)

        if ( self:GetCurrentLayer() == 'Water' ) then
            self:StartRocking()
            local surfaceAnim = self:GetBlueprint().Display.AnimationSurface
            if not self.SurfaceAnimator and surfaceAnim then
                self.SurfaceAnimator = CreateAnimator(self)
            end
            if surfaceAnim and self.SurfaceAnimator then
                self.SurfaceAnimator:PlayAnim(surfaceAnim):SetRate(1)
            end
        end

        if bp.Defense.LifeTime then
            self:ForkThread(self.LifeTimeThread)
        end

        self:PlayUnitSound('DoneBeingBuilt')

        self:PlayUnitAmbientSound( 'ActiveLoop' )

        if self.DisallowCollisions and builder then
            self.DisallowCollisions = false
            local healthPercentage = builder:GetHealthPercent()
            local newHealthAmount = healthPercentage * self:GetBlueprint().Defense.MaxHealth
            self:SetHealth(self, newHealthAmount)
        end

        # Turn off land bones if this unit has them.
        self:HideLandBones()
        self:DoOnStopBeingBuiltCallbacks()

        # Create any idle effects on unit
        self:CreateIdleEffects()

        # If we have a shield spec'd create it.
        if bp.Defense.Shield.ShieldSize > 0 then
            if bp.Defense.Shield.StartOn != false then
                if bp.Defense.Shield.PersonalShield == true then
                    self:CreatePersonalShield()
                else
                    self:CreateShield()
                end
                self:ToggleScriptBit('RULEUCC_ShieldToggle')
            end
        end

        if bp.Display.AnimationPermOpen then
            self.PermOpenAnimManipulator = CreateAnimator(self):PlayAnim(bp.Display.AnimationPermOpen)
            self.Trash:Add(self.PermOpenAnimManipulator)
        end

        # Initialize movement effects subsystems, idle effects, beam exhaust, and footfall manipulators
        local bpTable = bp.Display.MovementEffects
        if bpTable.Land or bpTable.Air or bpTable.Water or bpTable.Sub or bpTable.BeamExhaust then
            self.MovementEffectsExist = true
            if bpTable.BeamExhaust and (bpTable.BeamExhaust.Idle != false) then
                self:UpdateBeamExhaust( 'Idle' )
            end
            if not self.Footfalls and bpTable[layer].Footfall then
                #LOG('Creating Footfall Manips')
                self.Footfalls = self:CreateFootFallManipulators( bpTable[layer].Footfall )
            end
        else
            self.MovementEffectsExist = false
        end
    end,

    StartBeingBuiltEffects = function(self, builder, layer)
        self:SetMesh(self:GetBlueprint().Display.BuildMeshBlueprint, true)
    end,

    StopBeingBuiltEffects = function(self, builder, layer)
        local bp = self:GetBlueprint().Display
        local useTerrainType = false
        if bp then
            if bp.TerrainMeshes then
                local bpTM = bp.TerrainMeshes
                local pos = self:GetPosition()
                local terrainType = self.TerrainTypeOnCreate or GetTerrainType( pos[1], pos[3] )
                if bpTM[terrainType.Style] then
                    self:SetMesh(bpTM[terrainType.Style])
                    useTerrainType = true
                end
            end
            if not useTerrainType then
                self:SetMesh(bp.MeshBlueprint, true)
            end
        end
        self.OnBeingBuiltEffectsBag:Destroy()
    end,

    OnFailedToBeBuilt = function(self)
        self:Destroy()
    end,


    #############################################################################################
    ## CONSTRUCTING - BUILDING
    #############################################################################################
    SetupBuildBones = function(self)

        # Syntactical reference:
        #CreateBuilderArmController(unit,turretBone, [barrelBone], [aimBone])
        #BuilderArmManipulator:SetAimingArc(minHeading, maxHeading, headingMaxSlew, minPitch, maxPitch, pitchMaxSlew)
        self.BuildArmManipulator = CreateBuilderArmController(self, self.BuildBones.YawBone or 0 , self.BuildBones.PitchBone or 0, self.BuildBones.AimBone or 0)
        self.BuildArmManipulator:SetAimingArc(-180, 180, 90, -90, 90, 30)
        self.BuildArmManipulator:SetPrecedence(5)
        if self.BuildingOpenAnimManip and self.BuildArmManipulator then
            self.BuildArmManipulator:Disable()
        end
    end,

    BuildManipulatorSetEnabled = function(self, enable)
        if not self.BuildArmManipulator then return end
        if enable then
            self.BuildArmManipulator:Enable()
        else
            self.BuildArmManipulator:Disable()
        end
    end,

    OnStartBuild = function(self, unitBeingBuilt, order)
        local bp = self:GetBlueprint()
        if order != 'Upgrade' or bp.Display.ShowBuildEffectsDuringUpgrade then
            self:StartBuildingEffects(unitBeingBuilt, order)
        end
        self:SetActiveConsumptionActive()
        self:PlayUnitSound('Construct')
        self:PlayUnitAmbientSound('ConstructLoop')
        if bp.General.UpgradesTo and unitBeingBuilt:GetUnitId() == bp.General.UpgradesTo and order == 'Upgrade' then
            unitBeingBuilt.DisallowCollisions = true
        end
        self:UpdateConsumptionValues(unitBeingBuilt:GetBlueprint())
    end,

    OnStopBuild = function(self, unitBeingBuilt)
        self:StopBuildingEffects(unitBeingBuilt)
        self:SetActiveConsumptionInactive()
        self:DoOnUnitBuiltCallbacks(unitBeingBuilt)
        self:PlayUnitAmbientSound(nil)
        self:PlayUnitSound('ConstructStop')
    end,

    GetUnitBeingBuilt = function(self)
        return self.UnitBeingBuilt
    end,


    OnFailedToBuild = function(self)
        self:SetActiveConsumptionInactive()
        self:PlayUnitAmbientSound(nil)
    end,

    OnPrepareArmToBuild = function(self)
    end,

    OnStartBuilderTracking = function(self)
    end,

    OnStopBuilderTracking = function(self)
    end,

    OnBuildProgress = function(self, unit, oldProg, newProg)
    end,

    StartBuildingEffects = function(self, unitBeingBuilt, order)
        self.BuildEffectsBag:Add( self:ForkThread( self.CreateBuildEffects, unitBeingBuilt ) )
    end,

    CreateBuildEffects = function( self, unitBeingBuilt )
    end,

    StopBuildingEffects = function(self, unitBeingBuilt)
        self.BuildEffectsBag:Destroy()
    end,

    OnStartSacrifice = function(self, target_unit)
		EffectUtilities.PlaySacrificingEffects(self,target_unit)
    end,

    OnStopSacrifice = function(self, target_unit)
		EffectUtilities.PlaySacrificeEffects(self,target_unit)
        self:SetDeathWeaponEnabled(false)
        self:Destroy()
    end,


    ##########################################################################################
    ## INTEL
    ##########################################################################################
    #Setup the initial intelligence of the unit.  Return true if it can, false if it can't.
    SetupIntel = function(self)
        local bp = self:GetBlueprint()
        if bp.Intel then
            self.IntelDisables = {
                Radar = 1,
                Sonar = 1,
                Omni = 1,
                RadarStealth = 1,
                SonarStealth = 1,
                RadarStealthField = 1,
                SonarStealthField = 1,
                Cloak = 1,
                CloakField = 1,
                Spoof = 1,
                Jammer = 1,
            }
            self:EnableUnitIntel()
            return true
        end
        return false
    end,

    DisableUnitIntel = function(self, intel)
        if intel then
            self.IntelDisables[intel] = self.IntelDisables[intel] + 1
            self:DisableIntel(intel)
        else
            for k, v in self.IntelDisables do
                self.IntelDisables[k] = v + 1
                self:DisableIntel(k)
            end
            local bpVal = self:GetBlueprint().Intel.ReactivateTime or 10
            if bpVal and bpVal > 0 and not self:GetBlueprint().Intel.FreeIntel then
                if self.IntelThread then
                    self.IntelThread:Destroy()
                    self.IntelThread = nil
                end
                self.IntelThread = self:ForkThread(self.IntelReEnableThread)
            end
        end
        self:OnIntelDisabled()
    end,

    EnableUnitIntel = function(self, intel)
        local bpVal = self:GetBlueprint().Economy.MaintenanceConsumptionPerSecondEnergy
        local layer = self:GetCurrentLayer()
        local intEnabled = false
        if layer == 'Seabed' or layer == 'Sub' then
            self:EnableIntel('WaterVision')
        else
            self:EnableIntel('Vision')
        end
        if intel and (intel != 'Vision' or intel != 'WaterVision') then
            if self.IntelDisables[intel] == 1 then
                self:EnableIntel(intel)
                intEnabled = true
            end
            self.IntelDisables[intel] = self.IntelDisables[intel] - 1
        else
            for k, v in self.IntelDisables do
                if v == 1 then
                    self:EnableIntel(k)
                    intEnabled = true
                end
                self.IntelDisables[k] = v - 1
            end
            if bpVal and bpVal > 0 and not self:GetBlueprint().Intel.FreeIntel then
                if self.IntelThread then
                    self.IntelThread:Destroy()
                    self.IntelThread = nil
                end
                self.IntelThread = self:ForkThread(self.IntelWatchThread)
            end
        end
        if intEnabled then
            self:OnIntelEnabled()
        end
    end,

    OnIntelEnabled = function(self)

    end,

    OnIntelDisabled = function(self)

    end,


    #Watch the economy.  If this unit doesn't get all it needs, shut off the intel.
    IntelWatchThread = function(self)
        WaitSeconds(0.5)
        local fraction = self:GetResourceConsumed()
        while fraction == 1 do
            WaitSeconds(0.5)
            fraction = self:GetResourceConsumed()
        end
        self:DisableUnitIntel()
    end,

    IntelReEnableThread = function(self)
        local recharge = self:GetBlueprint().Intel.ReactivateTime or 10
        WaitSeconds(recharge)
        local layer = self:GetCurrentLayer()
        self:EnableUnitIntel()
    end,


    AddDetectedByHook = function(self,hook)
        if not self.DetectedByHooks then
            self.DetectedByHooks = {}
        end
        table.insert(self.DetectedByHooks,hook)
    end,

    RemoveDetectedByHook = function(self,hook)
        if self.DetectedByHooks then
            for k,v in self.DetectedByHooks do
                if v == hook then
                    table.remove(self.DetectedByHooks,k)
                    return
                end
            end
        end
    end,

    OnDetectedBy = function(self, index)
        if self.DetectedByHooks then
            for k,v in self.DetectedByHooks do
                v(self,index)
            end
        end

        local bp = self:GetBlueprint().Audio
        if bp then
            local aiBrain = ArmyBrains[index]
            local factionIndex = aiBrain:GetFactionIndex()
            if factionIndex == 1 then
                if bp['ExperimentalDetectedByUEF'] then
                    aiBrain:ExperimentalDetected(bp['ExperimentalDetectedByUEF'])
                elseif bp['EnemyForcesDetectedByUEF'] then
                    aiBrain:EnemyForcesDetected(bp['EnemyForcesDetectedByUEF'])
                end
            elseif factionIndex == 2 then
                if bp['ExperimentalDetectedByAeon'] then
                    aiBrain:ExperimentalDetected(bp['ExperimentalDetectedByAeon'])
                elseif bp['EnemyForcesDetectedByAeon'] then
                    aiBrain:EnemyForcesDetected(bp['EnemyForcesDetectedByAeon'])
                end
            elseif factionIndex == 3 then
                if bp['ExperimentalDetectedByCybran'] then
                    aiBrain:ExperimentalDetected(bp['ExperimentalDetectedByCybran'])
                elseif bp['EnemyForcesDetectedByCybran'] then
                    aiBrain:EnemyForcesDetected(bp['EnemyForcesDetectedByCybran'])
                end
            end
        end
    end,


    ##########################################################################################
    ## ENHANCEMENT
    ##########################################################################################

    OnEnhanceBegin = function(self, enhancement)
        self.BuildEnhancement = self:GetBlueprint().Enhancements[enhancement]
        self.EnhanceProgress = 0
        self:SetActiveConsumptionActive()
        self:PlayUnitSound('Construct')
        self:UpdateConsumptionValues()
        ChangeState(self,self.EnhancingState)
    end,

    OnEnhanceEnd = function(self, enhancement)
        self:SetActiveConsumptionInactive()
    end,

    OnEnhanceFail = function(self, enhancement)
        self:SetActiveConsumptionInactive()
    end,

    CreateEnhancement = function(self, enh)
        local bp = self:GetBlueprint().Enhancements[enh]
        if not bp then
            error('*ERROR: Got CreateEnhancement call with an enhancement that doesn\'t exist in the blueprint.', 2)
            return false
        end
        if bp.ShowBones then
            for k, v in bp.ShowBones do
                if self:IsValidBone(v) then
                    self:ShowBone(v, true)
                end
            end
        end
        if bp.HideBones then
            for k, v in bp.HideBones do
                if self:IsValidBone(v) then
                    self:HideBone(v, true)
                end
            end
        end
        AddUnitEnhancement(self, enh, bp.Slot or '')
        if bp.RemoveEnhancements then
            for k, v in bp.RemoveEnhancements do
                RemoveUnitEnhancement(self, v)
            end
        end
        self:RequestRefreshUI()
    end,

    HasEnhancement = function(self, enh)
        local entId = self:GetEntityId()
        local unitEnh = SimUnitEnhancements[entId]
        if unitEnh then
            for k,v in unitEnh do
                if v == enh then
                    return true
                end
            end
        end
        return false
    end,

    ##########################################################################################
    ##
    ##########################################################################################

    OnLayerChange = function(self, new, old)
        #LOG('LayerChange old=',old,' new=',new,' for ',self:GetBlueprint().BlueprintId )
        for i = 1, self:GetWeaponCount() do
            self:GetWeapon(i):SetValidTargetsForCurrentLayer(new)
        end

        local oldWater = (old == 'Seabed' or old == 'Sub')
        local newWater = (new == 'Seabed' or new == 'Sub')
        if oldWater != newWater then
            #Surfacing
            if oldWater then
                self:DisableIntel('WaterVision')
                self:EnableIntel('Vision')
            #Submerging
            else
                if EntityCategoryContains(categories.LAND, self) then
                    self:DisableIntel('Vision')
                end
                self:EnableIntel('WaterVision')
            end
        end

        if( new == 'Land' ) then
            self:PlayUnitSound('TransitionLand')
            self:PlayUnitAmbientSound('AmbientMoveLand')
        elseif(( new == 'Water' ) or ( new == 'Seabed' )) then
            self:PlayUnitSound('TransitionWater')
            self:PlayUnitAmbientSound('AmbientMoveWater')
        end

        local bpTable = self:GetBlueprint().Display.MovementEffects

        if not self.Footfalls and bpTable[new].Footfall then
            self.Footfalls = self:CreateFootFallManipulators( bpTable[new].Footfall )
        end

        self:CreateLayerChangeEffects( new, old )
    end,

    OnMotionHorzEventChange = function( self, new, old )
        if self:IsDead() then
            return
        end
        local layer = self:GetCurrentLayer()
        #LOG( 'OnMotionHorzEventChange, unit=', repr(self:GetBlueprint().BlueprintId), ' old = ', old, ', new = ', new, ', layer = ', self:GetCurrentLayer() )

        if ( old == 'Stopped' or (old == 'Stopping' and (new == 'Cruise' or new == 'TopSpeed'))) then
            # This will play the first appropriate StartMove sound that it finds
            if not (
                ((self:GetCurrentLayer() == 'Water') and self:PlayUnitSound('StartMoveWater')) or
                ((self:GetCurrentLayer() == 'Land') and self:PlayUnitSound('StartMoveLand')) or
                ((self:GetCurrentLayer() == 'Air') and self:PlayUnitSound('StartMoveAir'))
                )
            then
                self:PlayUnitSound('StartMove')
            end

            # Initiate the unit's ambient movement sound
            # Note that there is not currently an 'Air' version, and that
            # AmbientMoveWater plays if the unit is in either the Water or Seabed layer.
            if not (
                (((self:GetCurrentLayer() == 'Water') or (self:GetCurrentLayer() == 'Seabed')) and self:PlayUnitAmbientSound('AmbientMoveWater')) or
                ((self:GetCurrentLayer() == 'Land') and self:PlayUnitAmbientSound('AmbientMoveLand'))
                )
            then
                self:PlayUnitAmbientSound('AmbientMove')
            end

            self:StopRocking()
        end

        if ((new == 'Stopped' or new == 'Stopping') and (old == 'Cruise' or old == 'TopSpeed')) then
            # This will play the first appropriate StopMove sound that it finds
            if not (
                ((self:GetCurrentLayer() == 'Water') and self:PlayUnitSound('StopMoveWater')) or
                ((self:GetCurrentLayer() == 'Land') and self:PlayUnitSound('StopMoveLand')) or
                ((self:GetCurrentLayer() == 'Air') and self:PlayUnitSound('StopMoveAir'))
                )
            then
                self:PlayUnitSound('StopMove')
            end

            # Units in the water will rock back and forth a bit
            if ( self:GetCurrentLayer() == 'Water' ) then
                self:StartRocking()
            end
        end

        if( new == 'Stopped' or new == 'Stopping' ) then
            # Stop ambient sounds
            self:PlayUnitAmbientSound( nil )
        end

        if self.MovementEffectsExist then
            self:UpdateMovementEffectsOnMotionEventChange( new, old )
        end

        if old == 'Stopped' then
            self:DoOnHorizontalStartMoveCallbacks()
            for i = 1, self:GetWeaponCount() do
                local wep = self:GetWeapon(i)
                wep:OnMotionHorzEventChange(new, old)
            end
        end
    end,

    OnMotionVertEventChange = function( self, new, old )
        if self:IsDead() then
            return
        end
        local layer = self:GetCurrentLayer()

        #LOG( 'OnMotionVertEventChange, unit=', repr(self:GetBlueprint().BlueprintId), ' old = ', old, ', new = ', new, ', layer = ', self:GetCurrentLayer() )
        if (new == 'Down') then
            # Play the "landing" sound
            self:PlayUnitSound('Landing')
        elseif (new == 'Bottom') then
            # Play the "landed" sound
            self:PlayUnitSound('Landed')
        elseif (new == 'Up' or ( new == 'Top' and ( old == 'Down' or old == 'Bottom' ))) then
            # Play the "takeoff" sound
            self:PlayUnitSound('TakeOff')
        end

        # Adjust any beam exhaust
        if new == 'Bottom' then
            self:UpdateBeamExhaust('Landed')
        elseif old == 'Bottom' then
            self:UpdateBeamExhaust('Cruise')
        end

        # Surfacing and sinking, landing and take off idle effects
        if (new == 'Up' and old == 'Bottom') or
           (new == 'Down' and old == 'Top') then
            self:DestroyIdleEffects()
            if new == 'Up' and layer == 'Water' then
                self:PlayUnitSound('SurfaceStart')
            end
            if new == 'Down' and layer == 'Water' then
                self:PlayUnitSound('SubmergeStart')
                if self.SurfaceAnimator then
                    self.SurfaceAnimator:SetRate(-1)
                end
            end
        end

        if (new == 'Top' and old == 'Up') or
           (new == 'Bottom' and old == 'Down') then
            self:CreateIdleEffects()
            if new == 'Bottom' and layer == 'Water' then
                self:PlayUnitSound('SubmergeEnd')
            end
            if new == 'Top' and layer == 'Water' then
                self:PlayUnitSound('SurfaceEnd')
                local surfaceAnim = self:GetBlueprint().Display.AnimationSurface
                if not self.SurfaceAnimator and surfaceAnim then
                    self.SurfaceAnimator = CreateAnimator(self)
                end
                if surfaceAnim and self.SurfaceAnimator then
                    self.SurfaceAnimator:PlayAnim(surfaceAnim):SetRate(1)
                end
            end
        end

        self:CreateMotionChangeEffects(new,old)
    end,

    OnMotionTurnEventChange = function(self, newEvent, oldEvent)
        if self:IsDead() then
            return
        end

        if newEvent == 'Straight' then
            self:PlayUnitSound('MoveStraight')
        elseif newEvent == 'Turn' then
            self:PlayUnitSound('MoveTurn')
        elseif newEvent == 'SharpTurn' then
            self:PlayUnitSound('MoveSharpTurn')
        end
    end,

    OnTerrainTypeChange = function(self, new, old)
        #LOG('TerrainChange old=',repr(old.Description),' new=',repr(new.Description),' for ',self:GetBlueprint().BlueprintId)
        if self.MovementEffectsExist then
            self:DestroyMovementEffects()
            self:CreateMovementEffects( self.MovementEffectsBag, nil, new )
        end
    end,

    OnAnimCollision = function(self, bone, x, y, z)
        local layer = self:GetCurrentLayer()
        local bpTable = self:GetBlueprint().Display.MovementEffects

        if bpTable[layer].Footfall then
            bpTable = bpTable[layer].Footfall
            local effects = {}
            local scale = 1
            local offset = nil
            local army = self:GetArmy()
            local boneTable = nil

            if bpTable.Damage then
                local bpDamage = bpTable.Damage
                DamageArea(self, self:GetPosition(bone), bpDamage.Radius, bpDamage.Amount, bpDamage.Type, bpDamage.DamageFriendly )
            end

            if bpTable.CameraShake then
                local shake = bpTable.CameraShake
                self:ShakeCamera( shake.Radius, shake.MaxShakeEpicenter, shake.MinShakeAtRadius, shake.Interval )
            end

            for k, v in bpTable.Bones do
                if bone == v.FootBone then
                    boneTable = v
                    bone = v.FootBone
                    scale = boneTable.Scale or 1
                    offset = bone.Offset
                    if v.Type then
                        effects = self.GetTerrainTypeEffects( 'FXMovement', layer, self:GetPosition(v.FootBone), v.Type )
                    end
                    break
                end
            end

            if boneTable.Tread and self:GetTTTreadType(self:GetPosition(bone)) != 'None' then
                CreateSplatOnBone(self, boneTable.Tread.TreadOffset, 0, boneTable.Tread.TreadMarks, boneTable.Tread.TreadMarksSizeX, boneTable.Tread.TreadMarksSizeZ, 100, 15, army )
                local treadOffsetX = boneTable.Tread.TreadOffset[1]
                if x and x > 0 then
                    self:PlayUnitSound('FootFallLeft')
                elseif x and x < 0 then
                    self:PlayUnitSound('FootFallRight')
                end
            end

            for k, v in effects do
                CreateEmitterAtBone(self, bone, army, v):ScaleEmitter(scale):OffsetEmitter(offset.x or 0,offset.y or 0,offset.z or 0)
            end
        end
        self:PlayUnitSound('FootFallGeneric')
    end,

    UpdateMovementEffectsOnMotionEventChange = function( self, new, old )
        #LOG('UpdateMovementEffectsOnMotionEventChange ', new, ' ', old )
        local layer = self:GetCurrentLayer()
        local bpMTable = self:GetBlueprint().Display.MovementEffects

        if( old == 'TopSpeed' ) then
            # Destroy topspeed contrails, and exhaust effects
            self:DestroyTopSpeedEffects()
        end

        if new == 'TopSpeed' and self.HasFuel then
            if bpMTable[layer].Contrails and self.ContrailEffects then
                self:CreateContrails( bpMTable[layer].Contrails )
            end
            if bpMTable[layer].TopSpeedFX then
                self:CreateMovementEffects( self.TopSpeedEffectsBag, 'TopSpeed' )
            end
        end

        if (old == 'Stopped' and new != 'Stopping') or
           (old == 'Stopping' and new != 'Stopped') then
            self:DestroyIdleEffects()
            self:DestroyMovementEffects()
            self:CreateMovementEffects( self.MovementEffectsBag, nil )
            if bpMTable.BeamExhaust then
                self:UpdateBeamExhaust( 'Cruise' )
            end
            if self.Detector then
                self.Detector:Enable()
            end
        end

        if new == 'Stopped' then
            self:DestroyMovementEffects()
            self:DestroyIdleEffects()
            self:CreateIdleEffects()
            if bpMTable.BeamExhaust then
                self:UpdateBeamExhaust( 'Idle' )
            end
            if self.Detector then
                self.Detector:Disable()
            end
        end
    end,

    GetTTTreadType = function( self, pos )
        local TerrainType = GetTerrainType( pos.x,pos.z )
        return TerrainType.Treads or 'None'
    end,

    GetTerrainTypeEffects = function( FxType, layer, pos, type, typesuffix )
        local TerrainType

        # Get terrain type mapped to local position and if none defined use default
        if type then
            TerrainType = GetTerrainType( pos.x,pos.z )
        else
            TerrainType = GetTerrainType( -1, -1 )
            type = 'Default'
        end

        # Add in type suffix to type mask name
        if typesuffix then
            type = type .. typesuffix
        end

        # If our current masking is empty try and get the default layer effect
        if TerrainType[FxType][layer][type] == nil then
			TerrainType = GetTerrainType( -1, -1 )
		end

        #LOG( 'GetTerrainTypeEffects ', TerrainType.Name .. ' ' .. TerrainType.Description, ' ', layer, ' ', type )
        return TerrainType[FxType][layer][type] or {}
    end,

    CreateTerrainTypeEffects = function( self, effectTypeGroups, FxBlockType, FxBlockKey, TypeSuffix, EffectBag, TerrainType )
        local army = self:GetArmy()
        local pos = self:GetPosition()
        local effects = {}
        local emit = nil

        for kBG, vTypeGroup in effectTypeGroups do
            if TerrainType then
                effects = TerrainType[FxBlockType][FxBlockKey][vTypeGroup.Type] or {}
            else
                effects = self.GetTerrainTypeEffects( FxBlockType, FxBlockKey, pos, vTypeGroup.Type, TypeSuffix )
            end

            if not vTypeGroup.Bones or (vTypeGroup.Bones and (table.getn(vTypeGroup.Bones) == 0)) then
                LOG('*WARNING: No effect bones defined for layer group ',repr(self:GetUnitId()),', Add these to a table in Display.[EffectGroup].', self:GetCurrentLayer(), '.Effects { Bones ={} } in unit blueprint.' )
                continue
            end

            for kb, vBone in vTypeGroup.Bones do
                for ke, vEffect in effects do
                    emit = CreateAttachedEmitter(self,vBone,army,vEffect):ScaleEmitter(vTypeGroup.Scale or 1)
                    if vTypeGroup.Offset then
                        emit:OffsetEmitter(vTypeGroup.Offset[1] or 0, vTypeGroup.Offset[2] or 0,vTypeGroup.Offset[3] or 0)
                    end
                    if EffectBag then
                        table.insert( EffectBag, emit )
                    end
                end
            end
        end
    end,

    CreateIdleEffects = function( self )
        local layer = self:GetCurrentLayer()
        local bpTable = self:GetBlueprint().Display.IdleEffects

        if bpTable[layer] and bpTable[layer].Effects then
            self:CreateTerrainTypeEffects( bpTable[layer].Effects, 'FXIdle',  layer, nil, self.IdleEffectsBag )
        end
    end,

    CreateMovementEffects = function( self, EffectsBag, TypeSuffix, TerrainType )
        local layer = self:GetCurrentLayer()
        local bpTable = self:GetBlueprint().Display.MovementEffects

        if bpTable[layer] then
            bpTable = bpTable[layer]
            local effectTypeGroups = bpTable.Effects

            if bpTable.Treads then
                self:CreateTreads( bpTable.Treads )
            end

            if (not effectTypeGroups or (effectTypeGroups and (table.getn(effectTypeGroups) == 0))) then
                if not self.Footfalls and bpTable.Footfall then
                    LOG('*WARNING: No movement effect groups defined for unit ',repr(self:GetUnitId()),', Effect groups with bone lists must be defined to play movement effects. Add these to the Display.MovementEffects', layer, '.Effects table in unit blueprint. ' )
                end
                return false
            end

            if bpTable.CameraShake then
                self.CamShakeT1 = self:ForkThread(self.MovementCameraShakeThread, bpTable.CameraShake )
            end

            self:CreateTerrainTypeEffects( effectTypeGroups, 'FXMovement', layer, TypeSuffix, EffectsBag, TerrainType )
        end
    end,

    CreateLayerChangeEffects = function( self, new, old )
        local key = old..new
        local bpTable = self:GetBlueprint().Display.LayerChangeEffects[key]

        if bpTable then
            self:CreateTerrainTypeEffects( bpTable.Effects, 'FXLayerChange', key )
        end
    end,

    CreateMotionChangeEffects = function( self, new, old )
        local key = self:GetCurrentLayer()..old..new
        local bpTable = self:GetBlueprint().Display.MotionChangeEffects[key]

        if bpTable then
            self:CreateTerrainTypeEffects( bpTable.Effects, 'FXMotionChange', key )
        end
    end,

    DestroyMovementEffects = function( self )
        local bpTable = self:GetBlueprint().Display.MovementEffects
        local layer = self:GetCurrentLayer()

        EffectUtilities.CleanupEffectBag(self,'MovementEffectsBag')

        # Cleanup any camera shake going on.
        if self.CamShakeT1 then
            KillThread( self.CamShakeT1 )
            local shake = bpTable[layer].CameraShake
            if shake and shake.Radius and shake.MaxShakeEpicenter and shake.MinShakeAtRadius then
                self:ShakeCamera( shake.Radius, shake.MaxShakeEpicenter * 0.25, shake.MinShakeAtRadius * 0.25, 1 )
            end
        end

        # Cleanup treads
        if self.TreadThreads then
            for k, v in self.TreadThreads do
                KillThread(v)
            end
            self.TreadThreads = {}
        end
        if bpTable[layer].Treads.ScrollTreads then
            self:RemoveScroller()
        end
    end,

    DestroyTopSpeedEffects = function( self )
        EffectUtilities.CleanupEffectBag(self,'TopSpeedEffectsBag')
    end,

    DestroyIdleEffects = function( self )
        EffectUtilities.CleanupEffectBag(self,'IdleEffectsBag')
    end,

    UpdateBeamExhaust = function( self, motionState )
        local bpTable = self:GetBlueprint().Display.MovementEffects.BeamExhaust
        if not bpTable then
            return false
        end

        if motionState == 'Idle' then
            if self.BeamExhaustCruise  then
                self:DestroyBeamExhaust()
            end
            if self.BeamExhaustIdle and (table.getn(self.BeamExhaustEffectsBag) == 0) and (bpTable.Idle != false) then
                self:CreateBeamExhaust( bpTable, self.BeamExhaustIdle )
            end
        elseif motionState == 'Cruise' then
            if self.BeamExhaustIdle and self.BeamExhaustCruise then
                self:DestroyBeamExhaust()
            end
            if self.BeamExhaustCruise and (bpTable.Cruise != false) then
                self:CreateBeamExhaust( bpTable, self.BeamExhaustCruise )
            end
        elseif motionState == 'Landed' then
            if not bpTable.Landed then
                self:DestroyBeamExhaust()
            end
        end
    end,

    CreateBeamExhaust = function( self, bpTable, beamBP )
        local effectBones = bpTable.Bones
        if not effectBones or (effectBones and (table.getn(effectBones) == 0)) then
            LOG('*WARNING: No beam exhaust effect bones defined for unit ',repr(self:GetUnitId()),', Effect Bones must be defined to play beam exhaust effects. Add these to the Display.MovementEffects.BeamExhaust.Bones table in unit blueprint.' )
            return false
        end
        local army = self:GetArmy()
        for kb, vb in effectBones do
            table.insert( self.BeamExhaustEffectsBag, CreateBeamEmitterOnEntity(self, vb, army, beamBP ))
        end
    end,

    DestroyBeamExhaust = function( self )
        EffectUtilities.CleanupEffectBag(self,'BeamExhaustEffectsBag')
    end,

    CreateContrails = function(self, tableData )
        local effectBones = tableData.Bones
        if not effectBones or (effectBones and (table.getn(effectBones) == 0)) then
            LOG('*WARNING: No contrail effect bones defined for unit ',repr(self:GetUnitId()),', Effect Bones must be defined to play contrail effects. Add these to the Display.MovementEffects.Air.Contrail.Bones table in unit blueprint. ' )
            return false
        end
        local army = self:GetArmy()
        local ZOffset = tableData.ZOffset or 0.0
        for ke, ve in self.ContrailEffects do
            for kb, vb in effectBones do
                table.insert(self.TopSpeedEffectsBag, CreateTrail(self,vb,army,ve):SetEmitterParam('POSITION_Z', ZOffset))
            end
        end
    end,

    MovementCameraShakeThread = function( self, camShake )
        local radius = camShake.Radius or 5.0
        local maxShakeEpicenter = camShake.MaxShakeEpicenter or 1.0
        local minShakeAtRadius = camShake.MinShakeAtRadius or 0.0
        local interval = camShake.Interval or 10.0
        if interval != 0.0 then
            while true do
                self:ShakeCamera( radius, maxShakeEpicenter, minShakeAtRadius, interval )
                WaitSeconds(interval)
            end
        end
    end,

    CreateTreads = function(self, treads)
        if treads.ScrollTreads then
            self:AddThreadScroller(1.0, 0.2)
        end
        self.TreadThreads = {}
        if treads.TreadMarks then
            local type = self:GetTTTreadType(self:GetPosition())
            if type != 'None' then
                for k, v in treads.TreadMarks do
                    table.insert( self.TreadThreads, self:ForkThread(self.CreateTreadsThread, v, type ))
                end
            end
        end
    end,

    CreateTreadsThread = function(self, treads, type )
        local sizeX = treads.TreadMarksSizeX
        local sizeZ = treads.TreadMarksSizeZ
        local interval = treads.TreadMarksInterval
        local treadOffset = treads.TreadOffset
        local treadBone = treads.BoneName or 0
        local treadTexture = treads.TreadMarks
        local army = self:GetArmy()

        while true do
            # Syntatic reference
            # CreateSplatOnBone(entity, offset, boneName, textureName, sizeX, sizeZ, lodParam, duration, army)
            CreateSplatOnBone(self, treadOffset, treadBone, treadTexture, sizeX, sizeZ, 50, 5, army)
            WaitSeconds(interval)
        end
    end,

    CreateFootFallManipulators = function( self, footfall )
        if not footfall.Bones or (footfall.Bones and (table.getn(footfall.Bones) == 0)) then
            LOG('*WARNING: No footfall bones defined for unit ',repr(self:GetUnitId()),', ', 'these must be defined to animation collision detector and foot plant controller' )
            return false
        end

        self.Detector = CreateCollisionDetector(self)
        self.Trash:Add(self.Detector)
        for k, v in footfall.Bones do
            self.Detector:WatchBone(v.FootBone)
            if v.FootBone and v.KneeBone and v.HipBone then
                CreateFootPlantController(self, v.FootBone, v.KneeBone, v.HipBone, v.StraightLegs or true, v.MaxFootFall or 0):SetPrecedence(10)
            end
        end
        return true
    end,

    OnRunOutOfFuel = function(self)
        self.HasFuel = false
        self:DestroyTopSpeedEffects()
    end,

    OnGotFuel = function(self)
        self.HasFuel = true
    end,

    GetWeaponClass = function(self, label)
        return self.Weapons[label] or import('weapon.lua').Weapon
    end,

    # Return the total time in seconds, cost in energy, and cost in mass to build the given target type.
    GetBuildCosts = function(self, target_bp)
        return Game.GetConstructEconomyModel(self, target_bp.Economy)
    end,

    SetReclaimTimeMultiplier = function(self, time_mult)
        self.ReclaimTimeMultiplier = time_mult
    end,

    # Return the total time in seconds, cost in energy, and cost in mass to reclaim the given target from 100%.
    # The energy and mass costs will normally be negative, to indicate that you gain mass/energy back.
    GetReclaimCosts = function(self, target_entity)
        local bp = self:GetBlueprint()
        local target_bp = target_entity:GetBlueprint()
        if target_entity:IsUnit() then
            #LOG('Reclaim Target Is Unit')
            local mtime = target_bp.Economy.BuildCostEnergy / (bp.Economy.BuildRate or 1)
            local etime = target_bp.Economy.BuildCostMass / (bp.Economy.BuildRate or 1)
            local time = mtime
            if mtime < etime then
                time = etime
            end
            time = time * (self.ReclaimTimeMultiplier or 1)
            return (time/10), target_bp.Economy.BuildCostEnergy, target_bp.Economy.BuildCostMass
        elseif target_entity:IsProp() then
            local time, energy, mass =  target_entity:GetReclaimCosts(self)
            #LOG('*DEBUG: Reclaiming a prop.  Time = ', repr(time), ' Mass = ', repr(mass), ' Energy = ', repr(energy))
            return time, energy, mass
        end
    end,

    # Return the Bonus Build Multiplier for the target we are re-building if we are trying to rebuild the same
    # structure that was destroyed earlier.
    GetRebuildBonus = function(self, rebuildUnitBP)
        # for now everything is re-built is 50% complete to begin with
        return 0.5
    end,

    SetCaptureTimeMultiplier = function(self, time_mult)
        self.CaptureTimeMultiplier = time_mult
    end,

    # Return the total time in seconds, cost in energy, and cost in mass to capture the given target.
    # Calculation given by Jake - 6/26/06
    GetCaptureCosts = function(self, target_entity)
        local target_bp = target_entity:GetBlueprint().Economy
        local bp = self:GetBlueprint().Economy

        local time = ((target_bp.BuildTime or 10) / (bp.BuildRate or 1)) / 2
        local energy = target_bp.BuildCostEnergy or 100
        time = time * (self.CaptureTimeMultiplier or 1)

        return time, energy, 0
    end,

    GetUnpacker = function(self, unpackerspec)
        #LOG(repr(self.UnpackerSpecs))
       # LOG(repr(unpackerspec))
        if self.UnpackerSpecs[unpackerspec] then
            #GO THROUGH LIST OF UNPACKERS
            #IF THE WEAPON YOU ARE NEEDS THIS UNPACKER, RETURN THE HANDLE.
            #IF THE HANDLE DOESN'T EXIST, CREATE ONE.
            for uname, uspec in self.UnpackerSpecs do
                local unpackers = self.Unpackers or {}
                if not unpackers[uname] then
                    unpackers[uname] = WeaponUnpacker(self, uspec)
                    self.Trash:Add(unpackers[uname])
                    #LOG('*DEBUG: Creating Unpacker in unit.lua - ', repr(unpackers[uname]))
                end
                #LOG('*DEBUG: Unit:GetUnpacker RETURNING: ', repr(vu))
                self.Unpackers = unpackers
                #LOG('*DEBUG: self.Unpackers = ', repr(self.Unpackers))
                return unpackers[uname]
            end
        else
            LOG('*ERROR: TRYING TO GET AN UNPACKER FROM A UNIT WITH NO UNPACKERS SPECIFIED. ', repr(self))
        end
    end,



    GetHealthPercent = function(self)
        local health = self:GetHealth()
        local maxHealth = self:GetBlueprint().Defense.MaxHealth
        return health / maxHealth
    end,

    ValidateBone = function(self, bone)
        if self:IsValidBone(bone) then
            return true
        end
        error('*ERROR: Trying to use the bone, ' .. bone .. ' on unit ' .. self:GetUnitId() .. ' and it does not exist in the model.', 2)
        return false
    end,

    CheckBuildRestriction = function(self, target_bp)
        return true
    end,

    PlayUnitSound = function(self, sound)
        local bp = self:GetBlueprint().Audio
        if bp and bp[sound] then
            #LOG( 'Playing ', sound )
            self:PlaySound(bp[sound])
            return true
        end
        #LOG( 'Could not play ', sound )
        return false
    end,

    # Play the specified ambient sound for the unit, and if it has
    # AmbientRumble defined, play that too
    PlayUnitAmbientSound = function(self, sound)
        if sound == nil then
            self:SetAmbientSound( nil, nil )
            return true
        else
            local bp = self:GetBlueprint().Audio
            if bp and bp[sound] then
                if bp.Audio['AmbientRumble'] then
                    self:SetAmbientSound( bp[sound], bp.Audio['AmbientRumble'] )
                else
                    self:SetAmbientSound( bp[sound], nil )
                end
                return true
            end
            return false
        end
    end,
    
    PlayExtraUnitAmbientSound = function(self, sound)
        local bp = self:GetBlueprint()
        if not bp.Audio[sound] then return end
        if not self.AmbientSounds then
            self.AmbientSounds = {}
        end
        if not self.AmbientSounds[sound] then
            local sndEnt = Entity {}
            self.AmbientSounds[sound] = sndEnt
            self.unit.Trash:Add(sndEnt)
            sndEnt:AttachTo(self.unit,-1)
        end
        self.AmbientSounds[sound]:SetAmbientSound( bp.Audio[sound], nil )
    end,
    
    StopExtraUnitAmbientSound = function(self, sound)
        if not self.AmbientSounds then return end
        if not self.AmbientSounds[sound] then return end
        local bp = self:GetBlueprint()
        if not bp.Audio[sound] then return end
        self.AmbientSounds[sound]:SetAmbientSound(nil, nil)
    end,

    ##########################################################################################
    ## UNIT CALLBACKS
    ##########################################################################################


    AddProjectileDamagedCallback = function( self, fn )
        if not fn then
            error('*ERROR: tried to add a projectile damaged callback with a nil function')
            return
        end
        table.insert( self.EventCallbacks.ProjectileDamaged, fn )
    end,

    AddOnKilledCallback = function(self, fn)
        if not fn then
            error('*ERROR: Tried to add an OnKilled callback with a nil function')
            return
        end
        table.insert(self.EventCallbacks.OnKilled, fn)
    end,

    DoOnKilledCallbacks = function(self)
        if self.EventCallbacks.OnKilled then
            for k, cb in self.EventCallbacks.OnKilled do
                if cb then
                    cb(self)
                end
            end
        end
    end,

    AddOnDamagedCallback = function(self, fn, amount, repeatNum)
        if not fn then
            error('*ERROR: Tried to add an OnDamaged callback with a nil function')
            return
        end
        local num = amount or -1
        repeatNum = repeatNum or 1
        table.insert(self.EventCallbacks.OnDamaged, {Func=fn, Amount=num, Called=0, Repeat=repeatNum})
    end,

    DoOnDamagedCallbacks = function(self, instigator)
        if self.EventCallbacks.OnDamaged then
            for num, callback in self.EventCallbacks.OnDamaged do
                if (callback.Called < callback.Repeat or callback.Repeat == -1) and ( callback.Amount == -1 or (1 - self:GetHealthPercent() > callback.Amount) ) then
                    callback.Called = callback.Called + 1
                    callback.Func(self, instigator)
                end
            end
        end
    end,

    AddOnCapturedCallback = function(self, cbOldUnit, cbNewUnit)
        if not cbOldUnit and not cbNewUnit then
            error('*ERROR: Tried to add an OnCaptured callback without any functions', 2)
            return
        end
        if cbOldUnit then
            table.insert(self.EventCallbacks.OnCaptured, cbOldUnit)
        end
        if cbNewUnit then
            table.insert(self.EventCallbacks.OnCapturedNewUnit, cbNewUnit)
        end
    end,

    DoOnCapturedCallbacks = function(self,captor)
        if self.EventCallbacks.OnCaptured then
            for k, cb in self.EventCallbacks.OnCaptured do
                if cb then
                    cb(self,captor)
                end
            end
        end
    end,

    AddOnStartBeingCapturedCallback = function(self, fn)
        if not fn then
            error('*ERROR: Tried to add an OnStartBeingCaptured callback with a nil function')
            return
        end
        table.insert(self.EventCallbacks.OnStartBeingCaptured, fn)
    end,

    DoOnStartBeingCapturedCallbacks = function(self, captor)
        if self.EventCallbacks.OnStartBeingCaptured then
            for k, cb in self.EventCallbacks.OnStartBeingCaptured do
                if cb then
                    cb(self, captor)
                end
            end
        end
    end,

    AddOnStopBeingCapturedCallback = function(self, fn)
        if not fn then
            error('*ERROR: Tried to add an OnStopBeingCaptured callback with a nil function')
            return
        end
        table.insert(self.EventCallbacks.OnStopBeingCaptured, fn)
    end,

    DoOnStopBeingCapturedCallbacks = function(self, captor)
        if self.EventCallbacks.OnStopBeingCaptured then
            for k, cb in self.EventCallbacks.OnStopBeingCaptured do
                if cb then
                    cb(self, captor)
                end
            end
        end
    end,

    AddOnFailedBeingCapturedCallback = function(self, fn)
        if not fn then
            error('*ERROR: Tried to add an OnFailedBeingCaptured callback with a nil function')
            return
        end
        table.insert(self.EventCallbacks.OnFailedBeingCaptured, fn)
    end,

    DoOnFailedBeingCapturedCallbacks = function(self, captor)
        if self.EventCallbacks.OnFailedBeingCaptured then
            for k,cb in self.EventCallbacks.OnFailedBeingCaptured do
                if cb then
                    cb(self, captor)
                end
            end
        end
    end,

    AddOnStartCaptureCallback = function(self, fn)
        if not fn then
            error('*ERROR: Tried to add an OnStartCapture callback with a nil function')
            return
        end
        table.insert(self.EventCallbacks.OnStartCapture, fn)
    end,

    DoOnStartCaptureCallbacks = function(self, target)
        if self.EventCallbacks.OnStartCapture then
            for k, cb in self.EventCallbacks.OnStartCapture do
                if cb then
                    cb(self, target)
                end
            end
        end
    end,

    AddOnStopCaptureCallback = function(self, fn)
        if not fn then
            error('*ERROR: Tried to add an OnStopCapture callback with a nil function')
            return
        end
        table.insert(self.EventCallbacks.OnStopCapture, fn)
    end,

    DoOnStopCaptureCallbacks = function(self, target)
        if self.EventCallbacks.OnStopCapture then
            for k, cb in self.EventCallbacks.OnStopCapture do
                if cb then
                    cb(self, target)
                end
            end
        end
    end,

    AddOnFailedCaptureCallback = function(self, fn)
        if not fn then
            error('*ERROR: Tried to add an OnFailedCapture callback with a nil function')
            return
        end
        table.insert(self.EventCallbacks.OnFailedCapture, fn)
    end,

    DoOnFailedCaptureCallbacks = function(self, target)
        if self.EventCallbacks.OnFailedCapture then
            for k,cb in self.EventCallbacks.OnFailedCapture do
                if cb then
                    cb(self, target)
                end
            end
        end
    end,

    AddOnReclaimedCallback = function(self, fn)
        if not fn then
            error('*ERROR: Tried to add an OnReclaimed callback with a nil function')
            return
        end
        table.insert(self.EventCallbacks.OnReclaimed, fn)
    end,

    DoOnReclaimedCallbacks = function(self, entity)
        if self.EventCallbacks.OnReclaimed then
            for k, cb in self.EventCallbacks.OnReclaimed do
                if cb then
                    cb(self, entity)
                end
            end
        end
    end,

    AddOnStartReclaimCallback = function(self, fn)
        if not fn then
            error('*ERROR: Tried to add an OnStartReclaim callback with a nil function')
            return
        end
        table.insert(self.EventCallbacks.OnStartReclaim, fn)
    end,

    DoOnStartReclaimCallbacks = function(self)
        if self.EventCallbacks.OnStartReclaim then
            for k, cb in self.EventCallbacks.OnStartReclaim do
                if cb then
                    cb(self)
                end
            end
        end
    end,

    AddOnStopReclaimCallback = function(self, fn)
        if not fn then
            error('*ERROR: Tried to add an OnStopReclaim callback with a nil function')
            return
        end
        table.insert(self.EventCallbacks.OnStopReclaim, fn)
    end,

    DoOnStopReclaimCallbacks = function(self)
        if self.EventCallbacks.OnStopReclaim then
            for k, cb in self.EventCallbacks.OnStopReclaim do
                if cb then
                    cb(self)
                end
            end
        end
    end,

    AddOnLoadCallback = function(self, fn)
        if not fn then
            error('*ERROR: Tried to add an OnLoad callback with a nil function')
            return
        end
        table.insert(self.EventCallbacks.OnLoad, fn)
    end,

    DoOnLoadCallbacks = function(self)
        if self.EventCallbacks.OnLoad then
            for k, cb in self.EventCallbacks.OnLoad do
                if cb then
                    cb(self)
                end
            end
        end
    end,

    AddOnUnloadCallback = function(self, fn)
        if not fn then
            error('*ERROR: Tried to add an OnUnload callback with a nil function')
            return
        end
        table.insert(self.EventCallbacks.OnUnload, fn)
    end,

    DoOnUnloadCallbacks = function(self)
        if self.EventCallbacks.OnUnload then
            for k, cb in self.EventCallbacks.OnUnload do
                if cb then
                    cb(self)
                end
            end
        end
    end,

    AddOnStopBeingBuiltCallback = function(self, fn)
        if not fn then
            error('*ERROR: Tried to add an OnLoad callback with a nil function')
            return
        end
        table.insert(self.EventCallbacks.OnStopBeingBuilt, fn)
    end,

    DoOnStopBeingBuiltCallbacks = function(self)
        if self.EventCallbacks.OnStopBeingBuilt then
            for k, cb in self.EventCallbacks.OnStopBeingBuilt do
                if cb then
                    cb(self)
                end
            end
        end
    end,

    AddOnUnitBuiltCallback = function(self, fn, category)
        if not fn then
            error('*ERROR: Tried to add an OnLoad callback with a nil function')
            return
        end
        local insertedTable = {
                CallBackFunction = fn,
                Category = category
            }
        table.insert(self.EventCallbacks.OnUnitBuilt, insertedTable)
        #LOG('*DEBUG: CALLBACK ADDED')
    end,

    DoOnUnitBuiltCallbacks = function(self, unit)
        #LOG('*DEBUG: CALLBACK FIRED')
        if self.EventCallbacks.OnUnitBuilt then
            for k, v in self.EventCallbacks.OnUnitBuilt do
                if v and unit and not unit:BeenDestroyed() and EntityCategoryContains(v.Category, unit) then
                    #LOG('*DEBUG: CALLBACK FIRED, RUNNING FUNCTION')
                    #Function will call back with both the unit's and the unit being built's handle
                    v.CallBackFunction(self, unit)
                end
            end
        end
    end,

    AddOnHorizontalStartMoveCallback = function(self, fn)
        if not fn then
            error('*ERROR: Tried to add an OnHorizontalMove callback with a nil function')
            return
        end
        table.insert(self.EventCallbacks.OnHorizontalStartMove, fn)
    end,

    DoOnHorizontalStartMoveCallbacks = function(self)
        if self.EventCallbacks.OnHorizontalStartMove then
            for k, cb in self.EventCallbacks.OnHorizontalStartMove do
                if cb then
                    cb(self)
                end
            end
        end
    end,

    AddOnVeteranCallback = function(self, fn)
        if not fn then
            error('*ERROR: Tried to add an OnVeteran callback with a nil function', 2)
            return
        end
        table.insert(self.EventCallbacks.OnVeteran, fn)
    end,

    DoOnVeteranCallbacks = function(self, level)
        if self.EventCallbacks.OnVeteran then
            for k, cb in self.EventCallbacks.OnVeteran do
                if cb then
                    cb(self, level)
                end
            end
        end
    end,

    RemoveCallback = function(self, fn)
        for k, v in self.EventCallbacks do
            for kcb, vcb in v do
                if vcb == fn then
                    #LOG('*DEBUG: REMOVED TRIGGER ', repr(kcb))
                    v[kcb] = nil
                end
            end
        end
    end,


    ##########################################################################################
    ## STATES
    ##########################################################################################

    IdleState = State {
        Main = function(self)
        end,
    },

    DeadState = State {
        Main = function(self)
        end,
    },

    EnhancingState = State {
        Main = function(self)
            while self.EnhanceProgress < 1 and not self:IsDead() do
                WaitTicks(1)
            end
        end,

        OnEnhanceEnd = function(self, enhancement)
            self:SetActiveConsumptionInactive()
            self.BuildEnhancement = nil
            AddUnitEnhancement(self, enhancement)
            self:CreateEnhancement(enhancement)
            ChangeState(self, self.IdleState)
        end,
    },


    ##########################################################################################
    ## BUFFS
    ##########################################################################################

    AddBuff = function(self, buffTable, PosEntity)
        local bt = buffTable.BuffType

        if not bt then
            error('*ERROR: Tried to add a unit buff in unit.lua but got no buff table.  Wierd.', 1)
            return
        end
        #When adding debuffs we have to make sure that we check for permissions
        local allow = categories.ALLUNITS
        if buffTable.TargetAllow then
            allow = ParseEntityCategory(buffTable.TargetAllow)
        end
        local disallow
        if buffTable.TargetDisallow then
            disallow = ParseEntityCategory(buffTable.TargetDisallow)
        end

        if bt == 'STUN' then
           if buffTable.Radius and buffTable.Radius > 0 then
                #if the radius is bigger than 0 then we will use the unit as the center of the stun blast
                #and collect all targets from that point
                local targets = {}
                if PosEntity then
                    targets = utilities.GetEnemyUnitsInSphere(self, PosEntity, buffTable.Radius)
                else
                    targets = utilities.GetEnemyUnitsInSphere(self, self:GetPosition(), buffTable.Radius)
                end
                if not targets then
                    #LOG('*DEBUG: No targets in radius to buff')
                    return
                end
                for k, v in targets do
                    if EntityCategoryContains(allow, v) and (not disallow or not EntityCategoryContains(disallow, v)) then
                        v:SetStunned(buffTable.Duration or 1)
                    end
                end
            else
                #The buff will be applied to the unit only
                if EntityCategoryContains(allow, self) and (not disallow or not EntityCategoryContains(disallow, self)) then
                    self:SetStunned(buffTable.Duration or 1)
                end
            end
        elseif bt == 'MAXHEALTH' then
            self:SetMaxHealth(self:GetMaxHealth() + (buffTable.Value or 0))
        elseif bt == 'HEALTH' then
            self:SetHealth(self, self:GetHealth() + (buffTable.Value or 0))
        elseif bt == 'SPEEDMULT' then
            self:SetSpeedMult(buffTable.Value or 0)
        elseif bt == 'MAXFUEL' then
            self:SetFuelUseTime(buffTable.Value or 0)
        elseif bt == 'FUELRATIO' then
            self:SetFuelRatio(buffTable.Value or 0)
        elseif bt == 'HEALTHREGENRATE' then
            self:SetRegenRate(buffTable.Value or 0)
        end
    end,

    AddWeaponBuff = function(self, buffTable, weapon)
        local bt = buffTable.BuffType
        if not bt then
            error('*ERROR: Tried to add a weapon buff in unit.lua but got no buff table.  Wierd.', 1)
            return
        end
        if bt == 'RATEOFFIRE' then
            weapon:ChangeRateOfFire(buffTable.Value or 1)
        elseif bt == 'TURRETYAWSPEED' then
            weapon:SetTurretYawSpeed(buffTable.Value or 0)
        elseif bt == 'TURRETPITCHSPEED' then
            weapon:SetTurretPitchSpeed(buffTable.Value or 0)
        elseif bt == 'DAMAGE' then
            weapon:AddDamageMod(buffTable.Value or 0)
        elseif bt == 'MAXRADIUS' then
            weapon:ChangeMaxRadius(buffTable.Value or weapon:GetBlueprint().MaxRadius)
        elseif bt == 'FIRINGRANDOMNESS' then
            weapon:SetFiringRandomness(buffTable.Value or 0)
        else
            self:AddBuff(buffTable)
        end
    end,

    ##########################################################################################
    ## VETERANCY
    ##########################################################################################

    #Return the unit's current vet level
    GetVeteranLevel = function(self)
        return self.VeteranLevel
    end,


    #Check to see if we should veteran up.
    CheckVeteranLevel = function(self)
        local bp = self:GetBlueprint().Veteran
        #There is no veteran block in the bp, return
        if not bp then return end

        #We add 1 because we get this before the stat gets updated
        local unitKills = self:GetStat('KILLS', 0).Value + 1

        #We are already at the highest veteran level, return
        #For some reason, if you do a table.getn on a table with string keys other than integers it'll return 0.  Which is... odd.
        #So, I'm running a loop to get the number of vet levels
        local highVet = 0
        for k, v in bp do
            highVet = highVet + 1
        end
        if self.VeteranLevel == highVet then
            return
        end
        local nextLvl = self.VeteranLevel + 1
        local nextKills = bp[('Level' .. nextLvl)]
        if unitKills >= nextKills then
            self:SetVeteranLevel(nextLvl)
        end
    end,

    #Set the veteran level to the level specified
    SetVeteranLevel = function(self, level)
        local old = self.VeteranLevel
        self.VeteranLevel = level
        #Inform all weapons that have veteraned
        for i = 1, self:GetWeaponCount() do
            local wep = self:GetWeapon(i)
            wep:OnVeteranLevel(old, level)
        end

        #Check for unit buffs
        local bp = self:GetBlueprint().Buffs
        if bp then
            local lvlkey = 'VeteranLevel' .. level
            for k, v in bp do
                if v.Add[lvlkey] == true then
                    self:AddBuff(v)
                end
            end
        end
        self:GetAIBrain():OnBrainUnitVeterancyLevel(self, level)
        self:DoOnVeteranCallbacks(level)
    end,

    PlayVeteranFx = function(self, newLvl)
        #NOTE: Place vet FX here
        CreateAttachedEmitter(self, 0, self:GetArmy(), 'destruction_explosion_concussion_ring_03_emit.bp'):ScaleEmitter(1)
    end,

    ##########################################################################################
    ## SHIELDS
    ##########################################################################################
    CreateShield = function(self, shieldSpec)
        local bp = self:GetBlueprint()
        local bpShield = shieldSpec
        if not shieldSpec then
            bpShield = bp.Defense.Shield
        end
        if bpShield then
            self:DestroyShield()
            self.MyShield = Shield {
                Owner = self,
                FactionName = bp.General.FactionName,
                Size = bpShield.ShieldSize or 10,
                ShieldMaxHealth = bpShield.ShieldMaxHealth or 250,
                ShieldRechargeTime = bpShield.ShieldRechargeTime or 10,
                ShieldEnergyDrainRechargeTime = bpShield.ShieldEnergyDrainRechargeTime or 10,
                ShieldVerticalOffset = bpShield.ShieldVerticalOffset or -1,
                ShieldRegenRate = bpShield.ShieldRegenRate or 1,
                ShieldRegenStartTime = bpShield.ShieldRegenStartTime or 5,
            }
            self:EnableShield()
            self.Trash:Add(self.MyShield)
        end
    end,

    CreatePersonalShield = function(self, shieldSpec)
        local bp = self:GetBlueprint()
        local bpShield = shieldSpec
        if not shieldSpec then
            bpShield = bp.Defense.Shield
        end
        if bpShield then
            self:DestroyShield()
            if bpShield.OwnerShieldMesh then
                self.MyShield = UnitShield {
                    Owner = self,
                    FactionName = bp.General.FactionName,
                    CollisionSizeX = bp.SizeX * 0.75 or 1,
                    CollisionSizeY = bp.SizeY * 0.75 or 1,
                    CollisionSizeZ = bp.SizeZ * 0.75 or 1,
                    CollisionCenterX = bp.CollisionOffsetX or 0,
                    CollisionCenterY = bp.CollisionOffsetY or 0,
                    CollisionCenterZ = bp.CollisionOffsetZ or 0,
                    OwnerShieldMesh = bpShield.OwnerShieldMesh,
                    ShieldMaxHealth = bpShield.ShieldMaxHealth or 250,
                    ShieldRechargeTime = bpShield.ShieldRechargeTime or 10,
                    ShieldEnergyDrainRechargeTime = bpShield.ShieldEnergyDrainRechargeTime or 10,
                    ShieldRegenRate = bpShield.ShieldRegenRate or 1,
                    ShieldRegenStartTime = bpShield.ShieldRegenStartTime or 5,
                }
                self:EnableShield()
                self.Trash:Add(self.MyShield)
            else
                LOG('*WARNING: TRYING TO CREATE PERSONAL SHIELD ON UNIT ',repr(self:GetUnitId()),', but it does not have an OwnerShieldMesh=<meshBpName> defined in the Blueprint.')
            end
        end
    end,

    OnStartRefueling = function(self)
		#LOG('playing refueling sound')
        self:PlayUnitSound('Refueling')
    end,

    OnShieldEnabled = function(self)
        #self:PlayUnitSound('Activate')
        self:PlayUnitSound('ShieldOn')
        # Make the shield drain energy
        self:SetMaintenanceConsumptionActive()
    end,

    OnShieldDisabled = function(self)
        #self:PlayUnitSound('Deactivate')
        self:PlayUnitSound('ShieldOff')
        # Turn off the energy drain
        self:SetMaintenanceConsumptionInactive()
    end,

    EnableShield = function(self)
        if self.MyShield then
            self.MyShield:TurnOn()
        end
    end,

    DisableShield = function(self)
        if self.MyShield then
            self.MyShield:TurnOff()
        end
    end,

    DestroyShield = function(self)
        if self.MyShield then
            self.MyShield:TurnOff()
            self.MyShield:Destroy()
            self.MyShield = nil
        end
    end,

    ##########################################################################################
    ## TRANSPORTING
    ##########################################################################################
    OnStartTransportBeamUp = function(self, transport, bone)
        #LOG( 'Beaming Up' )
        self:DestroyIdleEffects()
        self:DestroyMovementEffects()
        local army =  self:GetArmy()
        table.insert( self.TransportBeamEffectsBag, AttachBeamEntityToEntity(self, -1, transport, bone, army, EffectTemplate.TTransportBeam01))
        table.insert( self.TransportBeamEffectsBag, AttachBeamEntityToEntity( transport, bone, self, -1, army, EffectTemplate.TTransportBeam02))
        #table.insert( self.TransportBeamEffectsBag, CreateEmitterAtBone( self, -1, army, 'effect') )
        table.insert( self.TransportBeamEffectsBag, CreateEmitterAtBone( transport, bone, army, EffectTemplate.TTransportGlow01) )
    end,

    OnStopTransportBeamUp = function(self)
        #LOG( 'Beaming Done' )
        for k, v in self.TransportBeamEffectsBag do
            v:Destroy()
        end
    end,
    
    OnTransportAborted = function(self)
        #LOG('TransportAborted')
    end,

    OnTransportOrdered = function(self)
        #LOG('TransportOrdered')
    end,
    
    MarkWeaponsOnTransport = function(self, unit, transport)
        #Mark the weapons on a transport
        for i = 1, unit:GetWeaponCount() do
            local wep = unit:GetWeapon(i)
            wep:SetOnTransport(transport)
        end
    end,

    OnTransportAttach = function(self, attachBone, unit)
        self:PlayUnitSound('Load')
        self:MarkWeaponsOnTransport(unit, true)
        if not EntityCategoryContains(categories.PODSTAGINGPLATFORM, self) then
            self:RequestRefreshUI()
        end
    end,

    OnTransportDetach = function(self, attachBone, unit)
        self:PlayUnitSound('Unload')
        self:MarkWeaponsOnTransport(unit, false)
        if not EntityCategoryContains(categories.PODSTAGINGPLATFORM, self) then
            self:RequestRefreshUI()
        end
    end,

    OnStartTransportLoading = function(self)
    end,

    OnStopTransportLoading = function(self)
    end,

    OnAddToStorage = function(self, unit)
        if EntityCategoryContains(categories.CARRIER, unit) then
            self:HideBone(0, true)
            self:SetCanTakeDamage(false)
            self:SetReclaimable(false)
            self:SetCapturable(false)
        end
    end,

    OnRemoveFromStorage = function(self, unit)
        if EntityCategoryContains(categories.CARRIER, unit) then
            self:SetCanTakeDamage(true)
            self:SetReclaimable(true)
            self:SetCapturable(true)
            self:ShowBone(0, true)
        end
    end,


    GetTransportClass = function(self)
        local bp = self:GetBlueprint().Transport
        return bp.TransportClass
    end,


    ##########################################################################################
    ## TELEPORTING
    ##########################################################################################
    OnTeleportUnit = function(self, teleporter, location, orientation)
        self:ForkThread(self.InitiateTeleportThread, teleporter, location, orientation)
    end,
    
    InitiateTeleportThread = function(self, teleporter, location, orientation)
        local tbp = teleporter:GetBlueprint()
        local ubp = self:GetBlueprint()
        self.UnitBeingTeleported = self
        self:SetImmobile(true)
        self:PlayUnitSound('TeleportStart')
        self:PlayExtraUnitAmbientSound('TeleportLoop')
        local bp = self:GetBlueprint().Economy
        local energyCost, time
        if bp then
            local mass = bp.BuildCostMass * (bp.TeleportMassMod or 0.01)
            local energy = bp.BuildCostEnergy * (bp.TeleportEnergyMod or 0.01)
            energyCost = mass + energy
            time = energyCost * (bp.TeleportTimeMod or 0.01)
        end

        #LOG('*UNIT DEBUG: TELEPORTING, energyCost = ', repr(energyCost), ' time = ', repr(time))
        self.TeleportDrain = CreateEconomyEvent(self, energyCost or 100, 0, time or 5)
        
        # create teleport charge effect
        self:PlayTeleportChargeEffects()
        
        WaitFor( self.TeleportDrain  ) # Perform fancy Teleportation FX here
        self:PlayTeleportOutEffects()
        self:CleanupTeleportChargeEffects()

        WaitSeconds( 0.1 )
        
        #Teleport Sound
        #LOG('DRAIN')
        #self.PlayUnitSound ('TeleportDrain')
        RemoveEconomyEvent(self, self.TeleportDrain )
        self.TeleportDrain = nil

        Warp(self, location, orientation)
        self:PlayTeleportInEffects()
        
        WaitSeconds( 0.1 ) # Perform cooldown Teleportation FX here
        #Landing Sound
        #LOG('DROP')
        self:StopExtraUnitAmbientSound('TeleportLoop')
        self:PlayUnitSound('TeleportEnd')
        self:SetImmobile(false)
        self.UnitBeingTeleported = nil
    end,
    
    PlayTeleportChargeEffects = function(self)
        local army = self:GetArmy()
        local bp = self:GetBlueprint() 
        
        self.TeleportChargeBag = {}
        for k, v in EffectTemplate.GenericTeleportCharge01 do
            local fx = CreateEmitterAtEntity(self,army,v):OffsetEmitter(0, (bp.Physics.MeshExtentsY or 1) / 2, 0) 
            self.Trash:Add(fx)
            table.insert( self.TeleportChargeBag, fx)            
        end
    end,  
   
    CleanupTeleportChargeEffects = function( self )
        if self.TeleportChargeBag then
            for keys,values in self.TeleportChargeBag do
                values:Destroy()
            end
            self.TeleportChargeBag = {}
        end    
    end,      
    
    PlayTeleportOutEffects = function(self)
        local army = self:GetArmy()
        local emit = nil
        for k, v in EffectTemplate.GenericTeleportOut01 do
            emit = CreateEmitterAtEntity(self,army,v)
        end
    end,    

    
    PlayTeleportInEffects = function(self)
        local army = self:GetArmy()
        for k, v in EffectTemplate.GenericTeleportIn01 do
            emit = CreateEmitterAtEntity(self,army,v):OffsetEmitter(0, (bp.Physics.MeshExtentsY or 1) / 2, 0)
        end    
    end,    

    #########################################################################################
    ## ROCKING
    ##########################################################################################
    # While not as exciting as Rock 'n Roll, this will make the unit rock from side to side slowly
    # in the water
    StartRocking = function(self)
        KillThread(self.StopRockThread)
        self.StartRockThread = self:ForkThread( self.RockingThread )
    end,

    StopRocking = function(self)
        KillThread(self.StartRockThread)
        self.StopRockThread = self:ForkThread( self.EndRockingThread )
    end,

    RockingThread = function(self)
        if not self.RockManip and not self:IsDead() then
            self.RockManip = CreateRotator( self, 0, 'z', nil, 0, self.MaxRockSpeed / 5, self.MaxRockSpeed * 3 / 5 )
            self.Trash:Add(self.RockManip)
            self.RockManip:SetPrecedence(0)
            while (true) do
                WaitFor( self.RockManip )
                if self:IsDead() then break end # abort if the unit died
                self.RockManip:SetTargetSpeed( -self.MaxRockSpeed )
                WaitFor( self.RockManip )
                if self:IsDead() then break end # abort if the unit died
                self.RockManip:SetTargetSpeed( self.MaxRockSpeed )
            end
        end
    end,

    EndRockingThread = function(self)
        if self.RockManip then
            self.RockManip:SetGoal( 0 )
            self.RockManip:SetSpeed( self.MaxRockSpeed / 4 )
            WaitFor( self.RockManip )

            if self.RockManip then
                self.RockManip:Destroy()
                self.RockManip = nil
            end
        end
    end,

}

