--[[
This file contains a table which defines options available to the user in the options dialog

tab data is what populates each tab and defines each option in a tab
Each tab has:
    .title - the text that will appear on the tab
    .key - the key that will group the options (ie in the prefs file gameplay.help_prompts or video.resolution)
    .items - an array of the items for this key (this is an array rather than a genericly keyed table so display order can be imposed)
        .title - the text that will label the option line item
        .tip - the text that will appear in the tooltip for the item
        .key - the prefs key to identify this property
        .default - the default value of the property
        .set - an optional function that takes a value parameter and is responsible for determining what happens when the option is applied (function(key, value))
        .update - a optional function that is called when the control has a new value, also receives the control (function(control, value)), not always used,
                  but if you need additonal control (say of other controls) when this value changes (for example one control may change other controls) you can
                  set up that behavior here
        .type - the type of control used to display this property
            valid types are:
            toggle - multi state toggle button (TODO - add list to replace more than 2 states?)
            button - momentary button (usually open another dialog)
            slider - a value slider
        .custom - a table of data required by the control type, different for each control type.
            slider
                .min - the minimum value for the slider
                .max - the maximum value for the slider
                .inc - the increment between slider detents, if 0 its "analog"
            toggle
                .states - table of states the toggle switch can have
                    .text = text for each state
                    .key = the key or value for each state to be stored in the pref
            button
                .text - the text label of the button

the optionsOrder table is just an array of keys in to the option table, and their order will determine what
order the tabs show in the dialog
--]]

optionsOrder = {
    "gameplay",
    "video",
    "sound",
    "about",
}

options = {
    gameplay = {
        title = "<LOC _Gameplay>",
        key = 'gameplay',
        items = {
            {
                title = "<LOC OPTIONS_0001>Zoom Wheel Sensitivity",
                key = 'wheel_sensitivity',
                type = 'slider',
                default = 40,
                set = function(key,value,startup)
                    ConExecute("cam_ZoomAmount " .. tostring(value / 100))
                end,
                custom = {
                    min = 1,
                    max = 100,
                    inc = 0,
                },
            },
            {
                title = "<LOC OPTIONS_0109>Always Render Strategic Icons",
                key = 'strat_icons_always_on',
                type = 'toggle',
                default = 0,
                set = function(key,value,startup)
                    ConExecute("ui_AlwaysRenderStrategicIcons " .. tostring(value))
                end,
                custom = {
                    states = {
                        {text = "<LOC _Off>", key = 0 },
                        {text = "<LOC _On>", key = 1 },
                    },
                },
            },
            {
                title = "<LOC OPTIONS_0107>Construction Popup Format",
                tip = "<LOC OPTIONS_0108>Change the layout that information is displayed in the rollover window for units in the construction manager.",
                key = 'uvd_format',
                type = 'toggle',
                default = 'full',
                set = function(key,value,startup)
                    ## needs logic to set priority (do we really want to do this though?)
                end,
                custom = {
                    states = {
                        {text = "<LOC _Full>", key = 'full'},
                        {text = "<LOC _Limited>", key = 'limited'},
                        {text = "<LOC _Off>", key = 'off'},
                    },
                },
            },
            {
                title = "<LOC OPTIONS_0106>Strategic View",
                key = 'stratview',
                type = 'toggle',
                default = false,
                custom = {
                    states = {
                        {text = "<LOC _On>", key = true},
                        {text = "<LOC _Off>", key = false},
                    },
                },
            },
            {
                title = "<LOC OPTIONS_0003>Dual Monitor Screen Edge",
                key = 'dual_mon_edge',
                type = 'toggle',
                default = 'open',
                set = function(key,value,startup)
                    ## probably just an option to read, but not 100% sure
                end,
                custom = {
                    states = {
                        {text = "<LOC _Open>", key = 'open'},
                        {text = "<LOC _Closed>", key = 'closed'},
                    },
                },
            },
            {
                title = "<LOC OPTIONS_0005>Dislpay Tooltips",
                key = 'tooltips',
                type = 'toggle',
                default = true,
                custom = {
                    states = {
                        {text = "<LOC _On>", key = true},
                        {text = "<LOC _Off>", key = false},
                    },
                },
            },
            {
                title = "<LOC OPTIONS_0078>Tooltip Delay",
                key = 'tooltip_delay',
                type = 'slider',
                default = 0,
                set = function(key,value,startup)
                end,
                custom = {
                    min = 0,
                    max = 3,
                    inc = 0,
                },
            },
            {
                title = "<LOC OPTIONS_0076>Economy Warnings",
                key = 'econ_warnings',
                type = 'toggle',
                default = true,
                custom = {
                    states = {
                        {text = "<LOC _On>", key = true,},
                        {text = "<LOC _Off>", key = false,},
                    },
                },
            },
            {
                title = "<LOC OPTIONS_0062>UI Animation",
                key = 'ui_animation',
                type = 'toggle',
                default = true,
                custom = {
                    states = {
                        {text = "<LOC _On>", key = true},
                        {text = "<LOC _Off>", key = false},
                    },
                },
            },
            {
                title = "<LOC OPTIONS_0089>MP Taunt Head",
                tip = "<LOC OPTIONS_0090>Sets your MP taunt persona",
                key = 'mp_taunt_head',
                type = 'toggle',
                default = 'Min',
                set = function(key,value,startup)
                    ## needs logic to set priority (do we really want to do this though?)
                end,
                custom = {
                    states = {
                        {text = "<LOC OPTIONS_0091>Leopard", key = 'Leopard'},
                        {text = "<LOC OPTIONS_0092>Min", key = 'Min'},
                        {text = "<LOC OPTIONS_0093>Eris", key = 'Eris'},
                        {text = "<LOC OPTIONS_0094>Gauge", key = 'Gauge'},
                        {text = "<LOC OPTIONS_0095>Mach", key = 'Mach'},
                        {text = "<LOC OPTIONS_0096>Arnold", key = 'Arnold'},
                        {text = "<LOC OPTIONS_0097>RedFog", key = 'RedFog'},
                        {text = "<LOC OPTIONS_0098>Ariel", key = 'Ariel'},
                        {text = "<LOC OPTIONS_0099>Berry", key = 'Berry'},
                        {text = "<LOC OPTIONS_0100>Godwyn", key = 'Godwyn'},
                        {text = "<LOC OPTIONS_0101>Aiko", key = 'Aiko'},
                        {text = "<LOC OPTIONS_0104>Blake", key = 'Blake'},
                        {text = "<LOC OPTIONS_0105>Marxon", key = 'Marxon'},
                    },
                },
            },
            {
                title = "<LOC OPTIONS_0102>MP Taunts",
                tip = "<LOC OPTIONS_0103>Enable or Disable displaying taunts in multiplayer.",
                key = 'mp_taunt_head_enabled',
                type = 'toggle',
                default = 'true',
                set = function(key,value,startup)
                    ## needs logic to set priority (do we really want to do this though?)
                end,
                custom = {
                    states = {
                        {text = "<LOC _On>", key = 'true'},
                        {text = "<LOC _Off>", key = 'false'},
                    },
                },
            },
        },
    },
    video = {
        title = "<LOC _Video>",
        key = 'video',
        items = {
            {
                title = "<LOC OPTIONS_0010>Primary Adapter",
                key = 'primary_adapter',
                type = 'toggle',
                default = '1024,768,60',
                set = function(key,value,startup)
                    if not startup then
                        ConExecute("SC_PrimaryAdapter " .. tostring(value))
                    end
                end,
                custom = {
                    states = {
                        { text = "<LOC OPTIONS_0070>Windowed", key = 'windowed' },
                        ## the remaining values are populated at runtime from device caps
                        ## what follows is just an example of the data which will be encountered
                        { text =  "1024x768(60)", key = '1024,768,60' },
                        { text = "1152x864(60)", key = '1152,864,60' },
                        { text = "1280x768(60)", key = '1280,768,60' },
                        { text = "1280x800(60)", key = '1280,800,60' },
                        { text = "1280x1024(60)", key = '1280,1024,60' },
                    },
                },
            },
            {
                title = "<LOC OPTIONS_0071>Secondary Adapter",
                key = 'secondary_adapter',
                type = 'toggle',
                default = 'disabled',
                set = function(key,value,startup)
                    if not startup then
                        ConExecute("SC_SecondaryAdapter " .. tostring(value))
                    end
                end,
                custom = {
                    states = {
                        { text = "<LOC OPTIONS_000>Disabled", key = 'disabled' },
                    },
                },
            },
            {
                title = "<LOC OPTIONS_0017>Background Image",
                key = 'bg_image',
                type = 'toggle',
                default = 1,
                set = function(key,value,startup)
                    ConExecute("ren_SkyCube " .. tostring(value))
                end,
                custom = {
                    states = {
                        {text = "<LOC _Off>", key = 0 },
                        {text = "<LOC _On>", key = 1 },
                    },
                },
            },
            {
                title = "<LOC OPTIONS_0018>Fidelity",
                key = 'fidelity',
                type = 'toggle',
                default = 2,
                set = function(key,value,startup)
                    ConExecute("graphics_Fidelity " .. tostring(value))
                end,
                update = function(control, newValue)
                    ## set other controls here?
                end,
                custom = {
                    states = {
                        {text = "<LOC _Low>", key = 0},
                        {text = "<LOC _Medium>", key = 1},
                        {text = "<LOC _High>", key = 2},
                    },
                },
            },
            {
                title = "<LOC OPTIONS_0024>Shadow Fidelity",
                key = 'shadow_quality',
                type = 'toggle',
                default = 2,
                set = function(key,value,startup)
                    ConExecute("shadow_Fidelity " .. tostring(value))
                end,
                custom = {
                    states = {
                        {text = "<LOC _Off>", key = 0},
                        {text = "<LOC _Low>", key = 1},
                        {text = "<LOC _Medium>", key = 2},
                        {text = "<LOC _High>", key = 3},
                    },
                },
            },
            {
                title = "<LOC OPTIONS_0015>Anti-Aliasing",
                key = 'antialiasing',
                type = 'toggle',
                default = 0,
                set = function(key,value,startup)
                    if not startup then ConExecute("D3D_AntiAliasingSamples " .. tostring(value)) end
                end,
                custom = {
                    states = {
                        {text = "<LOC OPTIONS_0029>Off", key = 0},
                        ## the remaining values are populated at runtime from device caps
                        ## what follows is just an example of the data which will be encountered
                        {text =  "2", key =  2},
                        {text =  "4", key =  4},
                        {text =  "8", key =  8},
                        {text = "16", key = 16},
                    },
                },
            },
            {
                title = "<LOC OPTIONS_0019>Texture Detail",
                key = 'texture_level',
                type = 'toggle',
                default = 0,
                set = function(key,value,startup)
                    ConExecute("ren_MipSkipLevels " .. tostring(value))
                end,
                custom = {
                    states = {
                        { text = "<LOC OPTIONS_0110>High", key = 0 },
                        { text = "<LOC OPTIONS_0111>Medium", key = 1 },
                        { text = "<LOC OPTIONS_0112>Low", key = 2 },
                    },
                },
            },
        },
    },
    sound = {
        title = "<LOC _Sound>",
        items = {
            {
                title = "<LOC OPTIONS_0028>Master Volume",
                key = 'master_volume',
                type = 'slider',
                default = 100,
                set = function(key,value,startup)
                    SetVolume("Global", value / 100)
                end,
                custom = {
                    min = 0,
                    max = 100,
                    inc = 0,
                },
            },
            {
                title = "<LOC OPTIONS_0026>FX Volume",
                key = 'fx_volume',
                type = 'slider',
                default = 100,
                set = function(key,value,startup)
                    SetVolume("World", value / 100)
                end,
                custom = {
                    min = 0,
                    max = 100,
                    inc = 0,
                },
            },
            {
                title = "<LOC OPTIONS_0027>Music Volume",
                key = 'music_volume',
                type = 'slider',
                default = 100,
                set = function(key,value,startup)
                    SetVolume("Music", value / 100)
                end,
                custom = {
                    min = 0,
                    max = 100,
                    inc = 0,
                },
            },
            {
                title = "<LOC OPTIONS_0064>Interface Volume",
                key = 'ui_volume',
                type = 'slider',
                default = 100,
                set = function(key,value,startup)
                    SetVolume("Interface", value / 100)
                end,
                custom = {
                    min = 0,
                    max = 100,
                    inc = 0,
                },
            },
            {
                title = "<LOC OPTIONS_0066>VO Volume",
                key = 'vo_volume',
                type = 'slider',
                default = 100,
                set = function(key,value,startup)
                    SetVolume("Voice", value / 100)
                    SetMovieVolume(value / 100)
                end,
                custom = {
                    min = 0,
                    max = 100,
                    inc = 0,
                },
            },
        },
    },
    about = {
        title = "<LOC _About>",
        key = 'about',
        items = {
--[[
            {
                title = "<LOC OPTIONS_0073>Credits",
                key = 'credits',
                type = 'button',
                default = true,
                update = function(control, value)
                    
                end,
                custom = {
                    text = "<LOC OPTIONS_0075>Play"
                },
            },
--]]            
            {
                title = "<LOC OPTIONS_0086>EULA",
                key = 'eula',
                type = 'button',
                default = true,
                update = function(control, value)
                    import('/lua/modules/ui/menus/eula.lua').CreateEULA()
                end,
                custom = {
                    text = "<LOC OPTIONS_0088>EULA"
                },
            },
        },
    },
}
