--[[ ------------------------------------------------------------------------------------------------------------------
Mods

All mods must be in subdirectories somewhere under /mods. Each mod has a
mod_info.lua file which contains various bits of info:

    name = "Happy mod"                          # Name to use for this mod
    version = 123                               # Version number to use for this mod
    copyright = "Copyright  2006, Someone"     # Optional copyright info
    description = "A long description of happy mod and why it will make you happy"
    author = "Joe"                              # Optional author info
    url = "http://www.gaspoweredgames.com"      # Optional URL to anywhere author likes

    selectable = true
        Flag for whether to list this mod in the mod manager window, where the
        player can select it. Examples where you would set it to false are:

        1.  A mod containing just custom maps, along with textures and props used
            on those maps. The mod will be automatically enabled when one of its
            maps is used.

        2.  A mod supplying textures and props for use by other mods. Those mods
            indicate that they require this one, and it is automatically enabled
            when they are.

    exclusive = true
        A simple form of preventing conflicting mods. Only one 'exclusive' mod
        can be active at a time. The default is true.

    ui_only = false
        Set true to indicate that this mod only affects the user interface. If
        this flag is false (the default), all players in a multiplayer game must
        have it in order for anyone to use it. If true, one player can have it
        active independently of the other players. (Games on GPGNet may restrict
        this further.)

        TODO: If ui_only is set true, the mod's files should not even be accessible
        from within the sim, but we need the VFS stuff implemented properly first.

    icon = "mod_icon.dds"
        Name of icon to use for this mod in the mod selection window. The default is
        mod_icon.dds in the same directory as the mod_info.lua file.

    requires = {
        # Optionally indicates that this mod only works if another mod is also present.
        "OtherMod1"             # Requires any version of OtherMod1
        {"OtherMod2", 123 }     # Requires version 123 of OtherMod2
        {"OtherMod3", 11, 22 }  # Requires any version between 11 and 22 of OtherMod3
    }

    conflicts = {
        Indicate any other mods that this mod is known to conflict with; the game
        will refuse to enable both of them at the same time.
        Same format as 'requires'.
    }

    before = {
        List of other mod names. If this mod happens to be active at the same time
        as any of the named other mods, it will be applied before them.
        Same format as 'requires'.
    }

    after = {
        List of other mod names. If this mod happens to be active at the same time
        as any of the named other mods, it will be applied after them. If you do
        not supply an 'after' list, the 'requires' list will be used in its place.
        Same format as 'requires'.
    }

    auto_apply = {
        A list of Lua regular expressions, to be checked against *_scenario.lua
        names. This mod will automatically be applied to any single-player games
        matching one of the expressions.
    }


Fields 'id' and 'location' also get filled in during initialization, to the full
path to the mod_info file and the directory containing it.



Adding extra maps

   Any extra maps in a mod will automatically show up on the available maps list on
   the single- and multi-player skirmish screen, as long as they have an appropriate
   Configuration set up.

   If you select a map from a mod for play, that mod (and any other mods it depends on)
   will automatically be active for that session. This allows you to define custom
   units or props just for a particular set of maps.


 Adding custom units

   If an active mod has any .bp files defining units, they will be automatically
   included during blueprint loading.

   If a mod contains a blueprint matching the ID of an existing unit (e.g. uel0001)
   then the mod's blueprint will replace the original. If the "Merge=true" flag is
   set in the mod blueprint, it will be merged with the original can can override
   some fields while leaving others intact.


 Balance changes

   There are two ways to make large-scale balance changes.

   First, you can define one or more .bp files with "Merge=true", that override
   particular fields for many units. This is how the campaign balance mod works.

   Second, you can define a function <TODO>, which is called on each blueprint
   as it is loaded. This function can then manipulate the blueprint in arbitrary
   ways.

   First, a mod can simply add a new blueprint file that defines a new blueprint.

   Second, a mod can contain a blueprint with the same ID as an existing blueprint.
   In this case it will completely override the original blueprint. Note that in
   order to replace an original non-unit blueprint, the mod must set the "BlueprintId"
   field to name the blueprint to be replaced. Otherwise the BlueprintId is defaulted
   off the source file name. (Units don't have this problem because the BlueprintId is
   shortened and doesn't include the original path).

   Finally, a mod can define a ModBlueprints() function which manipulates the
   original_blueprints table in arbitrary ways. [How/when exactly is this done?]


 Custom skins

   A mod can add a ??? file to indicate that it contains a custom skin. The skin
   will automatically be made available on the list of available skins.
   TODO: need help from CBlackwell to work out and document custom skin system.


 Other changes

   A mod can contain a 'shadow' folder, which contains files matching the names
   and directory structure of the game's top-level folder. These files will REPLACE
   the game's files on all lookups. You can use this mechanism to replace entire
   textures, audio files, scripts, blueprints, or any other game data. Note that
   when a game file is shadowed, the original file becomes completely inaccessible.
   This is ok for textures and such but usually not what you want for scripts.

   A mod can also contain a 'hook' folder, which contains files named the same way,
   matching the structure of the game's main data folder. After any script file is
   loaded, we check in the active mods to see if they define a hook of the same
   name; if so, we run the hook in the same environment as the original script. This
   allows hooks to tweak or replace individual functions, classes, or other data
   while still using the original code.


 Mod ordering

   It often matters in what order mods take effect. For example, perhaps two mods
   both adjust the tuning of some units, and they have a few in common. The mod
   that is applied last will take precedence.

   By default, mods are applied in simple alphabetical order. However, you can
   get more control by listing other mods in the 'before' and 'after' sections.

   If two mods specify an inconsistent ordering (e.g. they both ask to be before
   the other), SC will choose an arbitrary ordering and log a warning, but both
   mods will still be active.

'
-------------------------------------------------------------------------------------------------------------------- ]]

# Table of all mods, indexed by id
local all_mods = nil

# List of active mods in order they should be applied
local active_mods = nil

# List of requested mod ids in arbitrary order
requested_active_ids = {}


local function LoadModInfo(filename)
    # Fill in some defaults to start with...
    local env = {
        id = filename,
        location = Dirname(filename),
        name = filename,
        description = "<LOC uimod_0006>(No description)",
        author = '',
        copyright = '',
        exclusive = true,
        icon = '/textures/ui/common/game/cursors/repair.dds',
        selectable = true,
    }
    local ok, result = pcall(doscript, filename, env)
    if ok then
        env.id = filename
        env.location = Dirname(filename)
        return env
    else
        WARN("Problem loading " .. filename .. ":\n" .. result)
        return nil
    end
end


# Update which mods are actually active, based on which ones have been requested
# from the mod manager and which ones get auto-applied to the current scenario.
local function UpdateActiveMods()

    local active = {}

    # First activate requested mods...
    for i,id in requested_active_ids do
        if all_mods[id] then
            active[id] = true
        end
    end

    # Then any auto-activate mods...
    for id,m in all_mods do
        if m.auto_apply and not active[id] then
            for i,pat in m.auto_apply do
                if __scenario_filename:find(pat) then
                    active[id] = true
                end
            end
        end
    end

    local new_active_mods = {}
    for id,m in sortedpairs(all_mods) do
        m.active = active[id] or false
        if m.active then
            table.insert(new_active_mods,m)
        end
    end
    active_mods = new_active_mods
end


# Return a table of all mods found
function InitModInfo()
    local r = {}
    for i,file in DiskFindFiles('/mods/*mod_info.lua') do
        local mod = LoadModInfo(file)
        if mod then
            r[mod.id] = mod
        end
    end
    all_mods = r

    # Initialize active mods list from preferences. It can be changed by calling RequestActiveMods().

    # FIXME: The GetPreference() call broke when I moved the rule
    # loading out of the user lua into it's own lua state.  [wlott]

    # requested_active_ids = GetPreference('active_mods') or {}
    requested_active_ids = {}

    UpdateActiveMods()
end


# Return a table of all mods found
function AllMods()
    if not all_mods then
        InitModInfo()
    end
    return all_mods
end


# Return a table of active mods
function ActiveMods()
    if not active_mods then
        InitModInfo()
    end
    return active_mods
end


# Set the list of active mods requested by the user from the mod manager
function RequestActiveMods(list)
    LOG("MOD LIST SET TO:")
    LOG(repr(list))

    requested_active_ids = list
    SetPreference('active_mods', list)
    UpdateActiveMods()
end
